@@ -12,7 +12,7 @@ PUBLIC void MODULE_MODULEN::usage (SSTRINGS &tb)
 	tb.add (new SSTRING (MSG_R(T_USAGE)));
 }
 
-PUBLIC int MODULE_MODULEN::execmain (int argc , char *argv[])
+PUBLIC int MODULE_MODULEN::execmain (int argc , char *argv[], bool standalone)
 {
 	int ret = LNCF_NOT_APPLICABLE;
 	const char *pt = strrchr(argv[0],'/');
@@ -24,7 +24,10 @@ PUBLIC int MODULE_MODULEN::execmain (int argc , char *argv[])
 	if (strcmp(pt,"MODULEN")==0){
 		ret = -1;
 		if (argc == 1){
-			// ### Place call to main menu of the module
+			CONTEXT_LOCK l("MODULEN");
+			if (l.isok()){
+				// ### Place call to main menu of the module
+			}
 		}else{
 			// ### Add some option parsing for the module
 			::usage();
