@@ -16,7 +16,10 @@ PUBLIC int MODULE_MODULEN::domenu (
 {
 	if (context == MENUID){
 		if (key == keymenu){
-			// ### Place the call to the edit function here
+			CONTEXT_LOCK l("MODULEN");
+			if (l.isok()){
+				// ### Place the call to the edit function here
+			}
 		}
 	}
 	return 0;
