@@ -1,4 +1,5 @@
-/* diajava.cc 02/02/2000 10.39.02 */
+/* diajava.cc 13/08/2001 21.16.30 */
+PUBLIC UISTATE::UISTATE (void);
 PUBLIC DIAJAVA::DIAJAVA (bool guiok);
 PUBLIC DIAJAVA::DIAJAVA (int handlein,
 	 int handleout,
@@ -22,6 +23,7 @@ PUBLIC int DIAJAVA::getvals (const char *diapath,
 PUBLIC int DIAJAVA::wait (POPENWAITS&tbpopen,
 	 char dianame[200],
 	 char actionid[100],
+	 char menubarid[100],
 	 int &menu,
 	 int &but);
 PUBLIC void DIAJAVA::send (const char *ctl, ...);
