@@ -22,31 +22,40 @@ class extrstr{
 	// Get the next word. A word start with a non-white and end
 	// at a white char
 	public String nextword(){
+		String ret = "";
 		skip();		
-		int start = offset;
-		while (true){
-			if (offset == len) break;
-			char c = s.charAt(offset);
-			if (c <= ' ') break;
-			offset++;
+		if (offset < len){
+			int start = offset;
+			while (true){
+				if (offset == len) break;
+				char c = s.charAt(offset);
+				if (c <= ' ') break;
+				offset++;
+			}
+			ret = s.substring (start,offset);
 		}
-		return s.substring (start,offset);
+		return ret;
 	}
 	// Get the next "word", but understand quoting
 	// The quote will be removed
 	public String nextwordq(){
 		skip();		
-		String ret = null;
-		if (s.charAt(offset) == '"'){
+		String ret = "";
+		if (offset < len && s.charAt(offset) == '"'){
 			offset++;
-			int start = offset;
 			while (true){
 				if (offset == len) break;
 				char c = s.charAt(offset);
-				if (c == '"') break;
+				if (c == '"'){
+					break;
+				}else if (c == '\\' && offset < len-1){
+					offset++;
+					ret += s.charAt(offset);
+				}else{
+					ret += c;
+				}
 				offset++;
 			}
-			ret = s.substring (start,offset);
 			if (offset < len) offset++;
 		}else{
 			ret = nextword();
@@ -64,5 +73,34 @@ class extrstr{
 		skip();
 		return s.substring (offset);
 	}
+	public int getparms(String tb[]){
+		int nb = 0;
+		for (int i=0; i<100; i++){
+			skip();		
+			if (offset >= len){
+				break;
+			}else{
+				tb[nb++] = nextwordq();
+			}
+		}
+		return nb;
+	}
+	public static int splitpath (String path, String tb[]){
+		int ret = 0;
+		int len = path.length();
+		int offset = 0;
+		int start = 0;
+		while (offset < len){
+			if (path.charAt(offset)== '.'){
+				tb[ret++] = new String(path.substring(start,offset));
+				start = offset+1;
+			}
+			offset++;
+		}
+		if (offset > start){
+			tb[ret++] = new String(path.substring(start,offset));
+		}
+		return ret;
+	}
 }
 
