@@ -49,5 +49,14 @@ class formbase extends Panel{
 		}
 		return ret;
 	}
+	public mainform gettop(){
+		mainform ret = null;
+		if (parent == null){
+			ret = (mainform)this;
+		}else{
+			ret = parent.gettop();
+		}
+		return ret;
+	}
 }
 
