@@ -18,11 +18,11 @@ class mainform extends mform{
 		init();
 	}
 	public void Popup(){
-		System.err.println ("Avant pack");
+		// System.err.println ("Avant pack");
 		fram.pack();
-		System.err.println ("Avant show");
+		// System.err.println ("Avant show");
 		fram.show();
-		System.err.println ("Apres show");
+		// System.err.println ("Apres show");
 	}
 	public void setTitle(String title){
 		fram.setTitle(title);
