@@ -48,9 +48,9 @@ class mform_layout implements LayoutManager {
 		int haut,
 		int bas,
 		boolean use_cur_size){
-		if (fl.parent == null) System.err.println ("fit start");
+		// if (fl.parent == null) System.err.println ("fit start");
 		System.err.flush();
-		//System.out.println ("Fitstrategie " + mform.getabspath(fl) + " " + fl.was_modified());
+		//System.err.println ("Fitstrategie " + mform.getabspath(fl) + " " + fl.was_modified());
 		//if (!fl.was_modified()) return;
 		int nb = fl.nbc;
 		gauche += fl.marge_gauche;
@@ -284,7 +284,7 @@ class mform_layout implements LayoutManager {
 		Dimension ret = new Dimension (h_pos+droit, v_pos + bas);
 		if (ret.width < fl.min_width) ret.width = fl.min_width;
 		fl.modified = false;
-		if (fl.parent == null) System.err.println ("fit end");
+		// if (fl.parent == null) System.err.println ("fit end");
 		return ret;
 	}
 
@@ -304,14 +304,14 @@ class mform_layout implements LayoutManager {
 	}
 	/* Required by LayoutManager. */
 	public Dimension preferredLayoutSize(Container parent) {
-		if (fl.parent == null) System.err.println ("preferredLayoutSize");
+		// if (fl.parent == null) System.err.println ("preferredLayoutSize");
 		if (sizeUnknown || fl.was_modified()){
 			Dimension ret = setsizes (parent,false);
 			minWidth = preferredWidth = ret.width;
 			minHeight = preferredHeight = ret.height;
 			sizeUnknown = false;
 		}
-		if (fl.parent == null) System.err.println ("fin preferredLayoutSize");
+		// if (fl.parent == null) System.err.println ("fin preferredLayoutSize");
 		return new Dimension (preferredWidth, preferredHeight);
 	}
 
@@ -385,6 +385,8 @@ class mform_c{
 	static final int T_FILL=14;
 	static final int T_TEXT=15;
 	static final int T_LABEL=16;
+	static final int T_BOOK=17;
+	static final int T_CLIST=18;
 	public int type;
 	public mform_c(String _id){
 		weightx = weighty = 0;
@@ -443,7 +445,7 @@ class mform extends formbase{
 	public boolean action(Event e, Object arg) {
 		boolean ret = false;
 		if (e.id == Event.WINDOW_DESTROY) {
-			System.out.println ("destroy");
+			System.err.println ("destroy");
 		}else if (id != null && id.compareTo("")!=0){
 			Object target = e.target;
     	    for (int i=0; i<nbc; i++){
@@ -454,7 +456,9 @@ class mform extends formbase{
 					while (mainf.parent != null) mainf = mainf.parent;
 					mainf.dump();
 					String path = getabspath(this);
-					System.out.println ("action " + path + " " + tbc[i].id);
+System.err.println ("action " + path + " " + tbc[i].id);
+					mformscript.out.println ("action " + path + " " + tbc[i].id);
+					mformscript.out.flush();
 					ret = true;
 					break;
 				}
@@ -536,10 +540,10 @@ class mform extends formbase{
 	public TextField New_string (int len, String initval){
 		return New_string ("",len,initval);
 	}
-	public TextArea New_text (String initval, int rows, int cols){
-		mform_c m = alloc_mf("");
-		m.s = initval;
-		TextArea ret = new TextArea(initval,rows,cols);
+	public TextArea New_text (String _id, int rows, int cols){
+		mform_c m = alloc_mf(_id);
+		m.s = "";
+		TextArea ret = new TextArea("",rows,cols);
 		m.c = ret;
 		m.type = mform_c.T_TEXT;
 		add(m.c);
@@ -603,9 +607,9 @@ class mform extends formbase{
 			}
 		}
 	}
-	public List New_list(String _id, int nbvisible, String val){
+	public java.awt.List New_list(String _id, int nbvisible, String val){
 		mform_c m = alloc_mf (_id);
-		List ret = new List (nbvisible,false);
+		java.awt.List ret = new java.awt.List (nbvisible,false);
 		m.c = ret;
 		m.s = val;
 		m.type = mform_c.T_LIST;
@@ -616,7 +620,7 @@ class mform extends formbase{
 		for (int i=0; i<nbc; i++){
 			mform_c m = tbc[i];
 			if (m.type == mform_c.T_LIST && m.id.compareTo(_id)==0){
-				List c = (List)m.c;
+				java.awt.List c = (java.awt.List)m.c;
 				c.addItem(val);
 				if (m.s.compareTo(val)==0) c.select(c.countItems()-1);
 				break;
@@ -656,6 +660,18 @@ class mform extends formbase{
 		add(m.c);
 		return ret;
 	}
+	public void New_book(book sub){
+		mform_c m = alloc_mf();
+		m.c = sub;
+		m.type = mform_c.T_BOOK;
+		add(sub);
+	}
+	public void New_clist(clist sub){
+		mform_c m = alloc_mf();
+		m.c = sub;
+		m.type = mform_c.T_CLIST;
+		add(sub);
+	}
 	public formbutton New_formbutton(String _id){
 		formbutton ret = new formbutton (this,_id);
 		New_form (ret);
@@ -744,12 +760,12 @@ class mform extends formbase{
 				}else if (c.type == mform_c.T_COMBO){
 					val = ((combo)c.c).getText();
 				}else if (c.type == mform_c.T_LIST){
-					int sel = ((List)c.c).getSelectedIndex();
+					int sel = ((java.awt.List)c.c).getSelectedIndex();
 					val = String.valueOf(sel);
 				}else{
 					System.err.println ("code dump " + c.type);
 				}
-				System.out.println ("dump " + getabspath(this)
+				mformscript.out.println ("dump " + getabspath(this)
 					+ " " + c.id + " " + val);
 			}
 		}
@@ -879,6 +895,30 @@ class mform extends formbase{
 			item.c.resize(d.width,d.height+diffy*item.weighty/totalv);
 		}
 	}
+	public boolean locate (String _id, mformctx ctx){
+		boolean ret = false;
+		for (int i=0; i<nbc; i++){
+			mform_c item = getitem(i);
+			if (item.type == mform_c.T_FORM){
+				mform m = (mform)item.c;
+				if (m.id.compareTo(_id)==0){
+					ctx.fl = m;
+					ctx.bk = null;
+					ret = true;
+					break;
+				}
+			}else if (item.type == mform_c.T_BOOK){
+				book m = (book)item.c;
+				if (m.id.compareTo(_id)==0){
+					ctx.fl = null;
+					ctx.bk = m;
+					ret = true;
+					break;
+				}
+			}
+		}
+		return ret;
+	}
 }
 
 
