@@ -52,3 +52,31 @@
 #define P_Setval 51
 #define P_Alive 52
 #define P_Setcontext 53
+#define P_Defdc 54
+#define P_Deffont 55
+#define P_Defpen 56
+#define P_Defbrush 57
+#define P_Drawline 58
+#define P_Drawrect 59
+#define P_Drawarc 60
+#define P_Fillrect 61
+#define P_Fillarc 62
+#define P_Menubar 63
+#define P_Popupmenu 64
+#define P_Submenu 65
+#define P_Menuentry 66
+#define P_Clear 67
+#define P_Drawclip 68
+#define P_Setcursor 69
+#define P_Blit 70
+#define P_Drawtext 71
+#define P_Refresh 72
+#define P_Menutitle 73
+#define P_Listen 74
+#define P_Prefsubmenu 75
+#define P_Show 76
+#define P_Inputgrid 77
+#define P_Fillpolygon 78
+#define P_Splash 79
+#define P_Hidesplash 80
+#define P_Deletechild 81
