@@ -54,5 +54,33 @@ public static String tb[]={
 "Setval",
 "Alive",
 "Setcontext",
+"Defdc",
+"Deffont",
+"Defpen",
+"Defbrush",
+"Drawline",
+"Drawrect",
+"Drawarc",
+"Fillrect",
+"Fillarc",
+"Menubar",
+"Popupmenu",
+"Submenu",
+"Menuentry",
+"Clear",
+"Drawclip",
+"Setcursor",
+"Blit",
+"Drawtext",
+"Refresh",
+"Menutitle",
+"Listen",
+"Prefsubmenu",
+"Show",
+"Inputgrid",
+"Fillpolygon",
+"Splash",
+"Hidesplash",
+"Deletechild",
 };
 }
