@@ -5,10 +5,15 @@
 class ELM_STR: public ARRAY_OBJ{
 public:
 	char *value;
+	char *shown;		// value shown. Generally the same as value
+						// except when we want to present the value
+						// in a different language
 	char *verbose;
 	/*~PROTOBEG~ ELM_STR */
 public:
-	ELM_STR (const char *_value, const char *_verbose);
+	ELM_STR (const char *_value,
+		 const char *_shown,
+		 const char *_verbose);
 	~ELM_STR (void);
 	/*~PROTOEND~ ELM_STR */
 };
@@ -26,8 +31,8 @@ public:
 class LIST_STR: public ARRAY{
 	/*~PROTOBEG~ LIST_STR */
 public:
-	ELM_STR *getitem (int no);
-	const char *getvalue (int no);
+	ELM_STR *getitem (int no)const;
+	const char *getshown (int no)const;
 	/*~PROTOEND~ LIST_STR */
 };
 
