@@ -3,7 +3,7 @@
 #include "fcombo.h"
 
 PUBLIC ELM_STR_V::ELM_STR_V(const char *_str, char &_selected)
-	: ELM_STR(" ",_str),selected(_selected)
+	: ELM_STR(" "," ",_str),selected(_selected)
 {
 }
 
@@ -14,7 +14,7 @@ PUBLIC ELM_STR_V::ELM_STR_V(const char *_str, char &_selected)
 PUBLIC FIELD_COMBO_MANY::FIELD_COMBO_MANY(
 	const char *_prompt,
 	SSTRING &_str)
-	: FIELD_COMBO(_prompt,_str)
+	: FIELD_COMBO(_prompt,_str,30)
 {
 }
 
