@@ -21,12 +21,20 @@ PUBLIC FIELD_STRING_HELP::FIELD_STRING_HELP(
 {
 }
 
+PUBLIC FIELD_STRING_HELP::FIELD_STRING_HELP(
+	const char *_prompt,
+	SSTRING &_str,
+	int _maxsiz)
+	: FIELD_SSTRING (_prompt,_str,_maxsiz)
+{
+}
+
 /*
 	Draw only the input part of a field
 */
-PUBLIC void FIELD_STRING_HELP::drawtxt (WINDOW *dialog)
+PUBLIC void FIELD_STRING_HELP::drawtxt (WINDOW *dialog, int offset)
 {
-	FIELD_STRING_BASE::drawtxt(dialog);
+	FIELD_STRING_BASE::drawtxt(dialog, offset);
 	if (!is_readonly()){
 		// Add the little button
 		wattrset(dialog, inputbox_attr);
@@ -35,15 +43,18 @@ PUBLIC void FIELD_STRING_HELP::drawtxt (WINDOW *dialog)
 	}
 }
 
-PUBLIC void FIELD_STRING_HELP::dokey (
+PUBLIC MENU_STATUS FIELD_STRING_HELP::dokey (
 	WINDOW *dialog,
 	int key,
-	FIELD_MSG &msg)
+	FIELD_MSG &msg,
+	bool &grab)
 {
+	MENU_STATUS ret = MENU_NULL;
 	if (key == 24 || key == KEY_F(4)){
 		if (!is_readonly()) assist(dialog);
 	}else{
-		FIELD_SSTRING::dokey(dialog,key,msg);
+		ret = FIELD_SSTRING::dokey(dialog,key,msg,grab);
 	}
+	return ret;
 }
 
