@@ -124,6 +124,11 @@ void html_stresc (char *dst, const char *src, int size)
 			dst = stpcpy (dst,"&amp;");
 		}else if (carac == '"'){
 			dst = stpcpy (dst,"&quot;");
+		}else if (carac == ' '){
+			// We put a invisible white space so it is preserved
+			// Later when we decode (check for nbsp later in this file)
+			// we turn the %A0 code into a normal space
+			dst = stpcpy (dst,"&nbsp;");
 		}else{
 			*dst++=carac;
 		}
@@ -187,7 +192,7 @@ void html_defvarcur (
 static char *html_buf;
 static int html_len;
 static int html_maxlen;
-
+static const int httpmaxsize=40000;
 /*
 	Add to the buffer which will be transmitted to the browser.
 	html_flush() should be called once all the html_printf have been done.
@@ -198,7 +203,7 @@ EXPORT int html_printf (const char *ctl, ...)
 {
 	va_list list;
 	va_start (list,ctl);
-	char buf[10000];
+	char buf[20000];
 	int len = vsprintf (buf,ctl,list);
 	va_end (list);
 	if (html_len + len > html_maxlen){
@@ -346,6 +351,7 @@ void html_formatkey (char *key, const char *ctl, ...)
 	while (*pt != '\0'){
 		char carac = *pt++;
 		if (carac == ' '
+			|| carac == '\n'
 			|| carac == '/'
 			|| carac == '>'
 			|| carac == '\t'
@@ -353,7 +359,7 @@ void html_formatkey (char *key, const char *ctl, ...)
 			*key++ = '=';
 			*key++ = '=';
 		}else if (carac >= 128){
-			// Encore the character so browsers stop mangling them
+			// Encode the character so browsers stop mangling them
 			*key++ = '=';
 			*key++ = tohex (carac >> 4);
 			*key++ = tohex (carac & 0xf);
@@ -421,7 +427,7 @@ void html_setdone()
 	Return the "cut" information recorded in a previous "run".
 	Record at the same time the cut level which will be used by setcutinfo.
 */
-const char *html_getcutinfo ()
+EXPORT const char *html_getcutinfo ()
 {
 	/* #Specification: html mode / cut
 		Complex dialog sequences in "HTML mode" work based on one important
@@ -539,7 +545,7 @@ const char *html_getcutinfo ()
 	html_getcutinfo() must have been called before this function, generally
 	1 or more dialogs before.
 */
-void html_setcutinfo (const char *s)
+EXPORT void html_setcutinfo (const char *s)
 {
 	assert (cut_level != -1);
 	tblevel[cut_level].key.setfrom (s);
@@ -594,16 +600,18 @@ void html_needpasswd()
 
 static void html_draw_history()
 {
+	html_printf ("<table border=0><tr>\n");
 	for (int i=0; i<history_level; i++){
 		char path[PATH_MAX];
 		html_setpath_level (path,i);
-		html_printf ("<font size=2 bgcolor=%s>"
+		html_printf ("<td bgcolor=%s><font size=2>"
 			"<a href=\"%s/%s\">%s</a></font>\n\n"
-			,i&1 ? "#ffff99" : "white"
+			,i&1 ? "orange" : "orange"
 			,html_key
 			,path
 			,tblevel[i].title.get());
 	}
+	html_printf ("</table>\n");
 	if (history_level > 0){
 		html_printf ("<br>\n");
 	}
@@ -611,25 +619,32 @@ static void html_draw_history()
 
 static const char K_HTML[]="html";
 static const char K_BODYPARM[]="bodyparm";
+static const char K_HEADPARM[]="headparm";
 
 /*
-	Return the configurable parameter for the html BODY directive
+	Return the configurable parameters for the html BODY and HEAD directives
 */
-const char *html_getbodyparm()
+void html_getpageparm(SSTRING &body, SSTRING &head)
 {
-	return linuxconf_getval (K_HTML,K_BODYPARM,"");
+	body.setfrom (linuxconf_getval (K_HTML,K_BODYPARM,""));
+	head.setfrom (linuxconf_getval (K_HTML,K_HEADPARM,""));
 }
 /*
-	Save the configurable parameter for the html BODY directive
+	Save the configurable parameter for the HTML BODY and HEAD directives.
 	The application must call linuxconf_save() after that.
 */
-void html_setbodyparm(const char *parm)
+void html_setpageparm(const char *body, const char *head)
 {
-	if (parm == NULL || parm[0] == '\0'){
+	linuxconf_setcursys (subsys_stationid);
+	if (body == NULL || body[0] == '\0'){
 		linuxconf_removeall (K_HTML,K_BODYPARM);
 	}else{
-		linuxconf_setcursys (subsys_stationid);
-		linuxconf_replace (K_HTML,K_BODYPARM,parm);
+		linuxconf_replace (K_HTML,K_BODYPARM,body);
+	}
+	if (head == NULL || head[0] == '\0'){
+		linuxconf_removeall (K_HTML,K_HEADPARM);
+	}else{
+		linuxconf_replace (K_HTML,K_HEADPARM,head);
 	}
 }
 
@@ -638,7 +653,7 @@ PRIVATE void DIALOG::html_draw_top()
 	/* #Specification: html mode / strategy / back on our feet
 		Maybe this is an already visited dialog. This has been visited
 		along the path followed while running through tblevel[]. While
-		doing so, we have stamp dialog title in each level (in
+		doing so, we have stamped dialog title in each level (in
 		DIALOG::edithtml()). We will search now to see if we can find
 		this dialog title in tblevel and correct target_level accordingly.
 
@@ -659,8 +674,7 @@ PRIVATE void DIALOG::html_draw_top()
 		go the next. So we have a very long path, but we have to
 		fall on our feet again with a short path.
 	*/
-	int i;
-	for (i=0; i<target_level; i++){
+	for (int i=0; i<target_level; i++){
 		if (tblevel[i].title.cmp(internal->title) ==0){
 			target_level = i;
 			level = i;
@@ -668,20 +682,21 @@ PRIVATE void DIALOG::html_draw_top()
 		}
 	}
 	html_sendintro ("text/html",-1,5,true);
+	SSTRING body,head;
+	html_getpageparm(body,head);
 	html_printf (
 		"<HTML>\n"
-		"<HEAD>\n"
+		"<HEAD>%s\n"
 		"<TITLE>%s:%s</TITLE>\n"
 		"</HEAD>\n"
-		,html_host,internal->title.get());
+		,head.get(),html_host,internal->title.get());
 	if (internal->html_bypass.top.is_empty()){
 		if (!internal->internal_title.is_empty()){
 			html_printf ("<center><h2>%s</h2></center>\n"
 				,internal->internal_title.get());
 		}
-		const char *body = html_getbodyparm();
-		if (body[0] == '\0') body = "bgcolor=white";
-		html_printf ("<body %s>\n",body);
+		if (body.is_empty()) body.setfrom ("bgcolor=white");
+		html_printf ("<body %s>\n",body.get());
 		// THis title was taking too much space
 		//html_printf ("<center><h1>%s</h1></center>\n",internal->title.get());
 		//html_printf ("<p>\n<hr>\n<p>\n");
@@ -786,24 +801,51 @@ PRIVATE void DIALOG::html_draw (int nof)
 	Return -1 if any error.
 */
 
-PRIVATE int DIALOG::html_validate ()
+PRIVATE int DIALOG::html_validate (SSTRING &msg)
 {
 	int ret = 0;
 	int lastf = getnb();
 	for (int i=0; i<lastf; i++){
 		FIELD *f = getitem(i);
-		if (!f->is_readonly()) ret |= f->html_validate (i);
+		if (!f->is_readonly()){
+			int ok = f->html_validate (i);
+			if (ok != 0){
+				if (msg.is_empty()){
+					msg.setfromf (MSG_U(F_DIALOG,"Dialog %s\r\n<p>\r\n")
+						,internal->title.get());
+				}
+				msg.appendf (MSG_U(F_FIELDSTATE
+					,"Field number %d (%s) out of sync<br>\n")
+					,i+1,f->prompt);
+			}
+			ret |= ok;
+		}
 	}
 	return ret;
 }
 
+static void html_encodespaces(char *dst, const char *src)
+{
+	while (*src != '\0'){
+		char car = *src++;
+		if (car == ' ' || car >= 128){
+			*dst++ = '=';
+			*dst++ = tohex (car >> 4);
+			*dst++ = tohex (car & 0xf);
+		}else{
+			*dst++ = car;
+		}
+	}
+	*dst = '\0';
+}
+
 PUBLIC void BUTTONS_INFO::html_draw()
 {
 	for (int i=0; i<nb; i++){
 		MENU_STATUS code = tbret[i];
 		const char *name = "nil";
 		if (code == MENU_HELP){
-			html_printf ("<a href=\"/help:%s.html\">"
+			html_printf ("<a href=\"/help:%s.html\" target=\"_blank\">"
 				"<img border=0 src=\"/images:images/%s.png\" ALT=\"%s\"></a>\n"
 				,helpfile.get()
 				,tb_icon[i]
@@ -842,9 +884,11 @@ PUBLIC void BUTTONS_INFO::html_draw()
 			}else{
 				fprintf (stderr,"old button\n");
 			}
+			char key[PATH_MAX];
+			html_encodespaces (key,tb_icon[i]);
 			html_printf ("<input type=image name=%s "
 				 "border=0 src=\"/images:images/%s.png\" ALT=\"%s\">\n"
-				,name,tb_icon[i],tb_title[i]);
+				,name,key,tb_title[i]);
 		}
 	}
 }
@@ -887,14 +931,16 @@ int html_locatefile (
 	int maxlen)
 {
 	int ret = -1;
-	html_initbasepaths();
-	path[0] = '\0';
-	for (int i=0; i<html_basepaths.getnb(); i++){
-		snprintf (path,maxlen-1,"%s/%s%s",html_basepaths.getitem(i)->get(),fname
-			,extension);
-		if (file_exist (path)){
-			ret = 0;
-			break;
+	if (strstr(fname,"..")==NULL){
+		html_initbasepaths();
+		path[0] = '\0';
+		for (int i=0; i<html_basepaths.getnb(); i++){
+			snprintf (path,maxlen-1,"%s/%s%s",html_basepaths.getitem(i)->get(),fname
+				,extension);
+			if (file_exist (path)){
+				ret = 0;
+				break;
+			}
 		}
 	}
 	return ret;
@@ -944,11 +990,24 @@ static void html_copy (const char *fname, int dointro)
 					,size,24*60*60,false);
 			}
 			html_flush();
-			char buf[10000];
+			char buf[3*4096];
 			int n;
-			while ((n=fread(buf,1,sizeof(buf),fin))> 0){
+			while ((n=fread(buf,1,sizeof(buf)-1,fin))> 0){
 				//if (debug) fwrite (buf,1,n,stderr);
-				write (html_cli,buf,n);
+				buf[n] = '\0';
+				char *pt = strstr(buf,"$(HOSTNAME)");
+				if (pt == NULL){
+					write (html_cli,buf,n);
+				}else{
+					if (pt > buf) write(html_cli,buf,(int)(pt-buf));
+					// Plug the hostname
+					write (html_cli,html_host,strlen(html_host));
+					pt += 11;
+					int len = n - (int)(pt-buf);
+					if (len > 0){
+						write (html_cli,pt,len);
+					}
+				}
 			}
 			html_printf ("\r\n");
 			html_flush();
@@ -1053,7 +1112,8 @@ PRIVATE MENU_STATUS DIALOG::edithtml(int &nof)
 				static MENU_STATUS was_button;	// Was it a POST with the
 												// button accept
 				if (curvars != NULL && !html_postdone){
-					if (html_validate() != -1){
+					SSTRING msg;
+					if (html_validate(msg) != -1){
 						// Those names need not be translated
 						// see BUTTONS_INFO::html_draw()
 						if (html_butexist("ok.x")){
@@ -1087,8 +1147,10 @@ PRIVATE MENU_STATUS DIALOG::edithtml(int &nof)
 						was_button = ret;
 						html_postdone = 1;
 					}else{
-						html_printf ("500 %s\r\n"
+						html_draw_top();
+						html_printf ("500 %s<p>\r\n"
 							,MSG_U(E_MISMATCH,"dialog state mismatch"));
+						html_printf ("<hr>%s\n",msg.get());
 						html_setdone();
 					}
 				}else{
@@ -1145,8 +1207,11 @@ PRIVATE MENU_STATUS DIALOG::edithtml(int &nof)
 					ret = pt->status;
 					other_vars = varval_get(pt->key.getval());
 					//printf ("Intermediate validate level %d %d\n",level,ret);
-					if (html_validate() == -1){
-						html_printf ("500 %s\r\n",MSG_R(E_MISMATCH));
+					SSTRING msg;
+					if (html_validate(msg) == -1){
+						html_draw_top();
+						html_printf ("500 %s<p>\r\n",MSG_R(E_MISMATCH));
+						html_printf ("<hr>%s\n",msg.get());
 						html_setdone();
 					}
 					//printf ("Intermediate validate level %d end\n",level);
@@ -1270,6 +1335,15 @@ static const char *html_decode (const char *str, char *buf, int sizbuf)
 	}
 	*pt = '\0';
 	strip_end (buf);
+	{
+		// Check if there was some &nbsp; placed by the encoding earlier
+		// This is done after the strip_end.
+		pt = buf;
+		while (*pt != '\0'){
+			if (*pt == 0xa0) *pt = ' ';
+			pt++;
+		}
+	}
 	if (*str == '\n') str++;
 	return str;
 }
@@ -1317,6 +1391,24 @@ const char *http_getbody()
 	return htmlbody.get();
 }
 
+
+/*
+	Decode spaces (and only spaces) encoded as =20
+*/
+static void html_decodespaces (char *dst, const char *src, int maxlen)
+{
+	while (*src != '\0' && maxlen > 0){
+		if (src[0] == '=' && src[1] == '2' && src[2] == '0'){
+			src += 3;	
+			*dst++ = ' ';
+		}else{
+			*dst++ = *src++;
+		}
+		maxlen--;
+	}
+	*dst = '\0';
+}
+
 /*
 	Parse a potentially completed header.
 	Extract the "get" command or "post" command.
@@ -1349,7 +1441,7 @@ static int html_parse (
 	history_level = level = target_level = 0;
 	targethost.setfrom ("");
 	while (*str != '\0' && ret != -1){
-		char buf[10000];
+		char buf[httpmaxsize];
 		str = html_decode (str,buf,sizeof(buf));
 		if (buf[0] == '\0'){
 			if (get_ok){
@@ -1394,7 +1486,8 @@ static int html_parse (
 				}else if (strncmp(path,"/help:",6)==0){
 					str_copyword (file_request,path+6,PATH_MAX-1);
 				}else if (strncmp(path,"/images:",8)==0){
-					str_copyword (file_request,path+8,PATH_MAX-1);
+					html_decodespaces (file_request,path+8,PATH_MAX-1);
+					// str_copyword (file_request,path+8,PATH_MAX-1);
 				}else if (strncmp(path,"/remadm:",8)==0){
 					remadmin = true;
 				}else if (strncmp(path,"/html:",6)==0){
@@ -1423,7 +1516,7 @@ static int html_parse (
 				targethost.setfromf ("//%s",str_skip(pt));
 			}else if (stricmp(cmd,"Authorization:")==0){
 				char basic[1000];
-				pt = str_copyword (basic,pt,sizeof(basic));
+				pt = str_copyword (basic,pt,sizeof(basic)-1);
 				if (stricmp(basic,"Basic")==0){
 					pt = str_copyword (basic,pt,sizeof(basic));
 					char pw[1000];
@@ -1476,7 +1569,7 @@ int html_get (
 	if (cmd == NULL) cmd = new CMDSOCK (debug ? port : -1,1);
 	int ret = -1;
 	while (1){
-		int ok = cmd->listen(timeout*60*1000000);
+		int ok = cmd->listen(timeout*60);
 		if (ok <= 0){
 			if (errno != EINTR) break;
 		}else{
@@ -1491,7 +1584,10 @@ int html_get (
 				}else if (tbs[cli] == NULL
 					&& html_access_check(cli)!=0){
 					html_cli = cli;
-					html_printf ("500 access denied: Check networking/linuxconf network access\r\n");
+					html_printf ("500 access denied: Check config/networking/misc/linuxconf network access\r\n");
+					html_printf ("<p>\r\n");
+					html_printf ("By default, linuxconf network access is disabled<br>\r\n");
+					html_printf ("Access is generally granted for few locations only<br>\r\n");
 					html_flush();
 					cmd->closecli (cli);
 				}else{
@@ -1500,12 +1596,12 @@ int html_get (
 						tbs[cli] = new SSTRING;
 					}
 					tbs[cli]->append (buf);
-					if (tbs[cli]->getlen()>10000){
+					if (tbs[cli]->getlen()>httpmaxsize){
 						/* #Specification: html mode / input overflow
 							If linuxconf receive an http request exceding
-							10000 bytes, it is silently flushed.
+							20000 bytes, it is silently flushed.
 						*/
-						html_access_log(cli,MSG_U(E_IVLDHTMLREQ,"Invalid html request"));
+						html_access_log(cli,MSG_U(E_IVLDHTMLREQ,"Invalid html request, too long"));
 						cmd->closecli(cli);
 						delete tbs[cli];
 						tbs[cli] = NULL;
@@ -1587,7 +1683,7 @@ int html_getourname(char *name)
 			ret = 0;
 		}else{
 			long a = ntohl (*(long*)(&adr.sin_addr));
-			syslog (LOG_ERR,"Can't convert IP number %u.%u.%u.%u to name, using main domain"
+			syslog (LOG_ERR,"Can't convert IP number %lu.%lu.%lu.%lu to name, using main domain"
 				,(a>>24)&0xff,(a>>16)&0xff,(a>>8)&0xff,a&0xff);
 		}
 	}else{
