@@ -37,7 +37,8 @@ EXPORT MENU_STATUS dialog_inputpass(
 {
 	DIALOG dia;
 	SSTRING tmp(inpstr);
-	dia.newf_pass ("",tmp);
+	FIELD_PASSWORD *pass = dia.newf_pass ("",tmp);
+	pass->set_guiparms("enter=B98");
 	MENU_STATUS ret = dia.edit (title,prompt,helpfile);
 	tmp.copy (inpstr);
 	return ret;
