@@ -5,13 +5,14 @@
 #include "dialog.m"
 #include "internal.h"
 #include "../diajava/proto.h"
+#include <diajava.h>
 
 
 PUBLIC FIELD_MENU::FIELD_MENU(
 	const char *_icon,
 	const char *_tag,
 	const char *_str)
-	: FIELD_STRING ("",(char*)_str,60)
+	: FIELD_STRING ("",(char*)_str,200,false)
 {
 	key_type = HTML_KEY_FULL;
 	icon = NULL;
@@ -126,6 +127,11 @@ PUBLIC const char *FIELD_MENU::getmenustr() const
 	return str;
 }
 
+PUBLIC const char *FIELD_MENU::getmenuicon() const
+{
+	return icon;
+}
+
 /*
 	A FIELD_MENU is not really a string editor.
 	The following function are just there to prevent writing
@@ -137,73 +143,102 @@ PROTECTED void FIELD_MENU::save()
 PROTECTED void FIELD_MENU::restore()
 {
 }
-PROTECTED void FIELD_MENU::dokey(
+PROTECTED MENU_STATUS FIELD_MENU::dokey(
 	WINDOW *,
 	int,
-	FIELD_MSG & )
+	FIELD_MSG &,
+	bool & )
 {
+	return MENU_NULL;
 }
+static void menubox_addch (
+	WINDOW *win,
+	const char ch,
+	const int offset,
+	int &pos)
+{
+	if (pos >= offset) waddch (win,ch);
+	pos++;
+}
+static void menubox_addstr (
+	WINDOW *win,
+	const char *str,
+	const int offset,
+	int &pos)
+{
+	while (*str != '\0') menubox_addch (win,*str++,offset,pos);
+}
+
 void menubox_drawcols(
 	const char *pt,
 	int tbcol[],
 	int max_width,
-	WINDOW *win)
+	WINDOW *win,
+	const int hoffset,
+	int pos)
 {
 	int nocol = 0;
 	int ptcol = 0;
-	while (*pt != '\0' && ptcol < max_width){
+	int offset = 0;
+	max_width += hoffset;
+	while (*pt != '\0' && offset < max_width){
 		if (*pt == '\t'){
 			int wcol = tbcol[nocol];
-			while (ptcol < wcol){
-				waddch(win,' ');
+			while (ptcol < wcol && offset < max_width){
+				menubox_addch(win,' ',hoffset,pos);
 				ptcol++;
+				offset++;
 			}
 			ptcol = 0;
 			nocol++;
 		}else{
-			waddch(win, *pt);
+			menubox_addch(win, *pt,hoffset,pos);
 			ptcol++;
+			offset++;
 		}
 		pt++;
 	}
 }
+
+
 /*
 	Print menu item
 */
-PUBLIC void FIELD_MENU::drawgen(WINDOW *win, int selected)
+PUBLIC void FIELD_MENU::drawgen(WINDOW *win, bool selected, int offset)
 {
 	wattrset(win, menubox_attr);
 	wmove(win, box.y, box.x);
-	int i;
 	// Cleanup
-	for (i = 0; i < box.width; i++) waddch(win, ' ');
+	for (int i = 0; i < box.width; i++) waddch(win, ' ');
 	wmove(win, box.y, box.x);
 	char *pttag = tag;
+	int pos = 0;
 	if (tag[0] != ' ' && tag[0] != '\0'){
 		wattrset(win, selected ? tag_key_selected_attr : tag_key_attr);
-		waddch(win, tag[0]);
+		menubox_addch(win, tag[0],offset,pos);
 		pttag++;
 	}
 	wattrset(win, selected ? tag_selected_attr : tag_attr);
-	waddstr(win, pttag);
-	wmove(win, box.y, box.x + tbcol[0]);
+	menubox_addstr(win, pttag,offset,pos);
+	while (pos < tbcol[0]) menubox_addch (win,' ',offset,pos);
+	// wmove(win, box.y, box.x + tbcol[0]);
 	wattrset(win, selected ? item_selected_attr : item_attr);
-	menubox_drawcols (buf,tbcol+1,box.width,win);
+	menubox_drawcols (buf,tbcol+1,box.width-tbcol[0],win,offset,pos);
 	wmove(win, box.y, box.x);
 }
 
-PUBLIC void FIELD_MENU::unselect(WINDOW *dialog)
+PUBLIC void FIELD_MENU::unselect(WINDOW *dialog, int offset)
 {
-	drawgen(dialog,0);
+	drawgen(dialog,false,offset);
 }
 
-PUBLIC void FIELD_MENU::setcursor(WINDOW *dialog)
+PUBLIC void FIELD_MENU::setcursor(WINDOW *dialog, int offset)
 {
-	drawgen (dialog,1);
+	drawgen (dialog,true,offset);
 }
-PUBLIC void FIELD_MENU::drawtxt(WINDOW *dialog)
+PUBLIC void FIELD_MENU::drawtxt(WINDOW *dialog, int offset)
 {
-	drawgen (dialog,0);
+	drawgen (dialog,false,offset);
 }
 
 PUBLIC void FIELD_MENU::html_draw(int nof)
@@ -246,31 +281,21 @@ PUBLIC void FIELD_MENU::html_draw(int nof)
 	html_printf ("\n");
 }
 
-static bool menubox_gfx = true;
-
-/*
-	Record that the front end do no want any icons (to save bandwidth)
-*/
-void menubox_setnogfx ()
-{
-	menubox_gfx = false;
-}
-
-PUBLIC void FIELD_MENU::gui_draw(int nof, int &)
+PUBLIC void FIELD_MENU::gui_draw(int nof, SSTRINGS &)
 {
 	if (key_type == HTML_KEY_TAG || key_type == HTML_KEY_INDEX){
 		// Hack to differentiate tables from menu. To fix later...
-		ftitle_clist (nof,tag,buf,false,NULL,false);
+		ftitle_clist (nof,tag,buf,false,"\"\"",false);
 	}else{
-		if (menubox_gfx){
+		if (!diajava_nogfx){
 			if (icon != NULL){
 				char name_sent[PATH_MAX];
 				diagui_sendxpm (icon,name_sent);
-				if (may_select){
+				/*if (may_select){
 					diagui_sendcmd (P_Button_xpm,"M%d %s -\n",nof,name_sent);
-				}else{
+				}else{*/
 					diagui_sendcmd (P_Icon_xpm,"%s\n",name_sent);
-				}
+				//}
 			}else{
 				diagui_sendcmd (P_Skip,"1\n");
 			}
@@ -281,7 +306,7 @@ PUBLIC void FIELD_MENU::gui_draw(int nof, int &)
 		}else{
 			diagui_sendcmd (P_Skip,"1\n");
 		}
-		diagui_sendcmd (P_Dispolast,"l 1 c 1\n");
+		diagui_sendcmd (P_Dispolast,"c 1 c 1\n");
 		const char *pt = buf;
 		char tdbuf[300];
 		char *ptdst = tdbuf;
@@ -312,6 +337,26 @@ PUBLIC void FIELD_MENU::gui_draw(int nof, int &)
 		diagui_sendcmd (P_Dispolast,"l 1 c 1\n");
 	}
 }
+
+PUBLIC char FIELD_MENU::getidprefix ()
+{
+	return 'M';
+}
+
+
+PUBLIC void FIELD_MENU::popup_draw(int id, int &)
+{
+	if (buf[0] == '-'){
+		diagui_sendcmd (P_Menuentry,"0 -\n");
+	}else{
+		SSTRING s;
+		s.setfromf("%s %s",tag,buf);
+		char tmp[1000];
+		diagui_sendcmd (P_Menuentry,"%d %s\n",id,diagui_quote(s.get(),tmp));
+	}
+}
+
+
 PUBLIC int FIELD_MENU::html_validate(int)
 {
 	return 0;
@@ -390,7 +435,7 @@ PUBLIC void DIALOG::new_menuline (
 		*/
 		newf_title (prompt2,1,"",prompt2);
 	}else{
-		if (prompt1[0] == '\0') prompt1 = " ";
+		// if (prompt1[0] == '\0') prompt1 = " ";
 		FIELD_MENU *men = new FIELD_MENU (icon,prompt1,prompt2);
 		men->set_selectable (may_select);
 		add (men);
@@ -405,14 +450,14 @@ PUBLIC void DIALOG::new_menuline (
 */
 PUBLIC void DIALOG::new_menuitem (const char *prompt1, const char *prompt2)
 {
-	new_menuline (NULL,prompt1,prompt2,1);
+	new_menuline (NULL,prompt1,prompt2,true);
 }
 PUBLIC void DIALOG::new_menuitem (
 	const char *icon,
 	const char *prompt1,
 	const char *prompt2)
 {
-	new_menuline (icon,prompt1,prompt2,1);
+	new_menuline (icon,prompt1,prompt2,true);
 }
 /*
 	Redefine one line of a menu or list
@@ -424,25 +469,35 @@ PUBLIC void DIALOG::set_menuitem (
 {
 	FIELD *first = getitem(0);
 	if (first != NULL && first->is_head) no++;
-	if (prompt1[0] == '\0') prompt1 = " ";
+	// if (prompt1[0] == '\0') prompt1 = " ";
 	FIELD_MENU *old = (FIELD_MENU*)getitem(no);
 	if (old == NULL
 		|| strcmp(prompt1,old->tag)!= 0
 		|| strcmp(prompt2,old->buf)!= 0){
-		FIELD_MENU *men = new FIELD_MENU (NULL,prompt1,prompt2);
-		men->set_selectable (true);
-		if (no < getnb()){
-			ARRAY::remove_del (no);
-			insert (no,men);
+		bool setval;
+		if (old == NULL){
+			old = new FIELD_MENU(NULL,prompt1,prompt2);
+			old->set_selectable (true);
+			ARRAY::add (old);
+			setval = false;
 		}else{
-			ARRAY::add (men);
+			free (old->tag);
+			old->tag = strdup(prompt1);
+			strncpy (old->buf,prompt2,old->size);
+			old->buf[old->size] = '\0';
+			setval = true;
 		}
 		char tmp[200];
 		const char *dianame = setguiname(tmp);
 		if (dianame != NULL){
-			ftitle_clist (no,prompt1,prompt2,false,dianame,true);
+			char tmp1[1000];
+			if (first != NULL){
+				old->guipath.setfrom(first->guipath);
+				dianame = old->formatpath(tmp1,dianame);
+			}
+			ftitle_clist (no,prompt1,prompt2,false,dianame,setval);
 		}else{
-			internal->guidone = false;
+			reset_guidone();
 		}
 	}
 }
@@ -560,12 +615,12 @@ static void append_what (
 	}
 }
 
-static void (*fcollect)(const char *, const char *);
+static void (*fcollect)(const char *, const char *, const char *);
 /*
 	Register a collect function which will be called for each
 	menu option when running in tree mode
 */
-EXPORT void menubox_setcollect (void (*f)(const char *, const char *))
+EXPORT void menubox_setcollect (void (*f)(const char *, const char *, const char *))
 {
 	fcollect = f;
 }
@@ -595,7 +650,7 @@ PUBLIC VIRTUAL MENU_STATUS DIALOG::editmenu(
 	if (first != NULL && first->is_head) sel--;
 	return ret;
 }
-static void menubox_collect(int sublevel, const char *txt)
+static void menubox_collect(int sublevel, const char *txt, const char *icon)
 {
 	if (fcollect != NULL){
 		char key[100];
@@ -605,7 +660,7 @@ static void menubox_collect(int sublevel, const char *txt)
 		}
 		*ptkey = '\0';
 		//fprintf (stderr,"key :%s: :%s:\n",key,txt);
-		(*fcollect)(key,txt);
+		(*fcollect)(key,txt,icon);
 	}
 }
 
@@ -726,15 +781,16 @@ PUBLIC VIRTUAL MENU_STATUS DIALOG_MENU::editmenu(
 					treemenu_pos[ui_context.treemenu_level+1] = tblevel[sel][1]+1;
 					SSTRING ss;
 					ss.setfromf("%s %s",tags[sel],f->getmenustr());
+					const char *icon = f->getmenuicon();
 					if (f->may_select){
 						//FIELD_MENU *m = (FIELD_MENU*)f;
-						menubox_collect (sublevel,ss.get());
+						menubox_collect (sublevel,ss.get(),icon);
 						ret = MENU_OK;
 						// Set the level for the next dialog/menu
 						ui_context.treemenu_level += sublevel + 1;
 						break;
 					}else {
-						menubox_collect (0,ss.get());
+						menubox_collect (0,ss.get(),icon);
 					}
 				}
 				sel++;
@@ -861,3 +917,48 @@ PUBLIC void DIALOG_MENU::setmenu (MENU_CONTEXT ctx)
 	module_setmenu (*this,ctx);
 }
 
+/*
+	Present a Popup menu.
+	title may be NULL.
+	Return MENU_OK if a selection was made. sel contain the entry selected.
+*/
+PUBLIC MENU_STATUS DIALOG_MENUPOPUP::editmenu(const char *title, int &sel)
+{
+	MENU_STATUS ret = MENU_NULL;
+	if (title == NULL) title = "";
+	if (dialog_mode != DIALOG_GUI || !diajava_menu){
+		ret = DIALOG_MENU::editmenu (title,"",help_nil,sel,0);
+	}else{
+		char menuid[10];
+		extern int uithread_id;
+		sprintf (menuid,"menu-%d",uithread_id);
+		char tmp[1000];
+		diagui_sendcmd (P_Popupmenu,"%s %s\n",menuid,diagui_quote(title,tmp));
+		int curlevel = 0;
+		for (int i=0; i<getnb(); i++){
+			getitem(i)->popup_draw (i,curlevel);
+		}
+		for (int i=0; i<=curlevel; i++) diagui_sendcmd (P_End,"\n");
+		while (1){
+			SSTRING action,path,menubar;
+			diagui_sync (menuid,path,action,menubar);
+			if (!menubar.is_empty()){
+				sel = menubar.getval();
+				if (sel >= 0 && sel <getnb()){
+					ret = MENU_OK;
+				}else{
+					ret = MENU_QUIT;
+				}
+				break;
+			}
+		}
+	}
+	return ret;
+}
+
+
+PUBLIC DIALOG_MENUPOPUP::DIALOG_MENUPOPUP()
+{
+	settype (DIATYPE_MENUPOPUP);
+}
+
