@@ -1,5 +1,3 @@
-
-
 #include <stdio.h>
 #include <string.h>
 #include <context.h>
@@ -8,7 +6,8 @@
 #include "context.h"
 #include "internal.h"
 #include "../diajava/proto.h"
-
+#include <diajava.h>
+#include "modregister.h"
 /*
 	Evaluate the size of a text in a string.
 	(number of line, maximum width)
@@ -177,8 +176,8 @@ PRIVATE void DIALOG::setup ()
 		tmp += 4;
 		if (dialog_mode == DIALOG_CURSES
 			&& tmp >= COLS){
-			// Message should not be translated. It is for translator
-			//xconf_error ("Dialog introduction text too wide: deleted");
+			// The introduction is too wide, we fold the long
+			// lines in several lines
 			SSTRING newintro;
 			tmp_height = dialog_textfold (introtxt,COLS-4,newintro);
 			internal->intro.setfrom (newintro);
@@ -212,19 +211,24 @@ PRIVATE void DIALOG::setup ()
 		posy += f->vsize;
 		if (max_field < f->box.width) max_field = f->box.width;
 	}
-	int data_width = max_field + 5 + max_prompt;
+	int data_width = max_field + 6 + max_prompt;
 	if (dialog_mode == DIALOG_CURSES
 		&& data_width > COLS){
-		// Message should not be translated. It is for translator
-		xconf_error ("Dialog too wide: Max_prompt = %d Max_field = %d"
-			,max_prompt,max_field);
-		max_prompt = COLS - (max_field + 5);
-		if (max_prompt < 0){
+		if (getenv("LINUXCONF_TRANSLATOR")!=NULL){
+			// Message should not be translated. It is for translator
+			xconf_error ("Dialog too wide:\n"
+				"Max_prompt = %d Max_field =%d\n"
+				"field area will be truncated"
+				,max_prompt,max_field);
+		}
+		max_field = COLS - (max_prompt + 6);
+		if (max_field < 0){
 		}
 		for (int j=0; j<getnb(); j++){
 			FIELD *f = getitem(j);
-			int len = strlen(f->prompt);
-			if (len > max_prompt) f->prompt[max_prompt] = '\0';
+			//int len = strlen(f->prompt);
+			//if (len > max_prompt) f->prompt[max_prompt] = '\0';
+			if (f->box.width > max_field) f->box.width = max_field;
 		}
 		data_width = max_field + 5 + max_prompt;
 	}
@@ -234,10 +238,20 @@ PRIVATE void DIALOG::setup ()
 	int button_len = internal->buttons->evalwidth();
 	// Buttons can scroll if there are too many.
 	int button_y = internal->height - 3;
-	if (button_len > COLS - 2){
-		button_len = COLS-4;
-		// Add another button line
-		internal->height += 3;
+	internal->button_height = 3;
+	if (dialog_mode == DIALOG_CURSES){
+		if (button_len > COLS - 4){
+			button_len = COLS-5;
+			// Add another button line
+			internal->height += 3;
+			internal->button_height = 6;
+			int maxl = LINES - 1;
+			if (internal->height > maxl){
+				internal->nbvisible -= internal->height - maxl;
+				internal->height = maxl;
+				button_y = maxl - 6;
+			}
+		}
 	}
 	if (title_len > internal->width) internal->width = title_len;
 	if (button_len > internal->width) internal->width = button_len;
@@ -257,8 +271,10 @@ PRIVATE void DIALOG::setup ()
 	Set the offset of the first visible field and ajust
 	the coordinate of all field so they will know where to draw
 	themselve.
+
+	This is only useful in text mode. It does nothing otherwise.
 */
-PRIVATE void DIALOG::setoffset (int newoff)
+PUBLIC void DIALOG::setoffset (int newoff)
 {
 	FIELD *curpt = getitem(internal->offset);
 	FIELD *newpt = getitem(newoff);	
@@ -270,6 +286,15 @@ PRIVATE void DIALOG::setoffset (int newoff)
 		}
 	}
 }
+/*
+	Return the number of the first visible field (text mode)
+	This function is used along with setoffset to preserve and restore
+	the exact dialog state.
+*/
+PUBLIC int DIALOG::getoffset () const
+{
+	return internal->offset;
+}
 
 /*
 	Draw all visible field
@@ -280,7 +305,7 @@ PROTECTED void DIALOG::drawf(WINDOW *dialog)
 	int size = 0;
 	for (int i=internal->offset; i<lastf && size < internal->nbvisible; i++){
 		FIELD *f = getitem(i);
-		f->draw (dialog);
+		f->draw (dialog, internal->hoffset);
 		size += f->vsize;
 	}
 }
@@ -328,7 +353,8 @@ PRIVATE void DIALOG::drawarrow_if(
 	if (getnb() > 0){
 		FIELD *f = getitem(internal->offset);
 		int posx = f->box.x + f->box.width/2;
-		int posy = top ? f->box.y - 1 : internal->height - 5;
+		int posy = top ? f->box.y - 1
+			: internal->height - internal->button_height - 2;
 		if (condition){
 			if (!flag){
 				flag = true;
@@ -359,38 +385,22 @@ PRIVATE void DIALOG::dokeyup(int &nof, WINDOW *dialog)
 }
 PRIVATE void DIALOG::dokeydown(int &nof, WINDOW *dialog)
 {
-	#if 0
-	bool maygo = false;
-	for (int i=nof+1; i<getnb(); i++){
-		FIELD *f = getitem(i);
-		if (f != NULL){
-			if (!f->readonly || f->may_select){
-				maygo = true;
-				break;
-			}
+	int last_nof = nof;
+	nof++;
+	while (nof < getnb() && getitem(nof)->vsize == 0) nof++;
+	if (nof < getnb()){
+		FIELD *f = getitem(nof);
+		if (f->box.y >= internal->height-internal->button_height - 2){
+			int newoffset = internal->offset+1;
+			while (newoffset < getnb()
+				&& getitem(newoffset)->vsize == 0) newoffset++;
+			setoffset (newoffset);
+			drawf(dialog);
 		}
+	}else{
+		nof = last_nof;
 	}
-	if (maygo){
-	#endif
-		int last_nof = nof;
-		nof++;
-		while (nof < getnb() && getitem(nof)->vsize == 0) nof++;
-		if (nof < getnb()){
-			FIELD *f = getitem(nof);
-			if (f->box.y >= internal->height-5){
-				int newoffset = internal->offset+1;
-				while (newoffset < getnb()
-					&& getitem(newoffset)->vsize == 0) newoffset++;
-				setoffset (newoffset);
-				drawf(dialog);
-			}
-		}else{
-			nof = last_nof;
-		}
 
-	#if 0
-	}
-	#endif
 }
 /*
 	Interpret a cursor key.
@@ -401,7 +411,6 @@ PROTECTED VIRTUAL int DIALOG::keymove (WINDOW *dialog, int key, int &nof)
 	int ret = 0;
 	int nextkey = key;
 	switch (key){
-	case 2:		// ctrl-B (emacs style)
 	case KEY_PPAGE:
 		if (internal->offset == 0){
 			nof = 0;
@@ -417,7 +426,6 @@ PROTECTED VIRTUAL int DIALOG::keymove (WINDOW *dialog, int key, int &nof)
 	case KEY_UP:
 		dokeyup(nof,dialog);
 		break;
-	case 6:	// ctrl-F (emacs style)
 	case KEY_NPAGE:
 		{
 			int maxoffset = getnb() - internal->nbvisible;
@@ -470,6 +478,7 @@ PRIVATE void DIALOG::skipprotect (int &nof, int suggestkey, WINDOW *dialog)
 	}
 }
 DIALOG_MODE dialog_mode = DIALOG_CURSES;
+static DIALOG_MODE previous_mode = DIALOG_CURSES;
 int treemenu_pos[20];		// Assume maximum 20 levels of menu
 
 /*
@@ -479,6 +488,7 @@ int treemenu_pos[20];		// Assume maximum 20 levels of menu
 EXPORT DIALOG_MODE dialog_setmode (DIALOG_MODE mode)
 {
 	DIALOG_MODE old = dialog_mode;
+	previous_mode = old;
 	dialog_mode = mode;
 	if (mode == DIALOG_TREE){
 		ui_context.treemenu_level = 0;
@@ -514,17 +524,78 @@ EXPORT void dialog_jumpto (const char *menupath)
 	}
 }
 
+static bool jump2help = false;
+/*
+	The next dialog visited won't be shown. Instead its help screen
+	will be triggered.
+*/
+EXPORT void dialog_jump2help(bool mode)
+{
+	jump2help = mode;
+}
+
 int multi_alloc_gui_id()
 {
 	static int gui_id_alloc=1;
 	return gui_id_alloc++;
 }
 
+class DIALOGS: public ARRAY{
+public:
+	DIALOG *getitem(int no) const
+	{
+		return (DIALOG*)ARRAY::getitem(no);
+	}
+};
+
+
+static DIALOGS tbdia;
+
+/*
+	Send various listen directive to the GUI to inform about the "listening"
+	state of a dialog.
+*/
+void multi_setlistening ()
+{
+	if (diajava_listen){
+		for (int i=0; i<tbdia.getnb(); i++){
+			DIALOG *dia = tbdia.getitem(i);
+			if (dia->internal->guidone){
+				bool listening = dia->internal->listening;
+				if (listening != dia->internal->gui_listen){
+					char tmp[200];
+					if (dia->setguiname(tmp) != NULL){
+						diagui_sendcmd (P_Listen,"%s %d\n",tmp,listening ? 1 : 0);
+					}
+					dia->internal->gui_listen = listening;
+				}
+			}
+		}
+	}
+}
+
+/*
+	Invalidate all ncurses window so dialog_restart repaint the
+	screen properly. Called by dialog_restart().
+*/
+void multi_touchwins()
+{
+	for (int i=0; i<tbdia.getnb(); i++){
+		DIALOG *dia = tbdia.getitem(i);
+		WINDOW *win = dia->internal->cursw;
+		if (win != NULL){
+			touchwin (win);
+		}
+	}
+}
+
 
 PUBLIC DIALOG::DIALOG()
 {
 	internal = new DIALOG_INTERNAL;
 	internal->buttons = new BUTTONS_INFO;
+	internal->button = 0;
+	internal->selected_button = -1;
 	internal->offset = 0;
 	html_adddialog (this);
 	internal->guidone = false;
@@ -535,22 +606,60 @@ PUBLIC DIALOG::DIALOG()
 	internal->autonewline = true;
 	internal->height_hint = false;
 	internal->gui_id = multi_alloc_gui_id();
+	internal->thread_id = uithread_id;
 	internal->diatype = DIATYPE_STD;
 	internal->cursw = NULL;
 	internal->context_wasset = false;
+	internal->waitprivmsg.neverdelete();
+	internal->button_on_side = false;
+	internal->hoffset = 0;
+	internal->registry_id = NULL;
+	internal->subdia = NULL;
+	internal->last_nof = -1;		// For a curfield command in GUI
+	internal->listening = false;
+	internal->gui_listen = true;	// A dialog is created in listen mode
+									// not in gray out mode.
+	internal->insert_pos = -1;
+	tbdia.neverdelete();
+	tbdia.add (this);
 }
+
 PUBLIC DIALOG::~DIALOG()
 {
+	tbdia.remove (this);
 	html_forgetdialog (this);
 	guidelete();
 	delete internal->buttons;
 	delete internal;
 }
 
+/*
+	Record a unique string to identify the dialog. Combining
+	this string with a prompt, we can uniquely identify a field
+	of a given dialog.
+
+	We use this only when two or more dialogs are sharing the same
+	fields and we want to address those field using the registry.
+
+	The string content does not matter much. Only its address matters.
+	It is a static string.
+*/
+PUBLIC void DIALOG::set_registry_id(const char *id)
+{
+	internal->registry_id = id;
+}
+
+PUBLIC void DIALOG::set_selected_button (int button)
+{
+	internal->selected_button = button;
+}
+
 PUBLIC void DIALOG::remove_all()
 {
+	delete internal->subdia;
+	internal->subdia = NULL;
 	ARRAY::remove_all();
-	internal->guidone = false;
+	reset_guidone();
 	internal->buttons->delbutinfo();
 }
 
@@ -559,23 +668,70 @@ PUBLIC void DIALOG::remove_all()
 */
 PUBLIC void DIALOG::setcontext (const char *s)
 {
-	if (s != NULL){
+	if (s != NULL && diajava_context){
 		internal->context_wasset = true;
 		internal->context.setfrom (s);
 	}
 }
 
+/*
+	For the GUI to place the dialog buttons on the right side
+*/
+PUBLIC void DIALOG::set_button_on_side()
+{
+	internal->button_on_side = true;
+}
 
 PROTECTED void DIALOG::add (FIELD *f)
 {
-	internal->guidone = false;
-	ARRAY::add (f);
+	reset_guidone();
+	if (internal->insert_pos != -1){
+		ARRAY::insert (internal->insert_pos++,f);
+	}else{
+		ARRAY::add (f);
+	}
 }
 
+/*
+	Specify the position of the next field added to the dialog.
+	You call this function and then newf_something() and the field
+	will be inserted at the position you specified. Every field added from now
+	on will be inserted one after the other at this position. You call
+	the function again and set the position to -1 to disable to behavior.
+*/
+PUBLIC void DIALOG::set_nextfield (int no)
+{
+	internal->insert_pos = no;
+}
+
+PUBLIC int DIALOG::remove_del (FIELD *f)
+{
+	reset_guidone();
+	if (internal->subdia != NULL) internal->subdia->remove_del(f);
+	return ARRAY::remove_del (f);
+}
+
+/*
+	Set the DIALOG GUI state to "not done".
+	This means that the dialog will have to be transmitted again
+	to the GUI front-end. This is generally called when something
+	important has changed in the dialog (a new field for example) and it
+	can't be fixed using the GUI protocol. So the dialog will be deleted
+	on the GUI side and redone.
+*/
+PUBLIC void DIALOG::reset_guidone()
+{
+	guidelete();
+	internal->guidone = false;
+}
+
+
 PUBLIC int DIALOG::remove_del (int no)
 {
-	internal->guidone = false;
-	return ARRAY::remove_del (no);
+	int ret = -1;
+	FIELD *f = getitem(no);
+	if (f != NULL) ret = remove_del (f);
+	return ret;
 }
 
 /*
@@ -611,12 +767,14 @@ PUBLIC void DIALOG::show(
 	internal->buttons->set (but_options,helpfile);
 	fixwidth1();
 	setup ();
-	if (dialog_mode == DIALOG_HTML){
-		// showhtml (nof);
-	}else if (dialog_mode == DIALOG_GUI){
-		showgui (nof,but_options);
-	}else{
-		showterm (nof,but_options);
+	if (!jump2help){
+		if (dialog_mode == DIALOG_HTML){
+			// showhtml (nof);
+		}else if (dialog_mode == DIALOG_GUI){
+			showgui (nof,but_options);
+		}else{
+			showterm (nof,but_options);
+		}
 	}
 }
 /*
@@ -632,41 +790,96 @@ PUBLIC MENU_STATUS DIALOG::edit(
 					// Will contain the current field.
 	int but_options)		// MENUBUT_xxxxx
 {
+	if (dialog_mode == DIALOG_GET){
+		int n = getnb();
+		for (int i=0;i<n; i++){
+			master_registry.register_field(getitem(i),internal->registry_id);
+		}
+		return MENU_ESCAPE;
+	}
+	if (dialog_mode == DIALOG_SET){
+		int n = getnb();
+		for (int i=0;i<n; i++){
+			master_registry.retrieve(getitem(i),internal->registry_id);
+		}
+		bool allok = true;
+		for (int i=0; i<n; i++){
+			if (getitem(i)->post_validate()==-1){
+				allok = false;
+				break;
+			}
+		}
+		if (allok){
+			save();
+			// We switch back to the previous user interface mode.
+			// This allow the caller to present some error messages
+			// if need.
+			dialog_mode = previous_mode;
+			return MENU_ACCEPT;
+		}else{
+			return MENU_CANCEL;
+		}
+	}
 	if (dialog_mode == DIALOG_SILENT
 		|| dialog_mode == DIALOG_TREE){
 		// dialog_endlevel();
 		return MENU_ESCAPE;
 	}
-	show (_title,_intro,helpfile,nof,but_options);
 	MENU_STATUS ret = MENU_NULL;
-	while (1){
-		internal->button = getnb()==0 ? 0 : -1;	// Button currently selected
-		if (dialog_mode == DIALOG_HTML){
-			ret = edithtml (nof);
-		}else if (dialog_mode == DIALOG_GUI){
-			ret = editgui (nof,but_options);
-		}else{
-			ret = editterm (nof,but_options);
+	show (_title,_intro,helpfile,nof,but_options);
+	if (jump2help){
+		jump2help = false;
+		ret = MENU_ESCAPE;
+		internal->buttons->help(NULL);
+	}else{
+		if (ui_context.treemenu_level > 0){
+			// Enable uithread again, see uithread()
+			ui_context.treemenu_level = ui_context.treejump_level + 1;
 		}
-		if (ret != MENU_ESCAPE
-			&& ret != MENU_QUIT
-			&& ret != MENU_CANCEL){
-			int n = getnb();
-			int i;
-			for (i=0; i<n; i++){
-				if (getitem(i)->post_validate()==-1){
-					nof = i;
+		internal->listening = true;
+		while (1){
+			// Button currently selected
+			internal->button = internal->selected_button;
+			internal->selected_button = -1;
+			if (getnb()==0){
+				if (internal->button == -1) internal->button = 0;
+			}
+			if (dialog_mode == DIALOG_HTML){
+				ret = edithtml (nof);
+			}else if (dialog_mode == DIALOG_GUI){
+				ret = editgui (nof,but_options);
+			}else{
+				ret = editterm (nof,but_options);
+			}
+			if (ret != MENU_ESCAPE
+				&& ret != MENU_QUIT
+				&& ret != MENU_CANCEL){
+				int n = getnb();
+				int i;
+				for (i=0; i<n; i++){
+					if (getitem(i)->post_validate()==-1){
+						nof = i;
+						break;
+					}
+				}
+				if (i==n){
+					if (dialog_mode == DIALOG_CURSES){
+						delwin(internal->cursw);
+						internal->cursw = NULL;
+					}
 					break;
+				}else{
+					show (_title,_intro,helpfile,nof,but_options);
 				}
+			}else{
+				break;
 			}
-			if (i==n) break;
-		}else{
-			break;
 		}
-	}
-	if (ret == MENU_ACCEPT){
-		/* Save the content of the edit field into the buffer */
-		save();
+		internal->listening = false;
+		if (ret == MENU_ACCEPT){
+			/* Save the content of the edit field into the buffer */
+			save();
+		}
 	}
 	return ret;
 }
@@ -699,6 +912,23 @@ PUBLIC MENU_STATUS DIALOG::edit(
 		,MENUBUT_ACCEPT|MENUBUT_CANCEL);
 }
 
+/*
+	Request a dump from the dialog (even if no button was hit bu
+	the user). This function send the request and return immediatly.
+	Later, the dummy action MENU_DUMP will be returned by the
+	DIALOG::edit() function.
+
+	The application must be ready to process other request from
+	the front-end, which happened just about the dump.
+*/
+PUBLIC void DIALOG::request_dump()
+{
+	char tmp[200];
+	const char *diapath = setguiname(tmp);
+	if (diapath != NULL){
+		diagui_sendcmd (P_Dump,"%s B%d\n",tmp,BUTSPC_DUMP);
+	}
+}
 
 #ifdef TEST
 
