@@ -1,5 +1,23 @@
 #include "dialog.h"
 
+/*
+	Display a dialog box with two buttons - Yes and No
+	Return MENU_YES, MENU_NO or MENU_ESCAPE
+*/
+EXPORT MENU_STATUS dialog_yesno(
+	const char *title,
+	const char *prompt,
+	HELP_FILE &helpfile,		/* path of a help text file */
+								/* or help_nil */
+	bool default_is_yes)
+{
+	dialog_clear();
+	DIALOG dia;
+	dia.settype (DIATYPE_POPUP);
+	if (!default_is_yes) dia.set_selected_button(1);
+	int nof = 0;
+	return dia.edit (title,prompt,helpfile,nof,MENUBUT_YES|MENUBUT_NO);
+}
 
 /*
 	Display a dialog box with two buttons - Yes and No
@@ -11,11 +29,10 @@ EXPORT MENU_STATUS dialog_yesno(
 	HELP_FILE &helpfile)		/* path of a help text file */
 								/* or help_nil */
 {
-	DIALOG dia;
-	dia.settype (DIATYPE_POPUP);
-	int nof = 0;
-	return dia.edit (title,prompt,helpfile,nof,MENUBUT_YES|MENUBUT_NO);
+	return dialog_yesno (title,prompt,helpfile,true);
 }
+
+
 #ifdef TEST
 
 int main (int argc, char *argv[])
