@@ -52,5 +52,33 @@
 #define PROTO_Setval  51
 #define PROTO_Alive  52
 #define PROTO_Setcontext  53
-#define PROTO_last 53
+#define PROTO_Defdc  54
+#define PROTO_Deffont  55
+#define PROTO_Defpen  56
+#define PROTO_Defbrush  57
+#define PROTO_Drawline  58
+#define PROTO_Drawrect  59
+#define PROTO_Drawarc  60
+#define PROTO_Fillrect  61
+#define PROTO_Fillarc  62
+#define PROTO_Menubar  63
+#define PROTO_Popupmenu  64
+#define PROTO_Submenu  65
+#define PROTO_Menuentry  66
+#define PROTO_Clear  67
+#define PROTO_Drawclip  68
+#define PROTO_Setcursor  69
+#define PROTO_Blit  70
+#define PROTO_Drawtext  71
+#define PROTO_Refresh  72
+#define PROTO_Menutitle  73
+#define PROTO_Listen  74
+#define PROTO_Prefsubmenu  75
+#define PROTO_Show  76
+#define PROTO_Inputgrid  77
+#define PROTO_Fillpolygon  78
+#define PROTO_Splash  79
+#define PROTO_Hidesplash  80
+#define PROTO_Deletechild  81
+#define PROTO_last 81
 extern const char *protostr[];
