@@ -1,4 +1,5 @@
 #include <stdio.h>
+#include <assert.h>
 #include <string.h>
 #include <stdarg.h>
 #include <errno.h>
@@ -11,6 +12,8 @@
 #include <X11/Intrinsic.h>
 #include <netdb.h>
 #include <sys/time.h>
+#include <sys/types.h>
+#include <sys/socket.h>
 #include <netinet/in.h>
 
 FILE *ftty = NULL;
@@ -49,9 +52,16 @@ PUBLIC ICON_XPM::ICON_XPM(char **tbstr, int nbstr, const char *_name)
 	for (int i=0; i<nbstr; i++){
 		char *pt = tbstr[i];
 		if (pt[0] == '"'){
-			int len = strlen(pt);
-			if (len > 0) pt[len-2] = '\0'; 	// Kill ", sequence at the end
+			// Kill ", sequence at the end or " alone
+			int last = strlen(pt)-1;
+			if (pt[last] == ',' && pt[last-1] == '"'){
+				pt[last-1] = '\0';
+			}else if (pt[last] == '"'){
+				pt[last] = '\0';
+			}
+			//if (len > 0) pt[len-2] = '\0'; 	
 			tb[line++] = pt + 1;	// Skip the " at the beginning
+			// fprintf (stderr,"len %d\n",strlen(pt+1));
 		}
 	}
 	bitmap = new wxBitmap (tb);
@@ -65,28 +75,39 @@ PUBLIC void ICON_XPM::donothing()
 {
 }
 
-static ICON_XPM *icon_xpm_locate(const char *name)
+wxBitmap *icon_xpm_locate(const char *name)
 {
+	wxBitmap *ret = NULL;
 	ICON_XPM *x = first;
 	while (x != NULL){
 		if (strcmp(x->name,name)==0){
+			ret = x->bitmap;
 			break;
 		}
 		x = x->next;
 	}
-	if (x == NULL) fprintf (stderr,"Unknown XPM icon %s\n",name);
-	return x;
+	if (ret == NULL) fprintf (stderr,"Unknown XPM icon %s\n",name);
+	return ret;
 }
 class MFORMSCRIPT_CTX{
 public:
 	MFORM *tbfl[20];
 	BOOK  *tbbk[20];
+	wxMenu *tbmn[20];
+	wxMenuBar *menubar;
 	int level;
 	SETVAL_INFO info;
 	/*~PROTOBEG~ MFORMSCRIPT_CTX */
 public:
 	MFORMSCRIPT_CTX (void);
+	void newlevel (BOOK *bk);
+	void newlevel (MFORM *fl);
 	void newlevel (MFORM *fl, BOOK *bk);
+private:
+	void newlevel (void);
+public:
+	void newlevel (wxMenu *mn);
+	void newlevel (wxMenuBar *bar);
 	void poplevel (void);
 	/*~PROTOEND~ MFORMSCRIPT_CTX */
 };
@@ -113,7 +134,7 @@ public:
 	void readone (void);
 	void run (void);
 private:
-	void setcontext (MFORMSCRIPT_CTX&ctx,
+	int setcontext (MFORMSCRIPT_CTX&ctx,
 		 const char *id);
 public:
 	void setcursor (wxCursor *cur);
@@ -129,7 +150,7 @@ PUBLIC MFORMSCRIPT::MFORMSCRIPT (bool _must_exit)
 	maxmain = 100;
 	tbmain = new MAINFORM*[100];
 	for (int i=0; i<maxmain; i++) tbmain[i] = NULL;
-	tbstr = new char *[200];
+	tbstr = new char *[2000];
 	nbstr = 0;
 	current = NULL;
 }
@@ -184,19 +205,90 @@ static MAINFORM *remadmin_showerror (const char *ctl, ...)
 	return msg;
 }
 
+PUBLIC SETVAL_INFO::SETVAL_INFO()
+{
+	tarea = NULL;
+	tarea_buf = NULL;
+	tarea_size = 0;
+	tarea_pos = 0;
+	combo = NULL;
+}
+/*
+	Prepare the textarea append buffer, used by the Str primitive to
+	collect the content of a textarea
+*/
+PUBLIC void SETVAL_INFO::initbuf()
+{
+	free (tarea_buf);
+	tarea_buf = (char*)malloc(10000);
+	tarea_size = 10000;
+	tarea_pos = 0;
+	tarea_buf[0] = '\0';
+}
+
+PUBLIC void SETVAL_INFO::freebuf()
+{
+	free (tarea_buf);
+	tarea_buf = NULL;
+}
+/*
+	Append a line to the textarea buffer
+*/
+PUBLIC void SETVAL_INFO::append(const char *s)
+{
+	int len = strlen(s);
+	if (tarea_pos + len >= tarea_size){
+		tarea_size += 10000;
+		tarea_buf = (char*)realloc (tarea_buf,tarea_size);
+		assert (tarea_buf != NULL);
+	}
+	strcpy (tarea_buf+tarea_pos,s);
+	tarea_pos += len;
+	tarea_buf[tarea_pos++] = '\n';
+	tarea_buf[tarea_pos] = '\0';
+}
 
 PUBLIC MFORMSCRIPT_CTX::MFORMSCRIPT_CTX()
 {
 	level = -1;
-	info.tarea = NULL;
+}
+
+PRIVATE void MFORMSCRIPT_CTX::newlevel ()
+{
+	++level;
+	tbfl[level] = NULL;
+	tbbk[level] = NULL;
+	tbmn[level] = NULL;
 }
 
 PUBLIC void MFORMSCRIPT_CTX::newlevel (MFORM *fl, BOOK *bk)
 {
-	++level;
+	newlevel();
 	tbfl[level] = fl;
 	tbbk[level] = bk;
 }
+PUBLIC void MFORMSCRIPT_CTX::newlevel (MFORM *fl)
+{
+	newlevel();
+	tbfl[level] = fl;
+}
+PUBLIC void MFORMSCRIPT_CTX::newlevel (BOOK *bk)
+{
+	newlevel();
+	tbbk[level] = bk;
+}
+PUBLIC void MFORMSCRIPT_CTX::newlevel (wxMenu *mn)
+{
+	newlevel();
+	tbmn[level] = mn;
+}
+
+PUBLIC void MFORMSCRIPT_CTX::newlevel (wxMenuBar *bar)
+{
+	newlevel();
+	menubar = bar;
+}
+
 PUBLIC void MFORMSCRIPT_CTX::poplevel ()
 {
 	level--;
@@ -214,12 +306,14 @@ static bool pref_notree=false;
 static const char GCONTENT[]="--gcontent--";
 
 /*
-	Set the context to the form or book pointed by ID
+	Set the context to the form or book pointed by ID.
+	Return -1 if not found.
 */
-PRIVATE void MFORMSCRIPT::setcontext (
+PRIVATE int MFORMSCRIPT::setcontext (
 	MFORMSCRIPT_CTX &ctx,
 	const char *id)
 {
+	int oldlevel = ctx.level;
 	char mainf[strlen(id)+1];
 	strcpy (mainf,id);
 	char *pt = mainf;
@@ -240,7 +334,7 @@ PRIVATE void MFORMSCRIPT::setcontext (
 		// fprintf (stderr,"start :%s: %p %p\n",start,form,book);
 		if (form != NULL){
 			if (form->nbc == 1){
-				// Special case from Groups, see GCONTENT
+				// Special case for Groups, see GCONTENT
 				MFORM_C *c = form->tbc[0];
 				if (c->type == T_FORM){
 					MFORM *sub = (MFORM*)c->c;
@@ -262,8 +356,133 @@ PRIVATE void MFORMSCRIPT::setcontext (
 			ctx.newlevel (form,book);
 		}
 	}
+	return ctx.level > oldlevel ? 0 : -1;
 }
 
+struct TBVAR{
+	const char *name;
+	const char *val;
+};
+
+static TBVAR *tbvar;
+static int nbvar;
+
+/*
+	Return the value of a named parameter ($name=val).
+	Return NULL if this was not defined
+*/
+const char *rem_getvar (const char *name)
+{
+	const char *ret = NULL;
+	TBVAR *ptvar = tbvar;
+	for (int i=0; i<nbvar; i++,ptvar++){
+		if (strcmp(ptvar->name,name)==0){
+			ret = ptvar->val;
+			break;
+		}
+	}
+	return ret;
+}
+
+/*
+	Return the value of a variable or -1 of not defined
+*/
+int rem_getvarval (const char *name)
+{
+	int ret = -1;
+	const char *s = rem_getvar(name);
+	if (s != NULL) ret = atoi(s);
+	return ret;
+}
+
+static void remadmin_fctmenu (wxObject &, wxCommandEvent &)
+{
+	fprintf (stderr,"fctmenu\n");
+}
+
+static int last_mousex = 200;	// Last position of the mouse
+static int last_mousey = 200;	// for popup menus
+/*
+	Record the position of the last click the mouse
+*/
+void rem_setlastmouse (int x, int y)
+{
+	last_mousex = x;
+	last_mousey = y;
+}
+static char *popupid = NULL;
+static wxMenu *currentpopup = NULL;
+
+static void popup_func(wxObject &, wxCommandEvent &c)
+{
+	// fprintf (stderr,"func :%s: %d %ld :%s: :%s:\n",c.commandString
+	// ,c.commandInt,c.extraLong,c.labelString,c.clientData);
+	remadmin_setcursor (cursor_wait);
+	fprintf (mform_fout,"menubar %s %d\n",popupid,c.commandInt);
+	fflush (mform_fout);
+	free (popupid);
+	popupid = NULL;
+}
+
+struct ALIVE_EVENT {
+	void (*f) (void *data);
+	void *data;
+	ALIVE_EVENT *next;
+};
+
+static ALIVE_EVENT *event;
+
+/*
+	Register a callback which will be executed when the Alive
+	primitive is received.
+*/
+void remadmin_regalive (void (*f)(void *data), void *data)
+{
+	// Check if the event is already there
+	ALIVE_EVENT *pt = event;
+	while (pt != NULL){
+		if (pt->f == f && pt->data == data) break;
+		pt = pt->next;
+	}
+	if (pt == NULL){
+		ALIVE_EVENT *ev = new ALIVE_EVENT;
+		ev->f = f;
+		ev->data = data;
+		ev->next = event;
+		event = ev;
+	}
+}
+
+/*
+	Remove all event related to a given data
+*/
+void remadmin_delalive (void *data)
+{
+	ALIVE_EVENT **ptpt = &event;
+	while (*ptpt != NULL){
+		ALIVE_EVENT *pt = *ptpt;
+		if (pt->data == data){
+			*ptpt = pt->next;
+			delete pt;
+		}else{
+			ptpt = &pt->next;
+		}
+	}
+}
+/*
+	Process all alive event
+*/
+static void  remadmin_doalive ()
+{
+	ALIVE_EVENT *pt = event;
+	while (pt != NULL){
+		pt->f(pt->data);
+		ALIVE_EVENT *next = pt->next;
+		delete pt;
+		pt = next;
+	}
+	event = NULL;
+}
 // Process one line
 // Return true if this was a End statement
 PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
@@ -278,11 +497,14 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 		fl = ctx.tbfl[ctx.level];
 		bk = ctx.tbbk[ctx.level];
 	}
-	// fprintf (stderr,"level %d %p %p\n",ctx.level,fl,bk);
+	// fprintf (stderr,"level %d %p %p :%s:\n",ctx.level,fl,bk,buf);
 	bool end = false;
 	if (buf[0] == '\0') return false;
 	char args[20][1000];
 	const char *ptargs[20];
+	TBVAR vars[20];
+	nbvar = 0;
+	tbvar = vars;
 	int nbargs = 0;
 	{
 		for (int i=0; i<20; i++){
@@ -291,11 +513,16 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 		}
 		char *pt = str_skip(buf);
 		if (pt[0]=='#') return false;
+		bool in_variables = false;
 		while (*pt != '\0'){
 			pt = str_skip(pt);
+			if (*pt == '$'){
+				in_variables = true;
+				pt = str_skip(pt+1);
+			}
 			if (*pt == '"'){
 				pt++;
-				char *dst = args[nbargs++];
+				char *dst = args[nbargs];
 				while (*pt != '"' && *pt != '\0'){
 					if (*pt == '\\'){
 						pt++;
@@ -306,8 +533,20 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 				if (*pt == '"') pt++;
 			}else if (*pt > ' '){
 				pt = str_copyword (args[nbargs],pt,sizeof(args[nbargs]));
-				nbargs++;
 			}
+			if (in_variables){
+				char *arg = args[nbargs];
+				char *eq = strchr(arg,'=');
+				if (eq != NULL){
+					vars[nbvar].name = arg;
+					*eq++ = '\0';
+					vars[nbvar].val = eq;
+					nbvar++;
+				}else{
+					fprintf (stderr,"variable %s without value, ignored.\n",arg);
+				}
+			}
+			nbargs++;
 		}
 	}
 	char *verb = args[0];
@@ -328,14 +567,7 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 	if (num == PROTO_Str){
 		if (html_collect){
 		}else if (ctx.info.tarea != NULL){
-			const char *cur = ctx.info.tarea->GetValue();
-			int len1 = strlen(cur);
-			int len2 = strlen(args[1]);
-			char tmp[len1+len2+2];
-			strcpy (tmp,cur);
-			strcpy (tmp+len1,args[1]);
-			strcpy (tmp+len1+len2,"\n");
-			ctx.info.tarea->SetValue (tmp);
+			ctx.info.append (args[1]);
 		}else{
 			tbstr[nbstr++] = strdup(args[1]);
 		}
@@ -399,16 +631,16 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 		fl->New_button (args[1],atoi(args[2])!=0,args[3]);
 		break;
 	case PROTO_Buttonfill:
-		fl->New_buttonfill (args[1],args[2]);
+		fl->New_buttonfill (args[1],args[2],0,NULL);
 		break;
 	case PROTO_Button_xpmf:
 		//janne.New_button (fl,ex.nextword(),ex.nextword(),ex.getendskip());
 		break;
 	case PROTO_Button_xpm:
 		{
-			ICON_XPM *x = icon_xpm_locate(args[2]);
-			if (x != NULL){
-				fl->New_button (args[1],x->bitmap);
+			wxBitmap *bitmap = icon_xpm_locate(args[2]);
+			if (bitmap != NULL){
+				fl->New_button (args[1],bitmap);
 			}
 		}
 		break;
@@ -417,37 +649,60 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 		break;
 	case PROTO_Curfield:
 		{
-			MAINFORM *mf = locate (args[1]);
-			if (mf != NULL){
-				wxWindow *tbfocus[20];
-				int nbfocus = 0;
-				mf->setcurfield (args[2],tbfocus,nbfocus);
-				for (int f=nbfocus-1; f>=0; f--) tbfocus[f]->SetFocus();
+			//int level = ctx.level;
+			// FORMBASE*mf = NULL;
+			if (setcontext (ctx,args[1]) != -1){
+				FORMBASE *mf = ctx.tbfl[ctx.level];
+				if (mf == NULL) mf = ctx.tbbk[ctx.level];
+				// fprintf (stderr,"focus :%s: :%s: %p\n",args[1],args[2],mf);
+				if (mf != NULL){
+					wxWindow *tbfocus[20];
+					int nbfocus = 0;
+					mf->setcurfield (args[2],tbfocus,nbfocus);
+					// fprintf (stderr,"nbfocus = %d\n",nbfocus);
+					if (rem_getvarval("focus")!=0){
+						for (int f=nbfocus-1; f>=0; f--) tbfocus[f]->SetFocus();
+					}
+				}
+				ctx.poplevel();
+			}else{
+				fprintf (stderr,"curfield failed :%s:\n",args[1]);
 			}
 		}
 		break;
 	case PROTO_Alive:
+		remadmin_doalive();
 		setcursor (cursor_normal);
 		break;
 	case PROTO_Icon_xpm:
 		{
-			ICON_XPM *x = icon_xpm_locate(args[1]);
-			if (x != NULL){
-				fl->New_icon_xpm (x->bitmap);
+			wxBitmap *bitmap = icon_xpm_locate(args[1]);
+			if (bitmap != NULL){
+				fl->New_icon_xpm (bitmap);
 			}
 		}
 		break;
 	case PROTO_Xfer_xpm:
 		{
+			//FILE *f = fopen ("/tmp/log.xpm","w");
+			//for (int i=0; i<nbstr; i++) fprintf (f,"%s\n",tbstr[i]);
+			//fclose (f);
 			ICON_XPM *x = new ICON_XPM (tbstr,nbstr,args[1]);
 			x->donothing();
 		}
 		break;
 	case PROTO_Choice:
-		fl->New_choice (args[1],args[2]);
+		ctx.info.combo = fl->New_choice (args[1],args[2]);
 		break;
 	case PROTO_Choice_item:
-		fl->New_choice_item (args[1],args[2],args[3]);
+		{
+			int no = atoi(args[2]+1);
+			if (args[1][0] == '\0' && ctx.info.combo != NULL){
+				ctx.info.combo->setItem(no,args[3],args[4]);
+			}else{
+				fl->New_choice_item (args[1],no,args[2],args[3]);
+			}
+		}
 		break;
 	case PROTO_List:
 		fl->New_list (args[1],atoi(args[2]),args[3]);
@@ -456,19 +711,26 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 		fl->New_list_item (args[1],args[2]);
 		break;
 	case PROTO_Combo:
-		fl->New_combo (args[1],atoi(args[2]),args[3]);
+		ctx.info.combo = fl->New_combo (args[1],atoi(args[2]),args[3]);
 		break;
 	case PROTO_Combo_item:
-		fl->New_combo_item (args[1],args[2],args[3]);
+		{
+			int no = atoi(args[2]+1);
+			if (args[1][0] == '\0' && ctx.info.combo != NULL){
+				ctx.info.combo->setItem(no,args[3],args[4]);
+			}else{
+				fl->New_combo_item (args[1],no,args[2],args[3]);
+			}
+		}
 		break;
 	case PROTO_Group:
 		{
 			GROUP *gr = new GROUP(fl,args[1],args[2]);
 			fl->New_form(gr);
-			// We "protecte" the group by creating a subform in which
+			// We "protect" the group by creating a subform in which
 			// we place the various widgets
 			MFORM *sub = gr->New_form(GCONTENT);
-			ctx.newlevel (sub,NULL);
+			ctx.newlevel (sub);
 		}
 		break;
 	case PROTO_Groupfit:
@@ -476,17 +738,18 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 			GROUP *gr = new GROUP_FIT(fl,args[1],args[2]);
 			fl->New_form(gr);
 			MFORM *sub = gr->New_form(GCONTENT);
-			ctx.newlevel (sub,NULL);
+			ctx.newlevel (sub);
 		}
 		break;
 	case PROTO_Formbutton:
 		{
 			FORMBUTTON *gr = new FORMBUTTON(fl,args[1],0);
 			fl->New_form(gr);
-			ctx.newlevel (gr,NULL);
+			ctx.newlevel (gr);
 		}
 		break;
 	case PROTO_Setcontext:
+		// fprintf (stderr,"Setcontext :%s:\n",args[1]);
 		setcontext (ctx,args[1]);
 		break;
 	case PROTO_Form:
@@ -497,16 +760,30 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 			}else{
 				MFORM *gr = new MFORM(fl,id);
 				fl->New_form(gr);
-				ctx.newlevel (gr,NULL);
+				ctx.newlevel (gr);
 				// fprintf (stderr,"Ctx level %d\n",ctx.level);
 			}
 		}
 		break;
 	case PROTO_Clist:
-		fl->New_clist(args[1],atoi(args[2]),ptargs+3);
+		{
+			CLIST *cl = fl->New_clist(args[1],atoi(args[2]),ptargs+3);
+			ctx.newlevel (cl);
+		}
 		break;
 	case PROTO_Clist_item:
-		fl->New_clist_item (args[1],args[2],ptargs+3);
+		if (fl != NULL){
+			if (args[1][0] == '\0'){
+				CLIST *cl = (CLIST*)fl;
+				cl->New_item(args[2],ptargs+3);
+			}else{
+				fl->New_clist_item (args[1],args[2],ptargs+3);
+			}
+		}else if (setcontext (ctx,args[1])!=-1){
+			CLIST *cl = (CLIST*)ctx.tbfl[ctx.level];
+			cl->Set_item(args[2],ptargs+3);
+			ctx.poplevel();
+		}
 		break;
 	case PROTO_Sheet:
 		fl->New_sheet (args[1],atoi(args[2]),ptargs+3);
@@ -518,15 +795,25 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 		{
 			MFORM *gr = new TREEMENU(fl,args[1]);
 			fl->New_form(gr);
-			ctx.newlevel (gr,NULL);
+			ctx.newlevel (gr);
 		}
 		break;
 	case PROTO_Treeelem:
-		((TREEMENU*)fl)->addelm (args[1],args[2]);
+		{
+			wxBitmap *icon = args[1][0] != '\0'
+				? icon_xpm_locate(args[1])
+				: NULL;
+			((TREEMENU*)fl)->addelm (icon,args[2]);
+		}
 		break;
 	case PROTO_Treesub:
-		((TREEMENU*)fl)->addsub (atoi(args[1]),args[2],args[3]);
-		tree_collect++;
+		{
+			wxBitmap *icon = args[2][0] != '\0'
+				? icon_xpm_locate(args[2])
+				: NULL;
+			((TREEMENU*)fl)->addsub (atoi(args[1]),icon,args[3]);
+			tree_collect++;
+		}
 		break;
 	case PROTO_Page:
 		{
@@ -536,7 +823,7 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 			}else{
 				MFORM *gr = new MFORM(bk,id);
 				bk->New_page(gr,args[2]);
-				ctx.newlevel (gr,NULL);
+				ctx.newlevel (gr);
 			}
 		}
 		break;
@@ -544,7 +831,7 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 		{
 			BOOK *b = new BOOK(fl,args[1]);
 			fl->New_book(b);
-			ctx.newlevel (NULL,b);
+			ctx.newlevel (b);
 		}
 		break;
 	case PROTO_MainForm:
@@ -561,7 +848,8 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 					gr->delall();
 				}
 				gr->logicaltop = true;
-				ctx.newlevel (gr,NULL);
+				gr->max_width = 10000;
+				ctx.newlevel (gr);
 			}else if (bk != NULL){
 				MFORM *gr;
 				BOOK *dummy;
@@ -574,7 +862,8 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 				}
 				bk->selpage (gr);
 				gr->logicaltop = true;
-				ctx.newlevel (gr,NULL);
+				gr->max_width = 10000;
+				ctx.newlevel (gr);
 			}else{
 				MAINFORM *gr = NULL;
 				for (int i=0; i<maxmain; i++){
@@ -595,39 +884,68 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 						}
 					}
 				}
-				ctx.newlevel (gr,NULL);
+				ctx.newlevel (gr);
 			}
 		}
 		break;
 	case PROTO_End:
 		if (ctx.info.tarea != NULL){
+			ctx.info.tarea->SetValue (ctx.info.tarea_buf);
+			ctx.info.freebuf();
 			ctx.info.tarea = NULL;
+		}else if (ctx.info.combo != NULL){
+			ctx.info.combo = NULL;
 		}else if (html_collect){
 			html_collect = false;
 		}else if (tree_collect){
 			((TREEMENU*)fl)->endsub ();
 			tree_collect--;
 		}else{
+			wxMenu *mn = ctx.tbmn[ctx.level];
+			if (mn == NULL && ctx.menubar != NULL){
+				ctx.menubar = NULL;
+			}
 			if (ctx.level == 0){
 				// Normally, this is always the end of a mainform
-				// except for a setcontext, where it can be then end of
+				// except for a setcontext, where it can be the end of
 				// a sub-form or a book
-				if (fl != NULL){
-					fl->gettop()->Popup();
-				}else if (bk != NULL){
-					bk->gettop()->Popup();
+
+				if (rem_getvarval("nopopup")!=1){
+					if (mn != NULL){
+						// Trigger a popup
+						PopupMenu (NULL,mn,last_mousex,last_mousey);
+					}else if (fl != NULL){
+						fl->gettop()->Popup();
+					}else if (bk != NULL){
+						bk->gettop()->Popup();
+					}
+				}else{
+					if (fl != NULL){
+						fl->gettop()->waitpopup();
+					}
 				}
 			}
 			ctx.poplevel();
 			end = true;
 		}
 		break;
+	case PROTO_Show:
+		if (setcontext (ctx,args[1])!=-1){
+			MFORM *fl = ctx.tbfl[ctx.level];
+			MAINFORM *mf = fl->gettop();
+			if (strcmp(args[2],"1")==0){
+				mf->show();
+			}else{
+				mf->hide();
+			}
+			ctx.poplevel();
+		}
+		break;
 	case PROTO_Delete:
 		{
 			const char *id = args[1];
 			if (strchr(id,'.')!=NULL){
-				setcontext (ctx,id);
-				if (ctx.level >= 0){
+				if (setcontext (ctx,id) != -1){
 					MFORM *fl = ctx.tbfl[ctx.level];
 					FORMBASE *parent = fl->getparent();
 					parent->delform (fl);
@@ -656,12 +974,31 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 			}
 		}
 		break;
+	case PROTO_Deletechild:
+		{
+			const char *id = args[1];
+			if (setcontext (ctx,id) != -1){
+				MFORM *fl = ctx.tbfl[ctx.level];
+				if (fl != NULL){
+					fl->delchild();
+				}
+				ctx.poplevel ();
+			}
+		}
+		break;
 	case PROTO_Dump:
 		{
-			MAINFORM *mf = locate(args[1]);
-			if (mf != NULL){
-				mf->dump();
-				fflush (mform_fout);
+			int level = ctx.level;
+			setcontext (ctx,args[1]);
+			if (ctx.level > level){
+				FORMBASE *mf = ctx.tbfl[ctx.level];
+				if (mf == NULL) mf = ctx.tbbk[ctx.level];
+				if (mf != NULL){
+					mf->report_button (args[2],true,NULL,"");
+				}
+				ctx.poplevel();
+			}else{
+				fprintf (stderr,"Dump unknown form %s\n",args[1]);
 			}
 		}
 		break;
@@ -673,6 +1010,8 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 		break;
 	case PROTO_Textarea:
 		ctx.info.tarea = fl->New_text (args[1],atoi(args[2]),atoi(args[3]));
+		fl->Setweightlast (1,1);
+		ctx.info.initbuf();
 		break;
 	case PROTO_Slider:
 		fl->New_slider (args[1],atoi(args[2]),atoi(args[3]),atoi(args[4])
@@ -690,7 +1029,10 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 				if (mf == NULL) mf = ctx.tbbk[ctx.level];
 				mf->setval (args[2],(const char **)ptargs+3,ctx.info);
 				ctx.poplevel();
+			}else{
+				fprintf (stderr,"Setval unknown field %s %s\n",args[1],args[2]);
 			}
+
 		}	
 		break;
 	case 500:
@@ -703,6 +1045,141 @@ PRIVATE bool MFORMSCRIPT::oneline(char *buf, MFORMSCRIPT_CTX &ctx)
 			end = true;
 		}
 		break;
+	case PROTO_Defdc:
+		defs_setdc (args[1]);
+		break;
+	case PROTO_Deffont:
+		defs_setfont (args[1],atoi(args[2]),atoi(args[3]),atoi(args[4])
+			,atoi(args[5]),atoi(args[6])!=0);
+		break;
+	case PROTO_Defpen:
+		defs_setpen (args[1],args[2],atoi(args[3]),atoi(args[4]));
+		break;
+	case PROTO_Defbrush:
+		defs_setbrush (args[1],args[2],atoi(args[3]));
+		break;
+	case PROTO_Drawline:
+		fl->record_draw (PROTO_Drawline,atoi(args[1]),atoi(args[2])
+			,atoi(args[3]),atoi(args[4]));
+		break;
+	case PROTO_Drawrect:
+		fl->record_draw (PROTO_Drawrect,atoi(args[1]),atoi(args[2])
+			,atoi(args[3]),atoi(args[4]));
+		break;
+	case PROTO_Fillrect:
+		fl->record_draw (PROTO_Fillrect,atoi(args[1]),atoi(args[2])
+			,atoi(args[3]),atoi(args[4]));
+		break;
+	case PROTO_Fillpolygon:
+		{
+			int tb[20];
+			int i;
+			for (i=1; i<20; i++){
+				const char *arg = args[i];
+				if (!isdigit(arg[0])) break;
+				tb[i-1] = atoi(arg);
+			}
+			fl->record_draw (PROTO_Fillpolygon,tb,i-1);
+		}
+		break;
+	case PROTO_Fillarc:
+	case PROTO_Drawarc:
+		fl->record_draw (num,atoi(args[1]),atoi(args[2])
+			,atoi(args[3]),atoi(args[4])
+			,atoi(args[5]),atoi(args[6]));
+		break;
+	case PROTO_Clear:
+		fl->record_draw (PROTO_Clear,0,0,0,0);
+		break;
+	case PROTO_Drawclip:
+		fl->drawclip (atoi(args[1]),atoi(args[2]),atoi(args[3]),atoi(args[4]));
+		break;
+	case PROTO_Drawtext:
+		fl->record_draw (PROTO_Drawtext,atoi(args[1]),atoi(args[2]),args[3]);
+		break;
+	case PROTO_Refresh:
+		fl->do_drawing();
+		break;
+	case PROTO_Blit:
+		fl->blit (atoi(args[1]),atoi(args[2])
+			,atoi(args[3]),atoi(args[4])
+			,atoi(args[5]),atoi(args[6]));
+		break;
+	case PROTO_Menubar:
+		{
+			wxMenuBar *mn = new wxMenuBar;
+			ctx.newlevel(mn);
+			fl->getframe()->SetMenuBar(mn);
+		}
+		break;
+	case PROTO_Popupmenu:
+		{
+			free (popupid);
+			popupid = strdup(args[1]);
+			// We delete the last popup
+			delete currentpopup;
+			currentpopup = new wxMenu (args[2][0] != '\0'
+				? args[2] : (char*)NULL
+				,popup_func);
+			ctx.newlevel (currentpopup);
+		}
+		break;
+	case PROTO_Submenu:
+		{
+			wxMenu *mn = new wxMenu(NULL,remadmin_fctmenu);
+			wxMenu *cur = ctx.tbmn[ctx.level];
+			if (cur == NULL){
+				ctx.menubar->Append (mn,args[1]);
+				ctx.newlevel (mn);
+			}else{
+				cur->Append (0,args[1],mn);
+				ctx.newlevel (mn);
+			}
+		}
+		break;
+	case PROTO_Menuentry:
+		if (args[2][0] == '-'){
+			ctx.tbmn[ctx.level]->AppendSeparator();
+		}else{
+			ctx.tbmn[ctx.level]->Append (atoi(args[1]),args[2]);
+		}
+		break;
+	case PROTO_Menutitle:
+		if (currentpopup != NULL) currentpopup->SetTitle (args[1]);
+		break;
+	case PROTO_Setcursor:
+		break;
+	case PROTO_Listen:
+		{
+			int old_level = ctx.level;
+			setcontext (ctx,args[1]);
+			if (old_level == ctx.level){
+				fprintf (stderr,"Listen failed, missing dialog :%s: :%s:\n",args[1],args[2]);
+			}else{
+				bool en = atoi(args[2]) != 0;
+				MFORM *m = ctx.tbfl[ctx.level];
+				BOOK *b  = ctx.tbbk[ctx.level];
+				if (m != NULL){
+					m->Enable(en);
+					m->SetCursor (en ? cursor_normal : cursor_locked);
+				}else if (b != NULL){
+					b->Enable(en);
+				}
+				ctx.poplevel();
+			}
+		}
+		break;
+	case PROTO_Prefsubmenu:
+		// Do nothing for now
+		break;
+	case PROTO_Inputgrid:
+		fl->Inputgrid (args[1],atoi(args[2]),atoi(args[3]),atoi(args[4])
+			,atoi(args[5]),atoi(args[6]),atoi(args[7]));
+		break;
+	case PROTO_Splash:
+		break;
+	case PROTO_Hidesplash:
+		break;
 	default:
 		fprintf (stderr,"Unknown command %s\n",verb);
 	}
@@ -927,7 +1404,7 @@ static wxFrame *startup()
 {
 	//fprintf (stderr,"Start\n");
 	// Send our preference
-	fprintf (mform_fout,"prefer %s %s %s %s reconfdia slider gauge setval alive context\n"
+	fprintf (mform_fout,"prefer %s %s %s %s reconfdia slider gauge setval alive context menu listen clistdel\n"
 		,pref_modal ? "modal" : ""
 		,pref_nogfx ? "nogfx" : ""
 		,pref_notree ? "" : "treemenu"
