@@ -7,7 +7,8 @@ if [ $# -lt 2 ] ; then
 	echo publish_spec.sh guide_file base_name_of_output_file base_html_path subdir
 else
 	export LANG=en
-	proto -ceo+d -f/tmp/tmp.nap -q/tmp/tmp.cls *.cc *.h
+	tlproto -ceo+d -f/tmp/tmp.nap -q/tmp/tmp.cls *.h \
+		`find . -maxdepth 1 -name \*.cc -o -name \*.tlcc`
 	nadoc -a"Linuxconf team" -p -g $1 /tmp/tmp.nap /tmp/$2.sgml
 	(cd /tmp && sgml2html $2.sgml)
 	(cd /tmp && /usr/lib/linuxconf-tools/crossref $2*.html >$2.xref)
@@ -20,7 +21,7 @@ else
 		mkdir -p $3/$4/$2
 		# bug with virtualfs, mv fails because of utimes
 		#mv /tmp/$2*.html $3/$4/$2
-		cp /tmp/$2*.html $3/$4/$2/. && rm /tmp/$2*.html
+		bigcp -v /tmp/$2*.html $3/$4/$2/. && rm /tmp/$2*.html
 		(cd $3/$4/$2 && cp $2.html index.html)
 		if [ "$5" != "" ] ; then
 			rm -fr $5/$4/$2
