@@ -178,32 +178,13 @@ PUBLIC FIXPERM_SPEC::FIXPERM_SPEC(SPEC_ONE &sp)
 	if (fixperm_parseopts(sp,opts)==-1){
 		valid = false;
 	}
-	struct passwd *p = getpwnam (sp.owner);
-	struct group  *g = getgrnam (sp.group);
-	uid = gid = 0;
-	if (p == NULL){
-		fixperm_error (sp,MSG_U(E_NOUSER
-			,"No user \"%s\" defined on this system")
-			,sp.owner);
-		valid = false;
-	}else{
-		uid = p->pw_uid;
-	}
-	if (g == NULL){
-		fixperm_error (sp,MSG_U(E_NOGROUP
-			,"No group \"%s\" defined on this system")
-			,sp.group);
-		valid = false;
-	}else{
-		gid = g->gr_gid;
-	}
 }
 
 /*
 	Create an empty file or directory
 	Return -1 if any error.
 */
-PUBLIC int FIXPERM_SPEC::create()
+PUBLIC int FIXPERM_SPEC::create(PERMINFO &p)
 {
 	int ret = 0;
 	const char *fpath = path.get();
@@ -220,8 +201,8 @@ PUBLIC int FIXPERM_SPEC::create()
 		}
 	}
 	if (ret != 0
-		|| chown (fpath,uid,gid) == -1
-		|| chmod (fpath,perm) == -1){
+		|| chown (fpath,p.uid,p.gid) == -1
+		|| chmod (fpath,p.perm) == -1){
 		ret = -1;
 	}
 	return ret;
@@ -256,13 +237,25 @@ int fixperm_readperm (
 	p.uid = p.gid = (gid_t)-1;
 	int ret = 0;
 	if (pe == NULL){
-		if (!silent) xconf_error (MSG_R(E_NOUSER),owner);
+		if (!silent){
+			xconf_error (MSG_U(E_NOUSER
+				,"No user \"%s\" defined on this system\n"
+				 "Can't check permissions/ownership of the file or directory\n"
+				 "%s")
+				,owner,path);
+		}
 		ret = -1;
 	}else{
 		p.uid = pe->pw_uid;
 	}
 	if (ge == NULL){
-		if (!silent) xconf_error (MSG_R(E_NOGROUP),group);
+		if (!silent){
+			xconf_error (MSG_U(E_NOGROUP
+				,"No group \"%s\" defined on this system\n"
+				 "Can't check permissions/ownership of the file or directory\n"
+				 "%s")
+				,group,path);
+		}
 		ret = -1;
 	}else{
 		p.gid = ge->gr_gid;
@@ -276,85 +269,104 @@ int fixperm_readperm (
 PUBLIC int FIXPERM_SPEC::check()
 {
 	PERMINFO p;
-	if (valid
-		&& fixperm_readperm (path.get(),p,owner.get(),group.get(),perm
-			,false)!=-1){
-		uid = p.uid;
-		gid = p.gid;
-		perm = p.perm;
-		owner.setfrom (p.owner);
-		group.setfrom (p.group);
-		bool dook = !simul_ison();
+	if (valid){
+		/*
+			This spec may refer to a file normally owned by a missing
+			package. Further,the user/group of the spec are created
+			by the package, so do not exist in the system yet.
+
+			(For sure, fixperm specs should be supplied by the packages
+			 either using RPMS or something else. Unfortunatly, using
+			 would be too slow (checked everytime linuxconf exits).
+			)
+
+			In the mean time, we call fixperm_readperm only if we
+			we need to check the ownership and perm. This way, if
+			the file/directory is missing, then the missing
+			user/group are not important.
+		*/
 		struct stat st;
 		const char *fpath = path.get();
-		if (stat(fpath,&st)==-1){
-			/* #Specification: fixperm / symbolic links
-				Symbolic links are often used to move things
-				around in different file systems. The symbolic
-				links are faking the original hierarchy. This complicates
-				life for the "fixperm" functionnality of linuxconf.
-				In fact, in different time, linuxconf can't produce
-				a reliable status.
+		int ok = stat(fpath,&st);
+		if ((ok != -1 || opts.required)
+			&& fixperm_readperm (path.get(),p,owner.get(),group.get(),perm
+				,false)!=-1){
+			int uid = p.uid;
+			int gid = p.gid;
+			perm = p.perm;
+			owner.setfrom (p.owner);
+			group.setfrom (p.group);
 
-				For example my mail spool directory is on an NFS server.
-				Instead of mounting the server spool in /var/spool/mail,
-				I use the amd automounter and set a symbolic links like
-				this.
+			bool dook = !simul_ison();
+			if (ok==-1){
+				/* #Specification: fixperm / symbolic links
+					Symbolic links are often used to move things
+					around in different file systems. The symbolic
+					links are faking the original hierarchy. This complicates
+					life for the "fixperm" functionnality of linuxconf.
+					In fact, in different time, linuxconf can't produce
+					a reliable status.
 
-				ln -s /n/server/var/spool/mail /var/spool/mail
+					For example my mail spool directory is on an NFS server.
+					Instead of mounting the server spool in /var/spool/mail,
+					I use the amd automounter and set a symbolic links like
+					this.
 
-				Unfortunatly, linuxconf can't test the validity
-				of /var/spool/mail until the network is up. This
-				checking is annoying. It complains that /var/spool/mail
-				is not a directory and so on.
+					ln -s /n/server/var/spool/mail /var/spool/mail
 
-				So here is the patch. If a "something" has been
-				replaced by a symbolic links and the destination
-				of the link appears to be missing, linuxconf
-				won't complain at all.
-			*/
-			if (opts.required && lstat(fpath,&st)==-1){
-				net_prtlog (NETLOG_CMD,MSG_U(L_CREATING
-					,"Creating %s %s\n")
-					,S_ISDIR(perm)
-						? MSG_U(L_DIRECTORY,"directory")
-						: MSG_U(L_FILE,"file")
+					Unfortunatly, linuxconf can't test the validity
+					of /var/spool/mail until the network is up. This
+					checking is annoying. It complains that /var/spool/mail
+					is not a directory and so on.
+
+					So here is the patch. If a "something" has been
+					replaced by a symbolic links and the destination
+					of the link appears to be missing, linuxconf
+					won't complain at all.
+				*/
+				if (opts.required && lstat(fpath,&st)==-1){
+					net_prtlog (NETLOG_CMD,MSG_U(L_CREATING
+						,"Creating %s %s\n")
+						,S_ISDIR(perm)
+							? MSG_U(L_DIRECTORY,"directory")
+							: MSG_U(L_FILE,"file")
+						,fpath);
+					if (dook) create(p);
+				}
+			}else if ((st.st_mode & S_IFMT)
+				!= (mode_t)(perm & S_IFMT)){
+				net_prtlog (NETLOG_ERR
+					,MSG_U(E_CANTCHG
+						,"**** Can't change the type of file %s\n"
+						 "     manual action required!\n")
 					,fpath);
-				if (dook) create();
-			}
-		}else if ((st.st_mode & S_IFMT)
-			!= (mode_t)(perm & S_IFMT)){
-			net_prtlog (NETLOG_ERR
-				,MSG_U(E_CANTCHG
-					,"**** Can't change the type of file %s\n"
-					 "     manual action required!\n")
-				,fpath);
-		}else if ((S_ISBLK(perm) || S_ISCHR(perm))
-			&& st.st_rdev != (dev_t)dev){
-			net_prtlog (NETLOG_ERR
-				,MSG_U(E_WRONGDEV,"Device file %s wrongly created\n"
-					"\tExpected major %d, minor %d\n"
-					"\tFound    major %d, minor %d\n"
-					"\tManual action required\n")
-					,fpath,dev>>8,dev&0xff,st.st_rdev>>8,st.st_rdev&0xff);
-		}else{
-			char realpath[PATH_MAX];
-			file_followlink (fpath,realpath);
-			if (st.st_uid != (uid_t) uid
-				|| st.st_gid != (gid_t)gid){
-				if (dook) chown (realpath,uid,gid);
-				net_prtlog (NETLOG_CMD,MSG_U(L_CHGOWN,"Changing owner of file %s "
-					"to %s.%s\n")
-					,realpath,owner.get(),group.get());
-				stat(realpath,&st);
-			}
-			if (st.st_mode != (mode_t)perm){
-				if (dook) chmod (realpath,perm);
-				net_prtlog (NETLOG_CMD
-					,MSG_U(L_CHGPERM
-						,"Changing permissions  of file %s "
-						 "from %o to %o\n")
-					,realpath,st.st_mode,perm);
+			}else if ((S_ISBLK(perm) || S_ISCHR(perm))
+				&& st.st_rdev != (dev_t)dev){
+				net_prtlog (NETLOG_ERR
+					,MSG_U(E_WRONGDEV,"Device file %s wrongly created\n"
+						"\tExpected major %d, minor %d\n"
+						"\tFound    major %d, minor %d\n"
+						"\tManual action required\n")
+						,fpath,dev>>8,dev&0xff,st.st_rdev>>8,st.st_rdev&0xff);
+			}else{
+				char realpath[PATH_MAX];
+				file_followlink (fpath,realpath);
+				if (st.st_uid != (uid_t) uid
+					|| st.st_gid != (gid_t)gid){
+					if (dook) chown (realpath,uid,gid);
+					net_prtlog (NETLOG_CMD,MSG_U(L_CHGOWN,"Changing owner of file %s "
+						"to %s.%s\n")
+						,realpath,owner.get(),group.get());
+					stat(realpath,&st);
+				}
+				if (st.st_mode != (mode_t)perm){
+					if (dook) chmod (realpath,perm);
+					net_prtlog (NETLOG_CMD
+						,MSG_U(L_CHGPERM
+							,"Changing permissions  of file %s "
+							 "from %o to %o\n")
+						,realpath,st.st_mode,perm);
+				}
 			}
 		}
 	}
@@ -463,10 +475,39 @@ PUBLIC int FIXPERM_SPECS::check()
 {
 	return check (false,false);
 }
-
+/*
+	Fix the path of a perm file to select the one more appropriate to
+	the distribution version.
+*/
+static void fixperm_fixpath (
+	const char *prefix,			// General prefix
+	const char *file,
+	char path[PATH_MAX])
+{
+	char prefix_brev[PATH_MAX];	// Prefix for the base version of the distribution
+	char prefix_rev[PATH_MAX];	// For the distribution version
+	// prefix is something like /usr/lib/linuxconf/redhat/perm
+	// prefix_brev is prefix + /6 for any redhat 6.x
+	// prefix_rev is prefix + /
+	sprintf (prefix_rev,"%s/%s",prefix,distrib_getrelease());
+	strcpy (prefix_brev,prefix_rev);
+	{
+		char *pt = strchr(prefix_brev,'.');
+		if (pt != NULL) *pt = '\0';
+	}
+	if (file[0] != '\0'){
+		sprintf (path,"%s/%s",prefix_rev,file);
+		if (!file_exist(path)){
+			sprintf (path,"%s/%s",prefix_brev,file);
+			if (!file_exist(path)){
+				sprintf (path,"%s/%s",prefix,file);
+			}
+		}
+	}
+}
 
 static int fixperm_check (
-	const char *prefix,
+	const char *prefix,			// General prefix
 	SSTRINGS &lst,
 	bool boottime,
 	bool silentflag)
@@ -476,7 +517,7 @@ static int fixperm_check (
 		const char *ptf = lst.getitem(i)->get();
 		if (ptf[0] != '\0'){
 			char path[PATH_MAX];
-			sprintf (path,"%s/%s",prefix,ptf);
+			fixperm_fixpath (prefix,ptf,path);
 			FIXPERM_SPECS specs(path);
 			if (specs.check(boottime,silentflag) == -1) ret = -1;
 		}
@@ -486,19 +527,19 @@ static int fixperm_check (
 
 
 static void fixperm_getspecf(
-	SSTRINGS &usr_tb,
-	SSTRINGS &var_tb,
+	SSTRINGS &usr_tb,		// Will contains perm files in /usr/lib/linuxconf/...
+	SSTRINGS &var_tb,		// Will contains perm files in /var/lib/...
 	char pathperm[PATH_MAX])
 {
 	/* #Specification: fixperm / /var/lib/conf.permissions / strategy
-		Linuxconf provide its own set of permissions files
-		int /usr/lib/linuxconf/conf.permissions. These are
-		supplied with linuxconf and will probably get updated
+		Linuxconf provides its own set of permissions files
+		int /usr/lib/linuxconf/DIST/perm/. These are
+		supplied with linuxconf and are updated
 		at each new releases.
 
 		You can create permission files in /var/lib/conf.permissions.
 		If you create a permission file with the same name as
-		one in /usr/lib/linuxconf/conf.permissions, yours will
+		one in /usr/lib/linuxconf/DIST/perm/, yours will
 		take precedence. This allows you to do customisation
 		of those file without fearing the next update :-)
 	*/
@@ -687,7 +728,7 @@ static void fstab_loadspecs(
 		const char *ptf = lst.getitem(i)->get();
 		if (ptf[0] != '\0'){
 			char path[PATH_MAX];
-			sprintf (path,"%s/%s",prefix,ptf);
+			fixperm_fixpath (prefix,ptf,path);
 			specs.readspecf (path);
 		}
 	}
