@@ -38,14 +38,14 @@ public:
 	mode_t perm;
 	dev_t dev;
 	FIXPERM_OPTIONS opts;
-	uid_t uid;
-	gid_t gid;
+	//uid_t uid;
+	//gid_t gid;
 	bool valid;		// Is this record valid
 	/*~PROTOBEG~ FIXPERM_SPEC */
 public:
 	FIXPERM_SPEC (SPEC_ONE&sp);
 	int check (void);
-	int create (void);
+	int create (PERMINFO&p);
 	int edit (void);
 private:
 	void parsedev (SPEC_ONE&sp);
