@@ -1,5 +1,7 @@
 #pragma implementation
 #include <stdio.h>
+#include <stdlib.h>
+#include <ctype.h>
 #include <string.h>
 #include <configf.h>
 #include <misc.h>
@@ -357,7 +359,8 @@ PUBLIC void QUOTA_DEV::deluserdef_gspec (const char *name)
 	userdef_g.remove_del (getuserdef_gspec(name));
 }
 
-#if defined(__alpha__)
+#ifndef __GLIBC__
+#if defined (__alpha__)
 	#include <errno.h>
 	#include <syscall.h>
 	#include <asm/unistd.h>
@@ -372,6 +375,7 @@ PUBLIC void QUOTA_DEV::deluserdef_gspec (const char *name)
 
 	_syscall4(int, quotactl, int, cmd, const char *, special, int, id, caddr_t, addr);
 #endif
+#endif
 
 
 
@@ -403,10 +407,32 @@ PUBLIC void QUOTA_DEV::applyone (struct passwd *p)
 		if (g != NULL){
 			QUOTA_SPEC eff;
 			geteffuserspec (p->pw_name,g->gr_name,eff);
-			quota_apply (eff,p->pw_uid,USRQUOTA,device.get());
+			quota_apply (eff,p->pw_uid,USRQUOTA,realdev.get());
 		}
 	}
 }
+/*
+	Transform a LABEL= specification into a device
+	Return -1 if this can't be done.
+*/
+PUBLIC int QUOTA_DEV::setrealdevice()
+{
+	int ret = 0;
+	const char *dev = device.get();
+	realdev.setfrom (dev);
+	if (strncasecmp(dev,"LABEL=",6)==0){
+		dev = partition_findfromlabel(dev+6);
+		if (dev == NULL){
+			xconf_error (MSG_U(E_TRANSLATELABEL
+				,"Can't translate the file system label %s to any device.\n"
+				 "Won't apply quota"),device.get());
+			ret = -1;;
+		}else{
+			realdev.setfrom (dev);
+		}
+	}
+	return ret;
+}
 
 /*
 	Fill the user and group quota for all users and group on the system
@@ -414,18 +440,20 @@ PUBLIC void QUOTA_DEV::applyone (struct passwd *p)
 */
 PUBLIC void QUOTA_DEV::applyall()
 {
-	const char *devstr = device.get();
-	setpwent();
-	struct passwd *p;
-	while ((p=getpwent())!=NULL){
-		applyone (p);
-	}
-	setgrent ();
-	struct group *g;
-	while ((g=getgrent())!=NULL){
-		QUOTA_SPEC eff;
-		geteffgroupspec (g->gr_name,eff);
-		quota_apply (eff,g->gr_gid,GRPQUOTA,devstr);
+	if (setrealdevice()!=-1){
+		const char *devstr = realdev.get();
+		setpwent();
+		struct passwd *p;
+		while ((p=getpwent())!=NULL){
+			applyone (p);
+		}
+		setgrent ();
+		struct group *g;
+		while ((g=getgrent())!=NULL){
+			QUOTA_SPEC eff;
+			geteffgroupspec (g->gr_name,eff);
+			quota_apply (eff,g->gr_gid,GRPQUOTA,devstr);
+		}
 	}
 }
 
@@ -493,10 +521,10 @@ PUBLIC void QUOTACTL::editdef ()
 			const char *device = d->device.get();
 			dia.newf_title (device,1,"",device);
 			QUOTA_SPEC *s = &d->userdef;
-			dia.newf_title ("",MSG_U(T_USERDEF,"User default"));
+			dia.newf_title (MSG_U(T_USERDEF,"User default"),2,"",MSG_R(T_USERDEF));
 			s->setupdia (dia);
 			s = &d->groupdef;
-			dia.newf_title ("",MSG_U(T_GROUPDEF,"Group default"));
+			dia.newf_title (MSG_U(T_GROUPDEF,"Group default"),2,"",MSG_R(T_GROUPDEF));
 			s->setupdia (dia);
 		}
 		int nof = 0;
@@ -578,7 +606,8 @@ PUBLIC int QUOTACTL::applyone (const char *user)
 	struct passwd *p = getpwnam (user);
 	if (p != NULL){
 		for (int i=0; i<devs.getnb(); i++){
-			devs.getitem(i)->applyone(p);
+			QUOTA_DEV *dev = devs.getitem(i);
+			if (dev->setrealdevice()!=-1) dev->applyone(p);
 		}
 		ret = 0;
 	}
@@ -618,7 +647,8 @@ void quota_applyall ()
 	for (int i=0; i<fstab.getnb(); i++){
 		FSTAB_ENTRY *e = fstab.getitem(i);
 		if (e->has_quota_u() || e->has_quota_g()){
-			quota_applyall (e->getsource());
+			const char *dev = e->getsource();
+			quota_applyall (dev);
 		}
 	}
 }
@@ -763,31 +793,34 @@ PUBLIC int QUOTA_EDIT::save(PRIVILEGE *priv)
 		}
 		QUOTA_SPEC new_eff;
 		geteffspec (d,new_eff);
+
 		if (is_new || old_eff.isdiff (new_eff)){
-			const char *device = d->device.get();
-			if (type == QUOTA_USER){
-				struct passwd *p = getpwnam (nam);
-				if (p != NULL){
-					quota_apply (new_eff,p->pw_uid,USRQUOTA,device);
-				}
-			}else if (type == QUOTA_GROUP){
-				struct group *g = getgrnam (nam);
-				if (g != NULL){
-					quota_apply (new_eff,g->gr_gid,GRPQUOTA,device);
-				}
-			}else if (type == QUOTA_USERDEF_G){
-				// We have to walk all member of the group and maybe
-				// update their quota
-				struct group *g = getgrnam (nam);
-				if (g != NULL){
-					gid_t gid = g->gr_gid;
-					setpwent();
-					struct passwd *p;
-					while ((p=getpwent())!=NULL){
-						if (p->pw_gid == gid){
-							QUOTA_SPEC eff;
-							d->geteffuserspec (p->pw_name,nam,eff);
-							quota_apply (eff,p->pw_uid,USRQUOTA,device);
+			if (d->setrealdevice()!=-1){
+				const char *device = d->realdev.get();
+				if (type == QUOTA_USER){
+					struct passwd *p = getpwnam (nam);
+					if (p != NULL){
+						quota_apply (new_eff,p->pw_uid,USRQUOTA,device);
+					}
+				}else if (type == QUOTA_GROUP){
+					struct group *g = getgrnam (nam);
+					if (g != NULL){
+						quota_apply (new_eff,g->gr_gid,GRPQUOTA,device);
+					}
+				}else if (type == QUOTA_USERDEF_G){
+					// We have to walk all member of the group and maybe
+					// update their quota
+					struct group *g = getgrnam (nam);
+					if (g != NULL){
+						gid_t gid = g->gr_gid;
+						setpwent();
+						struct passwd *p;
+						while ((p=getpwent())!=NULL){
+							if (p->pw_gid == gid){
+								QUOTA_SPEC eff;
+								d->geteffuserspec (p->pw_name,nam,eff);
+								quota_apply (eff,p->pw_uid,USRQUOTA,device);
+							}
 						}
 					}
 				}
@@ -809,8 +842,8 @@ PUBLIC void QUOTA_EDIT::setupdia (DIALOG &dia, const char *maintitle)
 			QUOTA_SPEC *s = specs.getitem(i);
 			QUOTA_DEV *d = ctl->devs.getitem(i);
 			const char *title = d->device.get();
-			//dia.newf_title (title,2,"",title);
-			dia.newf_title ("",title);
+			dia.newf_title (title,2,"",title);
+			// dia.newf_title ("",title);
 			static const char *tbvals[]={
 				MSG_U(I_DEFAULT,"Default"),
 				MSG_R(I_NOLIMIT),
@@ -829,3 +862,103 @@ PUBLIC void QUOTA_EDIT::setupdia (DIALOG &dia, const char *maintitle)
 	}
 }
 
+PUBLIC QUOTA_DEV *QUOTACTL::getdev (const char *device)
+{
+	QUOTA_DEV *ret = NULL;
+	for (int i=0; i<devs.getnb(); i++){
+		QUOTA_DEV *dev = devs.getitem(i);
+		if (dev->device.cmp(device)==0){
+			ret = dev;
+			break;
+		}
+	}
+	return ret;
+}
+
+static void quota_setif (const char *val, int &field)
+{
+	if (val != NULL){
+		if (strcmp(val,"default")==0){
+			field = -1;
+		}else if(!isdigit(val[0])){
+			fprintf (stderr
+				,MSG_U(E_IVLDQUOTAVAL
+					,"Invalid quota value %s: expect default or a numeric value\n")
+				,val);
+		}else{
+			field = atoi(val);
+		}
+	}
+}
+
+/*
+	Command line support to set disk quota
+*/
+int quota_setquota (
+	bool is_group,		// Update group or user quota
+	const char *name,	// Group or user name
+	int argc,
+	char *argv[])
+{
+	int ret = -1;
+	bool some_errors = false;
+	const char *device = NULL;
+	const char *softk = NULL;
+	const char *hardk = NULL;
+	const char *softf = NULL;
+	const char *hardf = NULL;
+	int i;
+	for (i=0; i<argc-1; i+=2){
+		const char *opt = argv[i];
+		const char *arg = argv[i+1];
+		if (strcmp(opt,"--device")==0){
+			device = arg;
+		}else if (strcmp(opt,"--softk")==0){
+			softk = arg;
+		}else if (strcmp(opt,"--hardk")==0){
+			hardk = arg;
+		}else if (strcmp(opt,"--softf")==0){
+			softf = arg;
+		}else if (strcmp(opt,"--hardf")==0){
+			hardf = arg;
+		}else{
+			fprintf (stderr,MSG_U(E_IVLDQUOTAOPT,"Invalid setquota option %s\n")
+				,opt);
+			some_errors = true;
+			break;
+		}
+	}
+	if (i!=argc){
+		if (!some_errors){
+			fprintf (stderr,MSG_R(E_IVLDQUOTAOPT),argv[i]);
+		}
+	}else{
+		if (device == NULL){
+			fprintf (stderr,MSG_U(E_MISSDEVICE,"Missing --device option\n"));
+		}else{
+			QUOTACTL ctl;
+			QUOTA_DEV *dev = ctl.getdev(device);
+			if (device == NULL){
+				fprintf (stderr,MSG_U(E_NOQUOTADEV
+					,"Quota not available for device %s\n"),device);
+			}else{
+				QUOTA_SPECS *specs = is_group ? &dev->groups : &dev->users;
+				QUOTA_SPEC *spec = specs->getitem(name);
+				if (spec == NULL){
+					spec = new QUOTA_SPEC;
+					spec->name.setfrom (name);
+					specs->add (spec);
+				}
+				quota_setif (softk,spec->soft_maxk);
+				quota_setif (hardk,spec->hard_maxk);
+				quota_setif (softf,spec->soft_maxf);
+				quota_setif (hardf,spec->hard_maxf);
+				ret = ctl.write(NULL);
+				if (ret == 0) quota_applyall();
+			}
+		}
+	}
+	return ret;
+	return ret;
+}
+
