@@ -28,15 +28,18 @@ else
 	fi
 	NBHELP=`find . -name \*.help | wc -l`
 	NBHTML=`find . -name \*.html | wc -l`
+	if [ "`id -u`" = "0" ] ; then
+		INSTROOT="-o root -g root"
+	fi
 	if [ $NBHELP != 0 -a $NBHTML != 0 ] ; then
 		echo "    " Installing "*.html and *.help" in $HELPDIR
-		install -m 644 -o root -g root *.help *.html $HELPDIR
+		install -m 644 $INSTROOT *.help *.html $HELPDIR
 	elif [ $NBHELP != 0 ] ; then
 		echo "    " Installing "*.help" in $HELPDIR
-		install -m 644 -o root -g root *.help $HELPDIR
+		install -m 644 $INSTROOT *.help $HELPDIR
 	elif [ $NBHTML != 0 ] ; then
 		echo "    " Installing "*.html" in $HELPDIR
-		install -m 644 -o root -g root *.html $HELPDIR
+		install -m 644 $INSTROOT *.html $HELPDIR
 	else
 		echo "    " Nothing to install
 	fi
