@@ -15,21 +15,18 @@
 #include "../paths.h"
 #include <translat.h>
 #include "main.m"
+#include <usercomng.h>
 
 HELP_FILE help_mainintro (HELP_MAIN,"intro");
 static HELP_FILE help_control (HELP_MAIN,"control");
-static HELP_FILE help_shutdown (HELP_MAIN,"shutdown");
 static HELP_FILE help_features (HELP_MAIN,"features");
 static HELP_FILE help_logs (HELP_MAIN,"logs");
 static HELP_FILE help_ctrlfile (HELP_MAIN,"ctrlfile");
 static HELP_FILE help_keymap(HELP_MAIN,"keymap");
 
-static PRIVILEGE p_shutdown ("shutdown"
-	,P_MSG_U(T_PRIVSHUTDOWN,"May shutdown")
-	,P_MSG_U(T_PSYSCONTROL,"0-General system control"));
 static PRIVILEGE p_actchg ("actchg"
 	,P_MSG_U(T_PACTCHG,"May activate config changes")
-	,P_MSG_R(T_PSYSCONTROL));
+	,P_MSG_U(T_PSYSCONTROL,"0-General system control"));
 static PRIVILEGE p_netlevel ("netlevel"
 	,P_MSG_U(T_PNETLEVEL,"May switch network mode")
 	,P_MSG_R(T_PSYSCONTROL));
@@ -38,39 +35,13 @@ static PRIVILEGE p_logs ("viewlogs"
 	,P_MSG_R(T_PSYSCONTROL));
 
 CONFIG_FILE f_treemenu (VAR_STATE_TREEMENU_CACHE,help_nil
-	,CONFIGF_GENERATED|CONFIGF_OPTIONAL);
+	,CONFIGF_GENERATED|CONFIGF_OPTIONAL|CONFIGF_NOARCH);
 CONFIG_FILE f_treemenu_log (VAR_RUN_TREEMENU_LOG,help_nil
-	,CONFIGF_GENERATED|CONFIGF_OPTIONAL);
+	,CONFIGF_GENERATED|CONFIGF_OPTIONAL|CONFIGF_NOARCH);
 
-/*
-	Interactive control of a shutdown or reboot
-*/
-void linuxconf_boot()
-{
-	if (perm_access (&p_shutdown,MSG_U(P_SHUTDOWN
-		,"shutdown the computer"))){
-		DIALOG dia;
-		char oper=0;
-		dia.newf_radio ("",oper,0,MSG_U(F_REBOOT,"Reboot"));
-		dia.newf_radio ("",oper,1,MSG_U(F_HALT,"Shutdown & halt"));
-		int grace = 0;
-		dia.newf_num   (MSG_U(F_DELAI,"Delay before shutdown(minutes)"),grace);
-		SSTRING message;
-		dia.newf_str   (MSG_U(F_MESSAGE,"Message"),message);
-		if (dia.edit (MSG_U(T_SHUTDOWN,"Shutdown control panel")
-			,MSG_U(I_SHUTDOWN,"You can force a reboot or a final shutdown\n"
-				"of this computer")
-			,help_shutdown)==MENU_ACCEPT){
-			setuid (0);
-			chdir ("/");
-			char timestr[10];
-			sprintf (timestr,"+%d",grace);
-			execl ("/sbin/shutdown","/sbin/shutdown"
-				,oper == 0 ? "-r" : "-h"
-				,timestr,message.get(),NULL);	
-		}
-	}
-}
+static CONFIG_FILE f_dmesg (VAR_LOG_DMESG,help_nil
+	,CONFIGF_OPTIONAL|CONFIGF_NOARCH);
+		
 
 /*
 	control panel to activate different things, not to configure.
@@ -135,7 +106,7 @@ static int linuxconf_control()
 						}
 					}
 				}else if (key == reboot){
-					linuxconf_boot();
+					shutdown_control();
 				}else if (key == runlevel){
 					if (perm_access (&p_netlevel,MSG_U(P_NETLEVEL
 						,"switch network operation level"))){
@@ -233,6 +204,10 @@ int linuxconf_setkeymap()
 	static const char keymap_arch[]="sparc";
 #elif defined(__alpha__)
 	static const char keymap_arch[]="alpha";
+#elif defined(__mips__)
+	// Just so it compiles on Cobalt. I have not seen a real
+	// Mips/Linux workstation
+	static const char keymap_arch[]="mips";
 #elif defined(__powerpc__)
 	static const char keymap_arch[]="ppc";
 #elif defined(__mc68000__)
@@ -247,6 +222,8 @@ int linuxconf_setkeymap()
 	    { "Motorola", "mvme" },
 	    { NULL, NULL }
 	};
+#elif defined(__ia64__)
+	static const char keymap_arch[]="ia64";
 #else
 	#error Need a ifdef for your architecture here
 #endif
@@ -320,6 +297,7 @@ static void linuxconf_features()
 {
 	int html_timeout = linuxconf_gethtmltimeout();
 	DIALOG dia;
+	dia.newf_title (MSG_U(T_UIFEATURE,"User interface"),1,"",MSG_R(T_UIFEATURE));
 	SSTRING kmap;
 	kmap.setfrom (linuxconf_getkeymap());
 	SSTRING old_kmap(kmap);
@@ -377,13 +355,18 @@ static void linuxconf_features()
 		,"May use the colors in text mode"));
 	int trig = linuxconf_getprefixtrig();
 	dia.newf_num (MSG_U(F_PREFIXTRIG,"Trigger for filter"),trig);
-	SSTRING bodyparm;
-	bodyparm.setfrom (html_getbodyparm());
-	dia.newf_str (MSG_U(F_HTMLBODYPARM,"Html body parameters"),bodyparm);
+	SSTRING bodyparm,headparm;
+	html_getpageparm(bodyparm,headparm);
+	dia.newf_str (MSG_U(F_HTMLHEADPARM,"HTML HEAD parameters"),headparm);
+	dia.newf_str (MSG_U(F_HTMLBODYPARM,"HTML BODY parameters"),bodyparm);
 
 	char uselynx = dialog_usinglynx();
 	dia.newf_chk ("",uselynx,MSG_U(F_USELYNX,"Use lynx to display help"));
 
+	USERACCT_COMNGS comngs;
+	comngs.getall  ("features");
+	comngs.setupdia (dia);
+
 	int nof = 0;
 	while (1){
 		MENU_STATUS code = dia.edit (MSG_U(T_FEATURES,"Features")
@@ -395,7 +378,8 @@ static void linuxconf_features()
 			break;
 		}else if (html_timeout < 5){
 			xconf_error (MSG_U(E_HTMLTIMEOUT,"The html timeout must be 5 minutes or higher"));
-		}else{
+		}else if (comngs.validate(dia,nof) != -1){
+			comngs.save (NULL);
 			linuxconf_replace (LINUXCONF,HTMLTIMEOUT,html_timeout);
 			if (kmap.cmp(MSG_R(F_NONE))==0) kmap.setfrom ("");
 			if (kmap.is_empty()){
@@ -418,11 +402,11 @@ static void linuxconf_features()
 			linuxconf_setguimode(usegui != 0);
 			linuxconf_setcolormode(usecol != 0);
 			linuxconf_setprefixtrig(trig);
-			html_setbodyparm (bodyparm.get());
+			html_setpageparm (bodyparm.get(),headparm.get());
 			dialog_setuselynx (uselynx != 0);
 			linuxconf_save();
 			if (kmap.cmp(old_kmap)!=0
-				&& xconf_yesno(MSG_U(T_CHGKMAP,"Changing keyboard map")
+				&& dialog_yesno(MSG_U(T_CHGKMAP,"Changing keyboard map")
 					,MSG_U(I_CHGMAP
 						,"You have changed the keyboard mapping specification.\n"
 						 "Do you want to activate the change now ?\n"
@@ -448,15 +432,17 @@ void linuxconf_recordkeymap(const char *map)
 static void linuxconf_logs()
 {
 	int choice = 0;
+	static const char *dmesg = MSG_U(M_DMESGLOGS,"Kernel boot messages");
 	static const char *bootlogs = MSG_U(M_BOOTLOGS,"Boot messages");
 	static const char *mylogs   = MSG_U(M_LINUXCONFLOGS,"Linuxconf logs");
 
 	static const char *menuopt[]={
-		" ",		bootlogs,
-		" ",		mylogs,
+		"bootlogs:",		bootlogs,
+		"mylogs:",		mylogs,
 		NULL
 	};
 	DIALOG_MENU dia;
+	if (f_dmesg.exist()) dia.new_menuitem ("dmesg","",dmesg);
 	dia.new_menuitems (menuopt);
 	module_setmenu (dia,MENU_LOGS);
 	while (1){
@@ -465,7 +451,7 @@ static void linuxconf_logs()
 				,"This menu lets you see the different messages generated\n"
 				 "by the system. These messages are normally checked to\n"
 				 "either detect some problems or as a proof that some event\n"
-				 "happeneded.")
+				 "happened.")
 			,help_logs
 			,choice,0);
 		if (code != MENU_OK){
@@ -474,7 +460,9 @@ static void linuxconf_logs()
 			,"view system logs"))){
 			const char *key = dia.getmenustr(choice);
 			module_domenu (MENU_LOGS,key);
-			if (key == bootlogs){
+			if (key == dmesg){
+				dialog_textbox (dmesg,f_dmesg.getpath());
+			}else if (key == bootlogs){
 				boot_showlog();
 			}else if (key == mylogs){
 				net_showlog();
@@ -485,13 +473,14 @@ static void linuxconf_logs()
 
 void linuxconf_editctrl()
 {
-	static const char *config_file	= MSG_U(MNU_CONFIGS,"Configure all configuration files");
-	static const char *commands	= MSG_U(MNU_COMMANDS,"Configure all commands and daemons");
-	static const char *perms	= MSG_U(MNU_PERMS,"Configure file permission and ownership");
-	static const char *config_module = MSG_U(MNU_MODULE,"Configure Linuxconf modules");
-	static const char *add_on_local	= MSG_U(MNU_ADD_ON,"Override Linuxconf addons");
-	static const char *add_on_orig	= MSG_U(MNU_ADD_ON_ORIG,"Create Linuxconf addons");
-	static const char *config_version = MSG_U(M_CONFVER,"Configure system profiles");
+	static const char *config_file	= MSG_U(MNU_CONFIGS,"Configuration files");
+	static const char *commands	= MSG_U(MNU_COMMANDS,"Commands and daemons");
+	static const char *perms	= MSG_U(MNU_PERMS,"Permission and ownership");
+	static const char *config_module = MSG_U(MNU_MODULE,"Modules");
+	static const char *add_on_local	= MSG_U(MNU_ADD_ON,"Override addons");
+	static const char *add_on_orig	= MSG_U(MNU_ADD_ON_ORIG,"Create addons");
+	static const char *config_version = MSG_U(M_CONFVER,"System profiles");
+	static const char *config_features = MSG_U(M_FEATURES,"Miscellaneous");
 	int choice = 0;
 	static const char *menuopt[]={
 		" ",	config_file,
@@ -501,6 +490,7 @@ void linuxconf_editctrl()
 		" ",	config_version,
 		" ",    add_on_local,
 		" ",    add_on_orig,
+		" ",    config_features,
 		
 		NULL
 	};
@@ -533,6 +523,8 @@ void linuxconf_editctrl()
 				dropin_editorig();
 			}else if (key == config_version){
 				confver_edit();
+			}else if (key == config_features){
+				linuxconf_features();
 			}else{
 				module_domenu (MENU_CTRL_FILE,key);
 			}
@@ -540,17 +532,19 @@ void linuxconf_editctrl()
 	}
 }
 
-static void linuxconf_miscmenu()
+static void linuxconf_somemenu(
+	MENU_CONTEXT ctx,
+	const char *title,
+	const char *error)
 {
 	DIALOG_MENU dia;
-	module_setmenu (dia,MENU_MISCSERV);
+	module_setmenu (dia,ctx);
 	if (dia.getnb()==0){
-		xconf_error (MSG_U(E_NOMISCSERV,"No misc services registered"));
+		xconf_error (error);
 	}else{
 		int nof = 0;
 		while (1){
-			MENU_STATUS code = dia.editmenu (MSG_U(T_MISCSERV
-				,"Miscellaneous services")
+			MENU_STATUS code = dia.editmenu (title
 				,""
 				,help_nil
 				,nof,0);
@@ -558,7 +552,7 @@ static void linuxconf_miscmenu()
 				break;
 			}else{
 				const char *key = dia.getmenustr(nof);
-				module_domenu (MENU_MISCSERV,key);
+				module_domenu (ctx,key);
 			}
 		}
 	}
@@ -569,23 +563,30 @@ static const char *config_askrun ;
 static const char *view_logs	 ;
 static const char *config_fstab	 ;
 static const char *config_misc	 ;
+static const char *config_hard	 ;
 static const char *config_graphic;
 static const char *config_net	 ;
 static const char *config_user	;
 static const char *config_datetime ;
 static const char *control	;
 static const char *ctrlfile	;
-static const char *config_features ;
 
 static void ft(void *p){
 	const char *key = (const char *)p;
 	module_domenu(MENU_MAIN_CONFIG,key);
 	module_domenu(MENU_MAIN_CONTROL,key);
 	module_domenu(MENU_MAIN_STATUS,key);
+	module_domenu(MENU_GURUS,key);
 	if (key == config_fstab){
 		fstab_edit ();
 	}else if (key == config_misc){
-		linuxconf_miscmenu ();
+		linuxconf_somemenu (MENU_MISCSERV
+			,MSG_U(T_MISCSERV,"Miscellaneous services")
+			,MSG_U(E_NOMISCSERV,"No misc services registered"));
+	}else if (key == config_hard){
+		linuxconf_somemenu (MENU_HARDWARE
+			,MSG_U(T_PERIPHERAL,"Peripherals/hardware configuration")
+			,MSG_U(E_NOPERIPHERAL,"No module registered"));
 	}else if (key == config_graphic){
 		xconf_edit ();
 	}else if (key == config_net){
@@ -602,8 +603,6 @@ static void ft(void *p){
 		was_checked = linuxconf_control();
 	}else if (key == ctrlfile){
 		linuxconf_editctrl();
-	}else if (key == config_features){
-		linuxconf_features();
 	}
 }
 
@@ -630,19 +629,24 @@ void linuxconf_formatversion (char version[80])
 
 void linuxconf_setmaintitle(char title[80],char sidetitle[80])
 {
-	extern char *revision;
-	THISHOST host;
-	const char *name = host.getname1();
-	char version[80];
-	linuxconf_formatversion(version);
-	snprintf (title,79,"%s: %s",name,version);
-	#if LINUXCONF_SUBSUBREVISION > 0
-		sprintf (sidetitle,"LINUXCONF %s.%d",revision,LINUXCONF_SUBREVISION);
-	#elif LINUXCONF_SUBREVISION > 0
-		sprintf (sidetitle,"LINUXCONF %s.%d",revision,LINUXCONF_SUBREVISION);
-	#else
-		sprintf (sidetitle,"LINUXCONF %s",revision);
-	#endif
+	if (dialog_mode == DIALOG_HTML){
+		strcpy_cut (title,MSG_U(M_MAINMENU,"Main menu"),79);
+		sidetitle[0] = '\0';
+	}else{
+		extern char *revision;
+		THISHOST host;
+		const char *name = host.getname1();
+		char version[80];
+		linuxconf_formatversion(version);
+		snprintf (title,79,"%s: %s",name,version);
+		#if LINUXCONF_SUBSUBREVISION > 0
+			sprintf (sidetitle,"LINUXCONF %s.%d",revision,LINUXCONF_SUBREVISION);
+		#elif LINUXCONF_SUBREVISION > 0
+			sprintf (sidetitle,"LINUXCONF %s.%d",revision,LINUXCONF_SUBREVISION);
+		#else
+			sprintf (sidetitle,"LINUXCONF %s",revision);
+		#endif
+	}
 }
 
 /*
@@ -651,17 +655,17 @@ void linuxconf_setmaintitle(char title[80],char sidetitle[80])
 void linuxconf_mainmenu(bool dontcheck)
 {
 	int choice = 0;
-	config_askrun = MSG_U(MNU_BOOT,"boot mode");
-	view_logs	 = MSG_U(MNU_LOGS,"logs");
+	config_askrun = MSG_U(MNU_BOOT,"Boot");
+	view_logs	 = MSG_U(MNU_LOGS,"Logs");
 	config_fstab = MSG_U(MNU_FILESYS,"File systems");
-	config_misc	 = MSG_U(MNU_MISCSERV,"Miscellaneous services");
+	config_misc	 = MSG_U(MNU_MISCSERV,"Miscellaneous");
+	config_hard  = MSG_U(MNU_HARDWARE,"Peripherals");
 	config_graphic= MSG_U(MNU_GRAPH,"Graphic mode (X)");
 	config_net	 = MSG_U(MNU_NET,"Networking");
-	config_user	= MSG_U(MNU_USERS,"Users accounts");
-	config_datetime = MSG_U(MNU_DATE,"date & time");
+	config_user	= MSG_U(MNU_USERS,"Users");
+	config_datetime = MSG_U(MNU_DATE,"Date & time");
 	control	= MSG_U(MNU_CONTROL,"Control panel");
-	ctrlfile	= MSG_U(MNU_CTRLFILE,"Control files and systems");
-	config_features = MSG_U(M_FEATURES,"Features");
+	ctrlfile	= MSG_U(MNU_CTRLFILE,"Linuxconf management");
 
 	static const char *menuopt1[]={
 		"-",		MSG_U(M_CONFIG,"Config"),
@@ -669,6 +673,7 @@ void linuxconf_mainmenu(bool dontcheck)
 		"users:",	config_user,
 		"file:",	config_fstab,
 		"misc:",	config_misc,
+		"hardware:",	config_hard,
 		//"",			config_graphic,
 		NULL
 	};
@@ -677,7 +682,6 @@ void linuxconf_mainmenu(bool dontcheck)
 		"controlpanel:",	control,
 		"controlfile:",		ctrlfile,
 		"time:",		config_datetime,
-		"features:",	config_features,
 		NULL
 	};
 	static const char *menuopt3[]={
@@ -685,6 +689,10 @@ void linuxconf_mainmenu(bool dontcheck)
 		"log:",		view_logs,
 		NULL
 	};
+	static const char *menuopt4[]={
+		"-",		MSG_U(M_COMMONTASKS,"Tasks"),
+		NULL
+	};
 	DIALOG_MENU dia;
 	dia.new_menuitems (menuopt1);
 	dia.new_menuitem ("boot","",config_askrun);
@@ -695,6 +703,10 @@ void linuxconf_mainmenu(bool dontcheck)
 		module_setmenu (dia,MENU_MAIN_CONTROL);
 		dia.new_menuitems (menuopt3);
 		module_setmenu (dia,MENU_MAIN_STATUS);
+		int nbopt4 = dia.getnb();
+		dia.new_menuitems (menuopt4);
+		module_setmenu (dia,MENU_GURUS);
+		if (dia.getnb()==nbopt4+1) dia.remove_last (nbopt4);
 	}
 	dialog_clear();
 	char title[80],sidetitle[80];
@@ -742,6 +754,12 @@ void linuxconf_main(bool dontcheck)
 		dialog_setmode (DIALOG_TREE);
 		linuxconf_mainmenu (true);
 	}else{
+		static bool done = false;
+		if (!done){
+			dialog_clear();
+			dialog_splash (NULL);
+			done = true;
+		}
 		notice_show();
 		const char *argv[]={
 			dontcheck ? "1" : "0",
@@ -753,11 +771,11 @@ void linuxconf_main(bool dontcheck)
 	}
 }
 
-static SSTRINGS tree_keys,tree_titles,tree_modules;
+static SSTRINGS tree_keys,tree_titles,tree_icons,tree_modules;
 static SSTRING tree_version,tree_lang;
-static FILE *collect_fout = NULL;
+static FILE_CFG *collect_fout = NULL;
 
-static void linuxconf_collect (const char *key, const char *title)
+static void linuxconf_collect (const char *key, const char *title, const char *icon)
 {
 	if (collect_fout != NULL){
 		fprintf (collect_fout,"Will execute menu %s %s\n",key,title);
@@ -765,13 +783,14 @@ static void linuxconf_collect (const char *key, const char *title)
 	}
 	tree_keys.add (new SSTRING (key));
 	tree_titles.add (new SSTRING (title));
+	tree_icons.add (new SSTRING (icon));
 }
 
 /*
 	Check if two table contain the same strings
 */
 static bool linuxconf_diffmod (
-	SSTRINGS &tb1,
+	const SSTRINGS &tb1,
 	SSTRINGS &tb2)
 {
 	bool differ = true;
@@ -791,25 +810,38 @@ static bool linuxconf_diffmod (
 
 void linuxconf_gettree (
 	SSTRINGS &keys,
-	SSTRINGS &titles)
+	SSTRINGS &titles,
+	SSTRINGS &icons)
 {
 	static const char K_TREE[]="tree";
 	static const char K_KEY[]="key";
 	static const char K_TITLE[]="title";
+	static const char K_ICON[]="icon";
 	static const char K_MODULE[]="module";
+	static const char K_OTHER[]="other";
 	static const char K_VERSION[]="version";
 	static const char K_LANG[]="lang";
 	
+	SSTRINGS others;
 	if (tree_keys.getnb()==0){
 		CONFDB db (f_treemenu);
 		db.getall (K_TREE,K_KEY,tree_keys,1);
 		db.getall (K_TREE,K_TITLE,tree_titles,1);
+		db.getall (K_TREE,K_ICON,tree_icons,1);
 		db.getall (K_TREE,K_MODULE,tree_modules,1);
+		db.getall (K_TREE,K_OTHER,others,1);
 		tree_version.setfrom (db.getval (K_TREE,K_VERSION));
 		tree_lang.setfrom (db.getval (K_TREE,K_LANG));
 	}
-	SSTRINGS *curloaded = module_getlist();
+	const SSTRINGS *curloaded = module_getlist();
+	SSTRINGS curothers;
+	{
+		const char *arg[1]={(const char*)&curothers};
+		module_sendmessage ("treemenu-depend",1,arg);
+		curothers.sort();
+	}
 	if (linuxconf_diffmod (*curloaded,tree_modules)
+		|| linuxconf_diffmod (curothers,others)
 		|| tree_version.cmp(PACKAGE_REV)!=0
 		|| tree_lang.cmp(linuxconf_getlang())!=0){
 		xconf_notice (MSG_U(N_BUILDTREE
@@ -819,28 +851,31 @@ void linuxconf_gettree (
 			 "itself or because you have installed a new module."));
 		tree_keys.remove_all();
 		tree_titles.remove_all();
+		tree_icons.remove_all();
 		tree_modules.remove_all();
 		collect_fout = f_treemenu_log.fopen ("w");
 		DIALOG_MODE curmode  = dialog_setmode (DIALOG_TREE);
 		menubox_setcollect (linuxconf_collect);
-		perm_setbypass(1);
+		perm_setbypass(true);
 		linuxconf_mainmenu (true);
 		if (collect_fout != NULL){
 			fclose (collect_fout);
 			collect_fout = NULL;
 		}
-		perm_setbypass(0);
+		perm_setbypass(false);
 		dialog_setmode (curmode);
 		html_resetpopup();
 		CONFDB db (f_treemenu);
 		db.remove_all();
 		db.replace (K_TREE,K_KEY,tree_keys);
 		db.replace (K_TREE,K_TITLE,tree_titles);
+		db.replace (K_TREE,K_ICON,tree_icons);
 		for (int i=0; i<curloaded->getnb(); i++){
 			const char *mod = curloaded->getitem(i)->get();
 			tree_modules.add (new SSTRING(mod));
 		}
 		db.replace (K_TREE,K_MODULE,tree_modules);
+		db.replace (K_TREE,K_OTHER,curothers);
 		db.replace (K_TREE,K_VERSION,PACKAGE_REV);
 		tree_version.setfrom (PACKAGE_REV);
 		db.replace (K_TREE,K_LANG,linuxconf_getlang());
@@ -850,12 +885,15 @@ void linuxconf_gettree (
 	}
 	keys.remove_all();
 	titles.remove_all();
+	icons.remove_all();
 	keys.neverdelete();
 	titles.neverdelete();
+	icons.neverdelete();
 	int n = tree_keys.getnb();
 	for (int i=0; i<n; i++){
 		keys.add (tree_keys.getitem(i));
 		titles.add (tree_titles.getitem(i));
+		icons.add (tree_icons.getitem(i));
 	}
 }
 
