@@ -26,6 +26,7 @@ static CONFIG_FILE_LISTER *first_lister=NULL;
 void configf_required()
 {
 	fviews_required();
+	malloc_err(0);
 }
 
 
@@ -41,8 +42,9 @@ public:
 	int status;		// See CONFIGF_XXXXX
 	const char *owner;
 	const char *group;
-	int perm;
-	const char *subsys;		// This config file belong to this sub-system
+        int perm;
+        const char *perm_str;
+        const char *subsys;		// This config file belong to this sub-system
 	/*~PROTOBEG~ CONFIG_FILE_INTERNAL */
 public:
 	CONFIG_FILE_INTERNAL (HELP_FILE&_helpfile,
@@ -51,11 +53,29 @@ public:
 		 const char *_owner,
 		 const char *_group,
 		 int _perm,
+		 const char *_perm_str,
 		 const char *_subsys);
 	~CONFIG_FILE_INTERNAL (void);
 	/*~PROTOEND~ CONFIG_FILE_INTERNAL */
 };
 
+/* #Specification: CONFIG_FILE_LISTER / principles
+	Some configuration file are dynamic. For example the various
+	DNS zone file are known only while reading /etc/named.conf.
+	When linuxconf needs to operate on all known configuration file
+	it uses the CONFIG_FILE_LISTER objects to trigger the definition
+	of all dynamic CONFIG_FILE object.
+
+	A CONFIG_FILE_LISTER is a static object. The constructor is used
+	to register a void (*f)() function pointer. When linuxconf needs
+	wants to enumerate all the CONFIG_FILE, it walks all the
+	CONFIG_FILE_LISTER objects and trigger the function. All those
+	function enumerate all the dynamic CONFIG_FILE they know and create
+	a new CONFIG_FILE object for each. Linuxconf will delete the newly
+	created CONFIG_FILE object later. So the function only care about
+	creating the objects.
+*/
+#
 PUBLIC CONFIG_FILE_LISTER::CONFIG_FILE_LISTER (void (*f)())
 {
 	next = first_lister;
@@ -103,13 +123,15 @@ PUBLIC CONFIG_FILE_INTERNAL::CONFIG_FILE_INTERNAL(
 	const char *_owner,
 	const char *_group,
 	int _perm,
-	const char *_subsys)
+        const char *_perm_str,
+        const char *_subsys)
 	: helpf(_helpfile)
 {
 	owner = _owner;
 	group = _group;
 	perm = _perm;
-	key = strdup(_path);
+        perm_str = _perm_str;
+        key = strdup(_path);
 	stdpath  = NULL;
 	realpath = NULL;	// fixpath() will look it up
 				// later when more is initialised
@@ -128,10 +150,25 @@ PRIVATE void CONFIG_FILE::init(
 	const char *_owner,
 	const char *_group,
 	int _perm,
-	const char *_subsys)
+        const char *_subsys)
 {
 	intern = new CONFIG_FILE_INTERNAL (_helpfile
-		,_path,_status,_owner,_group,_perm,_subsys);
+		,_path,_status,_owner,_group,_perm,NULL,_subsys);
+	intern->next = first;
+	first = this;
+}
+
+PRIVATE void CONFIG_FILE::init(
+	HELP_FILE &_helpfile,
+	const char *_path,
+	int _status,
+	const char *_owner,
+	const char *_group,
+        const char *_perm_str,
+        const char *_subsys)
+{
+	intern = new CONFIG_FILE_INTERNAL (_helpfile
+		,_path,_status,_owner,_group,0,_perm_str,_subsys);
 	intern->next = first;
 	first = this;
 }
@@ -144,7 +181,7 @@ PRIVATE void CONFIG_FILE::init(
 	assemble on the fly the path of a config file.
 	See modules/mailconf/generate.cc.
 */
-PROTECTED void CONFIG_FILE::setkey(const char *newkey)
+PUBLIC void CONFIG_FILE::setkey(const char *newkey)
 {
 	free ((char*)intern->key);
 	free ((char*)intern->stdpath);
@@ -209,6 +246,29 @@ PUBLIC CONFIG_FILE::CONFIG_FILE(
 	init (_helpfile,_path,_status,_owner,_group,_perm,_subsys);
 }
 
+PUBLIC CONFIG_FILE::CONFIG_FILE(
+	const char *_path,
+	HELP_FILE &_helpfile,
+	int _status,
+	const char *_owner,
+	const char *_group,
+	const char *_perm_str)
+{
+	init (_helpfile,_path,_status,_owner,_group,_perm_str,"base");
+}
+
+PUBLIC CONFIG_FILE::CONFIG_FILE(
+	const char *_path,
+	HELP_FILE &_helpfile,
+	int _status,
+	const char *_owner,
+	const char *_group,
+	const char *_perm_str,
+	const char *_subsys)
+{
+	init (_helpfile,_path,_status,_owner,_group,_perm_str,_subsys);
+}
+
 PUBLIC CONFIG_FILE_INTERNAL::~CONFIG_FILE_INTERNAL()
 {
 	free ((char*)subsys);
@@ -251,6 +311,7 @@ PUBLIC const char *CONFIG_FILE::getsubsys() const
 {
 	return intern->subsys;
 }
+
 /*
 	Set the permissions and owner of a file to the same value as
 	the configuration file, if it has such a requirement.
@@ -263,10 +324,20 @@ PUBLIC int CONFIG_FILE::setperm (const char *fpath) const
 	if (intern->owner != NULL){
 		fixpath();
 		PERMINFO p;
-		p.uid = 0;
+                int fixperm_int;
+                p.uid = 0;
 		p.gid = 0;
-		fixperm_readperm (intern->stdpath,p,intern->owner,intern->group
-			,intern->perm,true);
+		char fixuser[100],fixgroup[100],fixperm[100];
+                if(intern->perm_str){
+                        configf_mapowner (intern->owner,intern->group,intern->perm_str,fixuser,fixgroup,fixperm);
+                        sscanf(fixperm, "%o", &fixperm_int);
+                        fixperm_readperm (intern->stdpath,p,fixuser,fixgroup
+		        	,fixperm_int,true);
+                }else{
+                        configf_mapowner (intern->owner,intern->group,fixuser,fixgroup);
+         		fixperm_readperm (intern->stdpath,p,fixuser,fixgroup
+	                	,intern->perm,true);
+                }
 		ret = -1;
 		if (p.uid == (uid_t)-1){
 			xconf_error (MSG_U(E_SETOWNER
@@ -302,27 +373,48 @@ static SSTRINGS tbkey,tbval;
 */
 void configf_readlookup ()
 {
-	char basepath[PATH_MAX],basepath_rev[PATH_MAX];
+	char basepath[PATH_MAX],basepath_rev[PATH_MAX],basepath_brev[PATH_MAX];
 	sprintf (basepath,"%s/%s",USR_LIB_LINUXCONF,linuxconf_getdistdir());
 	sprintf (basepath_rev,"%s/%s",basepath,distrib_getrelease());
+	strcpy (basepath_brev,basepath_rev);
+	{
+		char *pt = strchr(basepath_brev,'.');
+		if (pt != NULL) *pt = '\0';
+	}
 	SSTRINGS tb;
-	int nb = dir_getlist (basepath,".paths",tb);
-	nb += dir_getlist (basepath_rev,".paths",tb);
+	dir_getlist (basepath,".paths",tb);
+	dir_getlist (basepath_rev,".paths",tb);
+	dir_getlist (basepath_brev,".paths",tb);
 	tb.sort();
 	tb.remove_dups();
-	for (int i=0; i<nb; i++){
+	for (int i=0; i<tb.getnb(); i++){
 		char filepath[PATH_MAX];
 		const char *basename = tb.getitem(i)->get();
 		/* #Specification: *.paths files / location
 			We check first in /usr/lib/linuxconf/distribution/release
+			then in /usr/lib/linuxconf/distribution/base-release
 			then in /usr/lib/linuxconf/distribution. This allows
-			redefinition for a specific release.
+			redefinition for a specific release, a major version
+			or any version. For example, when running on a RedHat 6.2, we
+			look in the following directory
+
+			#
+			/usr/lib/linuxconf/redhat/6.2
+			/usr/lib/linuxconf/redhat/6
+			/usr/lib/linuxconf/redhat
+			#
+
+			This should offer enough flexibility to adapt to newer releases.
 		*/
 		sprintf (filepath,"%s/%s.paths",basepath_rev,basename);
 		FILE *fin = fopen (filepath,"r");
 		if (fin == NULL){
-			sprintf (filepath,"%s/%s.paths",basepath,basename);
+			sprintf (filepath,"%s/%s.paths",basepath_brev,basename);
 			fin = fopen (filepath,"r");
+			if (fin == NULL){
+				sprintf (filepath,"%s/%s.paths",basepath,basename);
+				fin = fopen (filepath,"r");
+			}
 		}
 		if (fin != NULL){
 			char buf[2*PATH_MAX];
@@ -342,7 +434,7 @@ void configf_readlookup ()
 	Find the corresponding path for the current distribution.
 	Return the key itself if the lookup has nothing for this key.
 */
-const char *configf_lookuppath(const char *key)
+EXPORT const char *configf_lookuppath(const char *key)
 {
 	#if 0
 		static bool is_init = false;
@@ -357,13 +449,61 @@ const char *configf_lookuppath(const char *key)
 	return ret;
 }
 
+EXPORT const char *configf_lookup (const char *prefix, const char *key)
+{
+	char tmp[strlen(prefix)+1+strlen(key)+1];
+	sprintf (tmp,"%s:%s",prefix,key);
+	const char *ret = configf_lookuppath (tmp);
+	if (ret == tmp){
+		fprintf (stderr,"Can't resolve generic mapping %s\n",tmp);
+		ret = "nobody";
+	}
+	return ret;
+}
+
+/*
+	Translate a user and group (if needed) into other user and group from
+	the resource system (.paths files)
+*/
+EXPORT void configf_mapowner (
+	const char *generic_user,
+	const char *generic_group,
+	char user[100],
+	char group[100])
+{
+	if (generic_user[0] == '$'){
+		generic_user = configf_lookup ("user",generic_user+1);
+	}
+	strcpy_cut (user,generic_user,99);
+	if (generic_group[0] == '$'){
+		generic_group = configf_lookup ("group",generic_group+1);
+	}
+	strcpy_cut (group,generic_group,99);
+}
+
+EXPORT void configf_mapowner (
+	const char *generic_user,
+	const char *generic_group,
+        const char *generic_perm,
+        char user[100],
+        char group[100],
+        char perm[100])
+{
+        configf_mapowner (generic_user, generic_group, user, group);
+        if (generic_perm[0] == '$'){
+                generic_perm = configf_lookup ("perm",generic_perm+1);
+        }
+        strcpy_cut (perm,generic_perm,99);
+}
+
 static char CONFIG[]="config";
 
 PRIVATE void CONFIG_FILE::fixpath() const
 {
+	extern CONFIG_FILE f_linuxconf;
 	if (intern->stdpath == NULL){
 		const char *std = intern->key;
-		if (strcmp(intern->key,ETC_CONF_LINUXCONF)!=0){
+		if (this != &f_linuxconf){
 			std = configf_lookuppath(intern->key);
 		}
 		intern->stdpath = strdup(std);
@@ -378,7 +518,7 @@ PRIVATE void CONFIG_FILE::fixpath() const
 			/etc/conf.linuxconf.
 		*/
 		const char *selp = intern->stdpath;
-		if (strcmp(selp,ETC_CONF_LINUXCONF)!=0){
+		if (this != &f_linuxconf){
 			selp = linuxconf_getval (CONFIG,selp);
 			if (selp == NULL) selp = intern->stdpath;
 		}
@@ -394,7 +534,7 @@ PRIVATE void CONFIG_FILE::fixpath() const
 	}
 }
 
-PRIVATE void CONFIG_FILE::sign (FILE *fout, const char *mode) const
+PRIVATE void CONFIG_FILE::sign (FILE_CFG *fout, const char *mode) const
 {
 	if (fout != NULL && strcmp(mode,"w")==0){
 		if (intern->status & CONFIGF_SIGNPOUND){
@@ -416,17 +556,18 @@ PRIVATE void CONFIG_FILE::sign (FILE *fout, const char *mode) const
 /*
 	Open the configuration file without permission checking.
 */
-PUBLIC FILE *CONFIG_FILE::fopen_ok(const char *mode) const
+PUBLIC FILE_CFG *CONFIG_FILE::fopen_ok(const char *mode) const
 {
-	FILE *ret = NULL;
+	FILE *fret = NULL;
 	fixpath();
 	const char *realp = intern->realpath;
 	if (strcmp(mode,"r")==0 && (intern->status & CONFIGF_OPTIONNAL) != 0){
-		ret = ::fopen (realp,mode);
+		fret = ::fopen (realp,mode);
 	}else{
-		ret = xconf_fopen(realp,mode);
+		fret = xconf_fopen(realp,mode);
 		setperm(realp);
 	}
+	FILE_CFG *ret = filecfg_new (fret,this);
 	sign (ret,mode);
 	return ret;
 }
@@ -436,9 +577,9 @@ static bool extracting = false;
 	It may even ask the user for the root password or
 	a user password if priv != NULL.
 */
-PUBLIC FILE *CONFIG_FILE::fopen(PRIVILEGE *priv, const char *mode) const
+PUBLIC FILE_CFG *CONFIG_FILE::fopen(PRIVILEGE *priv, const char *mode) const
 {
-	FILE *ret = NULL;
+	FILE_CFG *ret = NULL;
 	fixpath();
 	if (strchr(mode,'w')!=NULL
 		|| strchr(mode,'+')!=NULL
@@ -482,19 +623,46 @@ PUBLIC FILE *CONFIG_FILE::fopen(PRIVILEGE *priv, const char *mode) const
 		context (a different root) because of the netadm module. So we
 		take a copy of realpath before using it.
 	*/
+	fixpath();
 	char realp[PATH_MAX];
 	strcpy_cut (realp,intern->realpath,sizeof(realp)-1);
 	if (strcmp(mode,"r")==0){
 		if ((intern->status & CONFIGF_OPTIONNAL) != 0){
-			ret = ::fopen (realp,mode);
+			FILE *f = ::fopen (realp,mode);
+			ret = filecfg_new (f,this);
 		}else{
-			ret = xconf_fopencfg(priv,realp,mode);
+			FILE *f = xconf_fopencfg(priv,realp,mode);
+			ret = filecfg_new (f,this);
 		}
 	}else{
-		if ((intern->status & CONFIGF_TMPLOCK) != 0){
+		/* #Specification: CONFIG_FILE / update strategy / .TMP files
+			The flag CONFIGF_TMPLOCK was used to produce a temp file
+			which was renamed in place at fclose time. This was ensuring
+			a consistant state for the configuration file at all time
+			(since rename is atomic). So any competing process would either
+			read the old or the new version, both in a "completed" state.
+
+			This strategy has other advantages. If Linuxconf has a flaw
+			and fails (crashes) while updating the file, the original
+			is left in place.
+
+			Also, at fclose time, we have the old and new file side by side.
+			This may be used by modules supporting the "updatefile" message
+			so they can inspect the changes. The updatemon module
+			is doing that and allows the user to preview the change
+			and even reject them.
+
+			Because of those advantage, the CONFIGF_TMPLOCK is now the
+			default behavior.
+		*/
+		// bool opentmp = (intern->status & CONFIGF_TMPLOCK) != 0;
+		bool opentmp = true;
+		if (strncmp(realp,"/proc/",6)==0) opentmp = false;
+		if (strcmp(mode,"w")==0 && opentmp){
 			ret = fopen_tmp(priv,mode);
 		}else{
-			ret = xconf_fopencfg(priv,realp,mode);
+			FILE *f = xconf_fopencfg(priv,realp,mode);
+			ret = filecfg_new (f,this);
 			setperm(realp);
 		}
 	}
@@ -505,7 +673,7 @@ PUBLIC FILE *CONFIG_FILE::fopen(PRIVILEGE *priv, const char *mode) const
 	Open the configuration file with permission checking.
 	It may even ask the user for the root password.
 */
-PUBLIC FILE *CONFIG_FILE::fopen(const char *mode) const
+PUBLIC FILE_CFG *CONFIG_FILE::fopen(const char *mode) const
 {
 	return fopen ((PRIVILEGE*)NULL,mode);
 }
@@ -513,27 +681,64 @@ PUBLIC FILE *CONFIG_FILE::fopen(const char *mode) const
 	Open the temp configuration file with permission checking.
 	It may even ask the user for the root password.
 */
-PUBLIC FILE *CONFIG_FILE::fopen(PRIVILEGE *priv, const char *temp, const char *mode) const
+PUBLIC FILE_CFG *CONFIG_FILE::fopen(PRIVILEGE *priv, const char *temp, const char *mode) const
 {
-	FILE *ret = NULL;
+	FILE *fret = NULL;
 	if (strcmp(mode,"r")==0 && (intern->status & CONFIGF_OPTIONNAL) != 0){
-		ret = ::fopen (temp,mode);
+		fret = ::fopen (temp,mode);
 	}else{
-		ret = xconf_fopencfg(priv,temp,mode);
+		fret = xconf_fopencfg(priv,temp,mode);
 		setperm(temp);
 	}
+	FILE_CFG *ret = filecfg_new (fret,this);
 	sign (ret,mode);
 	return ret;
 }
 /*
-	Open a temporary file using the configuration and adding .tmp to its
+	Return the absolute file name of a configuration file.
+	Check if the path is a simlink. If this is the case, return the
+	path pointed by the symlink.
+*/
+PRIVATE const char *CONFIG_FILE::getlinkpath(char linkpath[PATH_MAX]) const
+{
+	/* #Specification: CONFIG_FILE / symlink and path
+		If the administrator move a configuration file and then setup
+		a symbolic link pointing to the new location, Linuxconf will
+		respect this symbolic link. Further, it will produce a backup
+		file (.OLD) in the same directory as the configuration file.
+	*/
+	const char *ret = getpath();
+	int len = readlink(ret,linkpath,PATH_MAX-1);
+	if (len > 0){
+		linkpath[len] = '\0';
+		if (linkpath[0] != '/'){
+			// The symlink is relative, make it absolute
+			const char *pt = strrchr(ret,'/');
+			if (pt != NULL){
+				char tmp[PATH_MAX];
+				len = (int)(pt-ret)+1;
+				memmove (tmp,ret,len);
+				tmp[len] = '\0';
+				strcat (tmp,linkpath);
+				strcpy (linkpath,tmp);
+			}
+		}
+		ret = linkpath;
+	}
+	return ret;
+}
+/*
+	Open a temporary file using the configuration and adding .TMP to its
 	name.
 */
-PUBLIC FILE *CONFIG_FILE::fopen_tmp(PRIVILEGE *priv, const char *mode) const
+PUBLIC FILE_CFG *CONFIG_FILE::fopen_tmp(PRIVILEGE *priv, const char *mode) const
 {
+	char linkpath[PATH_MAX];
+	const char *abspath = getlinkpath(linkpath);
 	char path_tmp[PATH_MAX];
-	snprintf (path_tmp,PATH_MAX-1,"%s.TMP",getpath());
-	FILE *ret = fopen (priv,path_tmp,mode);
+	snprintf (path_tmp,PATH_MAX-1,"%s.TMP",abspath);
+	FILE_CFG *ret = fopen (priv,path_tmp,mode);
+	filecfg_setrelink (ret);
 	setperm (path_tmp);
 	return ret;
 }
@@ -541,10 +746,10 @@ PUBLIC FILE *CONFIG_FILE::fopen_tmp(PRIVILEGE *priv, const char *mode) const
 /*
 	Rename to original to .OLD, rename the tmp to the original
 	(This function assumes that fopen_tmp() has been used to update
-	 the configuration file)
+	 the configuration file. It is called from filecfg.cc)
 	return -1 if any error.
 */
-PUBLIC int CONFIG_FILE::relink_tmp()
+PUBLIC int CONFIG_FILE::relink_tmp() const
 {
 	/* #Specification: saving multi-user config file / strategy
 		Here is the strategy used to update file like /etc/passwd
@@ -568,27 +773,33 @@ PUBLIC int CONFIG_FILE::relink_tmp()
 		Some provision are made so this works also if /etc/passwd did not
 		exist.
 	*/
-	const char *oripath = getpath();
-	char path_old[PATH_MAX];
-	sprintf (path_old,"%s.OLD",oripath);
+	char path[PATH_MAX], path_old[PATH_MAX], path_tmp[PATH_MAX];
+	{
+		char linkpath[PATH_MAX];
+		const char *oripath = getlinkpath(linkpath);
+		strcpy (path,oripath);
+		sprintf (path_tmp,"%s.TMP",oripath);
+		sprintf (path_old,"%s.OLD",oripath);
+	}
 	int ret = -1;
-	::unlink(path_old);
-	// If the original file do not exist, we must still
-	// rename the temporary file in place.
-	int link_ret = link(oripath, path_old);
-	bool end_ok = false;
-	if (link_ret == -1){
-		if (errno == ENOENT){
+	const char *argv[]={path,path_tmp};
+	if (module_sendmessage ("updatefile",2,argv)!=-1){
+		::unlink(path_old);
+		// If the original file do not exist, we must still
+		// rename the temporary file in place.
+		int link_ret = link(path, path_old);
+		bool end_ok = false;
+		if (link_ret == -1){
+			if (errno == ENOENT){
+				end_ok = true;
+			}
+		}else{
 			end_ok = true;
 		}
-	}else{
-		end_ok = true;
-	}
-	if (end_ok){
-		char path_tmp[PATH_MAX];
-		sprintf (path_tmp,"%s.TMP",oripath);
-		if (rename(path_tmp, oripath) != -1){
-			ret = 0;
+		if (end_ok){
+			if (rename(path_tmp, path) != -1){
+				ret = 0;
+			}
 		}
 	}
 	return ret;
@@ -597,7 +808,7 @@ PUBLIC int CONFIG_FILE::relink_tmp()
 	Open the temp configuration file with permission checking.
 	It may even ask the user for the root password.
 */
-PUBLIC FILE *CONFIG_FILE::fopen(const char *temp, const char *mode) const
+PUBLIC FILE_CFG *CONFIG_FILE::fopen(const char *temp, const char *mode) const
 {
 	return fopen (NULL,temp,mode);
 }
@@ -621,8 +832,8 @@ PROTECTED VIRTUAL int CONFIG_FILE::archive(SSTREAM &ss) const
 {
 	if (exist()){
 		configf_sendexist (ss,true);
-		FILE *fin = fopen ("r");
-		char buf[300];
+		FILE_CFG *fin = fopen ("r");
+		char buf[3000];
 		while (fgets(buf,sizeof(buf)-1,fin)!=NULL){
 			ss.puts (buf);
 		}
@@ -636,7 +847,7 @@ PROTECTED VIRTUAL int CONFIG_FILE::archive(SSTREAM &ss) const
 static const char *cfg_command=NULL;
 static const char *cfg_arg=NULL;
 static bool cfg_verbose=true;
-static SSTREAM *cfg_ssout;
+static SSTREAM *cfg_ssout = NULL;
 
 /*
 	Archive the config file if configured
@@ -644,15 +855,17 @@ static SSTREAM *cfg_ssout;
 PUBLIC VIRTUAL int CONFIG_FILE::archive() const
 {
 	int ret = -1;
-	fixpath();
 	if (is_archived()){
 		const char *arch = confver_getfamily(getsubsys());
 		if (arch == NULL){
 			ret = 0;
 		}else{
+			fixpath();
 			if (cfg_verbose){
 				net_prtlog (NETLOG_TITLE,MSG_U(L_ARCHIVING,"Archiving %s, version %s\n")
 					,intern->realpath,arch);
+				configf_forgetpath();
+				fixpath();
 			}
 			char args[2*PATH_MAX];
 			const char *argpath = intern->realpath;
@@ -661,7 +874,7 @@ PUBLIC VIRTUAL int CONFIG_FILE::archive() const
 			}
 			const char *comm = cfg_command == NULL ? "cfgarchive" : cfg_command;
 			const char *carg = cfg_arg == NULL ? "--arch" : cfg_arg;
-			sprintf (args,"%s %s %s",carg,argpath,arch);
+			sprintf (args,"%s %s \"%s\"",carg,argpath,arch);
 			POPEN pop (comm,args);
 			if (pop.isok()){
 				/* #Specification: config file / archiving / strategy
@@ -698,9 +911,11 @@ PUBLIC VIRTUAL int CONFIG_FILE::archive() const
 				SSTREAM_POPEN ss(pop);
 				ret = archive(ss);
 				if (ret == 0) ret = pop.close();
-				char line[1000];
-				while (pop.readout(line,sizeof(line)-1)==0){
-					cfg_ssout->puts (line);
+				if (cfg_ssout != NULL){
+					char line[1000];
+					while (pop.readout(line,sizeof(line)-1)==0){
+						cfg_ssout->puts (line);
+					}
 				}
 			}
 		}
@@ -735,7 +950,7 @@ PUBLIC VIRTUAL int CONFIG_FILE::md5sum(char *sum)
 PUBLIC VIRTUAL int CONFIG_FILE::extract(SSTREAM &ss)
 {
 	int ret = -1;
-	FILE *fout = fopen("w");
+	FILE_CFG *fout = fopen("w");
 	if (fout != NULL){
 		char line[1000];
 		while (ss.gets(line,sizeof(line)-1) != NULL){
@@ -789,12 +1004,18 @@ PUBLIC VIRTUAL int CONFIG_FILE::extract()
 								,"File %s can't be extracted properly\n")
 								,intern->realpath);
 						}
+					}else{
+						net_prtlog (NETLOG_ERR,MSG_U(E_NOSTATUS,"Command produced no status: %s %s\n"),cfg_command,args);
 					}
+				}else{
+					net_prtlog (NETLOG_ERR,MSG_U(E_SHORTEXEC,"Command produced no output: %s %s\n"),cfg_command,args);
 				}
 				char line[300];
 				while (pop.readerr(line,sizeof(line)-1) != -1){
 					net_prtlog (NETLOG_ERR,"%s",line);
 				}
+			}else{
+				net_prtlog (NETLOG_ERR,MSG_U(E_CANTEXEC,"Can't execute %s %s"),cfg_command,args);
 			}
 		}
 	}else{
@@ -805,14 +1026,9 @@ PUBLIC VIRTUAL int CONFIG_FILE::extract()
 
 
 
-PUBLIC int CONFIG_FILE::fclose (FILE *fout)
+PUBLIC int CONFIG_FILE::fclose (FILE_CFG *fout)
 {
-	int ret = ::fclose(fout);
-	if (ret != -1){
-		if ((intern->status & CONFIGF_TMPLOCK) != 0) relink_tmp();
-		//archive();
-	}
-	return ret;
+	return ::fclose(fout);
 }
 /*
 	Return the path of the configuration file
@@ -890,6 +1106,7 @@ PUBLIC int CONFIG_FILE::is_archived() const
 		/usr/lib/linuxconf
 		/var/run
 		/var/log
+		#
 	*/
 	fixpath();
 	const char *realp = intern->realpath;
@@ -909,6 +1126,49 @@ PUBLIC int CONFIG_FILE::is_probed() const
 {
 	return intern->status & CONFIGF_PROBED;
 }
+static int file_type (struct stat &st)
+{
+	int ret = -1;
+	if (S_ISREG(st.st_mode)){
+		ret = 0;
+	}else if (S_ISDIR(st.st_mode)){
+		ret = 1;
+	}else if (S_ISCHR(st.st_mode) || S_ISBLK(st.st_mode)){
+		ret = 2;
+	}else if (S_ISLNK(st.st_mode)){
+		ret = 3;
+	}else if (S_ISFIFO(st.st_mode)){
+		ret = 4;
+	}
+	return ret;
+}
+/*
+	Check the type of a file or directory.
+	If follow is 1, follow symbolic links.
+	Return -1 if the path does not exist.
+	Return  0 if this is a file
+			1 if this is a directory
+			2 if this is a device
+			3 if this is a symbolic link
+			4 if this is a fifo
+*/
+int file_type (const char *path, bool follow)
+{
+	struct stat st;
+	int ret = -1;
+	if (path[0] == '\0' || strcmp(path,"/")==0){
+		ret = 1;
+	}else if ((follow || lstat(path,&st)!=-1)
+		  && (!follow || stat(path,&st)!=-1)) {
+		ret = file_type (st);
+	}
+	return ret;
+}
+
+int file_type (const char *path)
+{
+	return file_type(path,false);
+}
 
 /*
 	Check the type of a file or directory.
@@ -918,25 +1178,17 @@ PUBLIC int CONFIG_FILE::is_probed() const
 			2 if this is a device
 			3 if this is a symbolic link
 			4 if this is a fifo
+
+	This function uses stat instead of lstat
 */
-int file_type (const char *path)
+int file_rtype (const char *path)
 {
 	struct stat st;
 	int ret = -1;
 	if (path[0] == '\0' || strcmp(path,"/")==0){
 		ret = 1;
-	}else if (lstat(path,&st)!=-1){
-		if (S_ISREG(st.st_mode)){
-			ret = 0;
-		}else if (S_ISDIR(st.st_mode)){
-			ret = 1;
-		}else if (S_ISCHR(st.st_mode) || S_ISBLK(st.st_mode)){
-			ret = 2;
-		}else if (S_ISLNK(st.st_mode)){
-			ret = 3;
-		}else if (S_ISFIFO(st.st_mode)){
-			ret = 4;
-		}
+	}else if (stat(path,&st)!=-1){
+		ret = file_type (st);
 	}
 	return ret;
 }
@@ -985,7 +1237,7 @@ int file_create (
 		CONFIG_FILE file (path,help_nil
 			,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
 			,owner,group,perm);
-		FILE *fin = file.fopen ("w");
+		FILE_CFG *fin = file.fopen ("w");
 		if (fin != NULL){
 			ret = fclose (fin);
 		}else{
@@ -1073,15 +1325,18 @@ static int cmp_config(const void *pt1, const void *pt2)
 	CONFIG_FILE *p2 = *(CONFIG_FILE**)pt2;
 	return strcmp(p1->getstdpath(),p2->getstdpath());
 }
-static void config_getsortedlist(CONFIG_FILE *tb[])
+
+static int config_getsortedlist(CONFIG_FILE *tb[])
 {
 	CONFIG_FILE *f = first;
 	int no = 0;
 	while (f != NULL){
-		tb[no++] = f;
+		if (tb != NULL) tb[no] = f;
+		no++;
 		f = f->getnext();
 	}
-	qsort (tb,no,sizeof(CONFIG_FILE*),cmp_config);
+	if (tb != NULL) qsort (tb,no,sizeof(CONFIG_FILE*),cmp_config);
+	return no;
 }
 
 static void config_setflags (CONFIG_FILE *f, char type[10])
@@ -1101,34 +1356,25 @@ static void config_setflags (CONFIG_FILE *f, char type[10])
 */
 void configf_show()
 {
-	int nbconfig = 0;
-	{
-		CONFIG_FILE *f = first;
-		while (f != NULL){
-			f = f->getnext();
-			nbconfig++;
-		}
-	}
+	int nbconfig = config_getsortedlist(NULL);
 	CONFIG_FILE *tb[nbconfig];
 	config_getsortedlist(tb);
 	int choice = 0;
 	DIALOG_RECORDS dia;
+	dia.newf_head ("",MSG_U(H_CONFIGP,"Path\tStatus\tSubsys"));
 	while (1){
-		if (dia.getnb()==0){
-			dia.newf_head ("",MSG_U(H_CONFIGP,"Path\tStatus\tSubsys"));
-			for (int i=0; i<nbconfig; i++){
-				CONFIG_FILE *f = tb[i];
-				char type[10];
-				config_setflags (f,type);
-				const char *subsys = f->getsubsys();
-				if (!f->is_archived()) subsys = "";
-				const char *path = f->getpath();
-				const char *stdp = f->getstdpath();
-				if (strcmp(path,stdp)!=0) type[7] = '*';
-				char buf[2*PATH_MAX];
-				sprintf (buf,"%s\t%s",type,subsys);
-				dia.new_menuitem (stdp,buf);
-			}
+		for (int i=0; i<nbconfig; i++){
+			CONFIG_FILE *f = tb[i];
+			char type[10];
+			config_setflags (f,type);
+			const char *subsys = f->getsubsys();
+			if (!f->is_archived()) subsys = "";
+			const char *path = f->getpath();
+			const char *stdp = f->getstdpath();
+			if (strcmp(path,stdp)!=0) type[7] = '*';
+			char buf[2*PATH_MAX];
+			sprintf (buf,"%s\t%s",type,subsys);
+			dia.set_menuitem (i,stdp,buf);
 		}
 		MENU_STATUS code = dia.editmenu (
 			MSG_U(T_LISTCONF,"List of configuration files")
@@ -1150,7 +1396,7 @@ void configf_show()
 					,"You can't edit a virtual config file"));
 			}else if (perm_rootaccess (
 				MSG_U(P_MODCONF,"modify a configuration file path"))){
-				if (cfgf->editpath()==0) dia.remove_all();
+				cfgf->editpath();
 			}
 		}
 	}
@@ -1163,14 +1409,7 @@ void configf_list()
 {
 	// Setup the list of all config file, including dynamic one
 	CONFIG_FILE *original_first = configf_calllisters();
-	int nbconfig = 0;
-	{
-		CONFIG_FILE *f = first;
-		while (f != NULL){
-			f = f->getnext();
-			nbconfig++;
-		}
-	}
+	int nbconfig = config_getsortedlist(NULL);
 	CONFIG_FILE *tb[nbconfig];
 	config_getsortedlist(tb);
 	for (int i=0; i<nbconfig; i++){
@@ -1215,6 +1454,24 @@ int configf_getsubsyslist (SSTRINGS &tb)
 	}	
 	return tb.getnb()-start;
 }
+
+/*
+	Get the list of configuration files member of a sub-system
+*/
+int configf_getsubsysmembers(const char *subsys, SSTRINGS &tb)
+{
+	CONFIG_FILE *original = configf_calllisters();
+	CONFIG_FILE *f = first;
+	while (f != NULL){
+		if (f->is_archived() && strcmp(f->getsubsys(),subsys)==0){
+			tb.add (new SSTRING (f->getstdpath()));
+		}
+		f = f->getnext();
+	}
+	while (first != original) delete first;
+	return tb.getnb();
+}
+
 /*
 	Erase some config file at boot time.
 */
@@ -1230,12 +1487,18 @@ void configf_booterase()
 	Archive all config file member of a subsystem
 */
 static int configf_archiveone (
-	const char *subsys)
+	const char *name)		// Sub-system or file if it starts with a slash
 {
 	int ret = 0;
 	CONFIG_FILE *f = first;
+	bool isfile = name[0] == '/';
 	while (f != NULL){
-		if (strcmp(f->getsubsys(),subsys)==0){
+		if (isfile){
+			if (strcmp(name,f->getpath())==0){
+				ret |= f->archive();
+				break;
+			}
+		}else if (strcmp(f->getsubsys(),name)==0){
 			ret |= f->archive();
 		}
 		f = f->getnext();
@@ -1244,7 +1507,8 @@ static int configf_archiveone (
 }
 
 int configf_archive(
-	const SSTRINGS &tb,
+	const SSTRINGS &tb,		// Sub-system or files to archive
+							// (A file starts with a /)
 	const char *command,	// Archiving command (just the name)
 							// normally cfgarchive
 	const char *arg,		// First argument of the command
@@ -1266,25 +1530,39 @@ int configf_archive(
 	for (int i=0; i<nbsys; i++){
 		const char *sub = tb.getitem(i)->get();
 		ret |= configf_archiveone (sub);
-		linuxconf_archive (sub);
+		if (sub[0] != '/') linuxconf_archive (sub);
 	}
 	// The CONFIG_FILE object are responsible from removing themselves
 	// from the list
 	while (first != original_first) delete first;
 	cfg_command = NULL;
 	cfg_arg = NULL;
+	cfg_ssout = NULL;
 	return ret;
 }
 
 /*
 	Extract all config file member of a subsystem
 */
-static int configf_extractone (const char *subsys, CONFIG_FILE *end)
+static int configf_extractone (
+	const char *name,	// Subsystem name or file path
+	CONFIG_FILE *end,	// Stop when we reach this CONFIG_FILE
+	bool &done)			// Was it extracted
 {
 	int ret = 0;
 	CONFIG_FILE *f = first;
+	bool isfile = name[0] == '/';
 	while (f != end){
-		if (strcmp(f->getsubsys(),subsys)==0) ret |= f->extract();
+		if (isfile){
+			if (strcmp(name,f->getpath())==0){
+				done = true;
+				ret |= f->extract();
+				break;
+			}
+		}else if (strcmp(f->getsubsys(),name)==0){
+			done = true;
+			ret |= f->extract();
+		}
 		f = f->getnext();
 	}
 	return ret;
@@ -1294,7 +1572,7 @@ static int configf_extractone (const char *subsys, CONFIG_FILE *end)
 	Extract all the file of the current configuration
 */
 int configf_extract (
-	const SSTRINGS &tb,
+	const SSTRINGS &tb,		// Sub-systems or files (starts with a /)
 	const char *command,	// Archiving command (just the name)
 							// normally cfgarchive
 	const char *arg)		// First argument of the command
@@ -1310,20 +1588,22 @@ int configf_extract (
 		we have no clue what must be extracted. For example, we
 		may archive a lot of .dconf file (PPP/Slip dialout config)
 		but we have no clue which one to extract. The same apply
-		to the DNS configuration. We know how to extract the /etc/named.boot
+		to the DNS configuration. We know how to extract the /etc/named.conf
 		but without its content, we have no clue about the various zone
 		file.
 
 		Extraction is done by iteration. First we extract the well
 		known config file. As a side effect of the extraction, the
 		various CONFIG_FILE are free to define new CONFIG_FILE object
-		as they learn about them. New CONFIG_FILE are record at
+		as they learn about them. New CONFIG_FILE are recorded at
 		the beginning of the linked list.
 
 		So by running the process until all file have been extracted
 		and no new CONFIG_FILE has been added, we end with a working
 		solution.
 	*/
+	bool tbdone[tb.getnb()];
+	memset (tbdone,0,sizeof(tbdone));
 	int ret = 0;
 	CONFIG_FILE *original_first = first;
 	CONFIG_FILE *start = NULL;
@@ -1332,11 +1612,59 @@ int configf_extract (
 		CONFIG_FILE *end = start;
 		start = first;
 		for (int i=0; i<nbsys; i++){
-			ret |= configf_extractone (tb.getitem(i)->get(),end);
+			if (!tbdone[i]){
+				ret |= configf_extractone (tb.getitem(i)->get()
+					,end,tbdone[i]);
+			}
+		}
+	}
+	/* #Specification: config versionning / extraction / principle part 2
+		After doing some iteration, we may have extracts all needed
+		sub-systems and files. But we may have missed some. Here is
+		a case. We can to extract /var/named/solucorp.qc.ca. This
+		is a DNS zone file. We only learn about this zone file if we
+		extract /etc/named.conf. Now, if we do not extract the dnsserv
+		sub-system (which own /etc/named.conf) or /etc/named.conf,
+		we will never learn about the zone file. So it won't be extracted.
+	
+		After doing the iteration explain above, where we extract
+		files and sub-systems, checking if new config file appears, we
+		may still have some sub-system and files not extracted.
+
+		THis time, we trigger the config file listers and we try again
+		to perform the extraction.
+	*/
+	for (int i=0; i<nbsys; i++){
+		if (!tbdone[i]){
+			// At least one is not extracted
+			CONFIG_FILE *end = first;
+			configf_calllisters();
+			for (int i=0; i<nbsys; i++){
+				if (!tbdone[i]){
+					ret |= configf_extractone (tb.getitem(i)->get()
+						,end,tbdone[i]);
+				}
+			}
+			break;
 		}
 	}
 	for (int i=0; i<nbsys; i++){
-		ret |= linuxconf_extract (tb.getitem(i)->get());
+		const char *name = tb.getitem(i)->get();
+		bool is_subsys = name[0] != '/';
+		if (!tbdone[i]){
+			if (is_subsys){
+				xconf_error (MSG_U(E_HOWEXTRACTSUB
+					,"Don't know how to exract %s: unknown sub-system")
+					,name);
+			}else{
+				xconf_error (MSG_U(E_HOWEXTRACTFILE
+					,"Don't know how to exract %s:"
+					 " not member of any sub-system")
+					,name);
+			}
+		}else if (is_subsys){
+			ret |= linuxconf_extract (name);
+		}
 	}
 	linuxconf_save();
 	while (first != original_first) delete first;
@@ -1354,7 +1682,7 @@ static int configf_md5sumone (const char *subsys, SSTREAM &ss)
 	int ret = 0;
 	CONFIG_FILE *f = first;
 	while (f != NULL){
-		if (strcmp(f->getsubsys(),subsys)==0){
+		if (strcmp(f->getsubsys(),subsys)==0 && f->is_archived()){
 			char sum[100];
 			ret |= f->md5sum(sum);
 			ss.printf ("%s\t%s\n",f->getpath(),sum);
@@ -1400,3 +1728,9 @@ CONFIG_FILE *configf_locate (const char *path)
 	}
 	return ret;
 }
+
+PUBLIC CONFIG_FILE *CONFIG_FILES::getitem (int no) const
+{
+	return (CONFIG_FILE*)ARRAY::getitem(no);
+}
+
