@@ -22,13 +22,14 @@ static void xconf_msg (
 	const char *msg,
 	va_list list)
 {
+	if (!batchmode) dialog_clear();
 	if (batchmode || !dialog_uiok()){
 		fprintf (stderr,"%s\n",title);
 		vfprintf (stderr,msg,list);
+		fputs ("\n",stderr);
 	}else{
 		char buf[2000];
 		vsnprintf (buf,sizeof(buf)-1,msg,list);
-		dialog_clear();
 		html_setpopup();
 		dialog_msgbox ((char*)title,buf,icon);
 	}
