@@ -1,4 +1,4 @@
-/* array.cc 29/12/1999 23.53.22 */
+/* array.cc 21/08/2001 13.21.34 */
 PUBLIC ARRAY_OBJ::ARRAY_OBJ (void);
 PUBLIC VIRTUAL ARRAY_OBJ::~ARRAY_OBJ (void);
 PUBLIC void ARRAY_OBJ::rstmodified (void);
@@ -9,17 +9,19 @@ PUBLIC ARRAY::ARRAY (void);
 PUBLIC void ARRAY::neverdelete (void);
 PUBLIC void ARRAY::setgrowth (int _increm);
 PUBLIC VIRTUAL ARRAY::~ARRAY (void);
+PRIVATE void ARRAY::grow_realloc (int newmax);
 PROTECTED void ARRAY::grow (void);
 PUBLIC int ARRAY::remove (ARRAY_OBJ *obj);
+PUBLIC int ARRAY::remove (int no);
 PUBLIC int ARRAY::lookup (ARRAY_OBJ *o)const;
 PUBLIC void ARRAY::moveto (ARRAY_OBJ *o, int newpos);
 PUBLIC void ARRAY::remove_all (void);
 PUBLIC int ARRAY::remove_del (ARRAY_OBJ *obj);
 PUBLIC int ARRAY::remove_del (int no);
-PUBLIC void ARRAY::delall (void);
 PUBLIC int ARRAY::getnb (void)const;
 PUBLIC void ARRAY::add (ARRAY_OBJ *pt);
 PUBLIC void ARRAY::insert (int pos, ARRAY_OBJ *pt);
+PUBLIC void ARRAY::set (int pos, ARRAY_OBJ *pt);
 PROTECTED ARRAY_OBJ *ARRAY::getitem (int no)const;
 PUBLIC void ARRAY::rstmodified (void);
 PUBLIC VIRTUAL int ARRAY::was_modified (void);
@@ -30,6 +32,7 @@ PUBLIC int ARRAY::manage_edit (ARRAY_OBJ *e, int code);
 PUBLIC int ARRAY::editone (int no);
 PUBLIC int ARRAY::editone (ARRAY_OBJ *e);
 PUBLIC void ARRAY::remove_last (int cut);
+PUBLIC void ARRAY::invert (void);
 /* arraykey.cc 12/12/1999 22.30.32 */
 PUBLIC ARRAY_KEY::ARRAY_KEY (const char *_key,
 	 ARRAY_OBJ *_obj,
@@ -50,7 +53,7 @@ PUBLIC void SSTRING_KEYS::add (const char *key, const char *val);
 PUBLIC SSTRING_KEY *SSTRING_KEYS::getitem (int no)const;
 PUBLIC SSTRING_KEY *SSTRING_KEYS::getobj (const char *key)const;
 PUBLIC const char *SSTRING_KEYS::getval (const char *key)const;
-/* confdb.cc 20/02/2000 14.59.46 */
+/* confdb.cc 24/11/2000 00.01.46 */
 PUBLIC CONFOBJ::CONFOBJ (const char *_sys,
 	 const char *_key,
 	 const char *_val);
@@ -132,12 +135,20 @@ PUBLIC void CONFDB::replace (const char *prefix,
 	 const char *key,
 	 const SSTRINGS&vals);
 PUBLIC VIRTUAL int CONFDB::archive (SSTREAM&ss, const char *_sys);
-PRIVATE void CONFDB::delsys (const char *_sys);
+PUBLIC void CONFDB::delsys (const char *_sys);
 PUBLIC VIRTUAL int CONFDB::extract (SSTREAM&ss, const char *_sys);
 PUBLIC void CONFDB::patchsys (void);
-/* configf.cc 28/01/2000 10.57.40 */
+/* configf.cc 05/11/2001 15.55.40 */
 PUBLIC CONFIG_FILE_LISTER::CONFIG_FILE_LISTER (void (*f)());
 PUBLIC CONFIG_FILE_INTERNAL::CONFIG_FILE_INTERNAL (HELP_FILE&_helpfile,
+	 const char *_path,
+	 int _status,
+	 const char *_owner,
+	 const char *_group,
+	 int _perm,
+	 const char *_perm_str,
+	 const char *_subsys);
+PRIVATE void CONFIG_FILE::init (HELP_FILE&_helpfile,
 	 const char *_path,
 	 int _status,
 	 const char *_owner,
@@ -149,9 +160,9 @@ PRIVATE void CONFIG_FILE::init (HELP_FILE&_helpfile,
 	 int _status,
 	 const char *_owner,
 	 const char *_group,
-	 int _perm,
+	 const char *_perm_str,
 	 const char *_subsys);
-PROTECTED void CONFIG_FILE::setkey (const char *newkey);
+PUBLIC void CONFIG_FILE::setkey (const char *newkey);
 PUBLIC CONFIG_FILE::CONFIG_FILE (const char *_path,
 	 HELP_FILE&_helpfile,
 	 int _status);
@@ -172,6 +183,19 @@ PUBLIC CONFIG_FILE::CONFIG_FILE (const char *_path,
 	 const char *_group,
 	 int _perm,
 	 const char *_subsys);
+PUBLIC CONFIG_FILE::CONFIG_FILE (const char *_path,
+	 HELP_FILE&_helpfile,
+	 int _status,
+	 const char *_owner,
+	 const char *_group,
+	 const char *_perm_str);
+PUBLIC CONFIG_FILE::CONFIG_FILE (const char *_path,
+	 HELP_FILE&_helpfile,
+	 int _status,
+	 const char *_owner,
+	 const char *_group,
+	 const char *_perm_str,
+	 const char *_subsys);
 PUBLIC CONFIG_FILE_INTERNAL::~CONFIG_FILE_INTERNAL (void);
 PUBLIC VIRTUAL CONFIG_FILE::~CONFIG_FILE (void);
 PUBLIC void CONFIG_FILE::forgetpath (void);
@@ -179,25 +203,27 @@ PUBLIC CONFIG_FILE *CONFIG_FILE::getnext (void)const;
 PUBLIC const char *CONFIG_FILE::getsubsys (void)const;
 PUBLIC int CONFIG_FILE::setperm (const char *fpath)const;
 PRIVATE void CONFIG_FILE::fixpath (void)const;
-PRIVATE void CONFIG_FILE::sign (FILE *fout, const char *mode)const;
-PUBLIC FILE *CONFIG_FILE::fopen_ok (const char *mode)const;
-PUBLIC FILE *CONFIG_FILE::fopen (PRIVILEGE *priv,
+PRIVATE void CONFIG_FILE::sign (FILE_CFG *fout,
 	 const char *mode)const;
-PUBLIC FILE *CONFIG_FILE::fopen (const char *mode)const;
-PUBLIC FILE *CONFIG_FILE::fopen (PRIVILEGE *priv,
+PUBLIC FILE_CFG *CONFIG_FILE::fopen_ok (const char *mode)const;
+PUBLIC FILE_CFG *CONFIG_FILE::fopen (PRIVILEGE *priv,
+	 const char *mode)const;
+PUBLIC FILE_CFG *CONFIG_FILE::fopen (const char *mode)const;
+PUBLIC FILE_CFG *CONFIG_FILE::fopen (PRIVILEGE *priv,
 	 const char *temp,
 	 const char *mode)const;
-PUBLIC FILE *CONFIG_FILE::fopen_tmp (PRIVILEGE *priv,
+PRIVATE const char *CONFIG_FILE::getlinkpath (char linkpath[PATH_MAX])const;
+PUBLIC FILE_CFG *CONFIG_FILE::fopen_tmp (PRIVILEGE *priv,
 	 const char *mode)const;
-PUBLIC int CONFIG_FILE::relink_tmp (void);
-PUBLIC FILE *CONFIG_FILE::fopen (const char *temp,
+PUBLIC int CONFIG_FILE::relink_tmp (void)const;
+PUBLIC FILE_CFG *CONFIG_FILE::fopen (const char *temp,
 	 const char *mode)const;
 PROTECTED VIRTUAL int CONFIG_FILE::archive (SSTREAM&ss)const;
 PUBLIC VIRTUAL int CONFIG_FILE::archive (void)const;
 PUBLIC VIRTUAL int CONFIG_FILE::md5sum (char *sum);
 PUBLIC VIRTUAL int CONFIG_FILE::extract (SSTREAM&ss);
 PUBLIC VIRTUAL int CONFIG_FILE::extract (void);
-PUBLIC int CONFIG_FILE::fclose (FILE *fout);
+PUBLIC int CONFIG_FILE::fclose (FILE_CFG *fout);
 PUBLIC const char *CONFIG_FILE::getpath (void)const;
 PUBLIC const char *CONFIG_FILE::getstdpath (void)const;
 PUBLIC const char *CONFIG_FILE::gethelp (void)const;
@@ -213,14 +239,24 @@ PUBLIC int CONFIG_FILE::create (void)const;
 PUBLIC long CONFIG_FILE::getdate (void)const;
 PUBLIC int CONFIG_FILE::unlink (void)const;
 PUBLIC int CONFIG_FILE::editpath (void);
-/* context.cc 07/06/1999 19.04.52 */
+PUBLIC CONFIG_FILE *CONFIG_FILES::getitem (int no)const;
+/* context.cc 18/09/2000 17.52.44 */
 PRIVATE void LINUXCONF_CONTEXT::init (void);
 PUBLIC LINUXCONF_CONTEXT::LINUXCONF_CONTEXT (const char *_path);
 PUBLIC LINUXCONF_CONTEXT::LINUXCONF_CONTEXT (void);
 PUBLIC void LINUXCONF_CONTEXT::set (LINUXCONF_CONTEXT&cur);
 PUBLIC void LINUXCONF_CONTEXT::settitle (const char *title);
+/* context_lock.cc 06/04/2001 23.04.18 */
+PRIVATE void CONTEXT_LOCK::init (const char *_key1,
+	 const char *_key2,
+	 int _flags);
+PUBLIC CONTEXT_LOCK::CONTEXT_LOCK (const char *key);
+PUBLIC CONTEXT_LOCK::CONTEXT_LOCK (const char *_key1,
+	 const char *_key2);
+PUBLIC CONTEXT_LOCK::~CONTEXT_LOCK (void);
+PUBLIC bool CONTEXT_LOCK::isok (void)const;
 /* _dict.cc 14/08/1996 14.39.06 */
-/* dictionary.cc 10/01/1999 21.43.42 */
+/* dictionary.cc 30/07/2001 17.02.06 */
 PUBLIC DICTIONARY::DICTIONARY (void);
 PRIVATE void DICTIONARY::set_type (const char prefix,
 	 const char *var,
@@ -237,23 +273,68 @@ PUBLIC void DICTIONARY::set_obj (const char *var, void *obj);
 PUBLIC void *DICTIONARY::get_obj (const char *var)const;
 PUBLIC const char *DICTIONARY::get_var (int no)const;
 PUBLIC const char *DICTIONARY::get_val (int no)const;
+PUBLIC int DICTIONARY::getnb (void)const;
 /* dir.cc 02/10/1998 01.20.08 */
-/* error.cc 02/02/1999 18.13.16 */
+/* error.cc 02/03/2001 12.28.04 */
 /* fgets1.cc 13/12/1996 23.39.36 */
-/* fgets.cc 20/07/1998 22.57.24 */
-/* file.cc 07/06/1999 19.06.18 */
+/* fgets.cc 31/03/2001 15.25.52 */
+/* file.cc 03/05/2001 11.18.06 */
+/* filecfg.cc 31/03/2001 15.27.24 */
+PUBLIC FILE_CFG::FILE_CFG (FILE *_f, const CONFIG_FILE *_cf);
 /* fopen1.cc 30/07/1998 02.03.12 */
 /* fopen.cc 22/12/1997 00.37.36 */
-/* fviews.cc 02/02/1999 23.33.28 */
-PUBLIC VIEWITEM::VIEWITEM (const SSTRING&_comment,
-	 const char *_line,
-	 const SSTRING&_endcomment);
+/* fviews.cc 29/12/2000 17.33.18 */
+PUBLIC VIEWITEMS_PARSER::VIEWITEMS_PARSER (void);
+PUBLIC VIRTUAL void VIEWITEMS_PARSER::init (VIEWITEMS&_vi);
+PUBLIC VIRTUAL bool VIEWITEMS_PARSER::is_comment (const char *line);
+PUBLIC VIRTUAL const char *VIEWITEMS_PARSER::skip_comment (const char *line);
+PUBLIC VIRTUAL void VIEWITEMS_PARSER::addline (const char *line);
+PUBLIC VIRTUAL const char *VIEWITEMS_PARSER::getline (bool reset,
+	 const char *&comment);
+PUBLIC VIEWITEM::VIEWITEM (const char *_line, int _type);
 PUBLIC VIEWITEM::VIEWITEM (const char *_line);
+PUBLIC int VIEWITEMS::realpos (int no, int type)const;
+PUBLIC int VIEWITEMS::virtpos (int no)const;
+PUBLIC VIEWITEM *VIEWITEMS::getitem (int no, int type)const;
 PUBLIC VIEWITEM *VIEWITEMS::getitem (int no)const;
+PUBLIC int VIEWITEMS::getnb (int type)const;
+PUBLIC int VIEWITEMS::getnb (void)const;
+PUBLIC void VIEWITEMS::insert (int pos, VIEWITEM *pt, int type);
+PUBLIC void VIEWITEMS::insert (int pos, VIEWITEM *pt);
+PUBLIC void VIEWITEMS::add (VIEWITEM *pt, int type);
+PUBLIC void VIEWITEMS::add (VIEWITEM *pt);
+PUBLIC int VIEWITEMS::lookup (VIEWITEM *o, bool real)const;
+PUBLIC int VIEWITEMS::lookup (VIEWITEM *o)const;
+PUBLIC void VIEWITEMS::moveto (VIEWITEM *o, int newpos, int type);
+PUBLIC void VIEWITEMS::moveto (VIEWITEM *o, int newpos);
+PUBLIC int VIEWITEMS::remove_del (int no, int type);
+PUBLIC int VIEWITEMS::remove_del (int no);
+PUBLIC int VIEWITEMS::remove_del (VIEWITEM *it);
+PUBLIC VIEWITEM *VIEWITEMS::locate (const char *key,
+	 int start,
+	 int end,
+	 int type);
+PUBLIC VIEWITEM *VIEWITEMS::locate (const char *key,
+	 int start,
+	 int end);
+PUBLIC VIEWITEM *VIEWITEMS::locate (const char *key, int type);
 PUBLIC VIEWITEM *VIEWITEMS::locate (const char *key);
+PUBLIC VIEWITEM *VIEWITEMS::locate (const char *key1,
+	 const char *key2,
+	 int start,
+	 int end,
+	 int type);
+PUBLIC VIEWITEM *VIEWITEMS::locate (const char *key1,
+	 const char *key2,
+	 int start,
+	 int end);
+PUBLIC VIEWITEM *VIEWITEMS::locate (const char *key1,
+	 const char *key2,
+	 int type);
 PUBLIC VIEWITEM *VIEWITEMS::locate (const char *key1,
 	 const char *key2);
 PUBLIC VIEWITEM *VIEWITEMS::locateassign (const char *var);
+PUBLIC VIEWITEM *VIEWITEMS::locatecommented (const char *var);
 PRIVATE int VIEWITEMS::locate_gen (const char *key,
 	 VIEWITEMS&items,
 	 bool fullmatch);
@@ -264,11 +345,13 @@ PUBLIC int VIEWITEMS::locate (const char *key1,
 	 const char *key2,
 	 VIEWITEMS&items);
 PUBLIC VIEWITEMS::VIEWITEMS (void);
+PUBLIC VIEWITEMS::VIEWITEMS (VIEWITEMS_PARSER&_vip);
 PUBLIC void VIEWITEMS::setcomcar (char com);
 PUBLIC int VIEWITEMS::read (CONFIG_FILE&fconf);
 PUBLIC int VIEWITEMS::write (CONFIG_FILE&fconf, PRIVILEGE *priv);
 PUBLIC const char *VIEWITEMS::locateval (const char *var,
 	 char tmp[1000]);
+PUBLIC int VIEWITEMS::locatebval (const char *var, int defval);
 PUBLIC int VIEWITEMS::locatebval (const char *var);
 PUBLIC int VIEWITEMS::locatehval (const char *var);
 PUBLIC int VIEWITEMS::locatenval (const char *var);
@@ -277,13 +360,20 @@ PUBLIC void VIEWITEMS::update (const char *var, const SSTRING&val);
 PUBLIC void VIEWITEMS::updatehval (const char *var, int val);
 PUBLIC void VIEWITEMS::update (const char *var, int val);
 PUBLIC void VIEWITEMS::updatebval (const char *var, bool val);
-/* helpf.cc 18/08/1998 13.56.58 */
+PUBLIC void VIEWITEMS::uncomment (VIEWITEM *vi);
+PUBLIC bool VIEWITEMS::uncomment (const char *var);
+PUBLIC void VIEWITEMS::comment (VIEWITEM *vi, const char *comstr);
+PUBLIC void VIEWITEMS::comment (VIEWITEM *vi);
+PUBLIC bool VIEWITEMS::comment (const char *var, const char *comstr);
+PUBLIC bool VIEWITEMS::comment (const char *var);
+/* helpf.cc 01/09/2000 00.19.10 */
 PUBLIC HELP_FILE::HELP_FILE (const char *_subdir, const char *_fname);
 PRIVATE void HELP_FILE::getpaths (char abspath[PATH_MAX],
 	 char relpath[PATH_MAX]);
 PUBLIC const char *HELP_FILE::getpath (void);
 PUBLIC void HELP_FILE::getrpath (char *rpath);
-/* ipnum.cc 10/12/1999 16.30.10 */
+PUBLIC bool HELP_FILE::is_nil (void)const;
+/* ipnum.cc 31/03/2001 22.00.00 */
 /* ipstuff.cc 07/04/1999 02.43.14 */
 PUBLIC IP_ADDR::IP_ADDR (void);
 PUBLIC IP_ADDR::IP_ADDR (const SSTRING&str);
@@ -309,16 +399,20 @@ PUBLIC void IPMAP::reset (void);
 PUBLIC int IPMAP::next (void);
 PUBLIC const char *IPMAP::getcur (void);
 PUBLIC void IPMAP::setuse (const IP_ADDR *adr);
-/* linuxconf.cc 20/02/2000 15.52.28 */
+/* libmodules.cc 03/12/2000 01.04.52 */
+LIBMODULE::LIBMODULE (char *name);
+LIBMODULE::~LIBMODULE (void);
+bool LIBMODULE::loaded (void);
+/* linuxconf.cc 04/12/2000 14.04.10 */
 PUBLIC CONFIG_FILE_LINUXCONF::CONFIG_FILE_LINUXCONF (const char *_path,
 	 const char *_subsys);
 PUBLIC int CONFIG_FILE_LINUXCONF::archive (SSTREAM&ss)const;
 PUBLIC int CONFIG_FILE_LINUXCONF::archive (void)const;
 PUBLIC int CONFIG_FILE_LINUXCONF::extract (SSTREAM&ss);
 PUBLIC int CONFIG_FILE_LINUXCONF::extract (void);
-/* malloc.cc 14/08/1996 14.40.26 */
-/* misc.cc 12/12/1999 22.25.52 */
-/* module_api.cc 13/12/1999 21.53.12 */
+/* malloc.cc 11/11/2000 11.41.16 */
+/* misc.cc 03/04/2001 19.54.12 */
+/* module_api.cc 10/09/2000 22.45.28 */
 PUBLIC MODULE_API::MODULE_API (void * (*_fctget)(),
 	 void (*_fctrelease)(void *));
 PUBLIC MODULE_API *MODULE_APIS::getitem (int no)const;
@@ -328,7 +422,7 @@ PUBLIC MODULE_APIREF::~MODULE_APIREF (void);
 PUBLIC void MODULE_APIREF::add (void * (*_fctget)(),
 	 void (*_fctrelease)(void *));
 PUBLIC MODULE_APIREF *MODULE_APIREFS::getitem (int no)const;
-/* module.cc 12/12/1999 22.23.12 */
+/* module.cc 28/06/2001 14.11.50 */
 PUBLIC LINUXCONF_MODULE *LINUXCONF_MODULES::getitem (int no);
 PUBLIC LINUXCONF_MODULE::LINUXCONF_MODULE (const char *_name);
 PUBLIC LINUXCONF_MODULE::~LINUXCONF_MODULE (void);
@@ -344,23 +438,26 @@ PUBLIC VIRTUAL int LINUXCONF_MODULE::dohtml (const char *);
 PUBLIC VIRTUAL int LINUXCONF_MODULE::domenu (MENU_CONTEXT, const char *);
 PUBLIC VIRTUAL int LINUXCONF_MODULE::domenu (const char *, const char *);
 PUBLIC VIRTUAL int LINUXCONF_MODULE::fixperm (bool, bool);
-PUBLIC VIRTUAL int LINUXCONF_MODULE::execmain (int , char *[]);
+PUBLIC VIRTUAL int LINUXCONF_MODULE::execmain (int , char *[], bool);
 PUBLIC VIRTUAL int LINUXCONF_MODULE::message (const char *,
 	 int ,
 	 const char *[]);
 PUBLIC VIRTUAL int LINUXCONF_MODULE::probe (int , int , bool);
 PUBLIC VIRTUAL void LINUXCONF_MODULE::usage (SSTRINGS&);
-PUBLIC MODULE_INFO::MODULE_INFO (char _active, const char *name);
+PUBLIC MODULE_INFO::MODULE_INFO (bool _active, const char *name);
 PUBLIC MODULE_INFO::MODULE_INFO (void);
 PUBLIC int MODULE_INFO::write (void);
+PUBLIC void MODULE_INFO::checkdep (void);
 PUBLIC int MODULE_INFO::locate (char realpath[PATH_MAX]);
 PUBLIC MODULE_INFO *MODULE_INFOS::getitem (int no);
 PUBLIC MODULE_INFOS::MODULE_INFOS (void);
 PUBLIC void MODULE_INFOS::read_all (void);
 PUBLIC int MODULE_INFOS::write (void);
-PUBLIC void MODULE_INFOS::setone (const char *path);
+PUBLIC void MODULE_INFOS::setone (const char *path,
+	 bool force,
+	 bool enabled);
 PUBLIC void MODULE_INFOS::unsetone (const char *path);
-/* popen.cc 25/01/2000 10.49.24 */
+/* popen.cc 20/10/2001 00.19.12 */
 PROTECTED void POPENFD::setfds (int fdin,
 	 int fdout,
 	 int fderr,
@@ -369,18 +466,20 @@ PROTECTED POPENFD::POPENFD (void);
 PUBLIC POPENFD::POPENFD (int handlein, int handleout);
 PUBLIC VIRTUAL POPENFD::~POPENFD (void);
 PUBLIC VIRTUAL bool POPENFD::isok (void);
+PUBLIC bool POPENFD::iseof (void);
 PRIVATE void POPENFD::readif (fd_set *in,
 	 int fd,
 	 SSTRING&buf,
 	 bool&ctlmsg);
-PROTECTED int POPENFD::setup (fd_set&in, int maxfd, int otherfd);
-PROTECTED int POPENFD::process (int ret_select,
+PUBLIC int POPENFD::setup (fd_set&in, int maxfd, int otherfd);
+PUBLIC int POPENFD::process (int ret_select,
 	 fd_set&in,
 	 int otherfd,
 	 bool&ctlmsg);
 PROTECTED int POPENFD::wait (int timeout, int otherfd, bool&ctlmsg);
 PUBLIC int POPENFD::wait (int timeout, int otherfd);
 PUBLIC int POPENFD::wait (int timeout);
+PUBLIC void POPENFD::seteof (void);
 PUBLIC void POPENFD::grabhandles (int &in, int &out, int &err);
 PRIVATE int POPENFD::readline (char *line, int size, SSTRING&buf);
 PUBLIC int POPENFD::readerr (char *line, int size);
@@ -391,14 +490,22 @@ PUBLIC FILE *POPENFD::getfout (void);
 PUBLIC void POPENFD::send (const char *msg);
 PUBLIC void POPENFD::flush (void);
 PUBLIC void POPENFD::sendf (const char *ctl, ...);
-PRIVATE void POPEN::init (const char *command, int uid);
+PRIVATE void POPEN::init (const char *command,
+	 int uid,
+	 bool keepenv,
+	 bool keepcwd);
 PUBLIC POPEN::POPEN (const char *command, int uid);
+PUBLIC POPEN::POPEN (const char *command,
+	 int uid,
+	 bool keepenv,
+	 bool keepcwd);
 PUBLIC POPEN::POPEN (const char *command);
 PRIVATE void POPEN::initarg (const char *command,
 	 const char *args,
 	 int uid);
 PUBLIC POPEN::POPEN (const char *command, const char *args);
 PUBLIC POPEN::POPEN (const char *command, const char *args, int uid);
+PUBLIC POPENUSER::POPENUSER (const char *cmd);
 PUBLIC POPEN::~POPEN (void);
 PUBLIC void POPEN::kill (void);
 PUBLIC void POPEN::forget (void);
@@ -414,12 +521,15 @@ PUBLIC SSTREAM_POPEN::SSTREAM_POPEN (POPENFD&_pop);
 PUBLIC SSTREAM_POPEN::~SSTREAM_POPEN (void);
 PUBLIC void SSTREAM_POPEN::puts (const char *s);
 PUBLIC char *SSTREAM_POPEN::gets (char *s, int maxsiz);
+PUBLIC long SSTREAM_POPEN::getoffset (void);
 PUBLIC POPENWAIT::POPENWAIT (POPENFD&_po, int _timeout);
 PUBLIC int POPENWAIT::getretcode (void);
 PUBLIC POPENWAIT *POPENWAITS::getitem (int no)const;
 PUBLIC int POPENWAITS::wait (void);
 PUBLIC bool POPENWAITS::hasdata (void);
-/* sstream.cc 08/12/1998 01.32.18 */
+/* samples.cc 06/04/2001 22.46.22 */
+int CONFIG_FILE_SAMPLE::extract (SSTREAM&ss);
+/* sstream.cc 26/11/2000 22.48.04 */
 PUBLIC VIRTUAL SSTREAM::~SSTREAM (void);
 PUBLIC int SSTREAM::printf (const char *ctl, ...);
 PUBLIC void SSTREAM::putch (char c);
@@ -428,13 +538,20 @@ PUBLIC SSTREAM_BUF::SSTREAM_BUF (const char *_buf);
 PUBLIC SSTREAM_BUF::~SSTREAM_BUF (void);
 PUBLIC void SSTREAM_BUF::puts (const char *s);
 PUBLIC char *SSTREAM_BUF::gets (char *s, int maxs);
+PUBLIC long SSTREAM_BUF::getoffset (void);
 PUBLIC const char *SSTREAM_BUF::getbuf (void)const;
 PUBLIC SSTREAM_FILE::SSTREAM_FILE (FILE *_f);
 PUBLIC void SSTREAM_FILE::puts (const char *s);
 PUBLIC char *SSTREAM_FILE::gets (char *s, int maxs);
+PUBLIC long SSTREAM_FILE::getoffset (void);
+PUBLIC SSTREAM_FILE_CFG::SSTREAM_FILE_CFG (FILE_CFG *_f);
+PUBLIC void SSTREAM_FILE_CFG::puts (const char *s);
+PUBLIC char *SSTREAM_FILE_CFG::gets (char *s, int maxs);
+PUBLIC long SSTREAM_FILE_CFG::getoffset (void);
 PUBLIC void SSTREAM_NUL::puts (const char *);
 PUBLIC char *SSTREAM_NUL::gets (char *, int);
-/* sstring.cc 12/12/1999 22.07.40 */
+PUBLIC long SSTREAM_NUL::getoffset (void);
+/* sstring.cc 22/04/2001 23.38.48 */
 PUBLIC SSTRING::SSTRING (void);
 PUBLIC SSTRING::SSTRING (const char *_str);
 PUBLIC SSTRING::SSTRING (const char *_str, int _maxsiz);
@@ -443,16 +560,23 @@ PUBLIC SSTRING&SSTRING::operator = (const SSTRING&_str);
 PUBLIC SSTRING::~SSTRING (void);
 PUBLIC void SSTRING::copy (char *dst)const;
 PUBLIC void SSTRING::copy (SSTRING&dst)const;
+PUBLIC void SSTRING::setempty (void);
 PUBLIC VIRTUAL void SSTRING::setfrom (const char *src);
 PUBLIC void SSTRING::setfrom (const SSTRING&src);
+PUBLIC void SSTRING::setfrom (const char *src, int len);
 PUBLIC void SSTRING::setfrom (int val);
+PUBLIC void SSTRING::append (const char *app, int len2);
 PUBLIC void SSTRING::append (const char *app);
+PUBLIC void SSTRING::append (const char car);
+PUBLIC void SSTRING::append (const SSTRING&s);
+PUBLIC void SSTRING::append (const SSTRING *s);
 PUBLIC int SSTRING::getval (void)const;
 PUBLIC double SSTRING::getvalf (void)const;
 PUBLIC int SSTRING::getlen (void)const;
 PUBLIC const char *SSTRING::get (void)const;
 PUBLIC int SSTRING::cmp (const char *other)const;
 PUBLIC int SSTRING::ncmp (const char *other, int len)const;
+PUBLIC int SSTRING::ncmp (const SSTRING&other, int len)const;
 PUBLIC int SSTRING::icmp (const char *other)const;
 PUBLIC int SSTRING::icmp (const SSTRING&other)const;
 PUBLIC const char *SSTRING::strchr (char carac)const;
@@ -461,7 +585,8 @@ PUBLIC const char *SSTRING::strstr (const char *sub)const;
 PUBLIC int SSTRING::cmp (const SSTRING&other)const;
 PUBLIC void SSTRING::setmaxsiz (int size);
 PUBLIC int SSTRING::getmaxsiz (void)const;
-PUBLIC int SSTRING::is_empty (void)const;
+PUBLIC bool SSTRING::is_empty (void)const;
+PUBLIC bool SSTRING::is_filled (void)const;
 PUBLIC void SSTRING::strip_end (void);
 PUBLIC char *SSTRING::copyword (const char *line);
 PUBLIC bool SSTRING::truncate (int newlen);
@@ -472,20 +597,25 @@ PUBLIC int SSTRING::setfromf (const char *ctl, ...);
 PUBLIC int SSTRING::appendf (const char *ctl, ...);
 PUBLIC void SSTRING::to_lower (void);
 PUBLIC void SSTRING::to_upper (void);
-/* sstrings.cc 11/09/1998 17.11.26 */
+PUBLIC SSTRING SSTRING::substr (int start)const;
+PUBLIC SSTRING SSTRING::substr (int start, int end)const;
+/* sstrings.cc 06/09/2001 22.51.54 */
 PUBLIC SSTRING *SSTRINGS::getitem (int no)const;
 PUBLIC void SSTRINGS::sort (void);
 PUBLIC int SSTRINGS::lookup (const char *str)const;
 PUBLIC int SSTRINGS::lookup (const SSTRING *str)const;
+PUBLIC int SSTRINGS::ilookup (const char *str)const;
+PUBLIC int SSTRINGS::ilookup (const SSTRING *str)const;
 PUBLIC void SSTRINGS::remove_empty (void);
 PUBLIC void SSTRINGS::remove_dups (void);
 PUBLIC void SSTRINGS::append (const SSTRINGS&tb);
+PUBLIC SSTRINGS&SSTRINGS::operator = (const SSTRINGS&_strs);
 /* str1.cc 12/11/1997 02.59.48 */
 /* str3.cc 12/12/1999 22.09.10 */
 /* str6.cc 29/11/1997 13.38.50 */
-/* str.cc 28/07/1998 14.20.02 */
-/* stub.cc 24/01/2000 15.52.46 */
-/* subsys.cc 28/02/2000 12.41.58 */
+/* str.cc 28/08/2000 20.49.44 */
+/* stub.cc 10/03/2001 15.07.54 */
+/* subsys.cc 27/11/2000 22.06.14 */
 PRIVATE void LINUXCONF_SUBSYS::init (const char *key);
 PUBLIC LINUXCONF_SUBSYS::LINUXCONF_SUBSYS (const char *key,
 	 TRANS_NOTLOAD *_title);
@@ -499,15 +629,16 @@ PUBLIC ONECONF::ONECONF (const char *_name);
 PUBLIC void ONECONF::write (void);
 PUBLIC void ONECONF::remove (void);
 PUBLIC const char *ONECONF::getfamily (const char *subsys);
-/* timestr.cc 12/12/1999 22.24.34 */
+/* timestr.cc 02/03/2001 16.19.22 */
 PRIVATE void TIMESTR::formatstr (void);
 PUBLIC void TIMESTR::setfrom (const char *_str);
 PUBLIC void TIMESTR::setfrom (long _seconds);
 PUBLIC void TIMESTR::setfrom (const TIMESTR&s);
 PUBLIC TIMESTR::TIMESTR (long _seconds);
+PUBLIC TIMESTR::TIMESTR (const char *_str);
 PUBLIC TIMESTR::TIMESTR (void);
 /* uname.cc 05/02/1999 00.45.54 */
 /* virtdb.cc 23/09/1997 01.14.20 */
 PUBLIC VIRTDB::VIRTDB (const char *_name);
 PUBLIC VIRTUAL VIRTDB::~VIRTDB (void);
-/* xconf1.cc 02/02/1999 17.56.48 */
+/* xconf1.cc 30/06/2000 23.07.54 */
