@@ -19,6 +19,8 @@
 #include <dialog.h>
 #include <modapi.h>
 #include "../paths.h"
+#include "libmodules.h"
+
 
 static HELP_FILE help_modules ("misc","modules");
 
@@ -38,7 +40,9 @@ static LINUXCONF_MODULES modules;
 
 PUBLIC LINUXCONF_MODULE::LINUXCONF_MODULE(const char *_name)
 {
+	context_lock_required();		// Make sure this is available for modules
 	module_api_required();
+	libmodules_required();
 	modules.neverdelete();
 	modules.add (this);
 	name = strdup(_name);
@@ -148,7 +152,8 @@ PUBLIC VIRTUAL int LINUXCONF_MODULE::fixperm (bool, bool )
 */
 PUBLIC VIRTUAL int LINUXCONF_MODULE::execmain (
 	int,		// argc
-	char *[])	// argv
+	char *[],	// argv
+	bool)		// standalone
 {
 	return LNCF_NOT_APPLICABLE;
 }
@@ -308,12 +313,12 @@ int module_fixperm (bool boottime, bool silentflag)
 	Try to pass control to a module based on argv[0]
 	Return LNCF_NOT_APPLICABLE if no module accept control.
 */
-int module_execmain (int argc, char *argv[])
+int module_execmain (int argc, char *argv[], bool standalone)
 {
 	int ret = LNCF_NOT_APPLICABLE;
 	int n = modules.getnb();
 	for (int i=0; i<n; i++){
-		int code = modules.getitem(i)->execmain (argc, argv);
+		int code = modules.getitem(i)->execmain (argc, argv,standalone);
 		if (code != LNCF_NOT_APPLICABLE){
 			ret = code;
 			break;
@@ -386,8 +391,9 @@ public:
 							// in conf.linuxconf
 	/*~PROTOBEG~ MODULE_INFO */
 public:
-	MODULE_INFO (char _active, const char *name);
+	MODULE_INFO (bool _active, const char *name);
 	MODULE_INFO (void);
+	void checkdep (void);
 	int locate (char realpath[PATH_MAX]);
 	int write (void);
 	/*~PROTOEND~ MODULE_INFO */
@@ -396,10 +402,10 @@ public:
 static const char K_MODULE[]="module";
 static const char K_LIST[]="list";
 
-PUBLIC MODULE_INFO::MODULE_INFO(char _active, const char *name)
+PUBLIC MODULE_INFO::MODULE_INFO(bool _active, const char *name)
 {
 	strcpy (path,name);
-	active = _active;
+	active = _active ? 1 : 0;
 	original_active = _active;
 }
 
@@ -459,7 +465,7 @@ int module_locate(
 
 			for example.
 		*/
-		long maxrev = 0;
+		long maxrev = -1;
 		int norev = -1;
 		char trypath[PATH_MAX];
 		int lentry = sprintf (trypath,"%s.so.",basepath);
@@ -498,6 +504,23 @@ int module_locate(
 	}
 	return ret;
 }
+/*
+	Load the libmodules required by this module
+*/
+PUBLIC void MODULE_INFO::checkdep()
+{
+	char dep[PATH_MAX];
+	snprintf (dep,sizeof(dep)-1,"/usr/lib/linuxconf/modules/%s.dep",path);
+	FILE *fin = fopen (dep,"r");
+	if (fin != NULL){
+		char buf[100];
+		while (fgets(buf,sizeof(buf)-1,fin)!=NULL){
+			strip_end(buf);
+			if (buf[0] != '\0') new LIBMODULE(buf);
+		}
+		fclose (fin);
+	}
+}
 
 /*
 	Locate a module based on the version of linuxconf
@@ -537,7 +560,9 @@ public:
 	MODULE_INFOS (void);
 	MODULE_INFO *getitem (int no);
 	void read_all (void);
-	void setone (const char *path);
+	void setone (const char *path,
+		 bool force,
+		 bool enabled);
 	void unsetone (const char *path);
 	int write (void);
 	/*~PROTOEND~ MODULE_INFOS */
@@ -594,7 +619,7 @@ PUBLIC void MODULE_INFOS::read_all()
 				break;
 			}
 		}
-		if(!state) add (new MODULE_INFO(0,fromdir));
+		if(!state) add (new MODULE_INFO(false,fromdir));
 	}
 }
 
@@ -642,7 +667,7 @@ static int cmp_by_name (const ARRAY_OBJ *o1, const ARRAY_OBJ *o2)
 {
 	const MODULE_INFO *i1 = (MODULE_INFO*)o1;
 	const MODULE_INFO *i2 = (MODULE_INFO*)o2;
-	return strcmp(i1->path,i2->path);
+	return strcasecmp(i1->path,i2->path);
 }
 
 void module_config()
@@ -703,6 +728,7 @@ static SSTRINGS tbloaded;
 
 void module_loadcheck(const char *path)
 {
+	if (tbloaded.lookup(path)!= -1) return;
 	/* #Specification: module / dlopen / RTLD_LAZY
 		Linuxconf is loading modules with dlopen flag RTLD_LAZY.
 		Some modules are using dlopen() internally and require that
@@ -714,7 +740,7 @@ void module_loadcheck(const char *path)
 		various interpreted language).
 
 		The case known is when a module is linked with a special library
-		ehich itself does dlopen() at some point. The libpam library does
+		which itself does dlopen() at some point. The libpam library does
 		that. With RTLD_LAZY flag, the various symbols of the module
 		are not exported, so the "sub-modules" can't be loaded (the
 		various pam_xxx.so modules).
@@ -741,7 +767,8 @@ void module_loadcheck(const char *path)
 		strstr (path,"/redhat.so") != NULL
 		|| strstr (path,"/conectiva.so") != NULL
 		|| strstr (path,"/suse.so") != NULL
-		|| strstr (path,"/debian.so") != NULL;
+		|| strstr (path,"/debian.so") != NULL
+		|| strstr (path,"/pythonmod.so") != NULL;
 	void *handle = dlopen (path
 		,need_global ? RTLD_GLOBAL|RTLD_NOW : RTLD_LAZY);
 	if (handle == NULL){
@@ -780,8 +807,10 @@ void module_loadcheck(const char *path)
 		if (pt == NULL || *pt != MODULE_API_VERSION){
 			xconf_error (MSG_U(E_COMPATMOD
 				,"Incompatible module %s\n"
-				 "get a new one or recompile it")
-				,path);
+				 "get a new one or recompile it against a recent linuxconf-devel package\n"
+				 "\n"
+				 "Expect API revision %d, got %d")
+				,path,MODULE_API_VERSION,*pt);
 			LINUXCONF_MODULE *mod = modules.getitem(modules.getnb()-1);
 			modules.remove (mod);
 		}else{
@@ -791,13 +820,20 @@ void module_loadcheck(const char *path)
 }
 
 /*
-	Get the path of all loaded modules
+	Get the path of all loaded modules.
+	It returns a SSTRINGS pointer. Do not delete it.
 */
-SSTRINGS *module_getlist()
+const SSTRINGS *module_getlist()
 {
 	return &tbloaded;
 }
 
+/*
+	Load all the linuxconf modules.
+	If a module is already loaded, it won't be loaded twice. So this
+	function may be called several time, generally after a module_setone()
+	call.
+*/
 void module_load ()
 {
 	#ifndef LINUXCONF_AOUT
@@ -806,6 +842,7 @@ void module_load ()
 		for (int i=0; i<infos.getnb(); i++){
 			MODULE_INFO *in = infos.getitem(i);
 			if (in->active){
+				in->checkdep();
 				char path[PATH_MAX];
 				if (in->locate(path)==-1){
 					fprintf (stderr,MSG_R(E_MODNOPATH),in->path);
@@ -834,18 +871,22 @@ void module_loaddistmod()
 	If the module is already in the list, let it there without
 	affecting its enabled flag.
 */
-PUBLIC void MODULE_INFOS::setone (const char *path)
+PUBLIC void MODULE_INFOS::setone (const char *path, bool force, bool enabled)
 {
 	bool found = false;
 	for (int i=0; i<getnb(); i++){
 		MODULE_INFO *in = getitem(i);
 		if (strcmp(in->path,path)==0){
 			found = true;
+			if (force){
+				in->active = enabled ? 1 : 0;
+				write();
+			}
 			break;
 		}
 	}
 	if (!found){
-		add (new MODULE_INFO (1,path));
+		add (new MODULE_INFO (true,path));
 		write();
 	}
 }
@@ -869,15 +910,75 @@ PUBLIC void MODULE_INFOS::unsetone (const char *path)
 	}
 }
 
-void module_setone (const char *path)
+/*
+	Add a module to the list of active modules.
+	If the module is already there, its status enabled is not changed
+	unless force==true.
+*/
+void module_setone (const char *path, bool force, bool enabled)
 {
 	MODULE_INFOS infos;
-	infos.setone (path);
+	infos.setone (path,force,enabled);
 }
-
+/*
+	Add a module to the list of active modules
+*/
+void module_setone (const char *path)
+{
+	module_setone (path,false,false);
+}
+/*
+	Remove a module from the list of active modules
+*/
 void module_unsetone (const char *path)
 {
 	MODULE_INFOS infos;
 	infos.unsetone (path);
 }
 
+/*
+	Return true if a module is enabled
+*/
+bool module_is_enabled(const char *path)
+{
+	bool ret = false;
+	MODULE_INFOS infos;
+	for (int i=0; i<infos.getnb(); i++){
+		MODULE_INFO *in = infos.getitem(i);
+		if (strcmp(in->path,path)==0){
+			ret = in->active != 0;
+			break;
+		}
+	}
+	return ret;
+}
+
+/*
+	Send a message to a the distribution specific module so it can check
+	if a given package is installed.
+
+	logical_name is something like "popserver" "dhcpserver" and so on.
+	The distribution module translate this name according to its
+	packaging and check if it is installed. If not, it propose to install it.
+
+	For example, the redhat module will check is imap is installed if popserver
+	is requested.
+
+	This strategy should help administrator get it right the first time.
+	This opens a new concept: What is coming first, the admin tool, or the
+	service. It seems the admin tool is a good ambassador here.
+
+	The function returns 1 if the package was installed, 0 otherwise (maybe
+	nothing was done or the package was already installed).
+*/
+int module_requestpkg (const char *logical_name)
+{
+	int ret = 0;
+	if (dialog_mode != DIALOG_TREE){
+		const char *tb[]={logical_name};
+		ret = module_sendmessage ("request_package",1,tb);
+	}
+	return ret;
+}
+
+
