@@ -2,7 +2,7 @@
 #ifndef MODULE_H
 #define MODULE_H
 
-#define MODULE_API_VERSION	15
+#define MODULE_API_VERSION	18
 #define MODULE_SYMBOL_VERSION "module_api_version"
 #define MODULE_DEFINE_VERSION(name) int module_api_version_##name=MODULE_API_VERSION
 
@@ -28,7 +28,7 @@ public:
 	virtual int dohtml (const char *);
 	virtual int domenu (MENU_CONTEXT, const char *);
 	virtual int domenu (const char *, const char *);
-	virtual int execmain (int , char *[]);
+	virtual int execmain (int , char *[], bool);
 	virtual int fixperm (bool, bool);
 	const char *getname (void);
 	virtual int message (const char *,
