@@ -9,6 +9,7 @@
 #include <utmp.h>
 #include <sys/wait.h>
 #include <signal.h>
+#include <time.h>
 #include <sys/time.h>
 #include <pwd.h>
 #include <sys/types.h>
@@ -66,8 +67,11 @@ PUBLIC POPENFD::POPENFD(int handlein, int handleout)
 PUBLIC VIRTUAL POPENFD::~POPENFD()
 {
 	if (fds.in != -1){
-		fclose (fds.fin);
-		// ::close (fds.in);
+		if (fds.fin != NULL){
+			fclose (fds.fin);
+		}else{
+			::close (fds.in);
+		}
 	}
 	::close (fds.out);
 	::close (fds.err);
@@ -80,6 +84,10 @@ PUBLIC VIRTUAL bool POPENFD::isok()
 {
 	return fds.in != -1;
 }
+PUBLIC bool POPENFD::iseof()
+{
+	return eof;
+}
 PRIVATE void POPENFD::readif (
 	fd_set *in,
 	int fd,
@@ -94,12 +102,12 @@ PRIVATE void POPENFD::readif (
 			buf.append (bufread);
 		}else{
 			// end of process
-			eof = true;
+			seteof();
 			ctlmsg = true;
 		}
 	}
 }
-PROTECTED int POPENFD::setup (fd_set &in, int maxfd, int otherfd)
+PUBLIC int POPENFD::setup (fd_set &in, int maxfd, int otherfd)
 {
 	if (!eof){
 		maxfd = popen_fdset (fds.out,in,maxfd);
@@ -110,7 +118,7 @@ PROTECTED int POPENFD::setup (fd_set &in, int maxfd, int otherfd)
 	}
 	return maxfd;
 }
-PROTECTED int POPENFD::process (
+PUBLIC int POPENFD::process (
 	int ret_select,
 	fd_set &in,
 	int otherfd,
@@ -138,11 +146,12 @@ PROTECTED int POPENFD::process (
 			if (popen_fd_isset(fds.ctl,in)){
 				char buf[10];
 				read (fds.ctl,buf,10);	// Just remove the message
+				// fprintf (stderr,"ctlmsg ret=%d\n",ret);
 				ctlmsg = true;
 				::close (fds.ctl);
 				fds.ctl = -1;
 			}
-		}else if (ret == -1 && errno == EINTR){
+		}else if (ret_select == -1 && errno == EINTR){
 			ret = 0;
 		}
 	}
@@ -167,8 +176,13 @@ PROTECTED int POPENFD::wait(int timeout, int otherfd, bool &ctlmsg)
 		struct timeval tim;
 		tim.tv_usec = 0;
 		tim.tv_sec = timeout;
-		ret = select (maxfd+1,&in,NULL,NULL,&tim);
+		while (1){
+			ret = select (maxfd+1,&in,NULL,NULL,&tim);
+			if (ret != -1 || errno != EINTR) break;
+			//fprintf (stderr,"POPEN eintr\n");
+		}
 		ret = process (ret,in,otherfd,ctlmsg);
+		// fprintf (stderr,"after process %d %d\n",ret,ctlmsg);
 	}
 	return ret;
 }
@@ -184,7 +198,10 @@ PROTECTED int POPENFD::wait(int timeout, int otherfd, bool &ctlmsg)
 PUBLIC int POPENFD::wait(int timeout, int otherfd)
 {
 	bool dummy=false;
-	return wait (timeout,otherfd,dummy);
+	int ret = wait (timeout,otherfd,dummy);
+	// fprintf (stderr,"wait %d %d\n",ret,dummy);
+	if (ret == 0 && dummy) ret = -1;
+	return ret;
 }
 /*
 	Wait for anything to be available from the child process
@@ -197,6 +214,11 @@ PUBLIC int POPENFD::wait(int timeout)
 	return wait (timeout,-1);
 }
 
+PUBLIC void POPENFD::seteof()
+{
+	eof = true;
+}
+
 /*
 	Grab the PIPE handles used by this object to prevent it from closing them.
 	Return the number of handle placed in tb[].
@@ -366,11 +388,27 @@ static void fchild(int )
 	child_counter++;
 	int status;
 	int pid = ::wait(&status);
-	for (unsigned i=0; i<sizeof(tbcod)/sizeof(tbcod[0]); i++){
-		if (tbcod[i].pid == pid){
-			tbcod[i].status = status;
-			tbcod[i].done = true;
-			break;
+	if (pid != -1){
+		unsigned i;
+		// fprintf (stderr,"fchild %d\n",pid);
+		for (i=0; i<sizeof(tbcod)/sizeof(tbcod[0]); i++){
+			if (tbcod[i].pid == pid){
+				// fprintf (stderr,"fchild found\n");
+				tbcod[i].status = status;
+				tbcod[i].done = true;
+				break;
+			}
+		}
+		if (i == sizeof(tbcod)/sizeof(tbcod[0])){
+			for (i=0; i<sizeof(tbcod)/sizeof(tbcod[0]); i++){
+				if (tbcod[i].pid == 0){
+					// fprintf (stderr,"fchild adding\n");
+					tbcod[i].pid = pid;
+					tbcod[i].status = status;
+					tbcod[i].done = true;
+					break;
+				}
+			}
 		}
 	}
 }
@@ -379,13 +417,22 @@ static void fchild(int )
 */
 static void popen_reservpid(int pid)
 {
-	for (unsigned i=0; i<sizeof(tbcod)/sizeof(tbcod[0]); i++){
-		if (tbcod[i].pid == 0){
-			tbcod[i].pid = pid;
-			tbcod[i].done = false;
+	// fprintf (stderr,"reservpid %d\n",pid);
+	unsigned i;
+	for (i=0; i<sizeof(tbcod)/sizeof(tbcod[0]); i++){
+		if (tbcod[i].pid == pid){
 			break;
 		}
 	}
+	if (i == sizeof(tbcod)/sizeof(tbcod[0])){
+		for (unsigned i=0; i<sizeof(tbcod)/sizeof(tbcod[0]); i++){
+			if (tbcod[i].pid == 0){
+				tbcod[i].pid = pid;
+				tbcod[i].done = false;
+				break;
+			}
+		}
+	}
 }
 /*
 	Forget about a PID (We don't care about it anymore)
@@ -400,14 +447,36 @@ static void popen_forgetpid(int pid)
 	}
 }
 
+static void fpipe(int){}
+
 void popen_initsignal ()
 {
+	signal (SIGPIPE,fpipe);
 	signal (SIGCHLD,fchild);
 }
 
 const char popen_SOCKETHOLDER[]="%SOCKET%";
 
-PRIVATE void POPEN::init (const char *command, int uid)
+static void popen_add2env (
+	const char *envvar,
+	char buf[100],
+	char *tbenv[],
+	int &nbenv)
+{
+	const char *val = getenv (envvar);
+	if (val != NULL){
+		snprintf (buf,100-1,"%s=%s",envvar,val);
+		tbenv[nbenv] = buf;
+		nbenv++;
+	}
+}
+
+
+PRIVATE void POPEN::init (
+	const char *command,
+	int uid,
+	bool keepenv,	// Do not create a clean environment
+	bool keepcwd)	// Do not chdir to /
 {
 	cur_dead = child_counter;
 	pid = -1;
@@ -429,41 +498,34 @@ PRIVATE void POPEN::init (const char *command, int uid)
 		&& pipe(fdctl)!=-1){
 		popen_initsignal();
 		pid = fork();
-		popen_reservpid (pid);
 		if (pid == 0){
-			// Setting up a clean environnement: Linuxconf is often setuid
-			char *tb[10];
-			tb[0] = "PATH=/sbin:/usr/sbin:/bin:/usr/bin";
-			char termstr[100],displaystr[300],homestr[300],langstr[100];
-			strcpy (termstr,"TERM=linux");
-			const char *term = getenv ("TERM");
-			if (term != NULL) snprintf (termstr,sizeof(termstr)-1,"TERM=%s",term);
-			tb[1] = termstr;
-			struct passwd *pw = getpwuid (uid);
-			const char *home;
-			if (pw != NULL){
-				home = pw->pw_dir;
-			}else{
-				home = getenv ("HOME");
-			}
-			snprintf (homestr,sizeof(homestr)-1,"HOME=%s",home);
-			tb[2] = homestr;
-			int nbenv = 3;
-			const char *display = getenv("DISPLAY");
-			if (display != NULL){
-				snprintf (displaystr,sizeof(displaystr)-1,"DISPLAY=%s",display);
-				tb[nbenv] = displaystr;
-				nbenv++;
-			}
-			const char *lang = getenv ("LANG");
-			if (lang != NULL){
-				snprintf (langstr,sizeof(langstr)-1,"LANG=%s",lang);
-				tb[nbenv] = langstr;
-				nbenv++;
-			}
-			tb[nbenv] = NULL;
+			if (!keepenv){
+				// Setting up a clean environnement: Linuxconf is often setuid
+				char *tb[10];
+				tb[0] = "PATH=/sbin:/usr/sbin:/bin:/usr/bin";
+				char termstr[100],displaystr[300],homestr[300],langstr[100];
+				char authstr[100];
+				strcpy (termstr,"TERM=linux");
+				const char *term = getenv ("TERM");
+				if (term != NULL) snprintf (termstr,sizeof(termstr)-1,"TERM=%s",term);
+				tb[1] = termstr;
+				struct passwd *pw = getpwuid (uid);
+				const char *home;
+				if (pw != NULL){
+					home = pw->pw_dir;
+				}else{
+					home = getenv ("HOME");
+				}
+				snprintf (homestr,sizeof(homestr)-1,"HOME=%s",home);
+				tb[2] = homestr;
+				int nbenv = 3;
+				popen_add2env ("DISPLAY",displaystr,tb,nbenv);
+				popen_add2env ("LANG",langstr,tb,nbenv);
+				popen_add2env ("XAUTHORITY",authstr,tb,nbenv);
+				tb[nbenv] = NULL;
 
-			environ = tb;
+				environ = tb;
+			}
 
 			::close (fderr[0]);
 			::close (fdctl[0]);
@@ -491,7 +553,7 @@ PRIVATE void POPEN::init (const char *command, int uid)
 			for (int i=3; i<100; i++){
 				if (i != fdctl[1] && i != fdinout[1]) ::close (i);
 			}
-			chdir ("/");
+			if (!keepcwd) chdir ("/");
 			int ret = system (command);
 			write (fdctl[1],"done\n",5);
 			#if 0
@@ -502,6 +564,7 @@ PRIVATE void POPEN::init (const char *command, int uid)
 			if ((ret & 0xff)==0) ret >>= 8;
 			_exit (ret);
 		}
+		popen_reservpid (pid);
 		::close (fderr[1]);
 		::close (fdctl[1]);
 		if (sockholder != NULL){
@@ -518,14 +581,25 @@ PUBLIC POPEN::POPEN(
 	const char *command,
 	int uid)		// Effective and real UID of the new process
 {
-	init (command,uid);
+	init (command,uid,false,false);
+}
+PUBLIC POPEN::POPEN(
+	const char *command,
+	int uid,		// Effective and real UID of the new process
+	bool keepenv,
+	bool keepcwd)
+{
+	init (command,uid,keepenv,keepcwd);
 }
 PUBLIC POPEN::POPEN(const char *command)
 {
-	init (command,geteuid());
+	init (command,geteuid(),false,false);
 }
 
-PRIVATE void POPEN::initarg(const char *command, const char *args, int uid)
+PRIVATE void POPEN::initarg(
+	const char *command,
+	const char *args,
+	int uid)
 {
 	pid = -1;
 	status = -1;
@@ -534,9 +608,9 @@ PRIVATE void POPEN::initarg(const char *command, const char *args, int uid)
 	if (path != NULL){
 		char cmd[strlen(path)+2+strlen(args)];
 		sprintf (cmd,"%s %s",path,args);
-		init (cmd,uid);
+		init (cmd,uid,false,false);
 	}else{
-		eof = true;
+		seteof();
 	}
 }
 /*
@@ -569,6 +643,17 @@ PUBLIC POPEN::POPEN(const char *command, const char *args, int uid)
 {
 	initarg(command,args,uid);
 }
+
+/*
+	Execute a command for a user.
+	Unlike POPEN which make sure tne environment is secure, POPENUSER
+	is really trusting the user. This is never called by Linuxconf
+*/
+PUBLIC POPENUSER::POPENUSER(const char *cmd)
+	: POPEN (cmd,getuid(),true,true)
+{
+}
+
 	
 PUBLIC POPEN::~POPEN()
 {
@@ -648,7 +733,7 @@ PRIVATE void POPEN::checksignal()
 
 /*
 	Wait for anything to be available from the child process
-	Return -1 if any error.
+	Return -1 if any error (or end of session).
 	Return  0 if the timeout has elapsed.
 	Return  1 if there is some data to read
 	Return  2 if there is some data on otherfd
@@ -668,7 +753,10 @@ PUBLIC int POPEN::wait(int timeout, int otherfd)
 		bool ctlmsg = false;
 		ret = POPENFD::wait (timeout,otherfd,ctlmsg);
 		checksignal();
-		if (ctlmsg) waitend();
+		if (ctlmsg){
+			waitend();
+			if (ret == 0) ret = -1;
+		}
 	}else if(!eof){
 		ret = POPENFD::wait (timeout,-1);
 	}
@@ -724,6 +812,7 @@ PUBLIC int POPEN::close()
 */
 PUBLIC SSTREAM_POPEN::SSTREAM_POPEN(POPENFD &_pop)
 {
+	offset = 0;
 	pop = &_pop;
 }
 PUBLIC SSTREAM_POPEN::~SSTREAM_POPEN()
@@ -733,12 +822,14 @@ PUBLIC SSTREAM_POPEN::~SSTREAM_POPEN()
 PUBLIC void SSTREAM_POPEN::puts(const char *s)
 {
 	pop->send (s);
+	offset += strlen(s);
 }
 PUBLIC char *SSTREAM_POPEN::gets(char *s, int maxsiz)
 {
 	char *ret = NULL;
 	while (1){
 		if (pop->readout(s,maxsiz)==0){
+			offset += strlen(s);
 			ret = s;
 			break;
 		}else if (pop->wait(1)< 0){
@@ -747,6 +838,10 @@ PUBLIC char *SSTREAM_POPEN::gets(char *s, int maxsiz)
 	}
 	return ret;
 }
+PUBLIC long SSTREAM_POPEN::getoffset()
+{
+	return offset;
+}
 
 /*
 	Object used to wait for several POPEN at once
