@@ -6,6 +6,7 @@ class SSTREAM{
 public:
 	virtual void puts(const char*)=0;
 	virtual char *gets(char *, int)=0;
+	virtual long getoffset()=0;
 	/*~PROTOBEG~ SSTREAM */
 public:
 	int printf (const char *ctl, ...);
@@ -24,6 +25,7 @@ public:
 	SSTREAM_BUF (const char *_buf);
 	SSTREAM_BUF (void);
 	const char *getbuf (void)const;
+	long getoffset (void);
 	char *gets (char *s, int maxs);
 	void puts (const char *s);
 	~SSTREAM_BUF (void);
@@ -35,14 +37,29 @@ class SSTREAM_FILE: public SSTREAM {
 	/*~PROTOBEG~ SSTREAM_FILE */
 public:
 	SSTREAM_FILE (FILE *_f);
+	long getoffset (void);
 	char *gets (char *s, int maxs);
 	void puts (const char *s);
 	/*~PROTOEND~ SSTREAM_FILE */
 };
 
+class FILE_CFG;
+
+class SSTREAM_FILE_CFG: public SSTREAM {
+	FILE_CFG *f;
+	/*~PROTOBEG~ SSTREAM_FILE_CFG */
+public:
+	SSTREAM_FILE_CFG (FILE_CFG *_f);
+	long getoffset (void);
+	char *gets (char *s, int maxs);
+	void puts (const char *s);
+	/*~PROTOEND~ SSTREAM_FILE_CFG */
+};
+
 class SSTREAM_NUL: public SSTREAM {
 	/*~PROTOBEG~ SSTREAM_NUL */
 public:
+	long getoffset (void);
 	char *gets (char *, int);
 	void puts (const char *);
 	/*~PROTOEND~ SSTREAM_NUL */
