@@ -43,6 +43,28 @@ PUBLIC int SSTRINGS::lookup(const SSTRING *str) const
 {
 	return lookup (str->get());
 }
+/*
+	Find the occurence of a string in the array, case insensitive.
+	Return -1 if not found or the index if found
+*/
+PUBLIC int SSTRINGS::ilookup(const char *str) const
+{
+	int ret = -1;
+	int nb = getnb();
+	for (int i=0; i<nb; i++){
+		SSTRING *s = getitem(i);
+		if (s->icmp(str)==0){
+			ret = i;
+			break;
+		}
+	}
+	return ret;
+}
+
+PUBLIC int SSTRINGS::ilookup(const SSTRING *str) const
+{
+	return ilookup (str->get());
+}
 
 
 /*
@@ -88,3 +110,12 @@ PUBLIC void SSTRINGS::append (const SSTRINGS &tb)
 }
 		
 
+PUBLIC SSTRINGS &SSTRINGS::operator = (const SSTRINGS &_strs)
+{
+	remove_all();
+	for (int i=0; i<_strs.getnb(); i++){
+		add (new SSTRING(_strs.getitem(i)->get()));
+	}
+	return *this;
+}
+
