@@ -11,15 +11,45 @@
 	to the liblinuxconf library.
 */
 #include <stdio.h>
+#include <unistd.h>
+#include <pwd.h>
+#include <grp.h>
+#include <sys/types.h>
+#include <fixperm.h>
+
 int revision;
 
 void html_access_log(int, const char *){}
 int html_access_check(int){return 0;}
 void perm_setaccess (const char *, const char *){}
-int fixperm_readperm(char const *, class PERMINFO &,
-	char const *, char const *, int, bool)
+int fixperm_readperm(
+	char const *,
+	class PERMINFO &pinfo,
+	char const *user,
+	char const *group,
+	int perm,
+	bool)
 {
-	return -1;
+	pinfo.perm = perm;
+	uid_t uid = getuid();
+	if (uid == 0){
+		struct passwd *p = getpwnam (user);
+		if (p != NULL){
+			pinfo.uid = p->pw_uid;
+		}else{
+			pinfo.uid = uid;
+		}
+		struct group *g = getgrnam (user);
+		if (g != NULL){
+			pinfo.gid = g->gr_gid;
+		}else{
+			pinfo.gid = getgid();
+		}
+	}else{
+		pinfo.uid = uid;
+		pinfo.gid = getgid();
+	}
+	return 0;
 }
 
 int perm_rootaccess(char const *,...)
@@ -56,9 +86,19 @@ void button_text2png (const char *, FILE *)
 {
 }
 
+#if 0
 enum MENU_CONTEXT{
 	MENU_STUB
 };
+#endif
 
 void module_setmenu (class DIALOG &, MENU_CONTEXT){}
 
+int module_sendmessage (
+	const char *msg,
+	int argc,
+	const char *argv[])
+{
+	return 0;
+}
+
