@@ -320,14 +320,18 @@ PUBLIC const char *ONECONF::getfamily(const char *subsys)
 */
 const char *confver_getcur ()
 {
-	if (curver == NULL){
-		const char *name = linuxconf_getval(K_CONFVER,K_CUR
-			,MSG_R(T_OFFICE));
-		curver = new ONECONF(name);
-	}
+	return linuxconf_getval(K_CONFVER,K_CUR,MSG_R(T_OFFICE));
+}
+
+
+static const char *confver_loadcur ()
+{
+	if (curver == NULL) curver = new ONECONF(confver_getcur());
 	return curver->name.get();
 }
 
+
+
 /*
 	Record the new configuration name of all the configuration files
 */
@@ -347,7 +351,7 @@ void confver_setcur (const char *newcur)
 const char *confver_getfamily (const char *subsys)
 {
 	const char *ret = NULL;
-	confver_getcur();
+	confver_loadcur();
 	if (curver != NULL){
 		ret = curver->getfamily (subsys);
 	}
@@ -360,7 +364,7 @@ const char *confver_getfamily (const char *subsys)
 */
 bool confver_getmode ()
 {
-	return linuxconf_getvalnum (K_CONFVER,K_MODE,0);
+	return linuxconf_getvalnum (K_CONFVER,K_MODE,1);
 }
 
 static void subsys_addif (SSTRINGS &tb, const char *str)
@@ -368,6 +372,21 @@ static void subsys_addif (SSTRINGS &tb, const char *str)
 	if (tb.lookup(str)==-1) tb.add (new SSTRING(str));
 }
 
+static void subsys_showmembers(const char *subsys)
+{
+	SSTRINGS tb;
+	configf_getsubsysmembers (subsys,tb);
+	SSTRING title;
+	title.setfromf(MSG_U(T_SUBSYSNAME,"Sus-system %s"),subsys);
+	dialog_textbox (title.get()
+		,MSG_U(I_SUBSYSNAME,"Here is the list of configuration file\n"
+			"member of the sub-system")
+		,help_nil,tb);
+}
+
+/*
+	Edit the definition of a profile
+*/
 static int confver_editone (const char *name)
 {
 	ONECONF conf (name);
@@ -388,9 +407,12 @@ static int confver_editone (const char *name)
 		}
 	}
 	dia.newf_title ("",MSG_U(T_SUBSYS,"Archiving families"));
-	for (int i=0; i<conf.subsyss.getnb(); i++){
+	int nbsys = conf.subsyss.getnb();
+	PRIVATE_MESSAGE tbmsg[nbsys];
+	for (int i=0; i<nbsys; i++){
 		dia.newf_str (conf.titles.getitem(i)->get()
 			,*conf.families.getitem(i));
+		dia.set_helpdia (tbmsg[i]);
 	}
 	int ret = -1;
 	int nof = 0;
@@ -411,8 +433,19 @@ static int confver_editone (const char *name)
 				ret = 1;
 				break;
 			}
+		}else if (code == MENU_MESSAGE){
+			for (int i=0; i<nbsys; i++){
+				if (dialog_testmessage(tbmsg[i])){
+					// For now, we present the members of this sub-system
+					subsys_showmembers (conf.subsyss.getitem(i)->get());
+					break;
+				}
+			}
 		}else{
 			conf.write();
+			// Flush the cached values
+			delete curver;
+			curver = NULL;
 			ret = 0;
 			break;
 		}
@@ -438,25 +471,23 @@ void confver_edit()
 	SSTRINGS tbconf;
 	confver_getconfs(tbconf);
 	char mode = confver_getmode();
-	DIALOG *dia = NULL;
+	DIALOG dia;
 	int nof = 0;
 	int start_list = 0;
 	while (1){
-		if (dia == NULL){
-			dia = new DIALOG;
-			dia->newf_chk (MSG_U(F_CONFVERMODE,"Archiving mode"),mode
+		if (dia.getnb() == 0){
+			dia.newf_chk (MSG_U(F_CONFVERMODE,"Archiving mode"),mode
 				,MSG_U(I_AUTO,"Automatic"));
-			int i;
-			dia->newf_title ("",MSG_U(T_CONFIGS,"Configurations"));
-			start_list = dia->getnb();
-			for (i=0; i<tbconf.getnb(); i++){
+			dia.newf_title ("",MSG_U(T_CONFIGS,"Configurations"));
+			start_list = dia.getnb();
+			for (int i=0; i<tbconf.getnb(); i++){
 				const char *conf = tbconf.getitem(i)->get();
 				ONECONF one (conf);
-				dia->new_menuitem (conf,one.title.get());
+				dia.new_menuitem (conf,one.title.get());
 			}
-			dia->addwhat (MSG_U(I_ADDVER,"Select [Add] to create a new configuration tree"));
+			dia.addwhat (MSG_U(I_ADDVER,"Select [Add] to create a new configuration tree"));
 		}
-		MENU_STATUS code = dia->edit (
+		MENU_STATUS code = dia.edit (
 			 MSG_U(T_CONFVER,"Configuration versioning")
 			,MSG_U(I_CONFVER
 				,"You can save/archive all the configuration of this station\n"
@@ -477,6 +508,9 @@ void confver_edit()
 					xconf_error (MSG_U(E_CONFEXIST
 						,"This configuration name already exist\n"
 						 "pick another one"));
+				}else if (strchr(conf,' ')!=NULL){
+					xconf_error (MSG_U(E_NOSPACE
+						,"No space allowed in the configuration name"));
 				}else{
 					linuxconf_setcursys (subsys_subsys);
 					int ok = confver_editone (conf);
@@ -511,11 +545,9 @@ void confver_edit()
 			break;
 		}
 		if (must_delete){
-			delete dia;
-			dia = NULL;
+			dia.remove_all();
 		}
 	}
-	delete dia;
 }
 
 static int subsys_archive (SSTRINGS &t)
@@ -539,7 +571,7 @@ int confver_selectprofile (const char *newver_name)
 	confver_getconfs (tbconf);
 	if (tbconf.lookup(newver_name)==-1){
 		xconf_error (MSG_U(E_NOCONF,"Profile %s does not exist"),newver_name);
-	}else if (strcmp(confver_getcur(),newver_name)==0){
+	}else if (strcmp(confver_loadcur(),newver_name)==0){
 		net_prtlog (NETLOG_VERB,MSG_U(I_ALREADYSEL
 			,"Profile %s is already selected\n"),newver_name);
 		ret = 0;
@@ -589,14 +621,15 @@ void confver_selnewver()
 {
 	SSTRINGS tbconf;
 	confver_getconfs(tbconf);
-	DIALOG dia;
-	const char *cur = confver_getcur();
+	DIALOG_RECORDS dia;
+	dia.newf_head ("",MSG_U(H_SELPROF,"Profile\tTitle"));
+	const char *cur = confver_loadcur();
 	int lookup[tbconf.getnb()];
 	for (int i=0; i<tbconf.getnb(); i++){
 		const char *s = tbconf.getitem(i)->get();
 		if (strcmp(s,cur)!=0){
 			ONECONF one (s);
-			lookup[dia.getnb()] = i;
+			lookup[dia.getnb()-1] = i;
 			dia.new_menuitem (s,one.title.get());
 		}
 	}
