@@ -23,19 +23,19 @@ MENU_STATUS xconf_yesno(
 */
 int xconf_quitwosave()
 {
-	return xconf_yesno(MSG_U(T_QUIT,"Quit without saving ?")
+	return dialog_yesno(MSG_U(T_QUIT,"Quit without saving ?")
 		,MSG_U(I_QUIT,"You have modified the information somewhat\n"
 		 "Are you sure you want to quit ?\n")
-		,help_nil)==MENU_YES;
+		,help_nil,false)==MENU_YES;
 }
 /*
 	Return != 0 if the user confirm something.
 */
 int xconf_areyousure(const char *msg)
 {
-	return xconf_yesno(MSG_U(T_AREYOUSURE,"Are you sure ?")
+	return dialog_yesno(MSG_U(T_AREYOUSURE,"Are you sure ?")
 		,msg
-		,help_nil)==MENU_YES;
+		,help_nil,false)==MENU_YES;
 }
 
 /*
