@@ -10,6 +10,9 @@ PUBLIC M_DIALOG_INTERNAL::M_DIALOG_INTERNAL()
 	fmany = NULL;
 	fenum = NULL;
 	list = NULL;
+	clist = NULL;
+	btext = NULL;
+	bicon = NULL;
 }
 
 static void internal_delete (M_DIALOG_OBJECT *first)
@@ -27,5 +30,8 @@ PUBLIC M_DIALOG_INTERNAL::~M_DIALOG_INTERNAL()
 	internal_delete (fmany);
 	internal_delete (fenum);
 	internal_delete (list);
+	internal_delete (clist);
+	internal_delete (btext);
+	internal_delete (bicon);
 }
 
