@@ -14,6 +14,9 @@ class M_DIALOG_INTERNAL {
 	M_FIELD_COMBO_MANY *fmany;
 	M_FIELD_ENUM *fenum;
 	M_FIELD_LIST *list;
+	M_FIELD_CLIST *clist;
+	M_FIELD_BUTTON_TEXT *btext;
+	M_FIELD_BUTTON_ICON *bicon;
 	/*~PROTOBEG~ M_DIALOG_INTERNAL */
 public:
 	M_DIALOG_INTERNAL (void);
