@@ -14,7 +14,8 @@ class MODULE_conectiva: public LINUXCONF_MODULE{
 public:
 	MODULE_conectiva (void);
 	int dohtml (const char *key);
-	int execmain (int argc, char *argv[]);
+	int execmain (int argc, char *argv[], bool);
+	void usage (SSTRINGS&tb);
     /*~PROTOEND~ MODULE_conectiva */
 };
 
