@@ -2,7 +2,7 @@ extern const char **_dictionary_conectiva;
 #ifndef DICTIONARY_REQUEST
 	#define DICTIONARY_REQUEST \
 	const char **_dictionary_conectiva;\
-	TRANSLATE_SYSTEM_REQ _dictionary_req_conectiva("conectiva",_dictionary_conectiva,26,0);\
+	TRANSLATE_SYSTEM_REQ _dictionary_req_conectiva("conectiva",_dictionary_conectiva,38,0);\
 	void dummy_dict_conectiva(){}
 #endif
 #ifndef MSG_U
@@ -39,3 +39,15 @@ extern const char **_dictionary_conectiva;
 #define F_PKGVER	23
 #define I_NOMANAGERPM	24
 #define B_PKGINFO	25
+#define I_CUSTOMLEVELS	26
+#define I_BASE	27
+#define I_RUNLEVELS	28
+#define I_LEVEL0	29
+#define I_LEVEL1	30
+#define I_LEVEL2	31
+#define I_LEVEL3	32
+#define I_LEVEL4	33
+#define I_LEVEL5	34
+#define I_LEVEL6	35
+#define F_RUNLEVEL	36
+#define I_RECOMMENDED	37
