@@ -16,7 +16,7 @@ public:
 	int dohtml (const char *key);
 	int domenu (MENU_CONTEXT context,
 		 const char *key);
-	int execmain (int argc, char *argv[]);
+	int execmain (int argc, char *argv[], bool);
 	void setmenu (DIALOG&dia, MENU_CONTEXT context);
 	void usage (SSTRINGS&tb);
     /*~PROTOEND~ MODULE_debian */
