@@ -1,5 +1,6 @@
 #!/bin/sh
-if linuxconf --modulemain dhcpd --updatedns
+export PATH=/bin:/usr/bin:/sbin:/usr/sbin
+if linuxconf --silent --modulemain dhcpd --updatedns $@ >/dev/null 2>/dev/null
 then
 	if [ -x /usr/sbin/named.reload ] ; then
 		/usr/sbin/named.reload >/dev/null
