@@ -24,8 +24,7 @@ static CONFIG_FILE f_dhcp (ETC_DHCPD_CONF,help_dhcp
 	 |CONFIGF_SIGNPOUND
 	,subsys_dhcpd);
 CONFIG_FILE f_lease (ETC_DHCPD_LEASES,help_dhcp
-	,CONFIGF_OPTIONNAL|CONFIGF_PROBED
-	,subsys_dhcpd);
+	,CONFIGF_PROBED,subsys_dhcpd);
 
 
 
@@ -56,7 +55,7 @@ PUBLIC void DHCP_OPTION::addvalue(const char *value)
 	values.add (new SSTRING (value));
 }
 
-PUBLIC void DHCP_OPTION::write (const char *indent, FILE *fout)
+PUBLIC void DHCP_OPTION::write (const char *indent, FILE_CFG *fout)
 {
 	if (values.getnb() > 0){
 		fprintf (fout,"%soption %s",indent,keyw.get());
@@ -89,7 +88,7 @@ PUBLIC DHCP_OPTION *DHCP_OPTIONS::getitem (const char *keyw)
 	return ret;
 }
 
-PUBLIC void DHCP_OPTIONS::write (const char *indent, FILE *fout)
+PUBLIC void DHCP_OPTIONS::write (const char *indent, FILE_CFG *fout)
 {
 	for (int i=0; i<getnb(); i++) getitem(i)->write (indent,fout);
 }
@@ -128,16 +127,21 @@ PUBLIC void DHCP_OPTIONS::fill()
 		{"domain-name",		MSG_U(F_DOMAIN,"Domain name"),1,1,NULL},
 		{"host-name",		MSG_U(F_DHHOSTNAME,"Host name"),1,1,NULL},
 
-		{"log-servers",		MSG_U(F_LOGSERVERS,"Log servers"),1,2,NULL},
-		{"lpr-servers",		MSG_U(F_LPRSERVERS,"Print servers"),1,2,NULL},
-		{"time-servers",	MSG_U(F_TIMESERVERS,"Time servers"),1,2,NULL},
-		{"cookie-servers",	MSG_U(F_COOKIESERVERS,"Cookie servers"),1,2,NULL},
-		{"netbios-name-servers",	MSG_U(F_WINS,"Name servers(Netbios)"),3,2,NULL},
-		{"netbios-node-type",	MSG_U(F_NETBIOSNODETYPE,"Node type(Netbios)"),1,2,optnode},
+		{"nis-domain",	MSG_U(F_NISDOMAIN,"NIS domain"),1,2,NULL},
+		{"nis-servers",	MSG_U(F_NISSERVER,"NIS servers"),1,2,NULL},
+
+		{"netbios-name-servers",	MSG_U(F_WINS,"Name servers(Netbios)"),3,3,NULL},
+		{"netbios-node-type",	MSG_U(F_NETBIOSNODETYPE,"Node type(Netbios)"),1,3,optnode},
+
+		{"log-servers",		MSG_U(F_LOGSERVERS,"Log servers"),1,4,NULL},
+		{"lpr-servers",		MSG_U(F_LPRSERVERS,"Print servers"),1,4,NULL},
+		{"time-servers",	MSG_U(F_TIMESERVERS,"Time servers"),1,4,NULL},
+		{"cookie-servers",	MSG_U(F_COOKIESERVERS,"Cookie servers"),1,4,NULL},
+
 		{NULL,NULL, 0}
 	};
-	// Put unknown option in section 2
-	for (int n=0; n<getnb(); n++) getitem(n)->section = 3;
+	// Put unknown option in section 5
+	for (int n=0; n<getnb(); n++) getitem(n)->section = 5;
 	// Known option receive a translated title and a section number
 	// as well as a max value.
 	for (int n=0; tb[n].keyw != NULL; n++){
@@ -196,7 +200,7 @@ PUBLIC DHCP_SUBNET::DHCP_SUBNET()
 {
 }
 
-PUBLIC void DHCP_SUBNET::write (FILE *fout)
+PUBLIC void DHCP_SUBNET::write (FILE_CFG *fout)
 {
 	fprintf (fout,"subnet %s netmask %s{\n",network.get(),netmask.get());
 	for (int i=0; i<ranges.getnb(); i++){
@@ -221,14 +225,14 @@ PUBLIC DHCP_SUBNET *DHCP_SUBNETS::getitem (int no)
 {
 	return (DHCP_SUBNET*)ARRAY::getitem(no);
 }
-PUBLIC void DHCP_SUBNETS::write (FILE *fout)
+PUBLIC void DHCP_SUBNETS::write (FILE_CFG *fout)
 {
 	for (int i=0; i<getnb(); i++) getitem(i)->write (fout);
 }
 
 // Definition for one bootp client
 
-PUBLIC void DHCP_HOST::write (const char *indent, FILE *fout)
+PUBLIC void DHCP_HOST::write (const char *indent, FILE_CFG *fout)
 {
 	fprintf (fout,"%shost %s{\n",indent,ip_host.get());
 	char sindent[10];
@@ -242,7 +246,7 @@ PUBLIC DHCP_HOST* DHCP_HOSTS::getitem (int no)
 	return (DHCP_HOST*)ARRAY::getitem(no);
 }
 
-PUBLIC void DHCP_HOSTS::write (const char *indent, FILE *fout)
+PUBLIC void DHCP_HOSTS::write (const char *indent, FILE_CFG *fout)
 {
 	for (int i=0; i<getnb(); i++){
 		getitem(i)->write(indent,fout);
@@ -251,7 +255,7 @@ PUBLIC void DHCP_HOSTS::write (const char *indent, FILE *fout)
 
 // Group of bootp entries which share a common definition
 
-PUBLIC void DHCP_BOOTP::write (FILE *fout)
+PUBLIC void DHCP_BOOTP::write (FILE_CFG *fout)
 {
 	fputs ("group {\n",fout);
 	options.write ("\t",fout);
@@ -267,12 +271,35 @@ PUBLIC DHCP_BOOTP *DHCP_BOOTPS::getitem (int no)
 static const char K_DHCP[]="DHCP";
 static const char K_UPDDNSDOM[]="upddnsdom";
 static const char K_CRONUPDDNS[]="cronupddns";
+static const char K_VALIDNAME[]="validname";
+
+void dhcp_createlease()
+{
+	if (!f_lease.exist()){
+		const char *path = f_lease.getpath();
+		net_prtlog (NETLOG_WHY
+			,MSG_U(I_CREATELEASE,"File %s missing, must be created\n")
+			,path);
+		if (!simul_ison()){
+			// Create the sub-directory if missing
+			char pathdir[strlen(path)+1];
+			strcpy (pathdir,path);
+			char *last = strrchr(pathdir,'/');
+			if (last != NULL){
+				*last = '\0';
+				file_mkdirp(pathdir,"root","root",0755);
+			}
+			FILE_CFG *fout = f_lease.fopen ("w");
+			fclose (fout);
+		}
+	}
+}
 
 PUBLIC int DHCP::write()
 {
 	int ret = -1;
 	if (!error){
-		FILE *fout = f_dhcp.fopen ("w");
+		FILE_CFG *fout = f_dhcp.fopen ("w");
 		if (fout != NULL){
 			fprintf (fout,"server-identifier %s;\n",identifier.get());
 			if (default_lease_time.seconds != 0){
@@ -284,13 +311,11 @@ PUBLIC int DHCP::write()
 			options.write("",fout);
 			subnets.write(fout);
 			ret = fclose (fout);
-			if (!f_lease.exist()){
-				FILE *fout = f_lease.fopen ("w");
-				fclose (fout);
-			}
+			dhcp_createlease();
 			linuxconf_setcursys (subsys_dhcpd);
 			linuxconf_replace (K_DHCP,K_UPDDNSDOM,dnsdom);
 			linuxconf_replace (K_DHCP,K_CRONUPDDNS,cronupddns);
+			linuxconf_replace (K_DHCP,K_VALIDNAME,validname);
 			linuxconf_save();
 		}
 	}
@@ -513,7 +538,7 @@ static int dhcp_readstop (DHCP_PARSE &par, SSTRING &stop)
 	int ok = dhcp_readone (par,word);
 	stop.setfrom ("");
 	if (ok == 1){
-		if (word[0] == ':') ret = 0;
+		if (word[0] == ';') ret = 0;
 	}else if (ok == 0){
 		if (ipnum_validip (word,false)
 			&& dhcp_readtok(par)==';'){
@@ -644,13 +669,21 @@ const char *dhcp_getdefdom()
 {
 	return linuxconf_getval (K_DHCP,K_UPDDNSDOM,"");
 }
+/*
+	Return the flag telling if only valid name must be exported to the DNS
+*/
+bool dhcp_getvalidname()
+{
+	return linuxconf_getvalnum(K_DHCP,K_VALIDNAME,0) != 0;
+}
 
 PUBLIC DHCP::DHCP()
 {
 	error = 0;
 	dnsdom.setfrom (dhcp_getdefdom());
+	validname = dhcp_getvalidname() ? 1 : 0;
 	cronupddns = linuxconf_getvalnum (K_DHCP,K_CRONUPDDNS,0);
-	FILE *fin = f_dhcp.fopen ("r");
+	FILE_CFG *fin = f_dhcp.fopen ("r");
 	if (fin != NULL){
 		DHCP_PARSE par;
 		par.fin = fin;
@@ -671,13 +704,15 @@ PUBLIC void DHCP_OPTIONS::setdia (DIALOG &dia)
 	static const char *tbsect[]={
 		MSG_U(T_BASIC,"Basic"),
 		MSG_U(T_DNS,"Dns"),
+		MSG_U(T_NIS,"NIS"),
+		MSG_U(T_METBIOS,"Netbios"),
 		MSG_U(T_SERVERS,"Servers"),
 		MSG_U(T_OTHERS,"Others"),
 	};
-	int nbsect = 3;
+	int nbsect = 5;
 	for (int i=0; i<getnb(); i++){
-		if (getitem(i)->section == 3){
-			nbsect = 4;
+		if (getitem(i)->section == 6){
+			nbsect = 5;
 			break;
 		}
 	}
@@ -686,6 +721,7 @@ PUBLIC void DHCP_OPTIONS::setdia (DIALOG &dia)
 		for (int i=0; i<getnb(); i++){
 			DHCP_OPTION *o = getitem(i);
 			if (o->section == s){
+				o->nofield = dia.getnb();
 				for (int n=o->values.getnb(); n <o->maxvalues; n++){
 					o->addvalue ("");
 				}
@@ -706,6 +742,56 @@ PUBLIC void DHCP_OPTIONS::setdia (DIALOG &dia)
 	}
 }
 
+static void dhcp_valid_net (
+	SSTRING &ip,
+	SSTRING &net,
+	int nofield,
+	int &new_nofield)
+{
+	if (new_nofield == -1){
+		const char *s_ip = ip.get();
+		const char *s_mask = net.get();
+		if (!ipnum_validnet (s_ip,s_mask)){
+			unsigned long b_ip = ipnum_aip2l (s_ip);
+			unsigned long b_mask = ipnum_aip2l (s_mask);
+			unsigned long b_newnet = b_ip & b_mask;
+			char hint[LEN_IP_ASC+1];
+			ipnum_ip2a (b_newnet,hint);
+			new_nofield = nofield;
+			xconf_error (MSG_U(E_IVLDNETWORKIP
+				,"Invalid Network number %s for netmask %s\n"
+				 "t\t\tHint: %s")
+				,ip.get(),net.get(),hint); 
+		}
+	}
+}
+
+static void dhcp_valid_range (
+	SSTRING &net,
+	SSTRING &mask,
+	SSTRING &ip,
+	int nofield,
+	int &new_nofield)
+{
+	if (new_nofield == -1){
+		unsigned long bnetwork = ipnum_aip2l (net.get());
+		unsigned long bnetmask = ipnum_aip2l (mask.get());
+		unsigned long bipaddr  = ipnum_aip2l (ip.get());
+		unsigned long network  = bipaddr & bnetmask; 
+
+		if (network != bnetwork){
+			new_nofield = nofield;
+			xconf_error (MSG_U(E_IVLDNETHOSTIP
+				,"Invalid IP range %s for network %s")
+				,ip.get(),net.get());
+		}
+	}
+}
+
+
+/*
+	Check if an IP number is valid
+*/
 static void dhcp_checkip (
 	SSTRING &ip,
 	bool ishost,
@@ -731,6 +817,25 @@ static void dhcp_checkip (
 	}
 }
 
+PUBLIC void DHCP_OPTIONS::validate (int &new_nof)
+{
+	for (int i=0; i<getnb(); i++){
+		DHCP_OPTION *o = getitem(i);
+		if (o->keyw.cmp("routers")==0
+			|| o->keyw.cmp("domain-name-servers")==0
+			|| o->keyw.cmp("netbios-name-servers")==0){
+			for (int j=0; j<o->values.getnb(); j++){
+				SSTRING *ss = o->values.getitem(j);
+				dhcp_checkip (*ss,true,true,o->nofield+j,new_nof);
+			}
+		}else if (o->keyw.cmp("subnet-mask")==0){
+			SSTRING *ss = o->values.getitem(0);
+			dhcp_checkip (*ss,false,true,o->nofield,new_nof);
+		}
+	}
+}
+
+
 PUBLIC int DHCP_SUBNET::edit()
 {
 	int ret = -1;
@@ -756,7 +861,9 @@ PUBLIC int DHCP_SUBNET::edit()
 	int nof = 0;
 	while (1){
 		MENU_STATUS code = dia.edit (MSG_U(T_ONESUB,"One subnet definition")
-			,MSG_U(I_ONESUB,"")
+			,MSG_U(I_ONESUB
+				,"You control here the information for\n"
+				 "one network. Anything left blank use the defaults.")
 			,help_dhcp
 			,nof
 			,MENUBUT_DEL|MENUBUT_CANCEL|MENUBUT_ACCEPT);
@@ -771,18 +878,34 @@ PUBLIC int DHCP_SUBNET::edit()
 			int new_nof = -1;
 			dhcp_checkip (network,false,false,0,new_nof);
 			dhcp_checkip (netmask,false,false,1,new_nof);
+			dhcp_valid_net(network,netmask,0,new_nof); // Added to check the whether user entered a valid
+								//  network address for the mask which he/she entered.
 			for (int i=0; i<ranges.getnb() && new_nof == -1; i++){
 				DHCP_RANGE *r = ranges.getitem(i);
-				bool may_be_empty = i!=0 && r->alloc_stop.is_empty();
-				dhcp_checkip (r->alloc_start,true,may_be_empty
-					,field_range+i*3,new_nof);
-				dhcp_checkip (r->alloc_stop,true,true
-					,field_range+1+i*3,new_nof);
+				// The first range must be filled
+				if (r->alloc_start.is_filled()
+					|| r->alloc_stop.is_filled()
+					|| i == 0){
+					int nofield = field_range + i*3 +1;
+					dhcp_checkip (r->alloc_start,true,false
+						,nofield,new_nof);
+					dhcp_valid_range (network,netmask,r->alloc_start
+						,nofield,new_nof);
+					if (r->alloc_stop.is_filled()){
+						nofield++;
+						dhcp_checkip (r->alloc_stop,true,true
+							,nofield,new_nof);
+						dhcp_valid_range (network,netmask,r->alloc_stop
+							,nofield,new_nof);
+					}
+				}
 			}
-			char dev[20];
+			options.validate(new_nof);
+
 			if (new_nof != -1){
 				nof = new_nof;
 			}else{
+				char dev[20];
 				if (netconf_finddevfromnet(network.get(),netmask.get()
 					,dev)== -1){
 					xconf_notice (MSG_U(E_NOMATCH
@@ -814,7 +937,7 @@ PUBLIC int DHCP_SUBNET::edit()
 	Edit default values which apply to
 	Return -1 if the changes were not accepted.
 */
-PRIVATE int DHCP::editdefaults()
+PUBLIC int DHCP::editdefaults()
 {
 	int ret = -1;
 	DIALOG dia;
@@ -828,6 +951,7 @@ PRIVATE int DHCP::editdefaults()
 		dia.newf_str (MSG_U(F_DEFDNSDOM,"Update DNS domain"),dnsdom);
 		dia.newf_chk (MSG_U(F_UPDDNS,"Update DNS"),cronupddns
 			,MSG_U(I_UPDDNS,"from cron"));
+		dia.newf_chk ("",validname,MSG_U(I_VALIDNAMEONLY,"Export only valid names"));
 	}		
 	options.setdia (dia);
 	int nof = 0;
@@ -847,18 +971,24 @@ PRIVATE int DHCP::editdefaults()
 				 "unless you provide the domain in which to\n"
 				 "register hosts."));
 		}else{
-			if (dns_api_ok){
-				// Update the crontab
-				const char *cmd = "/usr/lib/linuxconf/lib/dhcp2dns.sh";
-				if (cronupddns){
-					cron_addcmd ("root",cmd
-						,"0-59/5","*","*","*","*");
-				}else{
-					cron_delcmd ("root",cmd);
+			int new_nof = -1;
+			options.validate (new_nof);
+			if (new_nof != -1){
+				nof = new_nof;
+			}else{
+				if (dns_api_ok){
+					// Update the crontab
+					const char *cmd = "/usr/lib/linuxconf/lib/dhcp2dns.sh";
+					if (cronupddns){
+						cron_addcmd ("root",cmd
+							,"0-59/5","*","*","*","*");
+					}else{
+						cron_delcmd ("root",cmd);
+					}
 				}
+				ret = 0;
+				break;
 			}
-			ret = 0;
-			break;
 		}
 	}
 	if (ret != 0) dia.restore();
@@ -886,10 +1016,16 @@ PUBLIC void DHCP::editloop()
 		for (i=0; i<subnets.getnb(); i++){
 			DHCP_SUBNET *s = subnets.getitem(i);
 			DHCP_RANGE *r = s->ranges.getitem(0);
+			const char *start = "";
+			const char *stop = "";
+			if (r != NULL){
+				start = r->alloc_start.get();
+				stop  = r->alloc_stop.get();
+			}
 			char buf[100];
 			sprintf (buf,"%s\t%s\t\t%s"
 				,s->netmask.get()
-				,r->alloc_start.get(),r->alloc_stop.get());
+				,start,stop);
 			dia.new_menuitem (s->network.get(),buf);
 		}
 		int stop_subnets = dia.getnb();
@@ -944,7 +1080,7 @@ PUBLIC void DHCP::editloop()
 		}else if (dia.getmenustr(nof)==upddns){
 		  //hubert@id-pro.de: care about the ranges
 		  UWE_RANGES ranges(*this);
-		  leases_updatedns (dns_api,false,false,&ranges);
+		  leases_updatedns (dns_api,false,false,&ranges,f_lease.getpath(),dhcp_getdefdom());
 		}else if (dia.getmenustr(nof)==viewleases){
 			leases_showalloc ();
 		}else if (nof >= start_subnets && nof < stop_subnets){
@@ -970,7 +1106,7 @@ PUBLIC void DHCP::editloop()
 /*
 	Command line version to update the dns from the dhcpd.leases
 */
-int dhcpd_updatedns(bool verbose)
+int dhcpd_updatedns(bool verbose, const char *fname, const char *domain)
 {
 	int ret = -1;
 	DNSCONF_API *dns_api = dnsconf_api_init("dhcpd");
@@ -982,7 +1118,7 @@ int dhcpd_updatedns(bool verbose)
 	  //hubert@id-pro.de: care about the ranges
 	  DHCP dh;
 	  UWE_RANGES ranges(dh);
-	  ret = leases_updatedns (dns_api,true,verbose,&ranges);
+	  ret = leases_updatedns (dns_api,true,verbose,&ranges,fname,domain);
 	}
 	dnsconf_api_end(dns_api);
 	return ret;
@@ -1059,8 +1195,9 @@ PUBLIC int DHCP::probe()
 		*ptr = '\0';
 		net_hint ("DEVICES",devices);
 		net_hint ("ROUTES",routes);
+		dhcp_createlease();
 		DAEMON_INTERNAL *dae = daemon_find ("dhcpd");
-		if (dae != NULL){
+		if (dae != NULL && dae->is_managed()){
 			char cmdargs[100];
 			sprintf (cmdargs,"%s 2>&1",devices);
 			dae->setargs (cmdargs);
@@ -1093,3 +1230,106 @@ int dhcp_probe()
 	return ret;
 }
 
+PUBLIC void DHCP::editsub (DHCP_SUBNET *sub, bool isnew)
+{
+	int ok = sub->edit();
+	if (ok >= 0){
+		if (ok == 1){
+			subnets.remove_del (sub);
+		}else{
+			if (isnew) subnets.add (sub);
+		}
+		write ();
+	}else if (isnew){
+		delete sub;
+	}
+}
+
+PUBLIC void DHCP::locate (const char *net, bool setting)
+{
+	bool found = false;
+	for (int i=0; i<subnets.getnb(); i++){
+		DHCP_SUBNET *sub = subnets.getitem(i);
+		if (sub->network.cmp(net)==0){
+			editsub (sub,false);
+			found = true;
+			break;
+		}
+	}
+	if (!found && setting){
+		DHCP_SUBNET *sub = new DHCP_SUBNET;
+		editsub (sub,true);
+	}
+}
+
+static void dhcp_locate (const char *net, bool setting)
+{
+	DHCP dh;
+	dh.locate (net,setting);
+}
+
+static void dhcp_editdefaults()
+{
+	DHCP dh;
+	if (dh.editdefaults()==0) dh.write();
+
+}
+
+
+#include <modregister.h>
+static REGISTER_VARIABLE_LOOKUP_MSG dhcp_var_list[]={
+	// Variables for the default
+	{"serverid",NULL,P_MSG_R(F_IDENT),dhcp_editdefaults,NULL},
+	{"defleasetime",NULL,P_MSG_R(F_DEFLEASETIME),dhcp_editdefaults,NULL},
+	{"maxleasetime",NULL,P_MSG_R(F_MAXLEASETIME),dhcp_editdefaults,NULL},
+	{"upddnsdom",NULL,P_MSG_R(F_DEFDNSDOM),dhcp_editdefaults,NULL},
+	{"upddnscron",NULL,P_MSG_R(F_UPDDNS),dhcp_editdefaults,NULL},
+	{"dnsvalidname",NULL,P_MSG_R(I_VALIDNAMEONLY),dhcp_editdefaults,NULL},
+
+	// Variables common to the default and subnets
+	{"routers",NULL,P_MSG_R(F_ROUTERS),dhcp_editdefaults,NULL},
+	{"subnet-mask",NULL,P_MSG_R(F_SUBNETMASK),dhcp_editdefaults,NULL},
+	{"time-offset",NULL,P_MSG_R(F_TIMEOFFSET),dhcp_editdefaults,NULL},
+	{"domain-name-servers",NULL,P_MSG_R(F_DNS),dhcp_editdefaults,NULL},
+	{"domain-name",NULL,P_MSG_R(F_DOMAIN),dhcp_editdefaults,NULL},
+	{"host-name",NULL,P_MSG_R(F_DHHOSTNAME),dhcp_editdefaults,NULL},
+	{"nis-domain",NULL,P_MSG_R(F_NISDOMAIN),dhcp_editdefaults,NULL},
+	{"nis-servers",NULL,P_MSG_R(F_NISSERVER),dhcp_editdefaults,NULL},
+	{"netbios-name-servers",NULL,P_MSG_R(F_WINS),dhcp_editdefaults,NULL},
+	{"netbios-node-type",NULL,P_MSG_R(F_NETBIOSNODETYPE),dhcp_editdefaults,NULL},
+	{"log-servers",NULL,P_MSG_R(F_LOGSERVERS),dhcp_editdefaults,NULL},
+	{"lpr-servers",NULL,P_MSG_R(F_LPRSERVERS),dhcp_editdefaults,NULL},
+	{"time-servers",NULL,P_MSG_R(F_TIMESERVERS),dhcp_editdefaults,NULL},
+	{"cookie-servers",NULL,P_MSG_R(F_COOKIESERVERS),dhcp_editdefaults,NULL},
+
+	// Variables for one subnet
+	{"network",NULL,P_MSG_R(F_NETWORK),NULL,dhcp_locate},
+	{"netmask",NULL,P_MSG_R(F_NETMASK),NULL,dhcp_locate},
+	{"defleasetime",NULL,P_MSG_R(F_DEFLEASETIME),NULL,dhcp_locate},
+	{"maxleasetime",NULL,P_MSG_R(F_MAXLEASETIME),NULL,dhcp_locate},
+	{"dynbootp",NULL,P_MSG_R(F_DYNBOOTP),NULL,dhcp_locate},
+	{"startip",NULL,P_MSG_R(F_STARTIP),NULL,dhcp_locate},
+	{"stopip",NULL,P_MSG_R(F_STOPIP),NULL,dhcp_locate},
+	// The same variables are repeated for the subnet and the default
+	{"routers",NULL,P_MSG_R(F_ROUTERS),NULL,dhcp_locate},
+	{"subnet-mask",NULL,P_MSG_R(F_SUBNETMASK),NULL,dhcp_locate},
+	{"time-offset",NULL,P_MSG_R(F_TIMEOFFSET),NULL,dhcp_locate},
+	{"domain-name-servers",NULL,P_MSG_R(F_DNS),NULL,dhcp_locate},
+	{"domain-name",NULL,P_MSG_R(F_DOMAIN),NULL,dhcp_locate},
+	{"host-name",NULL,P_MSG_R(F_DHHOSTNAME),NULL,dhcp_locate},
+	{"nis-domain",NULL,P_MSG_R(F_NISDOMAIN),NULL,dhcp_locate},
+	{"nis-servers",NULL,P_MSG_R(F_NISSERVER),NULL,dhcp_locate},
+	{"netbios-name-servers",NULL,P_MSG_R(F_WINS),NULL,dhcp_locate},
+	{"netbios-node-type",NULL,P_MSG_R(F_NETBIOSNODETYPE),NULL,dhcp_locate},
+	{"log-servers",NULL,P_MSG_R(F_LOGSERVERS),NULL,dhcp_locate},
+	{"lpr-servers",NULL,P_MSG_R(F_LPRSERVERS),NULL,dhcp_locate},
+	{"time-servers",NULL,P_MSG_R(F_TIMESERVERS),NULL,dhcp_locate},
+	{"cookie-servers",NULL,P_MSG_R(F_COOKIESERVERS),NULL,dhcp_locate},
+	{ NULL, NULL, NULL, NULL }
+};
+
+static REGISTER_VARIABLES host_registry1("dhcpd",dhcp_var_list);
+
+
+
+
