@@ -16,6 +16,8 @@
 #include <sstream.h>
 #include <dialog.h>
 #include <module_apis/fwinfo_apidef.h>
+#include <module_apis/devconf_api.h>
+#include <private_msg.h>
 
 
 HELP_FILE help_ppp ("dialout","ppp");
@@ -73,10 +75,10 @@ void serial_setfield (
 {
 	FIELD_COMBO *comb = dia.newf_combo  (MSG_U(F_SERIALDEF
 		,"Serial device"),device);
-	comb->addopt ("/dev/cua0",MSG_U(F_FIRSTCOM,"first standard serial port COM1"));
-	comb->addopt ("/dev/cua1",MSG_U(F_SECONDCOM,"Second standard serial port COM2"));
-	comb->addopt ("/dev/ttyS0",MSG_U(F_COM1M,"COM1 when using mgetty"));
-	comb->addopt ("/dev/ttyS1",MSG_U(F_COM2M,"COM2 when using mgetty"));
+	comb->addopt ("/dev/ttyS0",MSG_U(F_COM1M,"COM1"));
+	comb->addopt ("/dev/ttyS1",MSG_U(F_COM2M,"COM2"));
+	comb->addopt ("/dev/ttyS2",MSG_U(F_COM3M,"COM3"));
+	comb->addopt ("/dev/ttyS3",MSG_U(F_COM4M,"COM4"));
 }
 
 /*
@@ -109,7 +111,12 @@ PRIVATE int PPPONE::lcpecho_valid()
 
 PRIVATE int PPPONE::validate_edit (
 	SSTRING &baudstr,
-	SSTRING &oldname)
+	SSTRING &oldname,
+	int &nof,
+	int field_name,
+	int field_device,
+	int field_lcpecho,
+	int field_ssh)
 {
 	char buferr[2000];
 	buferr[0] = '\0';
@@ -117,6 +124,7 @@ PRIVATE int PPPONE::validate_edit (
 	if (name.strchr(' ')!=NULL){
 		strcpy (buferr
 			,MSG_U(E_NOSPACE,"No spaces allowed in the configuration name\n"));
+		nof = 0;
 	}else if (!oldname.is_empty() && name.cmp(oldname)!=0){
 		char path[PATH_MAX];
 		sprintf (path,"%s/%s.dconf",ETC_PPP,name.get());
@@ -126,11 +134,13 @@ PRIVATE int PPPONE::validate_edit (
 				 "already exist\n"
 				 "Use another name\n\n")
 				,name.get());
+			nof = 0;
 		}
 	}
 	if (name.is_empty()){
 		strcat (buferr,MSG_U(E_NEEDNAME,"You must supply\n"
 			"a configuration name\n\n"));
+		nof = 0;
 	}
 	if (device.is_empty()
 		&& type != TYPE_PPP_SSH
@@ -138,10 +148,12 @@ PRIVATE int PPPONE::validate_edit (
 		&& type != TYPE_PLIP2){
 		strcat (buferr,MSG_U(E_NEEDDEV,"You must supply "
 			"a device (/dev/xxxx)\n\n"));
+		nof = field_device;
 	}
 	if (!lcpecho_valid ()){
 		strcat (buferr,MSG_U(E_LCPECHO,"You must supply a value pair\n"
 			"    delay,retry for the LCP echo protocol"));
+		nof = field_lcpecho;
 	}
 	if (type == TYPE_PPP_SSH){
 		#if 0
@@ -157,6 +169,7 @@ PRIVATE int PPPONE::validate_edit (
 			strcat (buferr,MSG_U(E_SSHSECT
 				,"You must fill all fields (except encryption)\n"
 				 "of the PPP over SSH section\n"));
+			nof = field_ssh;
 		}
 	}
 	int ret = 0;
@@ -179,12 +192,36 @@ PRIVATE void PPPONE::setdia_iproute (DIALOG &dia, int no)
 	
 static char privi_changed = 0;
 
+/*
+	Trigger a probe of a modem
+*/
+static int ppp_probeserial (SSTRING &dev)
+{
+	int ret = -1;
+	DEVCONF_API *tbapi[MAX_API_PROVIDERS];
+	int nbapi = devconf_apis_init ("gurus/network",tbapi);
+	for (int i=0; i<nbapi; i++){
+		SSTRINGS tbs;
+		if (tbapi[i]->probe (DEVCONF_MODEM,tbs)>0){
+			ret = 0;
+			dev.setfrom (tbs.getitem(0)->get());
+			break;
+		}
+	}
+	devconf_apis_end(tbapi,nbapi);
+	if (ret == -1){
+		xconf_error (MSG_U(E_NOMODEM,"No modem found"));
+	}
+	return ret;
+}
+
 PUBLIC int PPPONE::edit ()
 {
 	int ret = -1;
 	DIALOG dia;
 	SSTRING oldname (name);	
 	dia.newf_title (MSG_U(N_BASE,"Base info"),1,"",MSG_R(N_BASE));
+	int field_name = dia.getnb();
 	dia.newf_str  (MSG_U(F_PPPNAME,"Configuration name"),name);
 	dia.newf_radio (MSG_U(F_CONNECTTYPE,"Connection type"),type
 		,TYPE_PPP_SERIAL
@@ -214,7 +251,10 @@ PUBLIC int PPPONE::edit ()
 	dia.newf_str (MSG_U(F_OURIP,"Our IP number(opt)"),ourip);
 	dia.newf_str (MSG_U(F_REMOTEIP,"Remote IP number(opt)"),remoteip);
 	dia.newf_str  (MSG_U(F_LOGIN,"login"),login);
-	dia.newf_str  (MSG_U(F_PASSWORD,"Password"),passwd);
+	SSTRING newpasswd,newpasswd2;
+	int field_newpass = dia.getnb();
+	dia.newf_pass  (MSG_U(F_PASSWORD,"Password"),newpasswd);
+	dia.newf_pass  (MSG_U(F_PASSWORD2,"Password (Confirm)"),newpasswd2);
 
 	dia.newf_title (MSG_U(T_LOGINCHAT,"Chat"),1,"",MSG_R(T_LOGINCHAT));
 
@@ -229,7 +269,10 @@ PUBLIC int PPPONE::edit ()
 	dia.newf_title (MSG_U(N_MODEM,"Modem"),1,""
 		,MSG_U(T_MODEM,"Modem specifications"));
 
+	int field_device = dia.getnb();
 	serial_setfield (device,dia);
+	PRIVATE_MESSAGE msg_probe;
+	dia.new_button (MSG_U(B_PROBE,"Detect modem"),"",msg_probe);
 
 	dia.newf_str (MSG_U(F_MODEMCHAT,"Modem initialisation"),modeminit);
 	dia.newf_str  (MSG_U(F_PHONE,"Phone"),phone);
@@ -249,6 +292,7 @@ PUBLIC int PPPONE::edit ()
 	dia.newf_chk ("",dbgppp,MSG_U(F_RUNPPPD,"Run pppd in debug mode"));
 	dia.newf_chk ("",dbgchat,MSG_U(F_RUNCHAT,"Run chat in debug mode"));
 	dia.newf_str (MSG_U(F_ASYNCMAP,"Async map"),asyncmap);
+	int field_lcpecho = dia.getnb();
 	FIELD_COMBO *cecho = dia.newf_combo (MSG_U(F_LCPECHO,"LCP echo"),lcpecho);
 	cecho->addopt ("",MSG_U(F_NOLCPECHO,"Don't use LCP echo protocol"));
 	cecho->addopt ("15,3",MSG_U(F_ECHO15_3,"Send lcp echo every 15, hangup after 3 failures"));
@@ -274,6 +318,7 @@ PUBLIC int PPPONE::edit ()
 
 	dia.newf_title (MSG_U(N_PPPSSH,"SSH"),1,""
 		,MSG_U(T_PPPSSH,"PPP over SSH"));
+	int field_ssh = dia.getnb();
 	dia.newf_str   (MSG_U(F_SSHHOST,"Ssh server"),ssh.host);
 	dia.newf_str   (MSG_U(F_SSHUSER,"Account"),ssh.user);
 	static const char *tbacctype[]={
@@ -314,11 +359,14 @@ PUBLIC int PPPONE::edit ()
 	for (int r=0; r<iproutes.getnb(); r++){
 		setdia_iproute (dia,r);
 	}
+	PRIVATE_MESSAGE growmsg;
+	dia.new_button (MSG_U(B_GROWFORM,"Grow form")
+		,MSG_U(I_GROWFORM,"Add more lines to enter IP routes")
+		,growmsg);
 	int no = 0;
 	int extra_but = name.is_empty() ? 0 : MENUBUT_DEL;
-	dia.setbutinfo (MENU_USR1,MSG_U(B_CONNECT,"Connect")
+	dia.setbutinfo (MENU_USR1,MSG_U(B_CONNECT,"Connection")
 		,MSG_U(X_CONNECT,"Connect"));
-	
 	while (1){
 		char title[100];
 		sprintf (title,MSG_U(T_ONEPPPCONF,"%s PPP/SLIP configuration")
@@ -327,27 +375,50 @@ PUBLIC int PPPONE::edit ()
 			,MSG_U(I_ONEPPPCONF,"One PPP, SLIP or PLIP configuration")
 			,help_ppp
 			,no
-			,extra_but|MENUBUT_ACCEPT|MENUBUT_CANCEL|MENUBUT_USR1|MENUBUT_ADD);
+			,extra_but|MENUBUT_ACCEPT|MENUBUT_CANCEL|MENUBUT_USR1);
 		if (code == MENU_CANCEL || code == MENU_ESCAPE){
 			break;
-		}else if (code == MENU_ADD){
-			int r = iproutes.getnb();
-			iproutes.add (new PPPIPROUTE);
-			setdia_iproute (dia,r);
 		}else if (code == MENU_USR1){
 			dia.save();
-			if (validate_edit(baudstr,oldname) != -1){
-				connect (0);
+			if (validate_edit(baudstr,oldname,no,field_name
+				,field_device,field_lcpecho,field_ssh) != -1){
+				control ();
+			}
+		}else if (code == MENU_MESSAGE){
+			if (dialog_testmessage(msg_probe)){
+				if (ppp_probeserial (device)!=-1){
+					dia.reload (field_device);
+				}
+			}else if (dialog_testmessage(growmsg)){
+				// Remove the button
+				dia.remove_del (dia.getnb()-1);	// Remove the button
+				int r = iproutes.getnb();
+				iproutes.add (new PPPIPROUTE);
+				setdia_iproute (dia,r);
+				// Put the button back
+				dia.new_button (MSG_R(B_GROWFORM),MSG_R(I_GROWFORM)
+					,growmsg);		// Put it back
 			}
 		}else if (code == MENU_ACCEPT){
-			if (validate_edit(baudstr,oldname) != -1){
+			if (!newpasswd.is_empty() && newpasswd.cmp(newpasswd2)!=0){
+				xconf_error (MSG_U(E_PASSMATCH
+					,"Passwords do not match\n"
+					 "Please reenter the password twice"));
+				no = field_newpass;
+				newpasswd.setfrom ("");
+				newpasswd2.setfrom("");
+				dia.reload (no);
+				dia.reload (no+1);
+			}else if (validate_edit(baudstr,oldname,no,field_name
+				,field_device,field_lcpecho,field_ssh) != -1){
 				del(oldname.get());
+				if (!newpasswd.is_empty()) passwd.setfrom (newpasswd);
 				save ();
 				ret = 0;
 				break;
 			}
 		}else if (code == MENU_DEL){
-			if (xconf_yesno(MSG_U(Q_DELETED,"Are you sure")
+			if (dialog_yesno(MSG_U(Q_DELETED,"Are you sure")
 				,MSG_U(I_DELETED,"This configuration will be\n"
 				 "deleted permanently")
 				,help_ppp)){
@@ -533,16 +604,37 @@ int ppp_connect (int argc, char *argv[])
 	return ret;
 }
 
+static int ppp_disconnect(const char *config)
+{
+	int ret = -1;
+	PPPONE *one = ppp_getconfig (config);
+	if (one != NULL){
+		if (one->getpppd_pid()!=-1){
+			ret = one->disconnect ();
+		}
+		delete one;
+	}
+	return ret;
+}
+
+
 /*
 	Kill a PPP/SLIP connnection
 */
 int ppp_disconnect (int argc, char *argv[])
 {
 	int ret = -1;
-	PPPONE *one = ppp_getconfig (argc,argv);
-	if (one != NULL){
-		ret = one->disconnect ();
-		delete one;
+	for (int i=0; i<argc; i++){
+		const char *config = argv[i];
+		if (strcmp(config,"--all")==0){
+			SSTRINGS lst;
+			int nb = ppp_getlist (lst);
+			for (int j=0; j<nb; j++){
+				ret |= ppp_disconnect (lst.getitem(j)->get());
+			}
+		}else{
+			ret |= ppp_disconnect (config);
+		}
 	}
 	return ret;
 }
@@ -612,6 +704,34 @@ PUBLIC PPPONE *PPPONES::getitem(int no)
 }
 
 #endif
+/*
+	Let the user start or stop the link
+*/
+PUBLIC void PPPONE::control()
+{
+	if (getpppd_pid() == -1){
+		if (dialog_yesno(MSG_U(Q_DOCONN,"Activate PPP link")
+			,MSG_U(I_DOCONN
+				,"Do you want to activate the network link ?")
+			,help_nil)==MENU_YES){
+			connect(0);
+		}
+	}else{
+		if (dialog_yesno(MSG_U(Q_DODISCONN,"Terminate PPP link")
+			,MSG_U(I_DODISCONN
+				,"Do you want to terminate the network link ?")
+			,help_nil)==MENU_YES){
+			disconnect();
+			for (int i=0; i<4; i++){
+				sleep(1);
+				if (getpppd_pid() == -1){
+					break;
+				}
+			}
+		}
+	}
+}
+
 /*
 	Control panel allowing connection and disconnection of PPP dialout
 */
@@ -620,7 +740,6 @@ void ppp_control ()
 	int choice = 0;
 	DIALOG_LISTE dia;
 	while (1){
-		dia.remove_all();
 		SSTRINGS lst;
 		int nb = ppp_setupdia(dia,lst);
 		MENU_STATUS code = dia.editmenu(
@@ -637,27 +756,7 @@ void ppp_control ()
 			PRIVILEGE *privi = ppp_lookuppriv (sel);
 			if (perm_access(privi,MSG_U(P_CTLDIALOUT,"control dialout"))){
 				PPPONE one (sel);
-				if (one.getpppd_pid() == -1){
-					if (dialog_yesno(MSG_U(Q_DOCONN,"Activate PPP link")
-						,MSG_U(I_DOCONN
-							,"Do you want to activate the network link ?")
-						,help_nil)==MENU_YES){
-						one.connect(0);
-					}
-				}else{
-					if (dialog_yesno(MSG_U(Q_DODISCONN,"Terminate PPP link")
-						,MSG_U(I_DODISCONN
-							,"Do you want to terminate the network link ?")
-						,help_nil)==MENU_YES){
-						one.disconnect();
-						for (int i=0; i<4; i++){
-							sleep(1);
-							if (one.getpppd_pid() == -1){
-								break;
-							}
-						}
-					}
-				}
+				one.control();
 			}
 		}
 	}
@@ -679,6 +778,10 @@ void ppp_bootcleanup ()
 		const char *name = s->get();
 		PPPONE one (name);
 		one.delpidfile();
+		// Remove the control file for "on boot" configuration
+		char pathwatch[PATH_MAX];
+		sprintf (pathwatch,"%s.%s",VAR_RUN_PPPRESTART,name);
+		unlink (pathwatch);
 	}
 }
 
@@ -786,33 +889,46 @@ static int ppp_getdevlist  (SSTRINGS &devs, SSTRINGS &descs)
 	}
 	return n;
 }
-static int ppp_getdevinfo  (const char *devname, SSTRING &kerneldev)
+static int ppp_getdevinfo  (const char *devname, SSTRINGS &kerneldevs)
 {
 	int ret = -1;
 	if (strncmp(devname,"dialout/",8)==0){
 		PPPONE one (devname+8);
 		char pppdev[8];
 		if (one.getpppd_info (pppdev) != -1){
-			kerneldev.setfrom (pppdev);
+			kerneldevs.add (new SSTRING(pppdev));
 			ret = 0;
 		}
 	}
 	return ret;
 }
 
-// THe dialout module does not define any logical HOST
+/*
+	A logical host correspond to the local IP of the ppp connection
+*/
 static bool ppp_hostexist (const char *hostname)
 {
-	return false;
+	return ppp_devexist(hostname);
 }
 
-static int ppp_gethostlist  (SSTRINGS &, SSTRINGS &)
+static int ppp_gethostlist  (SSTRINGS &devs, SSTRINGS &descs)
 {
-	return 0;
+	return ppp_getdevlist (devs,descs);
 }
-static int ppp_gethostinfo  (const char *, SSTRING &)
+static int ppp_gethostinfo  (const char *host, SSTRINGS &tb)
 {
-	return -1;
+	int ret = -1;
+	if (strncmp(host,"dialout/",8)==0){
+		PPPONE one (host+8);
+		char pppdev[8];
+		IFCONFIG_INFO info;
+		if (one.getpppd_info (pppdev) != -1
+			&& ifconfig_getinfo (pppdev,info)!=-1){
+			tb.add (new SSTRING(info.ip_addr));
+			ret = 0;
+		}
+	}
+	return ret;
 }
 
 void *dialout_fwinfo_api_get()
@@ -831,3 +947,30 @@ void dialout_fwinfo_api_release(void *api)
 {
 	delete (FWINFO_API*)api;
 }
+
+
+static void ppp_editone(const char *config, bool setting)
+{
+	char path[PATH_MAX];
+	sprintf (path,"%s/%s.dconf",ETC_PPP,config);
+	if (file_exist(path) || setting){
+		PPPONE one (config);
+		one.edit();
+	}
+}
+
+#include <modregister.h>
+static REGISTER_VARIABLE_LOOKUP_MSG dialout_var_list[]={
+	{"phone",NULL,P_MSG_R(F_PHONE),NULL,ppp_editone},
+	{"device",NULL,P_MSG_R(F_SERIALDEF),NULL,ppp_editone},
+	{"login",NULL,P_MSG_R(F_LOGIN),NULL,ppp_editone},
+	{"password",NULL,P_MSG_R(F_PASSWORD),NULL,ppp_editone},
+	{"connectmode",NULL,P_MSG_R(F_PPPAUTH),NULL,ppp_editone},
+	{ NULL, NULL, NULL, NULL }
+};
+
+static REGISTER_VARIABLES dialout_registry("dialout",dialout_var_list);
+
+
+
+
