@@ -23,11 +23,11 @@ static HELP_FILE help_pppdev("dialout","pppdev");
 static CONFIG_FILE f_ipup (ETC_PPP_IPUP
 	,help_ipup
 	,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
-	,"root","root",0700,ppp_dialout);
+	,"root","root",0755,ppp_dialout);
 static CONFIG_FILE f_ipdown (ETC_PPP_IPDOWN
 	,help_ipup
 	,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
-	,"root","root",0700,ppp_dialout);
+	,"root","root",0755,ppp_dialout);
 
 static CONFIG_FILE f_pppdev (VAR_RUN_PPPDEV
 	,help_pppdev
@@ -65,7 +65,7 @@ static const char *script_ipup[]={
 };
 
 
-static void copy_script (FILE *fout, const char *verb, int copy_sh)
+static void copy_script (FILE_CFG *fout, const char *verb, int copy_sh)
 {
 	if (fout != NULL){
 		if (copy_sh) fprintf (fout,"%s\n",bin_sh);
@@ -82,8 +82,8 @@ static void copy_script (FILE *fout, const char *verb, int copy_sh)
 	Return -1 if any error
 */
 static int ipup_update(
-	FILE *fin,
-	FILE *fout,
+	FILE_CFG *fin,
+	FILE_CFG *fout,
 	int replace_add,	// 0: Do nothing
 						// 1: Add the script
 						// 2: Replace an old one
@@ -174,7 +174,7 @@ static void create_script (
 {
 	if (dialog_yesno (MSG_U(T_UPDSCRIPT,"Update script")
 		,msgbuf,help_ipup)==MENU_YES){
-		FILE *fout = cf.fopen ("w");
+		FILE_CFG *fout = cf.fopen ("w");
 		if (fout != NULL){
 			copy_script (fout,verb,1);
 			fclose (fout);
@@ -201,7 +201,7 @@ static void check_ipscript(CONFIG_FILE &cf, const char *verb)
 		 "\n"
 		 "May I install the modifications ?")
 		,cfpath);
-	FILE *fin = cf.fopen ("r");
+	FILE_CFG *fin = cf.fopen ("r");
 	if (fin == NULL){
 		create_script (cf,buf,verb);
 	}else{
@@ -234,7 +234,7 @@ static void check_ipscript(CONFIG_FILE &cf, const char *verb)
 				char pathbak[PATH_MAX];
 				if (format_fname (cf,"NEW",pathnew)!=-1
 					&& format_fname (cf,"BAK",pathbak)!=-1){
-					FILE *fout = cf.fopen (pathnew,"w");
+					FILE_CFG *fout = cf.fopen (pathnew,"w");
 					if (fout != NULL){	
 						ipup_update (fin,fout,replace_add, verb
 							,found,differ,empty);
@@ -371,10 +371,10 @@ PRIVATE int PPPONE::setpppchat (char *pathchat_sh)
 			}
 			char pathchat[PATH_MAX];
 			sprintf (pathchat,"%s-chat.%s",f_pppargs.getpath(),name.get());
-			FILE *fchat = f_pppargs.fopen (pathchat,"w");
+			FILE_CFG *fchat = f_pppargs.fopen (pathchat,"w");
 			if (fchat != NULL){
 				sprintf (pathchat_sh,"%s.sh",pathchat);
-				FILE *fchat_sh = f_pppargs.fopen (pathchat_sh,"w");
+				FILE_CFG *fchat_sh = f_pppargs.fopen (pathchat_sh,"w");
 				if (fchat_sh != NULL){
 					fprintf (fchat_sh
 						,"#!/bin/sh\n"
@@ -414,7 +414,7 @@ PRIVATE void PPPONE::setpapchap(char *pppopt, PRIVILEGE *priv)
 				*/
 				char uapath[PATH_MAX];
 				sprintf (uapath,"%s.%s",f_ppppap.getpath(),name.get());
-				FILE *fpap = f_ppppap.fopen (uapath,"w");
+				FILE_CFG *fpap = f_ppppap.fopen (uapath,"w");
 				if (fpap != NULL){
 					fprintf (fpap,"%s\n%s\n",login.get(),passwd.get());
 					fclose (fpap);
@@ -441,7 +441,7 @@ PRIVATE int PPPONE::connect_ppp(bool fore, PRIVILEGE *priv)
 	*/
 	char path[PATH_MAX];
 	sprintf (path,"%s.%s",f_pppargs.getpath(),name.get());
-	FILE *fargs = f_pppargs.fopen (path,"w");
+	FILE_CFG *fargs = f_pppargs.fopen (path,"w");
 	if (fargs != NULL){
 		DAEMON_INTERNAL *cmd_pppd = daemon_find ("pppd");
 		if (cmd_pppd != NULL){
@@ -661,7 +661,7 @@ PRIVATE int PPPONE::connect_pppssh(int )
 static int pppcon_readpid(const char *path)
 {
 	int ret = -1;
-	FILE *fin = f_ppp0.fopen (path,"r");
+	FILE_CFG *fin = f_ppp0.fopen (path,"r");
 	if (fin != NULL){
 		int pid;
 		if (fscanf (fin,"%d",&pid)==1) ret = pid;
@@ -682,7 +682,7 @@ PUBLIC int PPPONE::getpppd_info(char pppdev[8])
 	char pathdev[PATH_MAX];
 	sprintf (pathdev,"%s.%s",f_pppdev.getpath(),name.get());
 	if (file_exist(pathdev)){
-		FILE *fin = f_pppdev.fopen (pathdev,"r");
+		FILE_CFG *fin = f_pppdev.fopen (pathdev,"r");
 		if (fin != NULL){
 			char dev[100];
 			if (fscanf (fin,"%s\n",dev)==1){
@@ -935,7 +935,7 @@ PUBLIC int PPPONE::postconnect(const char *dev)
 	}
 	char pathid[PATH_MAX];
 	sprintf (pathid,"%s.%s",f_pppdev.getpath(),name.get());
-	FILE *fout = f_pppdev.fopen (pathid,"w");
+	FILE_CFG *fout = f_pppdev.fopen (pathid,"w");
 	if (fout != NULL){
 		fprintf (fout,"%s\n",dev);
 		fclose (fout);
