@@ -1,10 +1,12 @@
 #include <stdio.h>
+#include <stdlib.h>
 #include <assert.h>
 #include <string.h>
 #include <unistd.h>
 #include <limits.h>
 #include <sys/stat.h>
 #include <misc.h>
+#include <dialog.h>
 #include <sstream.h>
 #include "dnsconf.h"
 #include "dnsconf.m"
@@ -75,26 +77,30 @@ static const char K_MASTERS[] = "masters";
 static const char K_HINT[] = "hint";
 const char K_ALLOW_TRANSFER[]="allow-transfer";
 const char K_ALLOW_QUERY[]="allow-query";
+const char K_ALLOW_RECURSION[]="allow-recursion";
 const char K_ALLOW_UPDATE[]="allow-update";
 const char K_NOTIFY[]="notify";
 const char K_ALSO_NOTIFY[]="also-notify";
 static const char K_LISTEN_ON[]="listen-on";
 static const char K_PORT[]="port";
 static const char K_STUB[] = "stub";
-static const char K_IN[] = "in"; // Internet zone data type.  JM.
+static const char K_IN[] = "in";
+static const char K_CHAOS[] = "chaos";
+static const char K_HS[]="hs";
+static const char K_HESIOD[]="hesiod";
 
 
 
-static int dns_is_reverse(const char *str)
+static bool dns_is_reverse(const char *str)
 {
-	int ret = 0;
+	bool ret = false;
 	char buf[200];
 	strcpy (buf,str);
 	strupr (buf);
 	char *pt = strstr(buf,".IN-ADDR.ARPA");
 	if (pt != NULL){
 		if (pt[13] == '\0'){
-			ret = 1;
+			ret = true;
 		}
 	}
 	return ret;
@@ -124,11 +130,22 @@ PRIVATE void DNS::parse4(
 			,words[4],words[5],words[6],words[7]
 			,words[8],words[9]);
 		const char *args = str_skipword (buf);
-		if (strcmp(words[0],K_DIRECTORY)==0){
-			pathcfg.setfrom (words[1]);
-		}else if (strcmp(words[0],K_CACHE)==0){
+		if (stricmp(words[0],K_DIRECTORY)==0){
+			pathcfg.setfrom (linuxconf_getval(K_DNSCONF,K_CHROOT,""));
+			pathcfg.append (words[1]);
+			pathcfg_conf.setfrom (words[1]);
+			if (file_rtype(pathcfg.get())!=1){
+				xconf_error (MSG_U(E_DIRPATH
+					,"The \"directory\" directive in %s\n"
+					 "points to a missing directory (%s).\n"
+					 "Create the directory or fix the directive before\n"
+					 "using linuxconf to maintain your DNS.")
+					,fpath
+					,pathcfg.get());
+			}
+		}else if (stricmp(words[0],K_CACHE)==0){
 			cachefiles.add (new CACHEFILE (words[1],words[2]));
-		}else if (strcmp(words[0],K_PRIMARY)==0){
+		}else if (stricmp(words[0],K_PRIMARY)==0){
 			if (dns_is_reverse(words[1])){
 				primarys_rev.add (new PRIMARY_REV(words[1]
 					,words[2],pathcfg.get(),extract));
@@ -136,34 +153,34 @@ PRIVATE void DNS::parse4(
 				primarys.add (new PRIMARY(words[1],words[2]
 					,pathcfg.get(),extract));
 			}
-		}else if (strcmp(words[0],K_SECONDARY)==0){
+		}else if (stricmp(words[0],K_SECONDARY)==0){
 			const char *tbip[8];
 			for (int i=0; i<8; i++) tbip[i] = words[i+2];
 			secondarys.add (new SECONDARY (words[1],words[nb-1]
 				,pathcfg.get(),extract
 				,tbip,nb-3));
-		}else if (strcmp(words[0],K_FORWARDERS)==0){
+		}else if (stricmp(words[0],K_FORWARDERS)==0){
 			for (int f=1; f<nb; f++) forwarders.add (new SSTRING(words[f]));
-		}else if (strcmp(words[0],K_SLAVE)==0){
+		}else if (stricmp(words[0],K_SLAVE)==0){
 			options.forward_only = 1;
-		}else if (strcmp(words[0],K_XFRNETS)==0){
+		}else if (stricmp(words[0],K_XFRNETS)==0){
 			xfernets.setfrom (args);
-		}else if (strcmp(words[0],K_BOGUSNS)==0){
+		}else if (stricmp(words[0],K_BOGUSNS)==0){
 			bogusns.setfrom (args);
-		}else if (strcmp(words[0],K_SEARCHLIST)==0){
+		}else if (stricmp(words[0],K_SEARCHLIST)==0){
 			searchlist.setfrom (args);
-		}else if (strcmp(words[0],K_OPTIONS)==0){
+		}else if (stricmp(words[0],K_OPTIONS)==0){
 			for (int i=1; i<nb; i++){
 				const char *word = words[i];
-				if (strcmp(word,K_FORWARD_ONLY)==0){
+				if (stricmp(word,K_FORWARD_ONLY)==0){
 					options.forward_only = 1;
-				}else if (strcmp(word,K_NO_RECURSION)==0){
+				}else if (stricmp(word,K_NO_RECURSION)==0){
 					options.no_recursion = 1;
-				}else if (strcmp(word,K_NO_FETCH_GLUE)==0){
+				}else if (stricmp(word,K_NO_FETCH_GLUE)==0){
 					options.no_fetch_glue = 1;
-				}else if (strcmp(word,K_QUERY_LOG)==0){
+				}else if (stricmp(word,K_QUERY_LOG)==0){
 					options.query_log = 1;
-				}else if (strcmp(word,K_FAKE_IQUERY)==0){
+				}else if (stricmp(word,K_FAKE_IQUERY)==0){
 					options.fake_iquery = 1;
 				}else{
 					xconf_error (MSG_U(E_IVLDOPT
@@ -185,12 +202,12 @@ PRIVATE void DNS::parselistenon (
 {
 	const char *tok = lex.gettoken();
 	if (tok != NULL){
-		if (strcmp(tok,K_PORT)==0){
+		if (stricmp(tok,K_PORT)==0){
 			tok = lex.gettoken();
 			options.listenonport.setfrom (tok);
 			tok = lex.gettoken();
 		}
-		if (strcmp(tok,"{")==0){
+		if (tok != NULL && strcmp(tok,"{")==0){
 			parselist_end (lex,options.listenon);
 		}else{
 			lex.error (MSG_R(E_EXPECTED),"{");
@@ -210,32 +227,42 @@ PRIVATE void DNS::parseoptions (
 			}else if (strcmp(tok,"}")==0){
 				lex.expect (";");
 				break;
-			}else if (strcmp(tok,K_DIRECTORY)==0){
-				pathcfg.setfrom (lex.getarg1());
-			}else if (strcmp(tok,K_FORWARDERS)==0){
+			}else if (stricmp(tok,K_DIRECTORY)==0){
+				pathcfg.setfrom (linuxconf_getval(K_DNSCONF,K_CHROOT,""));
+				const char *dirpath = lex.getarg1();
+				pathcfg.append (dirpath);
+				pathcfg_conf.setfrom (dirpath);
+				if (file_rtype(pathcfg.get())!=1){
+					xconf_error (MSG_R(E_DIRPATH)
+						,lex.getfpath()
+						,pathcfg.get());
+				}
+			}else if (stricmp(tok,K_FORWARDERS)==0){
 				parselist (lex,forwarders);
-			}else if (strcmp(tok,K_ALLOW_TRANSFER)==0){
+			}else if (stricmp(tok,K_ALLOW_TRANSFER)==0){
 				parselist (lex,options.allowtrans);
-			}else if (strcmp(tok,K_ALLOW_QUERY)==0){
+			}else if (stricmp(tok,K_ALLOW_QUERY)==0){
 				parselist (lex,options.allowquery);
-			}else if (strcmp(tok,K_LISTEN_ON)==0){
+			}else if (stricmp(tok,K_ALLOW_RECURSION)==0){
+				parselist (lex,options.allowrecursion);
+			}else if (stricmp(tok,K_LISTEN_ON)==0){
 				parselistenon (lex);
-			}else if (strcmp(tok,K_FORWARD)==0){
+			}else if (stricmp(tok,K_FORWARD)==0){
 				tok = lex.getarg1();
 				if (tok != NULL){
-					if (strcmp(tok,K_FIRST)==0){
+					if (stricmp(tok,K_FIRST)==0){
 						options.forward_only = 0;
-					}else if (strcmp(tok,K_ONLY)==0){
+					}else if (stricmp(tok,K_ONLY)==0){
 						options.forward_only = 1;
 					}
 				}
-			}else if (strcmp(tok,K_FAKE_IQUERY)==0){
+			}else if (stricmp(tok,K_FAKE_IQUERY)==0){
 				options.fake_iquery = lex.getarg_yesno();
-			}else if (strcmp(tok,K_FETCH_GLUE)==0){
+			}else if (stricmp(tok,K_FETCH_GLUE)==0){
 				options.no_fetch_glue = !lex.getarg_yesno();
-			}else if (strcmp(tok,K_RECURSION)==0){
+			}else if (stricmp(tok,K_RECURSION)==0){
 				options.no_recursion = !lex.getarg_yesno();
-			}else if (strcmp(tok,K_NOTIFY)==0){
+			}else if (stricmp(tok,K_NOTIFY)==0){
 				options.notify = lex.getarg_yesno();
 			}else{
 				UNKNOWN *un = new UNKNOWN;
@@ -301,33 +328,50 @@ PRIVATE void DNS::parsezone (
 	bool extract)
 {
 	SSTRING zone,file;
-	SSTRINGS masters,tforwarders,allowtrans,allowquery,allowupdate,alsonotify;
+	SSTRINGS masters,tforwarders,allowtrans,allowquery;
+	SSTRINGS allowrecursion,allowupdate,alsonotify;
 	char notify = NOTIFY_DEFAULT; 
 	char option_forward_only = FORWARD_FIRST; // the default
+	ZONE_DATA_TYPE datatype = ZONE_DATA_NONE;
 	zone.setfrom (lex.gettoken());
 	const char *tok = lex.gettoken();
+	if (tok == NULL) return;
 	ZONE_TYPE type = ZONE_UNKNOWN;
-	if (strcmp(tok, K_IN)==0) tok = lex.gettoken(); // skip 'in' token if present. JM.
-	if (!zone.is_empty() && strcmp(tok,"{")==0){
+	if (stricmp(tok, K_IN)==0){
+		datatype = ZONE_DATA_IN;
+		tok = lex.gettoken();
+	}else if (stricmp(tok, K_CHAOS)==0){
+		datatype = ZONE_DATA_CHAOS;
+		tok = lex.gettoken();
+	}else if (stricmp(tok, K_HS)==0){
+		datatype = ZONE_DATA_HS;
+		tok = lex.gettoken();
+	}else if (stricmp(tok, K_HESIOD)==0){
+		datatype = ZONE_DATA_HESIOD;
+		tok = lex.gettoken();
+	}
+	if (!zone.is_empty() && tok != NULL && strcmp(tok,"{")==0){
 		while (!lex.waserr()){
 			tok = lex.gettoken();
-			if (strcmp(tok,"}")==0){
+			if (tok == NULL){
+				break;
+			}else if (strcmp(tok,"}")==0){
 				lex.expect (";");
 				break;
-			}else if (strcmp(tok,K_FILE)==0){
+			}else if (stricmp(tok,K_FILE)==0){
 				file.setfrom(lex.getarg1());
-			}else if (strcmp(tok,K_TYPE)==0){
+			}else if (stricmp(tok,K_TYPE)==0){
 				tok = lex.getarg1();
 				if (tok != NULL){
-					if (strcmp(tok,K_MASTER)==0){
+					if (stricmp(tok,K_MASTER)==0){
 						type = ZONE_PRIMARY;
-					}else if (strcmp(tok,K_SLAVE)==0){
+					}else if (stricmp(tok,K_SLAVE)==0){
 						type = ZONE_SECONDARY;
-					}else if (strcmp(tok,K_HINT)==0){
+					}else if (stricmp(tok,K_HINT)==0){
 						type = ZONE_HINT;
-					}else if (strcmp(tok,K_FORWARD)==0){
+					}else if (stricmp(tok,K_FORWARD)==0){
 						type = ZONE_FORWARD;
-					}else if (strcmp(tok,K_STUB)==0){
+					}else if (stricmp(tok,K_STUB)==0){
 						type = ZONE_STUB;
 						lex.error (MSG_U(E_STUBNOTIMPLEMENTED
 							,"This version of Linuxconf"
@@ -338,7 +382,7 @@ PRIVATE void DNS::parsezone (
 							 " slave, hint or forward"));
 					}
 				}
-			}else if (strcmp(tok,K_MASTERS)==0){
+			}else if (stricmp(tok,K_MASTERS)==0){
 				if (lex.expect("{")!=-1){
 					while (1){
 						tok = lex.gettoken();
@@ -356,7 +400,7 @@ PRIVATE void DNS::parsezone (
 						}
 					}
 				}
-			}else if (strcmp(tok,K_FORWARDERS)==0){
+			}else if (stricmp(tok,K_FORWARDERS)==0){
 				if (lex.expect("{")!=-1){
 					while (1){
 						tok = lex.gettoken();
@@ -374,22 +418,24 @@ PRIVATE void DNS::parsezone (
 						}
 					}
 				}
-			}else if (strcmp(tok,K_ALLOW_TRANSFER)==0){
+			}else if (stricmp(tok,K_ALLOW_TRANSFER)==0){
 				parselist (lex,allowtrans);
-			}else if (strcmp(tok,K_ALLOW_QUERY)==0){
+			}else if (stricmp(tok,K_ALLOW_QUERY)==0){
 				parselist (lex,allowquery);
-			}else if (strcmp(tok,K_ALLOW_UPDATE)==0){
+			}else if (stricmp(tok,K_ALLOW_RECURSION)==0){
+				parselist (lex,allowrecursion);
+			}else if (stricmp(tok,K_ALLOW_UPDATE)==0){
 				parselist (lex,allowupdate);
-			}else if (strcmp(tok,K_ALSO_NOTIFY)==0){
+			}else if (stricmp(tok,K_ALSO_NOTIFY)==0){
 				parselist (lex,alsonotify);
-			}else if (strcmp(tok,K_NOTIFY)==0){
+			}else if (stricmp(tok,K_NOTIFY)==0){
 				notify = lex.getarg_yesno();
-			}else if (strcmp(tok,K_FORWARD)==0){
+			}else if (stricmp(tok,K_FORWARD)==0){
 				tok = lex.getarg1();
 				if (tok != NULL){
-					if (strcmp(tok,K_FIRST)==0){
+					if (stricmp(tok,K_FIRST)==0){
 						option_forward_only = FORWARD_FIRST;
-					}else if (strcmp(tok,K_ONLY)==0){
+					}else if (stricmp(tok,K_ONLY)==0){
 						option_forward_only = FORWARD_ONLY;
 					}
 				}
@@ -408,9 +454,11 @@ PRIVATE void DNS::parsezone (
 			pri = new PRIMARY(zn,file.get(),pathcfg.get(),extract);
 			primarys.add (pri);
 		}
+		pri->datatype = datatype;
 		pri->notify = notify;
 		pri->allowtrans.append (allowtrans);
 		pri->allowquery.append (allowquery);
+		pri->allowrecursion.append (allowrecursion);
 		pri->allowupdate.append (allowupdate);
 		pri->alsonotify.append (alsonotify);
 	}else if (type == ZONE_FORWARD){
@@ -434,9 +482,11 @@ PRIVATE void DNS::parsezone (
 				,pathcfg.get(),extract
 				,tbip,n);
 		secondarys.add (sec);
+		sec->datatype = datatype;
 		sec->notify = notify;
 		sec->allowtrans.append (allowtrans);
 		sec->allowquery.append (allowquery);
+		sec->allowrecursion.append (allowrecursion);
 		sec->allowupdate.append (allowupdate);
 		sec->alsonotify.append (alsonotify);
 	}
@@ -460,7 +510,9 @@ PRIVATE void DNS::parseacl(BIND8_LEXPARSE &lex)
 				SSTRING ip (tok1);
 				SSTRING comment (lex.getcomment());
 				const char *tok2 = lex.gettoken();
-				if (strcmp(tok2,";")==0){
+				if (tok2 == NULL){
+					break;
+				}else if (strcmp(tok2,";")==0){
 					acl->add (new ADDRESS_MATCH(ip.get(),comment.get()));
 				}else if (strcmp(tok2,"/")==0){
 					const char *tok3 = lex.getarg1();
@@ -503,13 +555,13 @@ PRIVATE void DNS::parse8(
 		const char *comment = lex.getcomment();
 		if (tok == NULL){
 			break;
-		}else if (strcmp(tok,K_OPTIONS)==0){
+		}else if (stricmp(tok,K_OPTIONS)==0){
 			parseoptions (lex);
-		}else if (strcmp(tok,K_ZONE)==0){
+		}else if (stricmp(tok,K_ZONE)==0){
 			parsezone (lex,extract);
-		}else if (strcmp(tok,K_ACL)==0){
+		}else if (stricmp(tok,K_ACL)==0){
 			parseacl (lex);
-		}else if (strcmp(tok,K_LOGGING)==0){
+		}else if (stricmp(tok,K_LOGGING)==0){
 			logging.add (K_LOGGING,comment);
 			parseunknown (&logging,lex);
 		}else{
@@ -546,6 +598,7 @@ PRIVATE void DNS::init(bool extract)
 	options.fake_iquery=0;
 	options.notify=true;  //notify is enabled by default (bind8)
 	pathcfg.setfrom ("/var/named");
+	pathcfg_conf.setfrom ("/var/named");
 	/* #Specification: dns / bind4 and bind8
 		The dnsconf module supports both format (bind4 named.boot and
 		bind8 named.conf) on the fly. It first checks for /etc/named.conf.
@@ -559,7 +612,7 @@ PRIVATE void DNS::init(bool extract)
 			parse8 (lex,extract);
 		}
 	}else{
-		FILE *fin = f_boot.fopen ("r");
+		FILE_CFG *fin = f_boot.fopen ("r");
 		if (fin != NULL){
 			char buf[500];
 			int noline = 0;
@@ -628,17 +681,18 @@ PUBLIC void DNS::basiccheck()
 			const char *cname = c->getname();
 			if (strcmp(cname,root_cache)==0){
 				char buf[2000];
-				sprintf (buf,MSG_U(I_MISSCACHE
-						,"The file %s is not installed on your system\n"
-						 "This will prevent normal operation of your DNS\n"
-						 "I can install a copy of it for now.\n"
+				snprintf (buf,sizeof(buf)-1,MSG_U(I_MISSCACHE
+						,"The file %s\n"
+						 "is not installed on your system.\n"
+						 "This will prevent normal operation of your DNS.\n"
+						 "I can install a copy immediatly.\n"
 						 "\n"
 						 "Get an up to date version at\n"
 						 "ftp://ftp.rs.internic.net/domain/named.root\n"
 						 "\n"
 						 "Do I install a copy of root.cache now ?")
 						,abspath);
-				if (xconf_yesno(MSG_U(T_MISSCACHE,"Missing root file")
+				if (dialog_yesno(MSG_U(T_MISSCACHE,"Missing root file")
 					,buf,help_root)==MENU_YES){
 					if (file_copy (USR_LIB_LINUXCONF_ROOT_CACHE,abspath)!=0){
 						xconf_error (MSG_U(E_CANTCOPY
@@ -673,7 +727,7 @@ PUBLIC void DNS::basiccheck()
 	}
 }
 	
-static void dns_writeopt_if (FILE *fout, char opt, const char *title)
+static void dns_writeopt_if (FILE_CFG *fout, char opt, const char *title)
 {
 	if (opt) fprintf (fout,"%s %s\n",K_OPTIONS,title);
 }
@@ -687,7 +741,7 @@ PRIVATE void DNS::mkzonedir()
 	if (!context_fexist (cfg)) context_mkdir (cfg,"root","root",0755);
 }
 
-void dns_writeopts(const SSTRINGS &tb, const char *keyword, FILE *fout)
+void dns_writeopts(const SSTRINGS &tb, const char *keyword, FILE_CFG *fout)
 {
 	if (tb.getnb()>0){
 		fprintf (fout,"\t%s{\n",keyword);
@@ -714,18 +768,18 @@ PUBLIC int DNS::write ()
 {
 	int ret = -1;
 	/* #Specification: dnsconf / writing /etc/named.boot (or named.conf)
-		The file /etc/named.boot is rewritten everytime any
+		The file /etc/named.conf is rewritten everytime any
 		of the DNS files are rewritten. This give a hint
 		to netconf to instruct named to restart(reload) just
-		by looking at the revision date of /etc/named.boot.
+		by looking at the revision date of /etc/named.conf
 	*/
 	if (bind8){
-		FILE *fout = f_conf.fopen ("w");
+		FILE_CFG *fout = f_conf.fopen ("w");
 		if (fout != NULL){
 			mkzonedir();
 			acls.write (fout);
 			fputs ("options {\n",fout);
-			fprintf (fout,"\t%s \"%s\";\n",K_DIRECTORY,pathcfg.get());
+			fprintf (fout,"\t%s \"%s\";\n",K_DIRECTORY,pathcfg_conf.get());
 			if (options.forward_only){
 				fprintf (fout,"\t%s %s;\n",K_FORWARD,K_ONLY);
 			}
@@ -744,6 +798,7 @@ PUBLIC int DNS::write ()
 			dns_writeopts (forwarders,K_FORWARDERS,fout);
 			dns_writeopts (options.allowtrans,K_ALLOW_TRANSFER,fout);
 			dns_writeopts (options.allowquery,K_ALLOW_QUERY,fout);
+			dns_writeopts (options.allowrecursion,K_ALLOW_RECURSION,fout);
 			{
 				const char *keyw = K_LISTEN_ON;
 				SSTRING buf;
@@ -773,10 +828,10 @@ PUBLIC int DNS::write ()
 			ret |= fclose (fout);
 		}
 	}else{
-		FILE *fout = f_boot.fopen ("w");
+		FILE_CFG *fout = f_boot.fopen ("w");
 		if (fout != NULL){
 			mkzonedir();
-			fprintf (fout,"%s\t%s\n",K_DIRECTORY,pathcfg.get());
+			fprintf (fout,"%s\t%s\n",K_DIRECTORY,pathcfg_conf.get());
 			for (int i=0; i<cachefiles.getnb(); i++){
 				CACHEFILE *c = cachefiles.getitem(i);
 				fprintf (fout,"%s\t%s\t%s\n",K_CACHE,c->domain.get()
@@ -833,6 +888,7 @@ static void qualify(
 */
 PUBLIC void DNS::preload_rev (PRIMARY *pri)
 {
+	const char *domain_norange = pri->getdomain_norange();
 	for (int p=0; p<primarys.getnb(); p++){
 		ORIGINS *origins = &primarys.getitem(p)->origins;
 		for (int i=0; i<origins->getnb(); i++){
@@ -846,13 +902,14 @@ PUBLIC void DNS::preload_rev (PRIMARY *pri)
 					ip->setrev(revdom);
 					FQHOST rfq (revdom);
 					char left[100];
-					if (rfq.is_member(pri->domain.get(),left)){
+					if (rfq.is_member(domain_norange,left)
+						&& pri->matchrange(left)){
 						pri->unset_left (left);
 						char fqdn[PATH_MAX];
 						snprintf (fqdn,sizeof(fqdn)-1,"%s.%s",a->name.get()
 							,ori->origin.get());
 						pri->set (rfq
-						,new RECORD_IN_PTR("dummy",fqdn));
+							,new RECORD_IN_PTR("dummy",fqdn));
 					}
 				}
 			}
@@ -870,6 +927,7 @@ PUBLIC int DNS::set (
 				// relative to our own domain
 	const char *tbip[],	// May set several IP number for
 	int nbip,			// one host at once
+	const SSTRINGS &ttls,
 	char doreverse[],
 	int &matchrev)		// Number of reverse mapping domain which
 						// were updated
@@ -906,8 +964,9 @@ PUBLIC int DNS::set (
 	if (pri != NULL){
 		is_domain = strcmp(hostpart,"@")==0;
 		for (int i=0; i<nbip; i++){
-			pri->set (fq
-				,new RECORD_IN_A("dummy",tbip[i]));
+			RECORD_IN_A *rec = new RECORD_IN_A("dummy",tbip[i]);
+			rec->setttl (ttls.getitem(i));
+			pri->set (fq,rec);
 		}
 	}
 	matchrev = 0;
@@ -960,14 +1019,24 @@ PUBLIC int DNS::setfromip(const char *host, const char *ip)
 */
 PUBLIC int DNS::set (
 	const char *fqhost,	// Fully qualified name, or name
-				// relative to our own domain
+						// relative to our own domain
 	const char *tbip[],	// May set several IP number for
-	int nbip)		// one host at once
+	int nbip,			// one host at once
+	const SSTRINGS &ttls)
 {
 	int matchrev;
 	char doreverse[nbip];
 	memset (doreverse,1,nbip);
-	return set (fqhost,tbip,nbip,doreverse,matchrev);
+	return set (fqhost,tbip,nbip,ttls,doreverse,matchrev);
+}
+PUBLIC int DNS::set (
+	const char *fqhost,	// Fully qualified name, or name
+				// relative to our own domain
+	const char *tbip[],	// May set several IP number for
+	int nbip)		// one host at once
+{
+	SSTRINGS ttls;
+	return set (fqhost,tbip,nbip,ttls);
 }
 /*
 	Add or replace NS record for a host
@@ -975,9 +1044,11 @@ PUBLIC int DNS::set (
 */
 PUBLIC int DNS::setns (
 	const SSTRING &fqhost,	// Fully qualified name, or name
-				// relative to our own domain
-	const SSTRINGS &tbns)	// May set several NS for
-				// one host at once
+							// relative to our own domain
+	const SSTRINGS &tbns,	// May set several NS for
+							// one host at once
+
+	const SSTRINGS &ttls)
 {
 	/* #Specification: dnsconf / set a domain / multiple NS
 		A domain may have several NS record in the DNS. The
@@ -1000,13 +1071,38 @@ PUBLIC int DNS::setns (
 			? primarys_rev.getitem(fq,NULL,i)
 			: primarys.getitem(fq,NULL,i);
 		if (pri != NULL && pri != last){
+			/* #Specification: reverse zone / classless delegation / trick
+				Classless reverse zones have the an origin which looks like
+				this: x-y.c.b.a.in-addr.arpa. To help dispatch the
+				IP number in those zones (the ptr records), it is easier
+				to use the standard origin of a normal reverse zone:
+				c.b.a.in-addr.arpa.
+
+				After matching the proper zone, we just have to check that
+				the IP is in the proper range. But this fake origin
+				is causing problems when we are setting the NS record
+				for the zone. We end up with a record looking like this
+
+				x-y		IN	PTR	some_server.
+
+				instead of
+
+				@		IN	PTR	some_server.
+
+				So we use a special trick when setting the NS record
+				for those zone. We use the PRIMARY::getdomain_norange()
+				instead of the full zone name.
+			*/
+			FQHOST *ptfq = &fq;
+			FQHOST fq_norange (pri->getdomain_norange());
+			if (pri->domain.icmp(fqhost)==0) ptfq = &fq_norange;
 			int nbns = tbns.getnb();
 			for (int j=0; j<nbns; j++){
 				SSTRING *ns = tbns.getitem(j);
 				if (!ns->is_empty()){
-					pri->set (fq
-						,new RECORD_IN_NS("dummy"
-						,ns->get()));
+					RECORD_IN_NS *rec = new RECORD_IN_NS("dummy",ns->get());
+					rec->setttl (ttls.getitem(j));
+					pri->set (*ptfq,rec);
 				}
 			}
 			ret = 0;
@@ -1020,9 +1116,10 @@ PUBLIC int DNS::setns (
 */
 PUBLIC int DNS::setmx (
 	const SSTRING &fqhost,	// Fully qualified name, or name
-				// relative to our own domain
-	const SSTRINGS &tbmx)	// May set several MX for
-				// one host at once
+							// relative to our own domain
+	const SSTRINGS &tbmx,	// May set several MX for
+							// one host at once
+	const SSTRINGS &ttls)
 {
 	/* #Specification: dnsconf / set a host / multiple MX
 		A host may have several MX record in the DNS. dnsconf
@@ -1037,80 +1134,69 @@ PUBLIC int DNS::setmx (
 		for (int i=0; i<nbmx; i++){
 			SSTRING *mx = tbmx.getitem(i);
 			if (!mx->is_empty()){
-				/* #Specification: domain / MX / priority
-					Linuxconf does not let you enter the priority
-					of the MX hosts. It computes a priority based on the
-					order. Some need the ability to setup two or more
-					MXs with the same priority. We use a trick. We
-					specify the MX on the same line in the dialog
-					and they will get the same priority.
-
-					When editing back, they will be presented on the same
-					line as well.
-				*/
-				SSTRINGS mxs;
-				str_splitline (mx->get(),' ',mxs);
-				int priority = i*5+5;
-				for (int m=0; m<mxs.getnb(); m++){
-					const char *one = mxs.getitem(m)->get();
-					pri->set (fq
-						,new RECORD_IN_MX("dummy"
-						,priority,one));
-				}
+				SSTRING *ss = ttls.getitem(i);
+				const char *s = ss != NULL ? ss->get() : "";
+				pri->setonemx (fq,mx->get(),i*5+5,s);
 			}
 		}
 	}
-	return (pri != NULL) ? 0 : -1;
+	return (pri != NULL) ? 1 : -1;
 }
 
 /*
 	Remove all reference to a host in the DNS
 */
-PUBLIC void DNS::unset (const char *fqhost)
+PUBLIC int DNS::unset (const char *fqhost)
 {
+	int ret = -1;
 	FQHOST fq (fqhost);
 	char hostpart[200];
 	PRIMARY *pri = primarys.getitem(fq,hostpart);
 	if (pri != NULL){
-		pri->unset (new RECORD_IN_A(hostpart,"1.1.1.1"));
+		ret |= pri->unset (new RECORD_IN_A(hostpart,"1.1.1.1"));
 	}
 	char full[200];
 	fq.formatfull (full);
 	for (int i=0; i<primarys_rev.getnb(); i++){
 		PRIMARY *pri = primarys_rev.getitem(i);
-		pri->unset (new RECORD_IN_PTR("1",full));
+		ret |= pri->unset (new RECORD_IN_PTR("1",full));
 	}
+	return ret;
 }
 
 
 /*
 	Remove all reference to a host in the DNS
 */
-PUBLIC void DNS::unsetns (const SSTRING &fqhost)
+PUBLIC int DNS::unsetns (const SSTRING &fqhost)
 {
+	int ret = -1;
 	FQHOST fq (fqhost);
 	int is_reverse = fq.is_reverse();
 	for (int i=0; i<2; i++){
 		char hostpart[200];
 		PRIMARY *pri = is_reverse
-			? primarys_rev.getitem(fq,hostpart,i)
-			: primarys.getitem(fq,hostpart,i);
+			? primarys_rev.getitem(fq,hostpart,i!=0)
+			: primarys.getitem(fq,hostpart,i!=0);
 		if (pri != NULL){
-			pri->unset (new RECORD_IN_NS(hostpart,"dummy"));
+			ret = pri->unset (new RECORD_IN_NS(hostpart,"dummy"));
 		}
 	}
+	return ret;
 }
 /*
 	Remove all reference to a host in the DNS
 */
-PUBLIC void DNS::unsetmx (const SSTRING &fqhost)
+PUBLIC int DNS::unsetmx (const SSTRING &fqhost)
 {
+	int ret = -1;
 	FQHOST fq (fqhost);
 	char hostpart[200];
 	PRIMARY *pri = primarys.getitem(fq,hostpart);
 	if (pri != NULL){
-		pri->unset (new RECORD_IN_MX(hostpart,0,"dummy"));
+		ret = pri->unset (new RECORD_IN_MX(hostpart,0,"dummy"));
 	}
+	return ret;
 }
 
 /*
@@ -1128,7 +1214,21 @@ PUBLIC int DNS::setcname (const SSTRING &fqhost, const SSTRING &cname)
 				,cname.get()));
 		}
 	}
-	return (pri != NULL) ? 0 : -1;
+	return (pri != NULL) ? 1 : -1;
+}
+/*
+	Set a cname record (replace/add/remove)
+*/
+PUBLIC int DNS::unsetcname (const SSTRING &fqhost)
+{
+	int ret = -1;
+	FQHOST fq (fqhost);
+	char hostpart[200];
+	PRIMARY *pri = primarys.getitem(fq,hostpart);
+	if (pri != NULL){
+		ret = pri->unset (new RECORD_IN_CNAME(hostpart,"dummy"));
+	}
+	return ret;
 }
 
 /*
@@ -1163,7 +1263,7 @@ PRIVATE void DNS::setconfig(SSTRING &p)
 	const char *path = p.get();
 	CONFIG_FILE *cfg; // return value not used
 	if (path[0] != '/'){
-		sprintf (tmp,"%s/%s",pathcfg.get(),path);
+		snprintf (tmp,sizeof(tmp)-1,"%s/%s",pathcfg.get(),path);
 		path = tmp;
 	}
 	cfg=new CONFIG_FILE (tmp,help_dnsconf,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
@@ -1206,10 +1306,15 @@ static long dns_date (long maxdate, const char *basedir, const SSTRING &path)
 	char tmp[PATH_MAX];
 	const char *fname = path.get();
 	if (fname[0] != '/'){
-		sprintf (tmp,"%s/%s",basedir,fname);
+		snprintf (tmp,sizeof(tmp)-1,"%s/%s",basedir,fname);
 		fname = tmp;
 	}
 	long date = file_date(tmp);
+	if (getenv("DNSCONF_DEBUG")!=NULL){
+		fprintf (stderr,"Compare file %s: maxdate %ld date %ld %s\n"
+			,tmp,maxdate,date
+			,date > maxdate ? "File is newer" : "");
+	}
 	if (date > maxdate) maxdate = date;
 	return maxdate;
 }
