@@ -1,28 +1,44 @@
-/* dnsrecs.cc 09/11/1999 00.06.54 */
-PRIVATE int DNS::update_one (SSTRING&full,
-	 IP_ADDRS&tba,
-	 SSTRINGS&tbns,
-	 SSTRINGS&tbmx,
-	 SSTRING&cname,
-	 char doreverse[4]);
-PRIVATE int DNS::update_one (SSTRING&full,
-	 IP_ADDRS&tba,
-	 SSTRINGS&tbns,
-	 SSTRINGS&tbmx,
-	 SSTRING&cname);
-PUBLIC void DNS::getalladr (IP_ADDRS&adrs);
-PUBLIC void DNS::setip_in_dia (DIALOG&dia,
-	 int level,
-	 IP_ADDRS&tba,
-	 const char *title);
-PUBLIC void DNS::setip_in_dia (DIALOG&dia, IP_ADDRS&tba);
-PRIVATE void DNS::editone_rev (SSTRING&full);
-PRIVATE int DNS::check_dup (SSTRING&full,
-	 IP_ADDRS&tba,
-	 char doreverse[]);
-PUBLIC void DNS::editone (const char *name);
-PUBLIC void DNS::editrecs (const char *preset);
-/* dns.cc 13/12/1999 23.37.58 */
+/* _dict.cc 14/08/1996 15.08.30 */
+/* acl.cc 07/08/2000 10.54.36 */
+PUBLIC ADDRESS_MATCH::ADDRESS_MATCH (void);
+PUBLIC ADDRESS_MATCH::ADDRESS_MATCH (const char *_ip,
+	 const char *comment);
+PUBLIC ADDRESS_MATCH::ADDRESS_MATCH (const char *_ip,
+	 const char *_netsize,
+	 const char *comment);
+PUBLIC int ADDRESS_MATCH::edit (const ACLS&acls);
+PUBLIC void ADDRESS_MATCH::write (FILE_CFG *fout);
+PUBLIC bool ADDRESS_MATCH::is_empty (void);
+PUBLIC bool ADDRESS_MATCH::is_none (void);
+PUBLIC bool ADDRESS_MATCH::is_any (void);
+PUBLIC bool ADDRESS_MATCH::is_localnet (void);
+PUBLIC ADDRESS_MATCH *ACL::getitem (int no)const;
+PUBLIC bool ACL::is_empty (int val)const;
+PUBLIC ACL::ACL (const char *_name);
+PUBLIC void ACL::write (FILE_CFG *fout);
+PUBLIC ACL *ACLS::getitem (int no)const;
+PUBLIC void ACLS::write (FILE_CFG *fout);
+PUBLIC int ACL::editname (void);
+PUBLIC int ACL::edit (DNS&dns, const ACLS&acls);
+PUBLIC int ACLS::edit (DNS&dns);
+PUBLIC int ACLS::locate (const char *name)const;
+PUBLIC bool ACLS::exist (const char *name)const;
+/* bind8.cc 18/12/2000 12.19.44 */
+PUBLIC BIND8_LEXPARSE::BIND8_LEXPARSE (CONFIG_FILE&_cfg,
+	 SSTRING&_dir);
+PUBLIC BIND8_LEXPARSE::~BIND8_LEXPARSE (void);
+PUBLIC bool BIND8_LEXPARSE::isok (void);
+PRIVATE int BIND8_LEXPARSE::fillbuf (void);
+PUBLIC const char *BIND8_LEXPARSE::gettoken (bool keepquote);
+PUBLIC const char *BIND8_LEXPARSE::gettoken (void);
+PUBLIC const char *BIND8_LEXPARSE::getfpath (void);
+PUBLIC void BIND8_LEXPARSE::error (const char *ctl, ...);
+PUBLIC const char *BIND8_LEXPARSE::getarg1 (void);
+PUBLIC bool BIND8_LEXPARSE::getarg_yesno (void);
+PUBLIC bool BIND8_LEXPARSE::waserr (void);
+PUBLIC int BIND8_LEXPARSE::expect (const char *s);
+PUBLIC const char *BIND8_LEXPARSE::getcomment (void);
+/* dns.cc 02/09/2001 22.26.16 */
 PUBLIC CONFIG_FILE_DNS::CONFIG_FILE_DNS (const char *_path);
 PUBLIC int CONFIG_FILE_DNS::extract (SSTREAM&ss);
 PRIVATE void DNS::parse4 (char *buf,
@@ -53,18 +69,28 @@ PUBLIC void DNS::preload_rev (PRIMARY *pri);
 PUBLIC int DNS::set (const char *fqhost,
 	 const char *tbip[],
 	 int nbip,
+	 const SSTRINGS&ttls,
 	 char doreverse[],
 	 int &matchrev);
 PUBLIC int DNS::setfromip (const char *host, const char *ip);
+PUBLIC int DNS::set (const char *fqhost,
+	 const char *tbip[],
+	 int nbip,
+	 const SSTRINGS&ttls);
 PUBLIC int DNS::set (const char *fqhost,
 	 const char *tbip[],
 	 int nbip);
-PUBLIC int DNS::setns (const SSTRING&fqhost, const SSTRINGS&tbns);
-PUBLIC int DNS::setmx (const SSTRING&fqhost, const SSTRINGS&tbmx);
-PUBLIC void DNS::unset (const char *fqhost);
-PUBLIC void DNS::unsetns (const SSTRING&fqhost);
-PUBLIC void DNS::unsetmx (const SSTRING&fqhost);
+PUBLIC int DNS::setns (const SSTRING&fqhost,
+	 const SSTRINGS&tbns,
+	 const SSTRINGS&ttls);
+PUBLIC int DNS::setmx (const SSTRING&fqhost,
+	 const SSTRINGS&tbmx,
+	 const SSTRINGS&ttls);
+PUBLIC int DNS::unset (const char *fqhost);
+PUBLIC int DNS::unsetns (const SSTRING&fqhost);
+PUBLIC int DNS::unsetmx (const SSTRING&fqhost);
 PUBLIC int DNS::setcname (const SSTRING&fqhost, const SSTRING&cname);
+PUBLIC int DNS::unsetcname (const SSTRING&fqhost);
 PUBLIC PRIMARY *DNS::finddomain (const char *name)const;
 PUBLIC SECONDARY *DNS::findsecond (const char *name)const;
 PUBLIC FORWARD *DNS::findforward (const char *name)const;
@@ -72,12 +98,80 @@ PRIVATE void DNS::setconfig (SSTRING&p);
 PUBLIC void DNS::listconfigs (void);
 PUBLIC long DNS::getrevdate (void);
 PUBLIC PRIMARY *DNS::findprimary (const char *host);
-/* soa.cc 22/05/1999 12.32.08 */
-PUBLIC RECORD_IN_SOA::RECORD_IN_SOA (const RECORD_PARSE&p);
-PUBLIC RECORD_IN_SOA::RECORD_IN_SOA (void);
-PUBLIC void RECORD_IN_SOA::print (FILE *fout)const;
-PUBLIC void RECORD_IN_SOA::update (const char *name);
-/* ipmap.cc 06/01/1998 03.10.50 */
+/* dnscheck.cc 16/09/1995 13.32.38 */
+PUBLIC void DNS::check (void);
+/* dnsconf.cc 05/09/2001 00.08.16 */
+PUBLIC void DNS::editforwarders (void);
+PUBLIC int DNS::edit (void);
+PUBLIC int DNS::was_modified (void);
+PUBLIC int DNS::geta (const char *host, IP_ADDRS&tbip);
+PUBLIC bool DNS::condset (const char *hostname,
+	 const char *tbip[],
+	 int nbip,
+	 bool test);
+/* dnsmxs.cc 14/08/1996 15.09.00 */
+PUBLIC void DNS::editmxs (void);
+/* dnsrecs.cc 02/09/2001 23.35.52 */
+PRIVATE int DNS::update_one (SSTRING&full,
+	 IP_ADDRS&tba,
+	 SSTRINGS&tbattls,
+	 SSTRINGS&tbns,
+	 SSTRINGS&tbnsttls,
+	 SSTRINGS&tbmx,
+	 SSTRINGS&tbmxttls,
+	 SSTRING&cname,
+	 char doreverse[4]);
+PRIVATE int DNS::update_one (SSTRING&full,
+	 IP_ADDRS&tba,
+	 SSTRINGS&tbattls,
+	 SSTRINGS&tbns,
+	 SSTRINGS&tbnsttls,
+	 SSTRINGS&tbmx,
+	 SSTRINGS&tbmxttls,
+	 SSTRING&cname);
+PUBLIC void DNS::getalladr (IP_ADDRS&adrs);
+PUBLIC void DNS::setip_in_dia (DIALOG&dia,
+	 int level,
+	 IP_ADDRS&tba,
+	 SSTRINGS&ttls,
+	 const char *title);
+PUBLIC void DNS::setip_in_dia (DIALOG&dia,
+	 IP_ADDRS&tba,
+	 SSTRINGS&ttls);
+PRIVATE void DNS::editone_rev (SSTRING&full);
+PRIVATE int DNS::check_dup (SSTRING&full,
+	 IP_ADDRS&tba,
+	 char doreverse[]);
+PUBLIC void DNS::editone (const char *name);
+PUBLIC void DNS::editrecs (const char *preset);
+/* features.cc 02/04/2001 14.21.20 */
+PUBLIC void DNS::editaccess (void);
+PUBLIC void DNS::editfeatures (void);
+/* forward.cc 10/10/2001 16.34.16 */
+PUBLIC FORWARD::FORWARD (const char *_domain,
+	 const char **_ip_addr,
+	 int nb_ip);
+PUBLIC FORWARD::FORWARD (void);
+PUBLIC void FORWARD::write (bool bind8, FILE_CFG *fout)const;
+PUBLIC int FORWARD::edit (DNS&dns);
+PUBLIC FORWARD *FORWARDS::getitem (int no)const;
+PUBLIC FORWARD *FORWARDS::getitem (const char *name)const;
+PUBLIC void FORWARDS::write (bool bind8, FILE_CFG *fout)const;
+PUBLIC void FORWARDS::edit (DNS&dns);
+/* fqhost.cc 12/03/2001 16.30.52 */
+PRIVATE void FQHOST::init (const char *host);
+PUBLIC FQHOST::FQHOST (const char *host);
+PUBLIC FQHOST::FQHOST (const SSTRING&host);
+PUBLIC FQHOST::FQHOST (const char *host, const char *domain);
+PUBLIC int FQHOST::is_member (const char *domain, char *hostpart);
+PUBLIC int FQHOST::is_reverse (void);
+PUBLIC const char *FQHOST::gethostpart (const char *domain);
+PUBLIC void FQHOST::formatfull (char *full);
+PUBLIC void FQHOST::formatfull (SSTRING&full);
+/* helpf.cc 14/08/1996 15.09.36 */
+PUBLIC DNSCONF_HELP_FILE::DNSCONF_HELP_FILE (const char *fname);
+/* internal.cc 27/08/1995 21.36.10 */
+/* ipmap.cc 31/12/2000 13.47.46 */
 PUBLIC IPMAP *IPMAPS::getitem (int no);
 PUBLIC IPMAPS::IPMAPS (void);
 PUBLIC int IPMAPS::write (void);
@@ -88,130 +182,29 @@ PUBLIC int IPMAPS::edit (void);
 PUBLIC int IPMAPS::allocateip (const char *range,
 	 char *ip,
 	 DNS&dns);
+/* logging.cc 18/09/2000 17.52.44 */
+PUBLIC void UNKNOWN::add (const char *token, const char *comment);
+PUBLIC void UNKNOWN::write (FILE_CFG *fout, int level);
+PUBLIC UNKNOWN *UNKNOWNS::getitem (int no)const;
+PUBLIC void UNKNOWNS::write (FILE_CFG *fout, int level);
 /* misc.cc 30/07/1997 23.45.38 */
 PUBLIC CACHEFILE::CACHEFILE (const char *_domain, const char *_path);
 PUBLIC const char *CACHEFILE::getname (void);
 PUBLIC void CACHEFILE::abspath (const char *basepath, char *abspath);
 PUBLIC CACHEFILE *CACHEFILES::getitem (int no);
-/* dnsmxs.cc 14/08/1996 15.09.00 */
-PUBLIC void DNS::editmxs (void);
-/* helpf.cc 14/08/1996 15.09.36 */
-PUBLIC DNSCONF_HELP_FILE::DNSCONF_HELP_FILE (const char *fname);
-/* _dict.cc 14/08/1996 15.08.30 */
-/* primbydom.cc 22/04/1999 23.00.38 */
-PUBLIC RECORD_ITEM::RECORD_ITEM (RECORD *rec, const char *relname);
-PUBLIC RECORD_ITEM *RECORD_ITEMS::getitem (int no)const;
-PUBLIC void ZONE::edithosts (DNS&dns);
-PRIVATE void PRIMARYS::setselect (DIALOG&dia);
-PUBLIC void PRIMARYS::editbydom (DNS&dns);
-/* tbfile.cc 25/02/1998 14.37.04 */
-PUBLIC TBFILE::TBFILE (const char *_defpath, bool _extract);
-PUBLIC TBFILE::~TBFILE (void);
-PUBLIC FILE *TBFILE::fopen (const char *fname, const char *mode);
-PUBLIC void TBFILE::fclose (void);
-/* dnscheck.cc 16/09/1995 13.32.38 */
-PUBLIC void DNS::check (void);
-/* dnsconf.cc 15/11/1999 14.55.46 */
-PUBLIC void DNS::editforwarders (void);
-PUBLIC int DNS::edit (void);
-PUBLIC int DNS::was_modified (void);
-PUBLIC int DNS::geta (const char *host, IP_ADDRS&tbip);
-PUBLIC bool DNS::condset (const char *hostname,
-	 const char *tbip[],
-	 int nbip);
-/* features.cc 15/11/1999 14.54.38 */
-PUBLIC void DNS::editaccess (void);
-PUBLIC void DNS::editfeatures (void);
-/* record.cc 07/03/1999 14.08.00 */
-PUBLIC RECORD::RECORD (RECORD_TYPE _id);
-PUBLIC bool RECORD::is (RECORD_TYPE _id);
-PUBLIC VIRTUAL void RECORD::sethostpart (const char *);
-PROTECTED VIRTUAL int RECORD::cmpsame (const RECORD *);
-PUBLIC VIRTUAL int RECORD::set (const RECORD *);
-PUBLIC int RECORD::cmp (RECORD *other);
-PUBLIC VIRTUAL int RECORD::cmp_left (const char *);
-PUBLIC VIRTUAL const char *RECORD::get_left (void);
-PUBLIC VIRTUAL int RECORD::cmp_right (const char *);
-PUBLIC RECORD_IN::RECORD_IN (const RECORD_PARSE&p, RECORD_TYPE _id);
-PUBLIC RECORD_IN::RECORD_IN (RECORD_TYPE _id);
-PUBLIC void RECORD::setcomment (const char *str);
-PUBLIC RECORD_COMMENT::RECORD_COMMENT (const char *str);
-PUBLIC void RECORD_COMMENT::print (FILE *fout)const;
-PUBLIC RECORD_IN_A::RECORD_IN_A (const RECORD_PARSE&p);
-PUBLIC RECORD_IN_A::RECORD_IN_A (const char *_name, const char *_addr);
-PUBLIC void RECORD_IN_A::sethostpart (const char *hostpart);
-PROTECTED int RECORD_IN_A::cmpsame (const RECORD *other);
-PROTECTED int RECORD_IN_A::cmp_left (const char *str);
-PROTECTED int RECORD_IN_A::cmp_right (const char *str);
-PUBLIC VIRTUAL const char *RECORD_IN_A::get_left (void);
-PUBLIC int RECORD_IN_A::set (const RECORD *other);
-PUBLIC int RECORD_IN_A::cmphost (const char *host);
-PUBLIC void RECORD_IN_A::setip (const char *ip);
-PUBLIC void RECORD_IN_A::print (FILE *fout)const;
-PUBLIC RECORD_IN_PTR::RECORD_IN_PTR (const RECORD_PARSE&p);
-PUBLIC RECORD_IN_PTR::RECORD_IN_PTR (const char *iprev,
-	 const char *host);
-PROTECTED int RECORD_IN_PTR::cmpsame (const RECORD *other);
-PROTECTED int RECORD_IN_PTR::cmp_left (const char *str);
-PROTECTED int RECORD_IN_PTR::cmp_right (const char *str);
-PUBLIC int RECORD_IN_PTR::set (const RECORD *other);
-PUBLIC void RECORD_IN_PTR::sethostpart (const char *hostpart);
-PUBLIC void RECORD_IN_PTR::print (FILE *fout)const;
-PUBLIC int RECORD_IN_PTR::cmpip (const char *ip);
-PUBLIC RECORD_IN_NS::RECORD_IN_NS (const RECORD_PARSE&p);
-PUBLIC RECORD_IN_NS::RECORD_IN_NS (const char *_name, const char *_ns);
-PUBLIC RECORD_IN_NS::RECORD_IN_NS (void);
-PROTECTED int RECORD_IN_NS::cmpsame (const RECORD *other);
-PUBLIC int RECORD_IN_NS::set (const RECORD *other);
-PUBLIC void RECORD_IN_NS::sethostpart (const char *hostpart);
-PUBLIC int RECORD_IN_NS::cmp_left (const char *str);
-PUBLIC void RECORD_IN_NS::print (FILE *fout)const;
-PUBLIC RECORD_IN_CNAME::RECORD_IN_CNAME (const RECORD_PARSE&p);
-PUBLIC RECORD_IN_CNAME::RECORD_IN_CNAME (const char *cname,
-	 const char *realname);
-PUBLIC int RECORD_IN_CNAME::cmp_left (const char *str);
-PUBLIC void RECORD_IN_CNAME::sethostpart (const char *hostpart);
-PROTECTED int RECORD_IN_CNAME::cmpsame (const RECORD *other);
-PUBLIC void RECORD_IN_CNAME::print (FILE *fout)const;
-PUBLIC RECORD_IN_MX::RECORD_IN_MX (const RECORD_PARSE&p);
-PUBLIC RECORD_IN_MX::RECORD_IN_MX (const char *_mailname,
-	 int priority,
-	 const char *_servname);
-PROTECTED int RECORD_IN_MX::cmpsame (const RECORD *other);
-PUBLIC int RECORD_IN_MX::set (const RECORD *other);
-PUBLIC int RECORD_IN_MX::cmp_left (const char *str);
-PUBLIC void RECORD_IN_MX::print (FILE *fout)const;
-PUBLIC void RECORD_IN_MX::sethostpart (const char *hostpart);
-PUBLIC RECORD_IN_HINFO::RECORD_IN_HINFO (const char *_name,
-	 const char *_info);
-PROTECTED int RECORD_IN_HINFO::cmpsame (const RECORD *other);
-PUBLIC void RECORD_IN_HINFO::print (FILE *fout)const;
-PUBLIC RECORD_IN_RP::RECORD_IN_RP (const char *_name, const char *_rp);
-PROTECTED int RECORD_IN_RP::cmpsame (const RECORD *other);
-PUBLIC void RECORD_IN_RP::print (FILE *fout)const;
-PUBLIC RECORD_IN_TXT::RECORD_IN_TXT (const char *_name,
-	 const char *_txt);
-PROTECTED int RECORD_IN_TXT::cmpsame (const RECORD *other);
-PUBLIC void RECORD_IN_TXT::print (FILE *fout)const;
-PUBLIC RECORD_PARSE::RECORD_PARSE (void);
-PUBLIC RECORD_INCLUDE::RECORD_INCLUDE (const char *_path);
-PUBLIC void RECORD_INCLUDE::print (FILE *fout)const;
-PUBLIC RECORD_END_INCLUDE::RECORD_END_INCLUDE (void);
-PUBLIC void RECORD_END_INCLUDE::print (FILE *)const;
-PUBLIC RECORD *RECORDS::getitem (int no)const;
-PUBLIC int RECORDS::save (TBFILE&tbf)const;
-/* internal.cc 27/08/1995 21.36.10 */
-/* fqhost.cc 25/04/1998 11.28.48 */
-PRIVATE void FQHOST::init (const char *host);
-PUBLIC FQHOST::FQHOST (const char *host);
-PUBLIC FQHOST::FQHOST (const SSTRING&host);
-PUBLIC FQHOST::FQHOST (const char *host, const char *domain);
-PUBLIC int FQHOST::is_member (const char *domain, char *hostpart);
-PUBLIC int FQHOST::is_reverse (void);
-PUBLIC const char *FQHOST::gethostpart (const char *domain);
-PUBLIC void FQHOST::formatfull (char *full);
-PUBLIC void FQHOST::formatfull (SSTRING&full);
-/* origin.cc 26/07/1999 23.08.52 */
+/* mod_dnsconf.cc 23/05/2001 00.08.16 */
+PUBLIC MODULE_dnsconf::MODULE_dnsconf (void);
+PUBLIC void MODULE_dnsconf::setmenu (DIALOG&dia, MENU_CONTEXT context);
+PUBLIC int MODULE_dnsconf::domenu (MENU_CONTEXT context,
+	 const char *key);
+PUBLIC int MODULE_dnsconf::dohtml (const char *key);
+PUBLIC int MODULE_dnsconf::probe (int , int , bool);
+PUBLIC int MODULE_dnsconf::message (const char *msg,
+	 int ,
+	 const char *[]);
+PUBLIC void MODULE_dnsconf::usage (SSTRINGS&tb);
+PUBLIC int MODULE_dnsconf::execmain (int argc, char *argv[], bool);
+/* origin.cc 02/03/2001 13.13.00 */
 PUBLIC ORIGIN::ORIGIN (const char *_origin);
 PUBLIC void ORIGIN::print (bool save_ori, TBFILE&tbf)const;
 PUBLIC int ORIGIN::getalladr (IP_ADDRS&adrs);
@@ -231,11 +224,14 @@ PUBLIC int ORIGINS::read (const char *named_dir,
 	 bool extract);
 PUBLIC int ORIGINS::save (const char *named_dir,
 	 const char *fname)const;
-/* primary.cc 01/12/1999 11.54.26 */
+/* primary.cc 10/10/2001 16.28.14 */
+PUBLIC ZONE::ZONE (void);
+PRIVATE void PRIMARY::init (const char *_domain, const char *_file);
 PUBLIC PRIMARY::PRIMARY (const char *_domain,
 	 const char *_file,
 	 const char *named_dir,
 	 bool extract);
+PROTECTED PRIMARY::PRIMARY (const char *_domain, const char *_file);
 PUBLIC PRIMARY::PRIMARY (void);
 PUBLIC VIRTUAL void ZONE::setfromv (void);
 PUBLIC PRIMARY_REV::PRIMARY_REV (const char *_domain,
@@ -244,56 +240,87 @@ PUBLIC PRIMARY_REV::PRIMARY_REV (const char *_domain,
 	 bool extract);
 PUBLIC PRIMARY_REV::PRIMARY_REV (void);
 PUBLIC void PRIMARY_REV::setfromv (void);
+PUBLIC VIRTUAL bool PRIMARY::matchrange (const char *);
+PUBLIC bool PRIMARY_REV::matchrange (const char *host);
+PUBLIC VIRTUAL const char *ZONE::getdomain_norange (void);
+PUBLIC const char *PRIMARY_REV::getdomain_norange (void);
+PUBLIC VIRTUAL const char *PRIMARY::getrange (void)const;
+PUBLIC const char *PRIMARY_REV::getrange (void)const;
 PUBLIC int ZONE::was_modified (void);
-PUBLIC VIRTUAL int ZONE::is_reverse (void);
-PUBLIC int PRIMARY_REV::is_reverse (void);
+PUBLIC VIRTUAL bool ZONE::is_reverse (void);
+PUBLIC bool PRIMARY_REV::is_reverse (void);
 PUBLIC int ZONE::getalladr (IP_ADDRS&adrs);
-PRIVATE RECORD *ZONE::getfirst (RECORD_TYPE rtype);
-PUBLIC RECORD_IN_SOA *ZONE::getsoa (void);
+PRIVATE RECORD *ZONE::getfirst (RECORD_TYPE rtype)const;
+PUBLIC RECORD_IN_SOA *ZONE::getsoa (void)const;
+PUBLIC RECORD_TTL *ZONE::getdefaultttl (void);
 PUBLIC int ZONE::getns (SSTRING&dom, RECORDS&recs);
-PUBLIC int ZONE::getns (SSTRING&dom, SSTRINGS&strs);
+PUBLIC int ZONE::getns (SSTRING&dom,
+	 SSTRINGS&strs,
+	 SSTRINGS&ttls);
 PUBLIC int ZONE::getmx (SSTRING&dom, RECORDS&recs);
-PUBLIC int ZONE::getmx (SSTRING&dom, SSTRINGS&strs);
+PUBLIC int ZONE::getmx (SSTRING&dom,
+	 SSTRINGS&strs,
+	 SSTRINGS&ttls);
 PUBLIC int ZONE::geta (SSTRING&dom, RECORDS&recs);
-PUBLIC int ZONE::geta (SSTRING&dom, IP_ADDRS&adrs);
+PUBLIC int ZONE::geta (SSTRING&dom, IP_ADDRS&adrs, SSTRINGS&ttls);
 PUBLIC int ZONE::getcname (SSTRING&dom, SSTRING&cname);
 PUBLIC void ZONE::updatesoa (void);
 PUBLIC void ZONE::format_revision (char *buf);
-PROTECTED void ZONE::writeaccess (FILE *fout)const;
+PROTECTED void ZONE::writeaccess (FILE_CFG *fout)const;
 PUBLIC int PRIMARY::write (bool bind8,
-	 FILE *fout,
+	 FILE_CFG *fout,
 	 const char *named_dir,
 	 const DNS *dns)const;
 PUBLIC void ZONE::addrec (RECORD *rec);
+PUBLIC void ZONE::insrec (RECORD *rec);
 PROTECTED void ZONE::setupzoneaccess (DIALOG&dia, int level);
 PUBLIC void ZONE::setupdia (DIALOG&dia,
 	 DNS&dns,
 	 int level,
 	 PRIMARY_EDITINFO&info);
-PUBLIC int ZONE::editshow (DNS&dns, bool readonly);
+PROTECTED void ZONE::load_extrainfo (PRIMARY_EDITINFO&info);
+PROTECTED void ZONE::save_extrainfo (PRIMARY_EDITINFO&info);
+PROTECTED void ZONE::del_extrainfo (void);
+PUBLIC int ZONE::editshow (DNS&dns, bool readonly, SSTRING *range);
 PUBLIC void ZONE::show (DNS&dns);
-PUBLIC int PRIMARY::edit (DNS&dns);
+PUBLIC VIRTUAL int PRIMARY::edit (DNS&dns);
+PUBLIC int PRIMARY_REV::edit (DNS&dns);
 PUBLIC PRIMARY *PRIMARYS::getitem (int no)const;
 PUBLIC PRIMARY *PRIMARYS::getitem (const char *name)const;
 PUBLIC int PRIMARYS::getalladr (IP_ADDRS&adrs);
 PUBLIC PRIMARY *PRIMARYS::getitem (FQHOST&fq,
 	 char *hostpart,
-	 int dontitself);
+	 bool dontitself);
 PUBLIC PRIMARY *PRIMARYS::getitem (FQHOST&fq, char *hostpart);
 PUBLIC int PRIMARYS::write (bool bind8,
-	 FILE *fout,
+	 FILE_CFG *fout,
 	 const char *named_dir,
 	 const DNS *dns)const;
 PUBLIC VIRTUAL PRIMARY *PRIMARYS::new_PRIMARY (void);
+PUBLIC VIRTUAL bool PRIMARYS::is_reverse (void);
+PUBLIC bool PRIMARYS_REV::is_reverse (void);
 PUBLIC PRIMARY *PRIMARYS_REV::new_PRIMARY (void);
 PUBLIC void PRIMARYS::edit (DNS&dns);
+PUBLIC int PRIMARYS::initfrom (DNS&dns,
+	 PRIMARY *p,
+	 const char *example);
 PUBLIC int PRIMARYS::adddomain (const char *domain,
 	 const char *example,
 	 DNS&dns);
 PUBLIC int DNS::newdomain (const char *domain, const char *example);
 PUBLIC int DNS::deldomain (const char *domain);
-/* primquery.cc 22/04/1999 23.00.38 */
+/* primbydom.cc 05/03/2001 18.09.50 */
+PUBLIC RECORD_ITEM::RECORD_ITEM (RECORD *rec, const char *relname);
+PUBLIC RECORD_ITEM *RECORD_ITEMS::getitem (int no)const;
+PUBLIC void ZONE::edithosts (DNS&dns);
+PRIVATE void PRIMARYS::setselect (DIALOG&dia, PRIMARYS&sorted);
+PUBLIC void PRIMARYS::editbydom (DNS&dns);
+/* primquery.cc 02/09/2001 22.22.06 */
 PUBLIC int ZONE::set (FQHOST&fq, RECORD *new_rec);
+PUBLIC void PRIMARY::setonemx (FQHOST&fq,
+	 const char *mx,
+	 int priority,
+	 const char *ttl);
 PUBLIC int ZONE::unset (RECORD *new_rec);
 PUBLIC int ZONE::unset_left (const char *left);
 PUBLIC int ZONE::locate_left (FQHOST&fq,
@@ -308,7 +335,91 @@ PUBLIC int PRIMARYS::locate_right (const char *value,
 	 SSTRINGS&recs);
 PUBLIC int DNS::locate_ip (const char *ip, SSTRINGS&recs);
 PUBLIC PRIMARY *DNS::locate_domain (const char *domain);
-/* secondary.cc 21/07/1999 11.39.36 */
+/* record.cc 02/03/2001 17.03.16 */
+PUBLIC RECORD::RECORD (RECORD_TYPE _id);
+PUBLIC bool RECORD::is (RECORD_TYPE _id);
+PUBLIC VIRTUAL void RECORD::sethostpart (const char *);
+PROTECTED VIRTUAL int RECORD::cmpsame (const RECORD *);
+PUBLIC VIRTUAL int RECORD::set (const RECORD *);
+PUBLIC int RECORD::cmp (RECORD *other);
+PUBLIC VIRTUAL int RECORD::cmp_left (const char *);
+PUBLIC VIRTUAL const char *RECORD::get_left (void);
+PUBLIC VIRTUAL int RECORD::cmp_right (const char *);
+PUBLIC RECORD_IN::RECORD_IN (const RECORD_PARSE&p, RECORD_TYPE _id);
+PUBLIC RECORD_IN::RECORD_IN (RECORD_TYPE _id);
+PUBLIC const char *RECORD_IN::getttl (void);
+PUBLIC void RECORD_IN::setttl (const char *s);
+PUBLIC void RECORD_IN::setttl (const SSTRING *s);
+PUBLIC void RECORD::setcomment (const char *str);
+PUBLIC RECORD_COMMENT::RECORD_COMMENT (const char *str);
+PUBLIC void RECORD_COMMENT::print (FILE_CFG *fout)const;
+PUBLIC RECORD_IN_A::RECORD_IN_A (const RECORD_PARSE&p);
+PUBLIC RECORD_IN_A::RECORD_IN_A (const char *_name, const char *_addr);
+PUBLIC void RECORD_IN_A::sethostpart (const char *hostpart);
+PROTECTED int RECORD_IN_A::cmpsame (const RECORD *other);
+PROTECTED int RECORD_IN_A::cmp_left (const char *str);
+PROTECTED int RECORD_IN_A::cmp_right (const char *str);
+PUBLIC VIRTUAL const char *RECORD_IN_A::get_left (void);
+PUBLIC int RECORD_IN_A::set (const RECORD *other);
+PUBLIC int RECORD_IN_A::cmphost (const char *host);
+PUBLIC void RECORD_IN_A::setip (const char *ip);
+PUBLIC void RECORD_IN_A::print (FILE_CFG *fout)const;
+PUBLIC RECORD_IN_PTR::RECORD_IN_PTR (const RECORD_PARSE&p);
+PUBLIC RECORD_IN_PTR::RECORD_IN_PTR (const char *iprev,
+	 const char *host);
+PROTECTED int RECORD_IN_PTR::cmpsame (const RECORD *other);
+PROTECTED int RECORD_IN_PTR::cmp_left (const char *str);
+PROTECTED int RECORD_IN_PTR::cmp_right (const char *str);
+PUBLIC int RECORD_IN_PTR::set (const RECORD *other);
+PUBLIC void RECORD_IN_PTR::sethostpart (const char *hostpart);
+PUBLIC void RECORD_IN_PTR::print (FILE_CFG *fout)const;
+PUBLIC int RECORD_IN_PTR::cmpip (const char *ip);
+PUBLIC RECORD_IN_NS::RECORD_IN_NS (const RECORD_PARSE&p);
+PUBLIC RECORD_IN_NS::RECORD_IN_NS (const char *_name, const char *_ns);
+PUBLIC RECORD_IN_NS::RECORD_IN_NS (void);
+PROTECTED int RECORD_IN_NS::cmpsame (const RECORD *other);
+PUBLIC int RECORD_IN_NS::set (const RECORD *other);
+PUBLIC void RECORD_IN_NS::sethostpart (const char *hostpart);
+PUBLIC int RECORD_IN_NS::cmp_left (const char *str);
+PUBLIC void RECORD_IN_NS::print (FILE_CFG *fout)const;
+PUBLIC RECORD_IN_CNAME::RECORD_IN_CNAME (const RECORD_PARSE&p);
+PUBLIC RECORD_IN_CNAME::RECORD_IN_CNAME (const char *cname,
+	 const char *realname);
+PUBLIC int RECORD_IN_CNAME::cmp_left (const char *str);
+PUBLIC void RECORD_IN_CNAME::sethostpart (const char *hostpart);
+PROTECTED int RECORD_IN_CNAME::cmpsame (const RECORD *other);
+PUBLIC void RECORD_IN_CNAME::print (FILE_CFG *fout)const;
+PUBLIC RECORD_IN_MX::RECORD_IN_MX (const RECORD_PARSE&p);
+PUBLIC RECORD_IN_MX::RECORD_IN_MX (const char *_mailname,
+	 int priority,
+	 const char *_servname);
+PROTECTED int RECORD_IN_MX::cmpsame (const RECORD *other);
+PUBLIC int RECORD_IN_MX::set (const RECORD *other);
+PUBLIC int RECORD_IN_MX::cmp_left (const char *str);
+PUBLIC void RECORD_IN_MX::print (FILE_CFG *fout)const;
+PUBLIC void RECORD_IN_MX::sethostpart (const char *hostpart);
+PUBLIC RECORD_IN_HINFO::RECORD_IN_HINFO (const char *_name,
+	 const char *_info);
+PROTECTED int RECORD_IN_HINFO::cmpsame (const RECORD *other);
+PUBLIC void RECORD_IN_HINFO::print (FILE_CFG *fout)const;
+PUBLIC RECORD_IN_RP::RECORD_IN_RP (const char *_name, const char *_rp);
+PROTECTED int RECORD_IN_RP::cmpsame (const RECORD *other);
+PUBLIC void RECORD_IN_RP::print (FILE_CFG *fout)const;
+PUBLIC RECORD_IN_TXT::RECORD_IN_TXT (const char *_name,
+	 const char *_txt);
+PROTECTED int RECORD_IN_TXT::cmpsame (const RECORD *other);
+PUBLIC void RECORD_IN_TXT::print (FILE_CFG *fout)const;
+PUBLIC RECORD_TTL::RECORD_TTL (const char *arg);
+PUBLIC RECORD_TTL::RECORD_TTL (void);
+PUBLIC void RECORD_TTL::print (FILE_CFG *fout)const;
+PUBLIC RECORD_PARSE::RECORD_PARSE (void);
+PUBLIC RECORD_INCLUDE::RECORD_INCLUDE (const char *_path);
+PUBLIC void RECORD_INCLUDE::print (FILE_CFG *fout)const;
+PUBLIC RECORD_END_INCLUDE::RECORD_END_INCLUDE (void);
+PUBLIC void RECORD_END_INCLUDE::print (FILE_CFG *)const;
+PUBLIC RECORD *RECORDS::getitem (int no)const;
+PUBLIC int RECORDS::save (TBFILE&tbf)const;
+/* secondary.cc 10/10/2001 16.34.30 */
 PRIVATE int SECONDARY::readzone (const char *named_dir, bool extract);
 PUBLIC SECONDARY::SECONDARY (const char *_domain,
 	 const char *_file,
@@ -320,86 +431,34 @@ PUBLIC SECONDARY::SECONDARY (void);
 PRIVATE void SECONDARY::setzonefile (SSTRING&file2,
 	 const char *named_dir)const;
 PUBLIC void SECONDARY::write (bool bind8,
-	 FILE *fout,
+	 FILE_CFG *fout,
 	 const char *named_dir)const;
-PUBLIC int SECONDARY::edit (DNS&dns);
+PROTECTED void ZONE::remove_empty (void);
+PUBLIC int SECONDARY::edit (SECONDARYS&secs, DNS&dns, bool isnew);
 PUBLIC SECONDARY *SECONDARYS::getitem (int no)const;
 PUBLIC SECONDARY *SECONDARYS::getitem (const char *name)const;
 PUBLIC void SECONDARYS::write (bool bind8,
-	 FILE *fout,
+	 FILE_CFG *fout,
 	 const char *named_dir)const;
 PUBLIC void SECONDARYS::edit (DNS&dns);
 PUBLIC SECONDARY *DNS::locate_secondary (const char *domain);
 PUBLIC int DNS::delsecondary (const char *domain);
-/* mod_dnsconf.cc 15/11/1999 14.57.54 */
-PUBLIC MODULE_dnsconf::MODULE_dnsconf (void);
-PUBLIC void MODULE_dnsconf::setmenu (DIALOG&dia, MENU_CONTEXT context);
-PUBLIC int MODULE_dnsconf::domenu (MENU_CONTEXT context,
-	 const char *key);
-PUBLIC int MODULE_dnsconf::dohtml (const char *key);
-PUBLIC int MODULE_dnsconf::probe (int , int , bool);
-PUBLIC int MODULE_dnsconf::message (const char *msg,
-	 int ,
-	 const char *[]);
-PUBLIC void MODULE_dnsconf::usage (SSTRINGS&tb);
-PUBLIC int MODULE_dnsconf::execmain (int argc, char *argv[]);
-/* vhostcomng.cc 08/10/1998 11.22.38 */
+PUBLIC int DNS::newsecondary (const char *domain,
+	 const char *primaries[],
+	 int nbprim);
+/* soa.cc 02/03/2001 17.00.24 */
+PUBLIC RECORD_IN_SOA::RECORD_IN_SOA (const RECORD_PARSE&p);
+PUBLIC RECORD_IN_SOA::RECORD_IN_SOA (void);
+PUBLIC void RECORD_IN_SOA::print (FILE_CFG *fout)const;
+PUBLIC void RECORD_IN_SOA::update (const char *name);
+/* tbfile.cc 19/11/2000 01.14.10 */
+PUBLIC TBFILE::TBFILE (const char *_defpath, bool _extract);
+PUBLIC TBFILE::~TBFILE (void);
+PUBLIC FILE_CFG *TBFILE::fopen (const char *fname, const char *mode);
+PUBLIC void TBFILE::fclose (void);
+/* vhostcomng.cc 18/09/2000 17.52.44 */
 PUBLIC VIRTUAL_COMNG::VIRTUAL_COMNG (DICTIONARY&_dict);
 PUBLIC void VIRTUAL_COMNG::setupdia (DIALOG&dia);
 PUBLIC int VIRTUAL_COMNG::save (PRIVILEGE *priv);
 PUBLIC int VIRTUAL_COMNG::validate (DIALOG&, int &nof);
 PUBLIC int VIRTUAL_COMNG::deluser (PRIVILEGE *);
-/* bind8.cc 13/12/1999 23.38.48 */
-PUBLIC BIND8_LEXPARSE::BIND8_LEXPARSE (CONFIG_FILE&_cfg,
-	 SSTRING&_dir);
-PUBLIC BIND8_LEXPARSE::~BIND8_LEXPARSE (void);
-PUBLIC bool BIND8_LEXPARSE::isok (void);
-PRIVATE int BIND8_LEXPARSE::fillbuf (void);
-PUBLIC const char *BIND8_LEXPARSE::gettoken (bool keepquote);
-PUBLIC const char *BIND8_LEXPARSE::gettoken (void);
-PUBLIC void BIND8_LEXPARSE::error (const char *ctl, ...);
-PUBLIC const char *BIND8_LEXPARSE::getarg1 (void);
-PUBLIC bool BIND8_LEXPARSE::getarg_yesno (void);
-PUBLIC bool BIND8_LEXPARSE::waserr (void);
-PUBLIC int BIND8_LEXPARSE::expect (const char *s);
-PUBLIC const char *BIND8_LEXPARSE::getcomment (void);
-/* acl.cc 27/03/1999 11.12.28 */
-PUBLIC ADDRESS_MATCH::ADDRESS_MATCH (void);
-PUBLIC ADDRESS_MATCH::ADDRESS_MATCH (const char *_ip,
-	 const char *comment);
-PUBLIC ADDRESS_MATCH::ADDRESS_MATCH (const char *_ip,
-	 const char *_netsize,
-	 const char *comment);
-PUBLIC int ADDRESS_MATCH::edit (const ACLS&acls);
-PUBLIC void ADDRESS_MATCH::write (FILE *fout);
-PUBLIC bool ADDRESS_MATCH::is_empty (void);
-PUBLIC bool ADDRESS_MATCH::is_none (void);
-PUBLIC bool ADDRESS_MATCH::is_any (void);
-PUBLIC bool ADDRESS_MATCH::is_localnet (void);
-PUBLIC ADDRESS_MATCH *ACL::getitem (int no)const;
-PUBLIC bool ACL::is_empty (int val)const;
-PUBLIC ACL::ACL (const char *_name);
-PUBLIC void ACL::write (FILE *fout);
-PUBLIC ACL *ACLS::getitem (int no)const;
-PUBLIC void ACLS::write (FILE *fout);
-PUBLIC int ACL::editname (void);
-PUBLIC int ACL::edit (DNS&dns, const ACLS&acls);
-PUBLIC int ACLS::edit (DNS&dns);
-PUBLIC int ACLS::locate (const char *name)const;
-PUBLIC bool ACLS::exist (const char *name)const;
-/* forward.cc 31/05/1999 20.39.08 */
-PUBLIC FORWARD::FORWARD (const char *_domain,
-	 const char **_ip_addr,
-	 int nb_ip);
-PUBLIC FORWARD::FORWARD (void);
-PUBLIC void FORWARD::write (bool bind8, FILE *fout)const;
-PUBLIC int FORWARD::edit (DNS&dns);
-PUBLIC FORWARD *FORWARDS::getitem (int no)const;
-PUBLIC FORWARD *FORWARDS::getitem (const char *name)const;
-PUBLIC void FORWARDS::write (bool bind8, FILE *fout)const;
-PUBLIC void FORWARDS::edit (DNS&dns);
-/* logging.cc 28/03/1999 11.25.22 */
-PUBLIC void UNKNOWN::add (const char *token, const char *comment);
-PUBLIC void UNKNOWN::write (FILE *fout, int level);
-PUBLIC UNKNOWN *UNKNOWNS::getitem (int no)const;
-PUBLIC void UNKNOWNS::write (FILE *fout, int level);
