@@ -1,4 +1,5 @@
 #include "dnsconf.h"
+#include <string.h>
 
 PUBLIC void UNKNOWN::add (const char *token, const char *comment)
 {
@@ -9,7 +10,7 @@ PUBLIC void UNKNOWN::add (const char *token, const char *comment)
 }
 
 
-PUBLIC void UNKNOWN::write (FILE *fout, int level)
+PUBLIC void UNKNOWN::write (FILE_CFG *fout, int level)
 {
 	if (tb.getnb() > 0){
 		for (int j=0; j<level; j++) fputc ('\t',fout);
@@ -52,7 +53,7 @@ PUBLIC UNKNOWN *UNKNOWNS::getitem (int no) const
 	return (UNKNOWN*)ARRAY::getitem (no);
 }
 
-PUBLIC void UNKNOWNS::write (FILE *fout, int level)
+PUBLIC void UNKNOWNS::write (FILE_CFG *fout, int level)
 {
 	for (int i=0; i<getnb(); i++){
 		getitem(i)->write(fout,level);
