@@ -28,23 +28,30 @@ PUBLIC TBFILE::~TBFILE()
 
 	Return the FILE handle or NULL if can't open.
 */
-PUBLIC FILE *TBFILE::fopen (
+PUBLIC FILE_CFG *TBFILE::fopen (
 	const char *fname,
 	const char *mode)
 {
+	FILE_CFG *ret = NULL;
 	char abspath[PATH_MAX];
 	if (fname[0] != '/'){
 		sprintf (abspath,"%s/%s",defpath,fname);
 		fname = abspath;
+	}else{
+		sprintf (abspath,"%s%s",linuxconf_getval(K_DNSCONF,K_CHROOT,""),fname);
+		fname = abspath;
 	}
-	CONFIG_FILE conf (fname,help_nil,CONFIGF_OPTIONNAL,subsys_dnsserv);
-	if (extract){
-		conf.extract();
-	}
-	FILE *ret = conf.fopen (mode);
-	if (ret != NULL){
-		tb[nbopen++] = ret;
-		cur = ret;
+	CONFIG_FILE *conf = new CONFIG_FILE(fname,help_nil,CONFIGF_OPTIONAL,subsys_dnsserv);
+	if (conf != NULL){
+		if (extract){
+			conf->extract();
+		}
+		ret = conf->fopen (mode);
+		if (ret != NULL){
+			tb[nbopen++] = ret;
+			cur = ret;
+		}
+		tbconf.add (conf);
 	}
 	return ret;
 }
