@@ -21,12 +21,18 @@ public:
 	int dohtml (const char *key);
 	int domenu (MENU_CONTEXT context,
 		 const char *key);
-	int execmain (int argc, char *argv[]);
+	int execmain (int argc,
+		 char *argv[],
+		 bool);
+	int message (const char *msg,
+		 int argc,
+		 const char *argv[]);
 	int probe (int level,
 		 int target,
 		 bool simulation);
 	void setmenu (DIALOG&dia,
 		 MENU_CONTEXT context);
+	void usage (SSTRINGS&tb);
     /*~PROTOEND~ MODULE_fetchmailconf */
 };
 
