@@ -26,15 +26,18 @@ public:
 	/*~PROTOBEG~ SERVER */
 public:
 	SERVER (const char *_server_name);
-	SERVER ( void );
+	SERVER (void);
 	int edit (void);
-	bool input_error( void );
-	int write ( int button );
-	int write_server( FILE *fout );
-	void write_item( FILE *fout, VIEWITEM *item );
 private:
 	void init (void);
 public:
+	bool input_error (void);
+	int write (int button);
+	void write_item (FILE_CFG *fout,
+		 const VIEWITEMS&items,
+		 VIEWITEM *item,
+		 bool comments_only);
+	int write_server (FILE_CFG *fout);
 	/*~PROTOEND~ SERVER */
 };
 
@@ -42,12 +45,12 @@ class SERVERLIST: public ARRAY {
 private:
 	/*~PROTOBEG~ SERVERLIST */
 public:
-	SERVERLIST( );
+	SERVERLIST (void);
 	int edit (void);
-	SERVER *select( void );
-	SERVER *getitem (const char *id) const;
-	SERVER *getitem (int negative) const;
+	SERVER *getitem (const char *server_name)const;
+	SERVER *getitem (int no)const;
 	void read (void);
+	SERVER *select (void);
 	/*~PROTOEND~ SERVERLIST */
 };
 
