@@ -58,6 +58,7 @@
 	#define FIREWALL_NONE
 #endif
 extern HELP_FILE help_ipfw;
+static HELP_FILE help_control ("firewall","control");
 static const char subsys_firewall[]="firewall";
 static LINUXCONF_SUBSYS sube (subsys_firewall
 	,P_MSG_U(M_FIREWALLSYS,"Firewalling rules"));
@@ -96,7 +97,6 @@ private:
 	virtual IPFW_RULE *newrule()=0;
 	virtual IPFW_RULE *newrule(const char *pt)=0;
 	virtual int save()=0;
-	virtual int kernelok()=0;
 	virtual int enable(bool doit, SSTRING *collect)=0;
 public:
 	virtual int disable(bool doit, SSTRING *collect)=0;
@@ -112,7 +112,9 @@ protected:
 	int savek (const char *key,
 		 const char *key_active);
 public:
-	int setup (bool doit, SSTRING *collect);
+	int setup (FIREWALL_SECTION section,
+		 bool doit,
+		 SSTRING *collect);
 	/*~PROTOEND~ IPFW_RULES */
 };
 class IPFW_RULES_FORWARD: public IPFW_RULES{
@@ -121,7 +123,6 @@ public:
 	IPFW_RULES_FORWARD (CONFDB *_db);
 	int disable (bool doit, SSTRING *collect);
 	int enable (bool doit, SSTRING *collect);
-	int kernelok (void);
 protected:
 	IPFW_RULE *newrule (const char *pt);
 	IPFW_RULE *newrule (void);
@@ -135,7 +136,6 @@ public:
 	IPFW_RULES_INPUT (CONFDB *_db);
 	int disable (bool doit, SSTRING *collect);
 	int enable (bool doit, SSTRING *collect);
-	int kernelok (void);
 protected:
 	IPFW_RULE *newrule (const char *pt);
 	IPFW_RULE *newrule (void);
@@ -149,7 +149,6 @@ public:
 	IPFW_RULES_OUTPUT (CONFDB *_db);
 	int disable (bool doit, SSTRING *collect);
 	int enable (bool doit, SSTRING *collect);
-	int kernelok (void);
 protected:
 	IPFW_RULE *newrule (const char *pt);
 	IPFW_RULE *newrule (void);
@@ -166,9 +165,6 @@ public:
 	int enable (bool, SSTRING *);
 protected:
 	void initrules (void);
-public:
-	int kernelok (void);
-protected:
 	IPFW_RULE *newrule (const char *pt);
 	IPFW_RULE *newrule (void);
 public:
@@ -196,8 +192,8 @@ PROTECTED void IPFW_RULES::init (
 	if (_db == NULL) _db = linuxconf_getdb();
 	db = _db;
 	SSTRINGS strs;
-	active = db->getvalnum (FIREWALL,key_active,0);
-	db->getall (FIREWALL,key,strs,0);
+	active = db->getvalnum (K_FIREWALL,key_active,0);
+	db->getall (K_FIREWALL,key,strs,0);
 	int nb = strs.getnb();
 	for (int i=0; i<nb; i++){
 		SSTRING *s = strs.getitem(i);
@@ -208,20 +204,20 @@ PROTECTED void IPFW_RULES::init (
 
 PUBLIC IPFW_RULES_FORWARD::IPFW_RULES_FORWARD(CONFDB *_db)
 {
-	init (_db,FORWARD,ACTIVEF);
+	init (_db,K_FORWARD,ACTIVEF);
 }
 
 PUBLIC IPFW_RULES_INPUT::IPFW_RULES_INPUT(CONFDB *_db)
 {
-	init (_db,BLOCK,ACTIVEB);
+	init (_db,K_BLOCK,ACTIVEB);
 }
 PUBLIC IPFW_RULES_OUTPUT::IPFW_RULES_OUTPUT(CONFDB *_db)
 {
-	init (_db,OUTPUT,ACTIVEO);
+	init (_db,K_OUTPUT,ACTIVEO);
 }
 PUBLIC IPFW_RULES_ACCT::IPFW_RULES_ACCT(CONFDB *_db)
 {
-	init (_db,ACCT,ACTIVEA);
+	init (_db,K_ACCT,ACTIVEA);
 }
 
 
@@ -231,8 +227,8 @@ PROTECTED int IPFW_RULES::savek(const char *key, const char *key_active)
 	if (perm_rootaccess("update firewalling configuration")){
 		int nb = getnb();
 		db->setcursys (subsys_firewall);
-		db->replace (FIREWALL,key_active,active);
-		db->removeall(FIREWALL,key);
+		db->replace (K_FIREWALL,key_active,active);
+		db->removeall(K_FIREWALL,key);
 		for (int i=0; i<nb; i++){
 			getitem(i)->save(db);
 		}
@@ -244,7 +240,7 @@ PROTECTED int IPFW_RULES::savek(const char *key, const char *key_active)
 
 PUBLIC int IPFW_RULES_FORWARD::save()
 {
-	return IPFW_RULES::savek (FORWARD,ACTIVEF);
+	return IPFW_RULES::savek (K_FORWARD,ACTIVEF);
 }
 
 PROTECTED IPFW_RULE *IPFW_RULES_FORWARD::newrule()
@@ -258,7 +254,7 @@ PROTECTED IPFW_RULE *IPFW_RULES_FORWARD::newrule(const char *pt)
 
 PUBLIC int IPFW_RULES_INPUT::save()
 {
-	return IPFW_RULES::savek (BLOCK,ACTIVEB);
+	return IPFW_RULES::savek (K_BLOCK,ACTIVEB);
 }
 PROTECTED IPFW_RULE *IPFW_RULES_INPUT::newrule()
 {
@@ -271,7 +267,7 @@ PROTECTED IPFW_RULE *IPFW_RULES_INPUT::newrule(const char *pt)
 
 PUBLIC int IPFW_RULES_OUTPUT::save()
 {
-	return IPFW_RULES::savek (OUTPUT,ACTIVEO);
+	return IPFW_RULES::savek (K_OUTPUT,ACTIVEO);
 }
 PROTECTED IPFW_RULE *IPFW_RULES_OUTPUT::newrule()
 {
@@ -283,7 +279,7 @@ PROTECTED IPFW_RULE *IPFW_RULES_OUTPUT::newrule(const char *pt)
 }
 PUBLIC int IPFW_RULES_ACCT::save()
 {
-	return IPFW_RULES::savek (ACCT,ACTIVEA);
+	return IPFW_RULES::savek (K_ACCT,ACTIVEA);
 }
 PROTECTED IPFW_RULE *IPFW_RULES_ACCT::newrule()
 {
@@ -396,22 +392,24 @@ PUBLIC int IPFW_RULES_INPUT::enable(
 		,"127.0.0.1","255.255.255.255","",""
 		,NULL
 		,false,false,bf);
-	return ipfw_append (doit,collect,IP_FW_APPEND_IN,bf) == -1
+	return ipfw_append (FIRESECT_INTERNAL,-1,doit,collect,IP_FW_APPEND_IN,bf,NULL,NULL) == -1
 		|| ipfw_policy (doit,collect,IP_FW_POLICY_IN,0) == -1
 		? -1 : 0;
 }
 /*
 	check if the kernel do support blocking
 */
-PUBLIC int IPFW_RULES_INPUT::kernelok()
+static int firewall_kernelok()
 {
 	int ret = 1;
-	if (!ip_block.exist() && !ip_input.exist() && !ip_fwchains.exist()){
+	if (!ip_fwchains.exist()){
+		net_prtlog (NETLOG_WHY,"Ipchain firewalling not installed, loading kernel module\n");
+		netconf_system_if ("modprobe","ipchains");
 		static char shutoff = 0;
-		if (!shutoff){
-			xconf_error (MSG_U(ERR_BLOCKING
-				,"The kernel does not support\n"
-				"IP_BLOCKING, reconfigure it"));
+		if (!simul_ison() && !ip_fwchains.exist() && !shutoff){
+			xconf_error (MSG_U(E_KERNELOK
+				,"The kernel does not support ipchain firewalling\n"
+				"reconfigure it"));
 			shutoff = 1;
 		}
 		ret = 0;
@@ -452,7 +450,7 @@ PUBLIC int IPFW_RULES_OUTPUT::enable(
 		,"127.0.0.1","255.255.255.255","",""
 		,NULL
 		,false,false,bf);
-	return ipfw_append (doit,collect,IP_FW_APPEND_OUT,bf) == -1
+	return ipfw_append (FIRESECT_INTERNAL,-1,doit,collect,IP_FW_APPEND_OUT,bf,NULL,NULL) == -1
 		|| ipfw_policy (doit,collect,IP_FW_POLICY_OUT,0) == -1
 		? -1 : 0;
 }
@@ -496,61 +494,6 @@ PUBLIC int IPFW_RULES_ACCT::enable(
 	return 0;
 }
 
-
-/*
-	check if the kernel do support forwarding
-*/
-PUBLIC int IPFW_RULES_FORWARD::kernelok()
-{
-	int ret = 1;
-	if (!ip_forward.exist() && !ip_fwchains.exist()){
-		static bool shutoff = false;
-		if (!shutoff){
-			xconf_error (MSG_U(ERR_FORWARDING
-				,"The kernel does not support\n"
-				"firewalling reconfigure it"));
-			shutoff = true;
-		}
-		ret = 0;
-	}
-	return ret;
-}
-/*
-	check if the kernel do support outputint
-*/
-PUBLIC int IPFW_RULES_OUTPUT::kernelok()
-{
-	int ret = 1;
-	if (!ip_output.exist() && !ip_fwchains.exist()){
-		static char shutoff = 0;
-		if (!shutoff){
-			xconf_error (MSG_U(ERR_OUTPUT
-				,"The kernel does not support\n"
-				"IP_OUTPUTING, reconfigure it"));
-			shutoff = 1;
-		}
-		ret = 0;
-	}
-	return ret;
-}
-/*
-	check if the kernel do support accounting
-*/
-PUBLIC int IPFW_RULES_ACCT::kernelok()
-{
-	int ret = 1;
-	if (!ip_acct.exist() && !ip_fwchains.exist()){
-		static char shutoff = 0;
-		if (!shutoff){
-			xconf_error (MSG_U(ERR_ACCT
-				,"The kernel does not support\n"
-				"IP_ACCOUNTING, reconfigure it"));
-			shutoff = 1;
-		}
-		ret = 0;
-	}
-	return ret;
-}
 /*
 	check if the kernel do support accounting
 */
@@ -587,6 +530,7 @@ static int cmp_by_netmask_to (const ARRAY_OBJ *o1, const ARRAY_OBJ *o2)
 	Apply the firewalling rules
 */
 PUBLIC int IPFW_RULES::setup(
+	FIREWALL_SECTION section,
 	bool doit,
 	SSTRING *collect)
 {
@@ -608,13 +552,16 @@ PUBLIC int IPFW_RULES::setup(
 		while trying to set the rules. Quite often, setting rules
 		involve DNS access.
 	*/
+	for (int i=0; i<getnb(); i++){
+		IPFW_RULE *r = getitem(i);
+		r->rulenum = i;
+		r->sectnum = section;
+	}
 	if (active){
-		if (!kernelok()
-			|| disable(doit,collect)==-1){
+		if (disable(doit,collect)==-1){
 			ret = -1;
 		}else{
 			ret = 0;
-			ipfw_heading(collect);
 			int nbr = getnb();
 			int i;
 			SSTRING errmsg;
@@ -622,6 +569,7 @@ PUBLIC int IPFW_RULES::setup(
 			// doing that too often
 			FWINFO_API *tbapi[MAX_API_PROVIDERS];
 			int nbapi = fwinfo_apis_init ("IPFWRULES::setup",tbapi);
+			bool some_excluded = false;
 			for (i=0; i<nbr; i++){
 				IPFW_RULE *r = getitem(i);
 				if (r->active){
@@ -629,17 +577,23 @@ PUBLIC int IPFW_RULES::setup(
 						|| r->solve(r->to,tbapi,nbapi)==-1){
 						r->solved_valid = false;
 						if (collect != NULL){
+							if (!some_excluded){
+								some_excluded = true;
+								collect->appendf ("%s\n"
+									,MSG_U(I_EXCLRULE,"Excluded: Some information are missing to process those rules"));
+							}
 							char buf1[10],buf2[100];
 							r->present (buf1,sizeof(buf1),buf2,sizeof(buf2));
-							collect->appendf ("%s: %s\n"
-								,MSG_U(I_EXCLRULE,"Excluded rule"),buf2);
+							collect->appendf ("\t%02d: %s\n",r->rulenum+1,buf2);
 						}
 					}else{
 						r->solved_valid = true;
 					}
 				}
 			}
+			if (some_excluded && collect != NULL) collect->append ("\n");
 			fwinfo_apis_end (tbapi,nbapi);
+			ipfw_heading(collect);
 			// Now, generate the rules from "from" to "to"
 			sort (cmp_by_netmask_from);
 			if (doit) initrules();
@@ -674,12 +628,15 @@ PUBLIC int IPFW_RULES::setup(
 #ifndef FIREWALL_NONE
 
 static int firewall_setup (
+	FIREWALL_SECTION section,
 	IPFW_RULES &rules,
-	CONFIG_FILE &f_current)
+	CONFIG_FILE &f_current,
+	bool &msgdone,
+	const char *chain)
 {
 	int ret = 0;
 	SSTRING collect;
-	if (rules.setup(0,&collect) != -1){
+	if (rules.setup(section,false,&collect) != -1){
 		SSTRING current;
 		/* #Specification: firewalling / update needed
 			Probing the firewalling information to find
@@ -704,7 +661,7 @@ static int firewall_setup (
 		*/
 		time_t boottime = time(NULL) - sys_uptime();
 		if (f_current.getdate() > boottime){
-			FILE *fin = f_current.fopen ("r");
+			FILE_CFG *fin = f_current.fopen ("r");
 			if (fin != NULL){
 				char buf[1000];
 				while (fgets(buf,sizeof(buf)-1,fin)!=NULL){
@@ -723,9 +680,9 @@ static int firewall_setup (
 				}
 			}else{
 				if (!simul_ison()){
-					ret = rules.setup(1,NULL);
+					ret = rules.setup(section,true,NULL);
 					if (ret != -1){
-						FILE *fout = f_current.fopen ("w");
+						FILE_CFG *fout = f_current.fopen ("w");
 						if (fout != NULL){
 							fputs (collect.get(),fout);
 							fclose (fout);
@@ -733,7 +690,14 @@ static int firewall_setup (
 					}
 				}
 			}
+			if (!msgdone){
+				net_prtlog (NETLOG_WHY,MSG_U(I_FWCHANGED
+					,"Firewall state does not match configuration\n"));
+				msgdone = true;
+			}
 			net_prtlog ("%s",collect.get());
+		}else{
+			chains_confirm (chain);
 		}
 	}
 	return ret;
@@ -746,10 +710,18 @@ static const char *tbmodname[]={
 	"ip_masq_raudio",
 	"ip_masq_quake",
 	"ip_masq_vdolive",
+	"ip_masq_autofw",
+	"ip_masq_mfw",
+	"ip_masq_portfw",
+	"ip_masq_user",
+	"ip_masq_ipsec",
+	"ip_masq_pptp",
 };
 #define NBMOD (sizeof(tbmodname)/sizeof(tbmodname[0]))
-static const char FWSPCMOD[]="FWSPCMOD";
-static const char IRCPORTS[]="IRCPORTS";
+static const char K_FWSPCMOD[]="FWSPCMOD";
+static const char K_IRCPORTS[]="IRCPORTS";
+static const char K_OTHERKERNMOD[]="otherkernelmod";
+static const char K_SMARTUPDATES[]="smartupdates";
 
 /*
 	Install the firewalling rules in the kernel.
@@ -769,22 +741,25 @@ int firewall_setup(CONFDB *db)
 				 "support firewalling properly. No rule will be activated."));
 		#else
 			if (perm_rootaccess("activating the firewalling configuration")){
-				if (ipfw_open() != -1
-					&& firewall_setup (frules,f_current_forwd) != -1
-					&& firewall_setup (brules,f_current_block) != -1
-					&& firewall_setup (orules,f_current_output) != -1
-					&& firewall_setup (arules,f_current_acct) != -1){
+				bool msgdone = false;
+				if (firewall_kernelok() != -1
+					&& ipfw_open() != -1
+					&& firewall_setup (FIRESECT_FORWARD,frules,f_current_forwd,msgdone,"forward") != -1
+					&& firewall_setup (FIRESECT_INPUT,brules,f_current_block,msgdone,"input") != -1
+					&& firewall_setup (FIRESECT_OUTPUT,orules,f_current_output,msgdone,"output") != -1
+					&& firewall_setup (FIRESECT_ACCT,arules,f_current_acct,msgdone,"acct") != -1){
 					ipfw_close();
+					if (!simul_ison()) module_sendmessage ("firewall-updated",0,NULL);
 					// Load the module if needed
 					for (unsigned i=0; i<NBMOD; i++){
 						const char *name = tbmodname[i];
-						int needed = linuxconf_getvalnum (FWSPCMOD,name,0);
+						int needed = linuxconf_getvalnum (K_FWSPCMOD,name,0);
 						if (needed && !module_is_loaded(name)){
 							char buf[200];
 							strcpy (buf,name);
 							if (strcmp(name,"ip_masq_irc")==0){
 								SSTRING ircports;
-								ircports.setfrom(linuxconf_getval (FWSPCMOD,IRCPORTS));
+								ircports.setfrom(linuxconf_getval (K_FWSPCMOD,K_IRCPORTS));
 								if (!ircports.is_empty()){
 									sprintf (buf,"%s ports=%s",name,ircports.get());
 								}
@@ -792,6 +767,16 @@ int firewall_setup(CONFDB *db)
 							netconf_system_if ("modprobe",buf);
 						}
 					}
+					SSTRINGS tbmod;
+					int nbmod = linuxconf_getall (K_FIREWALL,K_OTHERKERNMOD,tbmod,false);
+					for (int i=0; i<nbmod; i++){
+						const char *s = tbmod.getitem(i)->get();
+						SSTRING mod;
+						mod.copyword (s);
+						if (!module_is_loaded(mod.get())){
+							netconf_system_if ("modprobe",s);
+						}
+					}
 					ret = 0;
 				}
 			}
@@ -816,6 +801,7 @@ int firewall_reset(CONFDB *db)
 		f_current_block.unlink();
 		f_current_output.unlink();
 		f_current_acct.unlink();
+		chains_resetfw();
 		if (ipfw_open() != -1
 			&& frules.disable(1,NULL) != -1
 			&& brules.disable(1,NULL) != -1
@@ -867,6 +853,13 @@ static int firewall_chkirc (SSTRING &val)
 	}
 	return ret;
 }
+/*
+	Return true if the firewall uses smart updates
+*/
+bool firewall_smartupdates()
+{
+	return linuxconf_getvalnum (K_FIREWALL,K_SMARTUPDATES,0) != 0;
+}
 
 /*
 	Edit the default configuration of the firewall
@@ -878,6 +871,7 @@ void firewall_editc(CONFDB *db)
 	IPFW_RULES_OUTPUT orules(db);
 	IPFW_RULES_ACCT  arules(db);
 	DIALOG dia;
+	dia.newf_title (MSG_U(T_RULESCTL,"Rules control"),1,"",MSG_R(T_RULESCTL));
 	dia.newf_chk (MSG_U(F_BRULES,"Inputing rules"),brules.active
 		,MSG_U(I_AREACTIVE,"are active"));
 	dia.newf_chk (MSG_U(F_FRULES,"forwarding rules"),frules.active
@@ -886,7 +880,8 @@ void firewall_editc(CONFDB *db)
 		,MSG_R(I_AREACTIVE));
 	dia.newf_chk (MSG_U(F_ARULES,"accounting rules"),arules.active
 		,MSG_R(I_AREACTIVE));
-	dia.newf_title ("",MSG_U(T_KERNMOD,"Special kernel modules"));
+	dia.newf_title (MSG_U(T_KERNMOD,"Special kernel modules"),1,""
+		,MSG_R(T_KERNMOD));
 
 	struct modinfo{
 		const char *title;
@@ -899,20 +894,41 @@ void firewall_editc(CONFDB *db)
 		MSG_U(F_MOD_RAUDIO,"Real Audio"),
 		MSG_U(F_MOD_QUAKE,"Quake"),
 		MSG_U(F_MOD_VDOLIVE,"Vdolive"),
+		MSG_U(F_MOD_AUTOFW,"Auto Forwarding"),
+		MSG_U(F_MOD_MFW,"Firewall Mark Forwarding"),
+		MSG_U(F_MOD_PORTFW,"Port Forwarding"),
+		MSG_U(F_MOD_USER,"Space Control"),
+		MSG_U(F_MOD_IPSEC,"IpSec VPN support"),
+		MSG_U(F_MOD_PPTP,"PPTP VPN support"),
 	};
 	char tbmod[NBMOD];
 	SSTRING ircports;
-	ircports.setfrom(linuxconf_getval (FWSPCMOD,IRCPORTS));
+	ircports.setfrom(linuxconf_getval (K_FWSPCMOD,K_IRCPORTS));
 	int f_irc = 0;
 	for (unsigned i=0; i<NBMOD; i++){
-		tbmod[i] = linuxconf_getvalnum (FWSPCMOD,tbmodname[i],0);
+		tbmod[i] = linuxconf_getvalnum (K_FWSPCMOD,tbmodname[i],0);
 		dia.newf_chk (tbt[i],tbmod[i],MSG_U(F_REQUIRED,"required"));
 		if (i==2){
 			f_irc = dia.getnb();
 			dia.newf_str (MSG_U(F_IRCPORTS,"Irc ports(opt)"),ircports);
 		}
 	}
-
+	dia.newf_title (MSG_U(T_OTHERKERNMOD,"Extra kernel modules"),1,""
+		,MSG_R(T_OTHERKERNMOD));
+	SSTRINGS tb;
+	linuxconf_getall (K_FIREWALL,K_OTHERKERNMOD,tb,true);
+	for (int i=0; i<3; i++) tb.add (new SSTRING);
+	for (int i=0; i<tb.getnb(); i++){
+		dia.newf_str (i==0
+			? MSG_U(F_KMODOPT,"Kernel module + options")
+			: ""
+			,*tb.getitem(i));
+	}
+	dia.newf_title (MSG_U(T_FEATURES,"Features"),1,""
+		,MSG_R(T_FEATURES));
+	char smart = firewall_smartupdates() ? 1 : 0;
+	dia.newf_chk (MSG_U(F_SMARTUPDATES,"Update the kernel rules"),smart
+		,MSG_U(I_SMARTUPDATES,"gracefully"));
 	int nof = 0;
 	while (1){
 		if (dia.edit(
@@ -922,7 +938,7 @@ void firewall_editc(CONFDB *db)
 				,"You are allowed to disable/enable all rule sets\n"
 				 "without deleting them. Beware that activating\n"
 				 "an empty rule set is closing all doors!!!")
-			,help_ipfw,nof)!=MENU_ACCEPT){
+			,help_control,nof)!=MENU_ACCEPT){
 			break;
 		}else{
 			if (firewall_chkirc (ircports)==-1){
@@ -934,9 +950,12 @@ void firewall_editc(CONFDB *db)
 			}else{
 				linuxconf_setcursys (subsys_firewall);
 				for (unsigned s=0; s<NBMOD; s++){
-					linuxconf_replace (FWSPCMOD,tbmodname[s],tbmod[s]);
+					linuxconf_replace (K_FWSPCMOD,tbmodname[s],tbmod[s]);
 				}
-				linuxconf_replace (FWSPCMOD,IRCPORTS,ircports);
+				linuxconf_replace (K_FWSPCMOD,K_IRCPORTS,ircports);
+				tb.remove_empty();
+				linuxconf_replace (K_FIREWALL,K_OTHERKERNMOD,tb);
+				linuxconf_replace (K_FIREWALL,K_SMARTUPDATES,smart);
 				brules.save();
 				frules.save();
 				orules.save();
