@@ -6,7 +6,7 @@ void fw_status()
 {
 	STATUS_API *api = status_api_init("firewall");
 	if (api != NULL){
-		api->showcommand (MSG_R(M_FW_STATUS),"ipchains","-L -n");
+		api->showcommand (MSG_R(M_FW_STATUS),"ipchains","-L -nv");
 		status_api_end (api);
 	}
 }
