@@ -26,6 +26,7 @@
 #include <ctype.h>
 #include <string.h>
 #include "firewall.h"
+#include "firewall.m"
 #include <netconf.h>
 #include <daemoni.h>
 #include <misc.h>
@@ -49,12 +50,16 @@ extern HELP_FILE help_ipfw;
 static CONFIG_FILE f_script_ipchain ("/var/run/ipchain.sh",help_ipfw
 	,CONFIGF_OPTIONAL|CONFIGF_GENERATED
 	,"root","root",0700);
-static FILE *ipchain_fout = NULL;
+static FILE_CFG *ipchain_fout = NULL;
 static bool ipchain_somecmds = false;
+static const char *ipchainpath = NULL;
+static const char *flushchains = NULL;
 
-static void ipfw_appendcmd (const char *ctl, ...)
+/*
+	Locate the ipchain command and return its path
+*/
+const char *ipfw_setipchainpath()
 {
-	static const char *ipchainpath = NULL;
 	if (ipchainpath == NULL){
 		DAEMON_INTERNAL *dae = daemon_find ("ipchains");
 		if (dae != NULL){
@@ -62,18 +67,35 @@ static void ipfw_appendcmd (const char *ctl, ...)
 		}else{
 			ipchainpath = "/sbin/ipchains";
 		}
+		dae = daemon_find ("firewall-flushchains");
+		if (dae != NULL){
+			flushchains = dae->getpath();
+		}else{
+			flushchains = "/usr/lib/linuxconf/lib/flushchains";
+		}
+	}
+	return ipchainpath;
+}
+/*
+	Add an ipchain command line to /var/run/ipchains.sh
+	The line does not include the path of the ipchains command.
+*/
+void ipfw_appendcmd (const char *ctl, ...)
+{
+	if (ipchain_fout != NULL){
+		ipfw_setipchainpath();
+		char buf[1000];
+		va_list list;
+		va_start (list,ctl);
+		strcpy (buf,ipchainpath);
+		int len = strlen(buf);
+		buf[len++] = ' ';
+		
+		ipchain_somecmds = true;
+		vsnprintf (buf+len,sizeof(buf)-1-len,ctl,list);
+		va_end (list);
+		fprintf (ipchain_fout,"%s\n",buf);
 	}
-	char buf[1000];
-	va_list list;
-	va_start (list,ctl);
-	strcpy (buf,ipchainpath);
-	int len = strlen(buf);
-	buf[len++] = ' ';
-	
-	ipchain_somecmds = true;
-	vsnprintf (buf+len,sizeof(buf)-1-len,ctl,list);
-	va_end (list);
-	fprintf (ipchain_fout,"%s\n",buf);
 }
 
 
@@ -95,12 +117,24 @@ static const char *ipfw_u2a (
 	return buf;
 }
 
+static const char *ipfw_u2a_not (
+	unsigned long nip,		// Network IP number
+	bool notflag,
+	char *buf)				// Will receive the Ascii IP number
+{
+	char *ret = buf;
+	if (notflag) *buf++ = '!';
+	ipfw_u2a (nip,buf);
+	return ret;
+}
+	
+
 void ipfw_heading(
 	SSTRING *collect)
 {
 	if (collect != NULL){
 			char buffer[300];
-			sprintf(buffer,"%s %5s %15s/%-15s -> %15s/%-15s %-15s %-15s\n"
+			sprintf(buffer,"%s %5s  %15s/%-15s ->  %15s/%-15s %-15s %-15s\n"
 				,"cmd","proto"
 				,"network","netmask"
 				,"network","netmask"
@@ -129,61 +163,106 @@ struct IPCHAIN_INFO{
 	char iface_verb[100];
 };
 
+/*
+	Check if a string contains a valid netmask.
+	The string may end with a '\0' or a comma.
+*/
+bool ipfw_validmask (const char *msk, unsigned long &bmsk)
+{
+	char tmp[strlen(msk)+1];
+	strcpy (tmp,msk);
+	char *pt = strchr(tmp,',');
+	if (pt != NULL) *pt = '\0';
+	bmsk = 0xffffffff;
+	bool ret = ipnum_validmask (tmp);
+	if (ret){
+		bmsk = ipnum_aip2l (tmp);
+	}
+	return ret;
+}
+
+
+static int nomark;		// mark id for ipchain rules
+
+
 static void ipfw_formatcmd (
+	FIREWALL_SECTION sectnum,
+	int rulenum,
+	const char *dispatch,
 	struct ip_fw &b,
 	IPCHAIN_INFO &inf,
 	const char *source_port,
 	const char *dest_port)
 {
+	const char *interface = b.fw_vianame;
 	char b1[LEN_IP_ASC],b2[LEN_IP_ASC],b3[LEN_IP_ASC],b4[LEN_IP_ASC];
 	char jump_verb[1000];
 	jump_verb[0] = '\0';
 	if (inf.verb != NULL){
 		snprintf (jump_verb,sizeof(jump_verb)-1,"--jump %s",inf.verb);
 	}
-	ipfw_appendcmd ("%s%s--append %s %s %s --source %s/%s %s"
-		 " --destination %s/%s %s %s"
+	char cmd[1000];
+	snprintf (cmd,sizeof(cmd)-1,"%s%s %s %s --source %s%s/%s %s%s"
+		 " --destination %s%s/%s %s%s %s"
 		,(b.fw_flg & IP_FW_F_PRN) ? "--log " : ""
 		,(b.fw_flg & IP_FW_F_TCPSYN) ? "--syn " : ""
-		,inf.chain,jump_verb,inf.proto_cmd
+		,jump_verb,inf.proto_cmd
+		,b.not_src ? "! " : ""
 		,ipfw_u2a(b.fw_src.s_addr,b1),ipfw_u2a(b.fw_smsk.s_addr,b2)
+		,b.not_srcport ? "! " : ""
 		,source_port
+		,b.not_dst ? "! " : ""
 		,ipfw_u2a(b.fw_dst.s_addr,b3),ipfw_u2a(b.fw_dmsk.s_addr,b4)
+		,b.not_dstport ? "! " : ""
 		,dest_port
 		,inf.iface_verb);
-
+	strip_end (cmd);	// In smart update mode, we compare commands
+						// and trailing blanks may fool us
+	bool noorder = b.fw_dmsk.s_addr == 0xffffffffl
+		&& b.fw_smsk.s_addr == 0xffffffffl;
+	chains_format (sectnum,rulenum,dispatch,inf.chain
+		,inf.proto
+		,b.fw_src.s_addr,b.fw_smsk.s_addr,source_port
+		,b.fw_dst.s_addr,b.fw_dmsk.s_addr,dest_port
+		,interface,cmd,noorder);
 }
 
 static void ipfw_formatdstports (
+	FIREWALL_SECTION sectnum,
+	int rulenum,
+	const char *dispatch,
 	struct ip_fw &b,
 	IPCHAIN_INFO &inf,
 	const char *source_port)
 {
-	int noport = b.fw_nsp;
-	int ndp = b.fw_ndp;
-	if (b.fw_flg & IP_FW_F_DRNG){
-		char dest_port[1000];
-		sprintf(dest_port, "%u:%u",b.fw_pts[noport],b.fw_pts[noport+1]);
-		noport += 2;
-		ndp -= 2;
-		ipfw_formatcmd (b,inf,source_port,dest_port);
+	int ndp = b.dst_ports.nb;
+	if (b.dst_ports.from != 0){
+		char dest_port[100];
+		snprintf(dest_port,sizeof(dest_port)-1, "%u:%u"
+			,b.dst_ports.from,b.dst_ports.to);
+		ipfw_formatcmd (sectnum,rulenum,dispatch,b,inf,source_port,dest_port);
 	}
 	for (int d=0; d<ndp; d++){
-		char bufp[1000];
-		ipfw_formatcmd (b,inf,source_port
-			,ipfw_getport(b.fw_pts[noport++],inf.proto,bufp));
+		char bufp[1000],dest_port[100];
+		snprintf (dest_port,sizeof(dest_port)-1,"%s%s"
+			,b.dst_ports.tb[d].not_flag ? "! " : ""
+			,ipfw_getport(b.dst_ports.tb[d].port,inf.proto,bufp));
+		ipfw_formatcmd (sectnum,rulenum,dispatch,b,inf,source_port,dest_port);
 	}
-	if (b.fw_ndp == 0){
-		ipfw_formatcmd (b,inf,source_port,"");
+	if (ndp == 0 && b.dst_ports.from == 0){
+		ipfw_formatcmd (sectnum,rulenum,dispatch,b,inf,source_port,"");
 	}
 }
 
-
 int ipfw_append (
+	FIREWALL_SECTION sectnum,
+	int rulenum,
 	bool doit,
 	SSTRING *collect,
 	int command,
-	struct ip_fw &b)
+	struct ip_fw &b,
+	const char *redirhost,
+	const char *dispatch)	// Chain dispatch
 {
 	int ret = 0;
 	if (collect != NULL){
@@ -196,8 +275,6 @@ int ipfw_append (
 			content of those file and tell if something as to be done.
 		*/
 		#ifndef FIREWALL_NONE
-			char buffer[300];
-			char b1[LEN_IP_ASC],b2[LEN_IP_ASC],b3[LEN_IP_ASC],b4[LEN_IP_ASC];
 
 			const char *verb = "dny";
 			if (b.fw_flg & IP_FW_F_MASQ){
@@ -210,14 +287,7 @@ int ipfw_append (
 				verb = "rjc";
 			}
 
-			const char *proto = "All";
-			if ((b.fw_flg & IP_FW_F_KIND) == IP_FW_F_TCP){
-				proto = "tcp";
-			}else if ((b.fw_flg & IP_FW_F_KIND) == IP_FW_F_UDP){
-				proto = "udp";
-			}else if ((b.fw_flg & IP_FW_F_KIND) == IP_FW_F_ICMP){
-				proto = "icmp";
-			}
+			const char *proto = b.protocol;
 
 			char iface[LEN_IP_ASC];
 			if (b.fw_vianame[0] != '\0'){
@@ -228,87 +298,74 @@ int ipfw_append (
 				ipfw_u2a (b.fw_via.s_addr,iface);
 			}
 
-			int len=sprintf(buffer,"%s %-5s %15s/%-15s -> %15s/%-15s %-15s"
+			char buffer[300];
+			char b1[LEN_IP_ASC],b2[LEN_IP_ASC],b3[LEN_IP_ASC],b4[LEN_IP_ASC];
+			int len=snprintf(buffer,sizeof(buffer)-1
+				,"%s %-5s %16s/%-15s -> %16s/%-15s %-15s"
 				,verb,proto
-				,ipfw_u2a(b.fw_src.s_addr,b1),ipfw_u2a(b.fw_smsk.s_addr,b2)
-				,ipfw_u2a(b.fw_dst.s_addr,b3),ipfw_u2a(b.fw_dmsk.s_addr,b4)
+				,ipfw_u2a_not(b.fw_src.s_addr,b.not_src,b1),ipfw_u2a(b.fw_smsk.s_addr,b2)
+				,ipfw_u2a_not(b.fw_dst.s_addr,b.not_dst,b3),ipfw_u2a(b.fw_dmsk.s_addr,b4)
 				,iface);
 			//len+=sprintf(buffer+len,"%u %u %-9lu %-9lu"
 			//	,b.fw_nsp,b.fw_ndp, b.fw_pcnt,b.fw_bcnt);
 
-			int noport = 0;
 			len += sprintf (buffer+len," [");
-			int nsp = b.fw_nsp;
-			if (b.fw_flg & IP_FW_F_SRNG){
-				len+=sprintf(buffer+len, " %u:%u",b.fw_pts[0],b.fw_pts[1]);
-				noport = 2;
-				nsp -= 2;
+			int nsp = b.src_ports.nb;
+			if (b.src_ports.from != 0){
+				len+=sprintf(buffer+len, " %u:%u",b.src_ports.from,b.src_ports.to);
 			}
 			for (int s=0; s<nsp; s++){
 				char bufp[100];
-				len += sprintf (buffer+len," %s"
-					,ipfw_getport(b.fw_pts[noport++],proto,bufp));
+				len += sprintf (buffer+len," %s%s"
+					,b.src_ports.tb[s].not_flag ? "! " : ""
+					,ipfw_getport(b.src_ports.tb[s].port,proto,bufp));
 			}
 			len += sprintf (buffer+len," ] -> [");
-			int ndp = b.fw_ndp;
-			if (b.fw_flg & IP_FW_F_DRNG){
-				len+=sprintf(buffer+len, " %u:%u",b.fw_pts[noport]
-					,b.fw_pts[noport+1]);
-				noport += 2;
-				ndp -= 2;
+			int ndp = b.dst_ports.nb;
+			if (b.dst_ports.from != 0){
+				len+=sprintf(buffer+len, " %u:%u",b.dst_ports.from
+					,b.dst_ports.to);
 			}
 			for (int d=0; d<ndp; d++){
 				char bufp[100];
-				len += sprintf (buffer+len," %s"
-					,ipfw_getport(b.fw_pts[noport++],proto,bufp));
+				len += sprintf (buffer+len," %s%s"
+					,b.dst_ports.tb[d].not_flag ? "! " : ""
+					,ipfw_getport(b.dst_ports.tb[d].port,proto,bufp));
 			}
 			len += sprintf (buffer+len," ]");
 
-			if (b.fw_flg & IP_FW_F_REDIR){
+			if (b.redir_port != 0){
 				char bufp[100];
 				len += sprintf (buffer+len," => %s"
-					,ipfw_getport(b.fw_pts[noport++],proto,bufp));
+					,ipfw_getport(b.redir_port,proto,bufp));
 			}
-			len += sprintf(buffer+len, " A%02X X%02X", b.fw_tosand, b.fw_tosxor);
+			//len += sprintf(buffer+len, " A%02X X%02X", b.fw_tosand, b.fw_tosxor);
 
 			if (b.fw_flg & IP_FW_F_PRN){
 				len += sprintf (buffer+len," logging");
 			}
 			strcpy (buffer+len,"\n");
-			collect->append (buffer);
+			if (dispatch != NULL && dispatch[0] != '\0'){
+				collect->appendf ("%s Dispatch %s",buffer,dispatch);
+			}else{
+				collect->append (buffer);
+			}
 		#endif
 	}
 	if (doit){
 		if (kernel_newer (2,2,0)){
 			char redir_verb[100];
+			int mark = nomark;
 
 			IPCHAIN_INFO inf;
 			inf.proto_cmd = "";
-			inf.proto = "All";
-			if ((b.fw_flg & IP_FW_F_KIND) == IP_FW_F_TCP){
-				inf.proto_cmd = "--proto TCP";
-				inf.proto = "tcp";
-			}else if ((b.fw_flg & IP_FW_F_KIND) == IP_FW_F_UDP){
-				inf.proto_cmd = "--proto UDP";
-				inf.proto = "udp";
-			}else if ((b.fw_flg & IP_FW_F_KIND) == IP_FW_F_ICMP){
-				inf.proto_cmd = "--proto ICMP";
-				inf.proto = "icmp";
+			inf.proto = b.protocol;
+			SSTRING proto_cmd;
+			if (strcasecmp(b.protocol,"all")!=0){
+				proto_cmd.setfromf ("--proto %s",b.protocol);
+				inf.proto_cmd = proto_cmd.get();
 			}
 			inf.verb = "DENY";
-			if (b.fw_flg & IP_FW_F_MASQ){
-				inf.verb = "MASQ";
-			}else if (b.fw_flg & IP_FW_F_REDIR){
-				char bufp[100];
-				int noport = b.fw_nsp + b.fw_ndp;
-				snprintf (redir_verb,sizeof(redir_verb)-1,"REDIRECT %s"
-					,ipfw_getport(b.fw_pts[noport],inf.proto,bufp));
-				inf.verb = redir_verb;
-			}else if (b.fw_flg & IP_FW_F_ACCEPT){
-				inf.verb = "ACCEPT";
-			}else if (b.fw_flg & IP_FW_F_ICMPRPL){
-				inf.verb = "REJECT";
-			}
 			inf.chain = NULL;
 			if (command == IP_FW_APPEND_IN){
 				inf.chain = "input";
@@ -322,6 +379,25 @@ int ipfw_append (
 				inf.verb = NULL;
 			}
 
+			if (b.fw_flg & IP_FW_F_MASQ){
+				inf.verb = "MASQ";
+			}else if (b.fw_flg & IP_FW_F_REDIR){
+				if (redirhost == NULL){
+					char bufp[100];
+					snprintf (redir_verb,sizeof(redir_verb)-1,"REDIRECT %s"
+						,ipfw_getport(b.redir_port,inf.proto,bufp));
+				}else{
+					snprintf (redir_verb,sizeof(redir_verb)-1,"ACCEPT --mark %d"
+						,mark);
+					nomark++;
+				}
+				inf.verb = redir_verb;
+			}else if (b.fw_flg & IP_FW_F_ACCEPT){
+				inf.verb = "ACCEPT";
+			}else if (b.fw_flg & IP_FW_F_ICMPRPL){
+				inf.verb = "REJECT";
+			}
+
 			inf.iface_verb[0] = '\0';
 			if (b.fw_vianame[0] != '\0'){
 				sprintf (inf.iface_verb,"--interface %s",b.fw_vianame);
@@ -333,22 +409,40 @@ int ipfw_append (
 
 
 
-			int noport = 0;
-			int nsp = b.fw_nsp;
-			if (b.fw_flg & IP_FW_F_SRNG){
+			if (b.src_ports.from != 0){
 				char source_port[1000];
-				sprintf(source_port, "%u:%u",b.fw_pts[0],b.fw_pts[1]);
-				noport = 2;
-				nsp -= 2;
-				ipfw_formatdstports(b,inf,source_port);
+				sprintf(source_port, "%u:%u",b.src_ports.from,b.src_ports.to);
+				ipfw_formatdstports(sectnum,rulenum,dispatch,b,inf,source_port);
 			}
+			int nsp = b.src_ports.nb;
 			for (int s=0; s<nsp; s++){
-				char bufp[100];
-				ipfw_formatdstports (b,inf
-					,ipfw_getport(b.fw_pts[noport+s],inf.proto,bufp));
+				char bufp[100],source_port[100];
+				snprintf (source_port,sizeof(source_port)-1,"%s%s"
+					,b.src_ports.tb[s].not_flag ? "! " : ""
+					,ipfw_getport(b.src_ports.tb[s].port,inf.proto,bufp));
+				ipfw_formatdstports (sectnum,rulenum,dispatch,b,inf
+					,source_port);
 			}
-			if (b.fw_nsp == 0){
-				ipfw_formatdstports (b,inf,"");
+			if (nsp == 0 && b.src_ports.from == 0){
+				ipfw_formatdstports (sectnum,rulenum,dispatch,b,inf,"");
+			}
+			if ((b.fw_flg & IP_FW_F_REDIR) != 0
+				&& redirhost != NULL
+				&& ipchain_fout != NULL){
+				static const char *ipmasqadmpath = NULL;
+				if (ipmasqadmpath == NULL){
+					DAEMON_INTERNAL *dae = daemon_find ("ipmasqadm");
+					if (dae != NULL){
+						ipmasqadmpath = dae->getpath();
+					}else{
+						ipmasqadmpath = "/usr/sbin/ipmasqadm";
+					}
+				}
+				if (mark == 1) fprintf (ipchain_fout,"%s mfw -F\n",ipmasqadmpath);
+				fprintf (ipchain_fout,"%s mfw -A -m %d -r %s %d\n"
+					,ipmasqadmpath,mark,redirhost
+					,b.redir_port);
+
 			}
 			ret = 0;
 		}else{
@@ -389,6 +483,10 @@ int ipfw_flush (
 					assert(0);
 				}
 				ipfw_appendcmd ("--flush %s",chain);
+				if (ipchain_fout != NULL){
+					fprintf (ipchain_fout,"%s %s %c-\n"
+						,flushchains,ipchainpath,toupper(chain[0]));
+				}
 			}
 			ret = 0;
 		}else{
@@ -481,6 +579,8 @@ int ipfw_policy (
 int ipfw_open ()
 {
 	int ret = -1;
+	nomark=1;
+	chains_reset();
 	if (kernel_newer (2,2,0)){
 		ipchain_somecmds = false;
 		ipchain_fout = f_script_ipchain.fopen ("w");
@@ -500,10 +600,15 @@ void ipfw_close()
 {
 	if (kernel_newer (2,2,0)){
 		if (ipchain_fout != NULL){
+			fputs ("exit 0\n",ipchain_fout);
 			fclose (ipchain_fout);
+			ipchain_fout = NULL;
 			if (ipchain_somecmds){
-				netconf_system (15,"/var/run/ipchain.sh");
+				if (chains_write()==-1){
+					netconf_system (60,f_script_ipchain.getpath());
+				}
 			}
+			chains_reset();
 		}
 	}else{
 		close (sockfd);
@@ -529,23 +634,22 @@ static int ipfw_a2ip (const char *adr, struct in_addr &ina)
 
 static int ipfw_setrange (
 	const char *range,
-	struct ip_fw &bf,
-	int flag,
-	int &noport,
-	unsigned short &nb)
+	struct ip_fw_ports &bf)
 {
 	int ret = 0;
+	bf.from = bf.to = 0;
 	const char *s = str_skip (range);
 	if (s[0] != '\0'){
-		bf.fw_flg |= flag;
-		bf.fw_pts[noport++] = atoi(s);
+		bf.from = atoi(s);
 		s = str_skipdig (s);
 		s = str_skip (s);
 		if (*s == ':'){
 			s++;
 			s = str_skip (s);
-			bf.fw_pts[noport++] = atoi(s);
-			nb = 2;
+			bf.to = atoi(s);
+			if (bf.to < bf.from){
+				ret = -1;
+			}
 		}else{
 			ret = -1;
 		}
@@ -559,19 +663,33 @@ static int ipfw_setrange (
 */
 int ipfw_checkrange (const SSTRING &range)
 {
-	struct ip_fw bf;
-	int noport = 0;
-	unsigned short nb = 0;
-	return ipfw_setrange (range.get(),bf,0,noport,nb);
+	struct ip_fw_ports bf;
+	return ipfw_setrange (range.get(),bf);
 }
 
-
+/*
+	Convert a port into numeric form
+*/
+static int ipfw_setport (
+	const char *portstr,
+	unsigned short &port,
+	const char *proto)
+{
+	int ret = 0;
+	struct servent *serv = getservbyname (portstr,proto);
+	if (serv != NULL){
+		port = ntohs(serv->s_port);
+	}else if (str_checkdig(portstr)){
+		port = atoi(portstr);
+	}else{
+		ret = -1;
+	}
+	return ret;
+}
 
 static int ipfw_setports (
 	const char *ports,
-	struct ip_fw &bf,
-	unsigned short &nb,
-	int &noport,
+	struct ip_fw_ports &bf,
 	const char *proto)
 {
 	int ret = 0;
@@ -582,27 +700,28 @@ static int ipfw_setports (
 			ret = -1;
 		}
 	}else{
+		int noport = 0;
 		while (1){
 			ports = str_skip (ports);
 			if (ports[0] == '\0'){
 				break;
 			}else{
+				bool not_flag = false;
+				if (ports[0] == '!'){
+					not_flag = true;
+					ports = str_skip(ports+1);
+				}
 				char word[200];
 				ports = str_copyword(word,ports,sizeof(word));
-				int port = -1;
-				struct servent *serv = getservbyname (word,proto);
-				if (serv != NULL){
-					port = ntohs(serv->s_port);
-				}else if (str_checkdig(word)){
-					port = atoi(word);
-				}else{
-					ret = -1;
-					break;
-				}
-				bf.fw_pts[noport++] = port;
-				nb++;
+				unsigned short port;
+				ret = ipfw_setport (word,port,proto);
+				if (ret != 0) break;
+				bf.tb[noport].not_flag = not_flag;
+				bf.tb[noport].port = port;
+				noport++;
 			}
 		}
+		bf.nb = noport;
 	}
 	return ret;
 }
@@ -615,10 +734,27 @@ int ipfw_checkports (
 	const SSTRING &ports,
 	const SSTRING &proto)
 {
-	struct ip_fw bf;
-	int noport = 0;
-	unsigned short nb = 0;
-	return ipfw_setports (ports.get(),bf,nb,noport,proto.get());
+	struct ip_fw_ports bf;
+	return ipfw_setports (ports.get(),bf,proto.get());
+}
+/*
+	Check if it is a valid protocol, or all. Accept the quotation mark
+	as well, to mean (not this protocol)
+
+	Return -1 if any error.
+*/
+int ipfw_checkproto (const char *protocol)
+{
+	int ret = -1;
+	if (strcasecmp(protocol,"all")==0){
+		ret = 0;
+	}else{
+		if (protocol[0] == '!') protocol = str_skip(protocol+1);
+		if (getprotobyname(protocol) != NULL){
+			ret = 0;
+		}
+	}
+	return ret;
 }
 	
 /*
@@ -652,30 +788,26 @@ int ipfw_baseinit (
 		We assume that all network device do begin with a letter.
 		This way we differentiate IP number for interface from name
 	*/
-	if (isalpha(iface[0])){
+	if (iface[0] == '!' || isalpha(iface[0])){
 		strcpy (bf.fw_vianame,iface);
 	}else{
 		ret |= ipfw_a2ip (iface,bf.fw_via);
 	}
-	if (stricmp(protocol,"all")==0){
-		bf.fw_flg = IP_FW_F_ALL;
-	}else if (stricmp(protocol,"tcp")==0){
-		bf.fw_flg = IP_FW_F_TCP;
-	}else if (stricmp(protocol,"udp")==0){
-		bf.fw_flg = IP_FW_F_UDP;
-	}else if (stricmp(protocol,"icmp")==0){
-		bf.fw_flg = IP_FW_F_ICMP;
+	if (ipfw_checkproto(protocol)!=-1){
+		strcpy_cut (bf.protocol,protocol,sizeof(bf.protocol));
 	}else{
 		ret = -1;
 	}
-	int noport = 0;
-	ret |= ipfw_setrange (sport_range,bf,IP_FW_F_SRNG,noport,bf.fw_nsp);
-	ret |= ipfw_setports (sports,bf,bf.fw_nsp,noport,protocol);
-	ret |= ipfw_setrange (dport_range,bf,IP_FW_F_DRNG,noport,bf.fw_ndp);
-	ret |= ipfw_setports (dports,bf,bf.fw_ndp,noport,protocol);
+	// if (ret == -1) fprintf (stderr,"ipfw_baseinit1 protocol %s error\n",protocol);
+	ret |= ipfw_setrange (sport_range,bf.src_ports);
+	ret |= ipfw_setports (sports,bf.src_ports,protocol);
+	// if (ret == -1) fprintf (stderr,"ipfw_baseinit2 protocol %s error\n",protocol);
+	ret |= ipfw_setrange (dport_range,bf.dst_ports);
+	ret |= ipfw_setports (dports,bf.dst_ports,protocol);
+	// if (ret == -1) fprintf (stderr,"ipfw_baseinit22 protocol %s error\n",protocol);
 
-	bf.fw_tosand = 0xFF;
-	bf.fw_tosxor = 0x00;
+	//bf.fw_tosand = 0xFF;
+	//bf.fw_tosxor = 0x00;
 	if (policy == FW_ACCEPT){
 		bf.fw_flg |= IP_FW_F_ACCEPT;
 	}else if (policy == FW_REJECT){
@@ -683,11 +815,12 @@ int ipfw_baseinit (
 	}
 	if (redirport != NULL){
 		bf.fw_flg |= IP_FW_F_REDIR;
-		unsigned short dummy = 0;
-		ret |= ipfw_setports (redirport,bf,dummy,noport,protocol);
+		ret |= ipfw_setport (redirport,bf.redir_port,protocol);
 	}
+	// if (ret == -1) fprintf (stderr,"ipfw_baseinit3 protocol %s error\n",protocol);
 	if (logging) bf.fw_flg |= IP_FW_F_PRN;
 	if (match_syn) bf.fw_flg |= IP_FW_F_TCPSYN;
+	// if (ret == -1) fprintf (stderr,"ipfw_baseinit4 protocol %s error\n",protocol);
 	return ret;
 }
 
@@ -703,6 +836,10 @@ int ipfw_checkrange (const SSTRING &)
 {
 	return 0;
 }
+int ipfw_checkproto (const char *protocol)
+{
+	return 0;
+}
 
 int ipfw_baseinit (
 	int,
