@@ -1,3 +1,4 @@
+//test dispatch si port vide
 #include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -45,9 +46,37 @@
 
 HELP_FILE help_ipfw ("firewall","firewall");
 
+PUBLIC IPFW_SOLVED_TRIPLET::IPFW_SOLVED_TRIPLET(
+	const char *_ip,
+	const char *_mask,
+	const char *_interface,
+	bool _not_flag)
+{
+	strcpy (ip,_ip);
+	strcpy (mask,_mask);
+	strcpy (interface,_interface);
+	not_flag = _not_flag;
+}
+
+PUBLIC IPFW_SOLVED_TRIPLET *IPFW_SOLVED_TRIPLETS::getitem (int no) const
+{
+	return (IPFW_SOLVED_TRIPLET*)ARRAY::getitem(no);
+}
+
+PUBLIC IPFW_SOLVED_TRIPLET *IPFW_SOLVED_TRIPLETS::add (
+	const char *_ip,
+	const char *_mask,
+	const char *_interface,
+	bool not_flag)
+{
+	IPFW_SOLVED_TRIPLET *ret = new IPFW_SOLVED_TRIPLET(_ip,_mask,_interface
+		,not_flag);
+	ARRAY::add (ret);
+	return ret;
+}	
+
 PUBLIC IPFW_SOLVED::IPFW_SOLVED()
 {
-	ip[0] = mask[0] = interface[0] = '\0';
 	nbbit_msk = 0;
 }
 
@@ -160,6 +189,14 @@ PROTECTED const char *IPFW_RULEP::extract_newf(const char *buf)
 	return buf;
 }
 
+PROTECTED const char *IPFW_RULEP::extract_chain(const char *buf)
+{
+	buf = str_extract (buf,from.dispatch);
+	buf = str_extract (buf,to.dispatch);
+	return buf;
+}
+
+
 PROTECTED IPFW_RULEP::IPFW_RULEP(const char * &buf)
 	: IPFW_RULE (buf)
 {
@@ -172,6 +209,7 @@ PUBLIC IPFW_RULE_FORWARD::IPFW_RULE_FORWARD(const char *buf)
 {
 	buf = firewall_extract (buf,masquerade,0);
 	buf = extract_newf (buf);
+	buf = extract_chain (buf);
 }
 PUBLIC IPFW_RULE_FORWARD::IPFW_RULE_FORWARD()
 {
@@ -182,6 +220,7 @@ PUBLIC IPFW_RULE_OUTPUT::IPFW_RULE_OUTPUT(const char *buf)
 	: IPFW_RULEP (buf)
 {
 	buf = extract_newf (buf);
+	buf = extract_chain (buf);
 }
 PUBLIC IPFW_RULE_OUTPUT::IPFW_RULE_OUTPUT()
 {
@@ -193,6 +232,8 @@ PUBLIC IPFW_RULE_INPUT::IPFW_RULE_INPUT(const char *buf)
 	buf = firewall_extract (buf,doredir,0);
 	buf = str_extract (buf,redirport);
 	buf = extract_newf(buf);
+	buf = str_extract (buf,redirhost);
+	buf = extract_chain (buf);
 }
 PUBLIC IPFW_RULE_INPUT::IPFW_RULE_INPUT()
 {
@@ -247,12 +288,17 @@ PROTECTED void IPFW_RULEP::append_newf (char *buf)
 	firewall_append (buf,logging);
 }
 
+PROTECTED void IPFW_RULEP::append_chain (char *buf)
+{
+	firewall_append (buf,from.dispatch);
+	firewall_append (buf,to.dispatch);
+}
 
-char FIREWALL[] = "firewall";
-char FORWARD[]  = "forward";
-char BLOCK[]  = "block";
-char OUTPUT[]  = "output";
-char ACCT[]  = "acct";
+const char K_FIREWALL[] = "firewall";
+const char K_FORWARD[]  = "forward";
+const char K_BLOCK[]  = "block";
+const char K_OUTPUT[]  = "output";
+const char K_ACCT[]  = "acct";
 
 /*
 	Save one forward line in /etc/conf.linuxconf
@@ -275,7 +321,8 @@ PUBLIC void IPFW_RULE_FORWARD::save(CONFDB *db)
 	IPFW_RULE::savek(buf);
 	firewall_append (buf,masquerade);
 	append_newf(buf);
-	db->add (FIREWALL,FORWARD,buf);
+	append_chain (buf);
+	db->add (K_FIREWALL,K_FORWARD,buf);
 }
 /*
 	Save one forward line in /etc/conf.linuxconf
@@ -287,7 +334,9 @@ PUBLIC void IPFW_RULE_INPUT::save(CONFDB *db)
 	firewall_append (buf,doredir);
 	firewall_append (buf,redirport);
 	append_newf(buf);
-	db->add (FIREWALL,BLOCK,buf);
+	firewall_append (buf,redirhost);
+	append_chain (buf);
+	db->add (K_FIREWALL,K_BLOCK,buf);
 }
 /*
 	Save one output line in /etc/conf.linuxconf
@@ -297,7 +346,8 @@ PUBLIC void IPFW_RULE_OUTPUT::save(CONFDB *db)
 	char buf[2000];
 	IPFW_RULE::savek(buf);
 	append_newf(buf);
-	db->add (FIREWALL,OUTPUT,buf);
+	append_chain (buf);
+	db->add (K_FIREWALL,K_OUTPUT,buf);
 }
 /*
 	Save one output line in /etc/conf.linuxconf
@@ -308,7 +358,7 @@ PUBLIC void IPFW_RULE_ACCT::save(CONFDB *db)
 	IPFW_RULE::savek(buf);
 	firewall_append (buf,weight);
 	firewall_append (buf,comment);
-	db->add (FIREWALL,ACCT,buf);
+	db->add (K_FIREWALL,K_ACCT,buf);
 }
 
 /*
@@ -322,7 +372,7 @@ PUBLIC void IPFW_RULE::present (
 	int size2)
 {
 	snprintf (buf1,size1-1,"[%c]",active ? 'X' : ' ');
-	snprintf (buf2,size2-1,"%s\t%s\t%s\t\t%s\t%s\t%d\t%-15.15s"
+	snprintf (buf2,size2-1,"%s\t%s\t%s\t\t%s\t%s\t%d\t%s"
 		,protocol.get()
 		,from.interface.get()
 		,from.host.get()
@@ -336,43 +386,101 @@ PUBLIC void IPFW_RULE::present (
 	try to convert something into an IP number
 	Return -1 if it can't.
 */
-static int ipfwrule_convert (
+static int ipfwrule_convertone (
 	const char *str,	// An IP number, a host name or an interface
-					// name (eth0)
-	char ip[],
-	char msk[],		// Corresponding (computed) netmask
-					// or extracted from the device
+						// name (eth0)
+	IPFW_SOLVED_TRIPLETS &triplets,	// Corresponding (computed) netmask
+									// or extracted from the device
 	SSTRING *errmsg,
 	FWINFO_API *tbapi[],
-	int nbapi)
+	int nbapi,
+	bool not_flag)
 {
 	int ret = -1;
 	if (strchr(str,'/')!=NULL){
 		// This is a logical host
 		for (int i=0; i<nbapi; i++){
-			SSTRING str_ip;
+			SSTRINGS str_ip;
 			if (tbapi[i]->gethostinfo(str,str_ip)!=-1){
-				str_ip.copy (ip);
-				netconf_computemsk (ip,msk);
+				int n = str_ip.getnb();
+				for (int j=0; j<n; j++){
+					const char *ip = str_ip.getitem(j)->get();
+					const char *pt = strchr (ip,'/');
+					if (pt != NULL){
+						// Ok, the netmask was not included, trying to figure out
+						char tmp[strlen(ip)+1];
+						strcpy (tmp,ip);
+						char *msk = tmp + (pt-ip);
+						*msk++ = '\0';
+						triplets.add (tmp,msk,"",not_flag);
+					}else{
+						char msk[LEN_IP_ASC];
+						netconf_computemsk (ip,msk);
+						triplets.add (ip,msk,"",not_flag);
+					}
+				}
 				ret = 0;
 				break;
 			}
 		}
 	}else{
+		char ip[LEN_IP_ASC],msk[LEN_IP_ASC];
 		ret = netconf_convert (str,ip,msk);
-		if (ret == -1 && errmsg != NULL){
-			char buf[300];
-			sprintf (buf,MSG_U(E_IVLDIP,"Invalid IP number or host name or interface: %s\n")
-				,str);
-			errmsg->append (buf);
+		if (ret == -1){
+			if (errmsg != NULL){
+				char buf[300];
+				sprintf (buf,MSG_U(E_IVLDIP,"Invalid IP number or host name or interface: %s\n")
+					,str);
+				errmsg->append (buf);
+			}
+		}else{
+			triplets.add (ip,msk,"",not_flag);
 		}
 	}
 	return ret;
 }
 
+/*
+	try to convert something into one or many IP numbers
+	Return -1 if it can't.
+*/
+static int ipfwrule_convert (
+	const char *str,	// An IP number, a host name or an interface
+						// name (eth0)
+						// May contains several entries splitted using commas
+	IPFW_SOLVED_TRIPLETS &triplets,	// Corresponding (computed) netmask
+									// or extracted from the device
+	SSTRING *errmsg,
+	FWINFO_API *tbapi[],
+	int nbapi)
+{
+	int ret = 0;
+	char tmp[strlen(str)+1];
+	strcpy (tmp,str);
+	char *pt = tmp;
+	while (*pt != '\0'){
+		pt = str_skip(pt);
+		char *start = pt;
+		while (*pt != ',' && *pt != '\0') pt++;
+		if (*pt == ',') *pt++ = '\0';
+		strip_end (start);
+		if (start[0] != '\0'){
+			bool not_flag = false;
+			if (start[0] == '!'){
+				not_flag = true;
+				start = str_skip(start+1);
+			}
+			ret |= ipfwrule_convertone (start,triplets,errmsg,tbapi,nbapi,not_flag);
+		}
+	}
+	return triplets.getnb()==0 ? -1 : ret;
+}
+
 static int ipfwrule_checkhn (const char *str)
 {
 	int ret = -1;
+	str = str_skip(str);
+	if (str[0] == '!') str++;
 	if (strchr(str,'/')!=NULL){
 		// logical host
 		FWINFO_API *tbapi[MAX_API_PROVIDERS];
@@ -393,9 +501,9 @@ static int ipfwrule_checkhn (const char *str)
 		}
 		ret = 0;
 	}else{
-		char msk[LEN_IP_ASC],ip[LEN_IP_ASC];
+		IPFW_SOLVED_TRIPLETS triplets;
 		SSTRING error;
-		ret = ipfwrule_convert (str,ip,msk,&error,NULL,0);
+		ret = ipfwrule_convert (str,triplets,&error,NULL,0);
 		if (ret == -1) xconf_error (error.get());
 	}
 	return ret;
@@ -419,6 +527,18 @@ static bool ipfwrule_isdevice (const char *prefix, int len, const char *name)
 	return ret;
 }
 
+const char *ipfwrule_getsect (FIREWALL_SECTION section)
+{
+	static const char *tbsect[]={
+		MSG_U(I_SECTINPUT,"Input firewalling"),
+		MSG_U(I_SECTFORWARD,"Forward firewalling"),
+		MSG_U(I_SECTOUTPUT,"Output firewalling"),
+		MSG_U(I_SECTACCT,"Packet accounting"),
+		MSG_U(I_SECTINTERNAL,"Linuxconf internal"),
+	};
+	return tbsect[section];
+}
+
 
 /*
 	Lookup the value of the various fields
@@ -431,66 +551,83 @@ PUBLIC int IPFW_RULE::solve (
 	int nbapi)
 {
 	IPFW_SOLVED &solved = f.solved;
+	solved.triplets.remove_all();
 	int ret = -1;
-	char std_msk[LEN_IP_ASC];
-	solved.not = false;
-	const char *host = f.host.get();
-	if (host[0] == '!'){
-		solved.not = true;
-		host = str_skip(host+1);
-	}
-	if (ipfwrule_convert(host,solved.ip,std_msk,NULL,tbapi,nbapi)!=-1){
+	if (ipfwrule_convert(f.host.get(),solved.triplets,NULL,tbapi,nbapi)!=-1){
 		// Lookup the interface
 		const char *inter = f.interface.get();
-		solved.interface[0] = '\0';
+		const char *not_inter = "";
+		if (inter[0] == '!'){
+			not_inter = "!";
+			inter = str_skip(inter+1);
+		}
 		if (strchr(inter,'/')!=NULL){
 			for (int i=0; i<nbapi; i++){
-				SSTRING str;
-				if (tbapi[i]->getdevinfo(inter,str)!=-1){
-					str.copy (solved.interface);
+				SSTRINGS strs;
+				if (tbapi[i]->getdevinfo(inter,strs)!=-1){
+					int n=strs.getnb();
+					for (int j=0; j<n; j++){
+						const char *sinter = strs.getitem(j)->get();
+						IPFW_SOLVED_TRIPLET *trip = solved.triplets.getitem(j);
+						if (trip != NULL){ // Should match triplets
+							sprintf (trip->interface,"%s%s",not_inter,sinter);
+						}
+					}
 					break;
 				}
 			}
 		}else{
+			char solved_interface[LEN_IP_ASC];
+			char *pt_solved = stpcpy (solved_interface,not_inter);
 			if (f.interface.is_empty()
 				|| f.interface.icmp("Any")==0){
-				strcpy (solved.interface,"0.0.0.0");
+				strcpy (solved_interface,"0.0.0.0");
 			}else{
 				/* #Specification: firewall / iface argument
 					We accept a host name, an IP number or
 					a network device name (eth0).
 				*/
-				if (ipfwrule_isdevice ("eth",3,inter)
+				if (strcmp(inter,"lo")==0
+					|| ipfwrule_isdevice ("eth",3,inter)
 					|| ipfwrule_isdevice ("ppp",3,inter)
 					|| ipfwrule_isdevice ("tr",2,inter)
-					|| ipfwrule_isdevice ("sl",2,inter)){
-					strcpy (solved.interface,inter);
+					|| ipfwrule_isdevice ("sl",2,inter)
+					|| ipfwrule_isdevice ("vlan",4,inter)){
+					strcpy (pt_solved,inter);
 				}else{
-					char ip[20],msk[20];
-					if (ipfwrule_convert (inter,ip,msk,NULL,tbapi,nbapi)!=-1){
+					IPFW_SOLVED_TRIPLETS trips;
+					if (ipfwrule_convert (inter,trips,NULL,tbapi,nbapi)!=-1){
+						IPFW_SOLVED_TRIPLET *trip = trips.getitem(0);
 						if(!kernel_newer(2,2,0)
-							|| netconf_finddevfromhost(ip,solved.interface)==-1){
-							strcpy (solved.interface,ip);
+							|| netconf_finddevfromhost(trip->ip,pt_solved)==-1){
+							strcpy (pt_solved,trip->ip);
 						}
 					}else{
 						xconf_error (MSG_U(E_IVLDINTER
-							,"Invalid interface %s for firewalling rule")
-							,inter);
+							,"Section %s, Rule %d\nInvalid interface %s for firewalling rule")
+							,ipfwrule_getsect(sectnum),rulenum,inter);
 					}
 				}
 			}
+			for (int i=0; i<solved.triplets.getnb(); i++){
+				IPFW_SOLVED_TRIPLET *trip = solved.triplets.getitem(i);
+				if (trip->interface[0] == '\0') strcpy (trip->interface,solved_interface);
+			}
 		}
-		if (solved.interface[0] != '\0'){
+		// Now fill the missing netmask if missing
+		for (int i=0; i<solved.triplets.getnb(); i++){
+			IPFW_SOLVED_TRIPLET *trip = solved.triplets.getitem(i);
 			// Compute the netmask if missing
+			char solved_mask[LEN_IP_ASC];
 			const char *msk = f.netmask.get();
 			if (*msk == '\0'){
 				/* #Specification: firewall / netmask
-					The netmask is optionnal. It is automacly
-					compute from the host/network number
+					The netmask is optionnal. It is
+					computed from the host/network number
 				*/
-				strcpy (solved.mask,std_msk);
+				strcpy (solved_mask,trip->mask);
 			}else{
-				f.netmask.copy (solved.mask);
+				f.netmask.copy (solved_mask);
 			}
 			// Compute the number of significant bits in the netmask
 			int num4[4];
@@ -501,8 +638,16 @@ PUBLIC int IPFW_RULE::solve (
 			int i;
 			for (i=0; i<32 && (nmsk & cmp) != 0; i++, cmp >>= 1);
 			solved.nbbit_msk = i;
+			strcpy (trip->mask,solved_mask);
 			ret = 0;
 		}
+		if (ret == 0){
+			// Make sure the interface is defined for everyone
+			for (int i=0; i<solved.triplets.getnb(); i++){
+				IPFW_SOLVED_TRIPLET *trip = solved.triplets.getitem(i);
+				if (trip->interface[0] == '\0') ret = -1;
+			}
+		}
 	}
 	return ret;
 }
@@ -529,8 +674,8 @@ PUBLIC int IPFW_RULE::nbbitmask_to()
 static int cmp_gen(
 	int netmask1,	// Number of bits == 1in the netmask
 	int netmask2,
-	const char *iface1,	// Interface
-	const char *iface2,
+	const IPFW_SOLVED_TRIPLET *trip1,
+	const IPFW_SOLVED_TRIPLET *trip2,
 	int masq1,		// Is it a masquerading rule
 	int masq2,
 	int redir1,		// Is it a redirection rule
@@ -547,6 +692,8 @@ static int cmp_gen(
 		If all equals, the rules with the interface Any is last
 			when compared to a rule with a specific interface
 	*/
+	const char *iface1 = trip1 != NULL ? trip1->interface : "";
+	const char *iface2 = trip2 != NULL ? trip2->interface : "";
 
 	int ret = redir2 - redir1;
 	if (ret == 0){
@@ -581,9 +728,13 @@ PUBLIC int IPFW_RULE::cmp_gen_to(
 	int redir1,		// Is it a redirection rule
 	int redir2)
 {
+	// We compare only the first triplets, assuming all
+	// elements of a logical interface are alike
+	IPFW_SOLVED_TRIPLET *tripto = to.solved.triplets.getitem(0);
+	IPFW_SOLVED_TRIPLET *triprto = r->to.solved.triplets.getitem(0);
 	return cmp_gen (
 		nbbitmask_to(),r->nbbitmask_to(),
-		to.solved.interface,r->to.solved.interface,
+		tripto,triprto,
 		masq1,masq2,
 		redir1,redir2);
 }
@@ -595,9 +746,11 @@ PUBLIC int IPFW_RULE::cmp_gen_from(
 	int redir1,		// Is it a redirection rule
 	int redir2)
 {
+	IPFW_SOLVED_TRIPLET *tripfrom = from.solved.triplets.getitem(0);
+	IPFW_SOLVED_TRIPLET *triprfrom = r->from.solved.triplets.getitem(0);
 	return cmp_gen (
 		nbbitmask_from(),r->nbbitmask_from(),
-		from.solved.interface,r->from.solved.interface,
+		tripfrom,triprfrom,
 		masq1,masq2,
 		redir1,redir2);
 }
@@ -698,7 +851,8 @@ PRIVATE int IPFW_RULE::diacheck (IPFW_SRC &s, int &nof)
 	}else if (ipfw_checkrange(s.portrange)==-1){
 		xconf_error (MSG_U(E_IVLDPRANGE
 			,"Invalid port range\n"
-			 "Expecting xxxx:yyyy"));
+			 "Expecting xxxx:yyyy where yyyy >= xxxx\n"
+			 "xxxx and yyyy are decimal numbers"));
 		nof = 2;
 	}else if (!allow_port && !s.ports.is_empty()){
 		xconf_error (MSG_U(E_ALLOWPORTLIST
@@ -721,9 +875,6 @@ struct DIALOG_CONTEXT {
 	int f_from_section;
 	int f_to_section;
 };
-static const char *tbproto[]={
-	"icmp","udp","tcp","all"
-};
 
 PROTECTED void IPFW_RULE::setup_top(
 	DIALOG &dia,
@@ -734,8 +885,24 @@ PROTECTED void IPFW_RULE::setup_top(
 	ctx.f_protocol = dia.getnb();
 	FIELD_COMBO *comb = dia.newf_combo (MSG_U(F_PROTOCOL,"Protocol")
 		,protocol);
-	for (int i=0; i<4; i++){
-		comb->addopt (tbproto[i]);
+	// We make sure those standard protocol are in front
+	comb->addopt ("All");
+	comb->addopt ("icmp");
+	comb->addopt ("tcp");
+	comb->addopt ("udp");
+	SSTRINGS tb;
+	struct protoent *p;
+	endprotoent();
+	while ((p=getprotoent())!=NULL){
+		if (strcasecmp(p->p_name,"tcp")!=0
+			&&strcasecmp(p->p_name,"udp")!=0
+			&& strcasecmp(p->p_name,"icmp")!=0){
+			tb.add (new SSTRING(p->p_name));
+		}
+	}
+	tb.sort();
+	for (int i=0; i<tb.getnb(); i++){
+		comb->addopt (tb.getitem(i)->get());
 	}
 }
 
@@ -760,6 +927,27 @@ PROTECTED void IPFW_RULE::setup_interface(
 	fwinfo_apis_end (tbapi,nb);
 }
 
+PROTECTED void IPFW_RULE::setup_host(
+	DIALOG &dia,
+	SSTRING &host)
+{
+	FIELD_COMBO *comb = dia.newf_combo (MSG_U(F_HOSTORNET,"Host(s) or Network(s)")
+		,host);
+	comb->addopt ("0.0.0.0",MSG_U(I_ALL,"All"));
+	FWINFO_API *tbapi[MAX_API_PROVIDERS];
+	int nb = fwinfo_apis_init ("ipfwrule",tbapi);
+	SSTRINGS hosts,descs;
+	for (int i=0; i<nb; i++){
+		tbapi[i]->gethostlist (hosts,descs);
+	}
+	for (int i=0; i<hosts.getnb(); i++){
+		comb->addopt (hosts.getitem(i)->get(),descs.getitem(i)->get());
+	}
+	fwinfo_apis_end (tbapi,nb);
+}
+
+
+
 PROTECTED void IPFW_RULE::setup_from(
 	DIALOG &dia,
 	DIALOG_CONTEXT &ctx,
@@ -767,10 +955,12 @@ PROTECTED void IPFW_RULE::setup_from(
 {
 	dia.newf_title (MSG_U(F_FROM,"From"),1,"",MSG_R(F_FROM));
 	ctx.f_from_section = dia.getnb();
-	dia.newf_str (MSG_U(F_HOSTORNET,"Host or Network"),from.host);
+	setup_host (dia,from.host);
 	FIELD_COMBO *comb = dia.newf_combo (MSG_U(F_HNETMASK,"Netmask"),from.netmask);
 	host_setmasklist(comb);
-	dia.newf_str (MSG_U(F_PORTRANGE,"Port range"),from.portrange);
+	comb  = dia.newf_combo (MSG_U(F_PORTRANGE,"Port range"),from.portrange);
+	comb->addopt ("",MSG_U(I_NORANGE,"None"));
+	comb->addopt ("1024:65535",MSG_U(I_CLIENTPORTS,"Client ports"));
 	dia.newf_str (MSG_U(F_OTHERPORTS,"Other ports"),from.ports);
 	dia.newf_chk (MSG_U(F_ACCEPTSYN,"Accept"),from.allow_syn
 		,MSG_U(I_ACCEPTSYN,"TCP Syn packet"));
@@ -785,7 +975,7 @@ PROTECTED void IPFW_RULE::setup_to(
 {
 	dia.newf_title (MSG_U(F_TO,"To"),1,"",MSG_R(F_TO));
 	ctx.f_to_section = dia.getnb();
-	dia.newf_str (MSG_R(F_HOSTORNET),to.host);
+	setup_host (dia,to.host);
 	FIELD_COMBO *comb = dia.newf_combo (MSG_R(F_HNETMASK),to.netmask);
 	host_setmasklist (comb);
 
@@ -807,6 +997,13 @@ PROTECTED void IPFW_RULE::setup_features(
 	dia.newf_num (MSG_U(F_ORDER,"Ordering factor"),weight);
 	dia.newf_chk (MSG_U(F_LOGGING,"Logging"),logging,MSG_U(I_LOGGING
 		,"is enabled"));
+	FIELD_COMBO *comb = dia.newf_combo (MSG_U(F_DISPATCHFROM
+			,"Chain dispatch (from -> to)")
+			,from.dispatch);
+	comb->addopt ("iface,proto,toport,from/255.255.255.192,to");
+	comb = dia.newf_combo (MSG_U(F_DISPATCHTO,"Chain dispatch (to -> from)")
+			,to.dispatch);
+	comb->addopt ("iface,proto,fromport,to/255.255.255.192,from");
 }
 
 /*
@@ -837,14 +1034,13 @@ PUBLIC int IPFW_RULE::editk(
 		}else if (diacheck(to,nof)==-1){
 			nof += ctx.f_to_section;
 		}else{
-			int i;
-			for (i=0; i<4; i++){
-				if (protocol.cmp(tbproto[i])==0) break;
-			}
-			if (i == 4){
+			if (ipfw_checkproto(protocol.get())==-1){
 				xconf_error (MSG_U(E_IVLDPROTO,"Invalid protocol"));
 				nof = ctx.f_protocol;
-			}else{
+			}else if (chains_validdispatch (from.dispatch.get(),protocol.get()
+					,from.interface.get())!=-1
+				&& chains_validdispatch (to.dispatch.get(),protocol.get()
+					,to.interface.get())!=-1){
 				ret = 0;
 				break;
 			}
@@ -907,8 +1103,12 @@ PUBLIC int IPFW_RULE_INPUT::edit()
 	setup_from (dia,ctx,MSG_U(F_INPUTINTER,"Input interface"));
 	setup_to  (dia,ctx,MSG_R(F_INPUTINTER));
 	setup_features (dia,ctx);
-	dia.newf_chk ("",doredir,MSG_U(F_DOREDIR,"Redirect to local port"));
-	dia.newf_str (MSG_U(F_REDIRPORT,"Redirection port"),redirport);
+	dia.newf_title (MSG_U(F_REDIR,"Redirections"),1,"",MSG_R(F_REDIR));
+	dia.newf_chk ("",doredir,MSG_U(F_DOREDIR,"Redirect to local port/host"));
+	dia.newf_str (MSG_U(F_REDIRPORT,"Redirect to port"),redirport);
+	if (kernel_newer(2,2,0)){
+		dia.newf_str (MSG_U(F_REDIRHOST,"Redir. to host"),redirhost);
+	}
 	return IPFW_RULE::editk (dia,MSG_U(T_INPUTING,"Firewall inputing rule")
 		,ctx);
 }
@@ -932,44 +1132,54 @@ PRIVATE int IPFW_RULE::setupone (
 	const IPFW_SRC &t,
 	FWTYPE type,
 	const char *redirport,
+	const char *redirhost,
 	bool syn_packets,	// This rule applies only to SYN packets
 	bool doit,			// Exec the command ?
 	SSTRING *collect,	// Will contain a copy of the
 						// command generated
 	SSTRING &errmsg)	// Error message are appended to this SSTRING
 {
-	const IPFW_SOLVED &solvedf = f.solved;
-	const IPFW_SOLVED &solvedt = t.solved;
-	int ret = -1;
-	struct ip_fw bf;
-	if (ipfw_baseinit(policy
-		,solvedf.interface,protocol.get()
-		,solvedf.ip,solvedf.mask
-		,f.portrange.get(),f.ports.get()
-		,solvedt.ip,solvedt.mask
-		,t.portrange.get(),t.ports.get()
-		,type == FWTYPE_REDIR ? redirport : (const char *)NULL
-		,logging != 0
-		,syn_packets
-		,bf)!=-1){
-		int command = 0;
-		if (type == FWTYPE_MASQ){
-			command = IP_FW_APPEND_FWD;
-			bf.fw_flg |= IP_FW_F_MASQ;
-		}else if (type == FWTYPE_FORWARD){
-			command = IP_FW_APPEND_FWD;
-		}else if (type == FWTYPE_OUTPUT){
-			command = IP_FW_APPEND_OUT;
-		}else if (type == FWTYPE_INPUT){
-			command = IP_FW_APPEND_IN;
-		}else if (type == FWTYPE_REDIR){
-			command = IP_FW_APPEND_IN;
-		}else if (type == FWTYPE_ACCT){
-			command = IP_ACCT_APPEND;
+	int ret = 0;
+	for (int i=0; i<f.solved.triplets.getnb(); i++){
+		IPFW_SOLVED_TRIPLET *ftrip = f.solved.triplets.getitem(i);
+		for (int j=0; j<t.solved.triplets.getnb(); j++){
+			IPFW_SOLVED_TRIPLET *ttrip = t.solved.triplets.getitem(j);
+			struct ip_fw bf;
+			if (ipfw_baseinit(policy
+				,ftrip->interface,protocol.get()
+				,ftrip->ip,ftrip->mask
+				,f.portrange.get(),f.ports.get()
+				,ttrip->ip,ttrip->mask
+				,t.portrange.get(),t.ports.get()
+				,type == FWTYPE_REDIR ? redirport : (const char *)NULL
+				,logging != 0
+				,syn_packets
+				,bf)!=-1){
+				bf.not_src = ftrip->not_flag;
+				bf.not_dst = ttrip->not_flag;
+				int command = 0;
+				if (type == FWTYPE_MASQ){
+					command = IP_FW_APPEND_FWD;
+					bf.fw_flg |= IP_FW_F_MASQ;
+				}else if (type == FWTYPE_FORWARD){
+					command = IP_FW_APPEND_FWD;
+				}else if (type == FWTYPE_OUTPUT){
+					command = IP_FW_APPEND_OUT;
+				}else if (type == FWTYPE_INPUT){
+					command = IP_FW_APPEND_IN;
+				}else if (type == FWTYPE_REDIR){
+					command = IP_FW_APPEND_IN;
+				}else if (type == FWTYPE_ACCT){
+					command = IP_ACCT_APPEND;
+				}
+				ret |= ipfw_append (sectnum,rulenum,doit,collect,command,bf,redirhost
+					,f.dispatch.get());
+			}else{
+				xconf_error (MSG_U(E_INTERNAL,"Internal error in section %s, rule %d")
+					,ipfwrule_getsect(sectnum),rulenum);
+				ret = -1;
+			}
 		}
-		ret = ipfw_append (doit,collect,command,bf);
-	}else{
-		ret = -1;
 	}
 	return ret;
 }
@@ -981,6 +1191,7 @@ PUBLIC int IPFW_RULE::setup(
 	const IPFW_SRC &t,
 	FWTYPE type,
 	const char *redirport,
+	const char *redirhost,
 	bool doit,			// Exec the command ?
 	SSTRING *collect,	// Will contain a copy of the
 						// command generated
@@ -993,10 +1204,10 @@ PUBLIC int IPFW_RULE::setup(
 			&& policy == FW_ACCEPT){
 			// We must block SYN packet first with a specific
 			// rule
-			ret = setupone (FW_DENY,f,t,type,redirport
+			ret = setupone (FW_DENY,f,t,type,redirport,redirhost
 				,true,doit,collect,errmsg);
 		}
-		ret |= setupone (policy,f,t,type,redirport
+		ret |= setupone (policy,f,t,type,redirport,redirhost
 			,false,doit,collect,errmsg);
 	}
 	return ret;
@@ -1007,12 +1218,13 @@ PROTECTED int IPFW_RULEP::setup(
 	IPFW_SRC &t,
 	FWTYPE type,
 	const char *redirport,
+	const char *redirhost,
 	bool doit,			// Exec the command ?
 	SSTRING *collect,	// Will contain a copy of the
 						// command generated
 	SSTRING &errmsg)	// Error message are appended to this SSTRING
 {
-	return IPFW_RULE::setup (policy,f,t,type,redirport,doit,collect,errmsg);
+	return IPFW_RULE::setup (policy,f,t,type,redirport,redirhost,doit,collect,errmsg);
 }
 
 /*
@@ -1022,9 +1234,9 @@ PUBLIC int IPFW_RULE_FORWARD::setup_left(bool doit, SSTRING *collect, SSTRING &e
 {
 	int ret = -1;
 	if (masquerade){
-		ret = setup (from,to,FWTYPE_MASQ,NULL,doit,collect,errmsg);
+		ret = setup (from,to,FWTYPE_MASQ,NULL,NULL,doit,collect,errmsg);
 	}else{
-		ret = setup (from,to,FWTYPE_FORWARD,NULL,doit,collect,errmsg);
+		ret = setup (from,to,FWTYPE_FORWARD,NULL,NULL,doit,collect,errmsg);
 	}
 	return ret;
 }
@@ -1035,7 +1247,7 @@ PUBLIC int IPFW_RULE_FORWARD::setup_right(bool doit, SSTRING *collect, SSTRING &
 {
 	int ret = 0;
 	if (!masquerade){
-		ret = setup (to,from,FWTYPE_FORWARD,NULL,doit,collect,errmsg);
+		ret = setup (to,from,FWTYPE_FORWARD,NULL,NULL,doit,collect,errmsg);
 	}
 	return ret;
 }
@@ -1044,14 +1256,14 @@ PUBLIC int IPFW_RULE_FORWARD::setup_right(bool doit, SSTRING *collect, SSTRING &
 */
 PUBLIC int IPFW_RULE_OUTPUT::setup_left(bool doit, SSTRING *collect, SSTRING &errmsg)
 {
-	return setup (from,to,FWTYPE_OUTPUT,NULL,doit,collect,errmsg);
+	return setup (from,to,FWTYPE_OUTPUT,NULL,NULL,doit,collect,errmsg);
 }
 /*
 	Setup one side of the forwarding rule (to -> from)
 */
 PUBLIC int IPFW_RULE_OUTPUT::setup_right(bool doit, SSTRING *collect, SSTRING &errmsg)
 {
-	return setup (to,from,FWTYPE_OUTPUT,NULL,doit,collect,errmsg);
+	return setup (to,from,FWTYPE_OUTPUT,NULL,NULL,doit,collect,errmsg);
 }
 
 
@@ -1064,9 +1276,11 @@ PUBLIC int IPFW_RULE_INPUT::setup_left(bool doit, SSTRING *collect, SSTRING &err
 	if (doredir){
 		const char *port = redirport.get();
 		if (redirport.is_empty()) port = "0";
-		ret = setup (from,to,FWTYPE_REDIR,port,doit,collect,errmsg);
+		const char *rdrhost = redirhost.get();
+		if (rdrhost[0] == '\0') rdrhost = NULL;
+		ret = setup (from,to,FWTYPE_REDIR,port,rdrhost,doit,collect,errmsg);
 	}else{
-		ret = setup (from,to,FWTYPE_INPUT,NULL,doit,collect,errmsg);
+		ret = setup (from,to,FWTYPE_INPUT,NULL,NULL,doit,collect,errmsg);
 	}
 	return ret;
 }
@@ -1077,7 +1291,7 @@ PUBLIC int IPFW_RULE_INPUT::setup_right(bool doit, SSTRING *collect, SSTRING &er
 {
 	int ret = 0;
 	if (!doredir){
-		ret = setup (to,from,FWTYPE_INPUT,NULL,doit,collect,errmsg);
+		ret = setup (to,from,FWTYPE_INPUT,NULL,NULL,doit,collect,errmsg);
 	}
 	return ret;
 }
@@ -1087,13 +1301,13 @@ PUBLIC int IPFW_RULE_INPUT::setup_right(bool doit, SSTRING *collect, SSTRING &er
 */
 PUBLIC int IPFW_RULE_ACCT::setup_left(bool doit, SSTRING *collect, SSTRING &errmsg)
 {
-	return setup (FW_ACCEPT,from,to,FWTYPE_ACCT,NULL,doit,collect,errmsg);
+	return setup (FW_ACCEPT,from,to,FWTYPE_ACCT,NULL,NULL,doit,collect,errmsg);
 }
 /*
 	Setup one side of the accounting rule (to -> from)
 */
 PUBLIC int IPFW_RULE_ACCT::setup_right(bool doit, SSTRING *collect, SSTRING &errmsg)
 {
-	return setup (FW_ACCEPT,to,from,FWTYPE_ACCT,NULL,doit,collect,errmsg);
+	return setup (FW_ACCEPT,to,from,FWTYPE_ACCT,NULL,NULL,doit,collect,errmsg);
 }
 
