@@ -34,9 +34,9 @@
 #include <net/if.h>
 #include <dirent.h>
 
-#include "lib/pathnames.h"
-#include "lib/net-support.h"
-#include "lib/util.h"
+void *xmalloc(size_t sz);
+#define new(p) ((p) = xmalloc(sizeof(*(p))))
+char *safe_strncpy(char *dst, const char *src, size_t size); 
 
 #define HAS_INODE	1
 #define PROGNAME_WIDTH	20
@@ -130,10 +130,6 @@ enum {
 #define SO_WAITDATA     (1<<17)	/* wait data to read            */
 #define SO_NOSPACE      (1<<18)	/* no space to write            */
 
-#define PROGNAME_WIDTHs PROGNAME_WIDTH1(PROGNAME_WIDTH)
-#define PROGNAME_WIDTH1(s) PROGNAME_WIDTH2(s)
-#define PROGNAME_WIDTH2(s) #s
-
 #define PRG_HASH_SIZE 211
 
 static struct prg_node {
@@ -146,12 +142,6 @@ static char prg_cache_loaded = 0;
 
 #define PRG_HASHIT(x) ((x) % PRG_HASH_SIZE)
 
-#define PROGNAME_BANNER "PID/Program name"
-
-#define print_progname_banner() do { printf("%-" PROGNAME_WIDTHs "s"," " PROGNAME_BANNER); } while (0)
-
-#define PRG_LOCAL_ADDRESS "local_address"
-#define PRG_INODE	 "inode"
 #define PRG_SOCKET_PFX    "socket:["
 #define PRG_SOCKET_PFXl (strlen(PRG_SOCKET_PFX))
 
@@ -159,6 +149,11 @@ static char prg_cache_loaded = 0;
 #define LINE_MAX 4096
 #endif
 
+#define _PATH_PROCNET_TCP		"/proc/net/tcp"
+#define _PATH_PROCNET_UDP		"/proc/net/udp"
+#define _PATH_PROCNET_TCP6		"/proc/net/tcp6"
+#define _PATH_PROCNET_UDP6		"/proc/net/udp6"
+
 #define PATH_PROC	  	"/proc"
 #define PATH_FD_SUFF	"fd"
 #define PATH_FD_SUFFl       strlen(PATH_FD_SUFF)
@@ -308,11 +303,6 @@ static void prg_cache_load(void)
 	closedir(dirfd);
 }
 
-/***********************************************************************
- * From net-tools-1.54/lib/af.c:
- */
-#define HAVE_AFINET	1
-
 /*
  * From net-tools-1.54/lib/util.c:
  */
@@ -326,14 +316,6 @@ char *safe_strncpy(char *dst, const char *src, size_t size)
  * From net-tools-1.54/lib/inet.c:
  */
 
-/* cache */
-struct addr {
-    struct sockaddr_in addr;
-    char *name;
-    int host;
-    struct addr *next;
-};
-
 static int INET_rresolve(char *name, size_t len, struct sockaddr_in *sin, 
 			 int numeric, unsigned int netmask)
 {
@@ -386,63 +368,12 @@ static char *INET_sprint(struct sockaddr *sap, int numeric)
     return (buff);
 }
 
-struct aftype inet_aftype =
-{
-	"inet",				/* name */
-	NULL,				/* title */
-	AF_INET,			/*af="DARPA Internet" */
-	sizeof(unsigned long),		/* alen */
-	NULL,				/* INET_print*/
-	INET_sprint,			/* INET_sprint */
-	NULL,				/*INET_input*/
-	NULL,				/*INET_reserror*/
-	NULL,				/*INET_rprint */
-	NULL,				/*INET_rinput */
-	NULL,				/*INET_getnetmask*/
-	-1,
-	NULL
-};
-
-static short sVafinit = 0;
-
-struct aftype *aftypes[] =
-{
-    &inet_aftype,
-    NULL
-};
-
-void afinit()
-{
-    inet_aftype.title = "DARPA Internet";
-    sVafinit = 1;
-}
-
-/* Check our protocol family table for this family. */
-struct aftype *get_afntype(int af)
-{
-//fprintf(stderr, "ports.cc:get_afntype: af=%d\n", af);
-    struct aftype **afp;
-
-    if (!sVafinit)
-	afinit();
-
-    afp = aftypes;
-    while (*afp != NULL) {
-	if ((*afp)->af == af)
-	    return (*afp);
-	afp++;
-    }
-    return (NULL);
-}
-/*
- * From net-tools/lib/af.c END
- **********************************************************************/
 FWPORT::FWPORT()
 {
 	port = 0;
 	uid = 0;
 	pid = 0;
-	active = 0;
+	accept = 0;
 }
 
 FWPORTLIST::FWPORTLIST()
@@ -454,11 +385,9 @@ PRIVATE void FWPORTLIST::tcp_do_one(int lnr, const char *line)
     unsigned long rxq, txq, time_len, retr, inode;
     int num, local_port, rem_port, d, state, uid, timer_run, timeout;
     char rem_addr[128], local_addr[128], more[512];
-    struct aftype *ap;
 #if HAVE_AFINET6
     struct sockaddr_in6 localaddr, remaddr;
     char addr6p[16][3], addr6[128];
-    extern struct aftype inet6_aftype;
 #else
     struct sockaddr_in localaddr, remaddr;
 #endif
@@ -514,11 +443,7 @@ PRIVATE void FWPORTLIST::tcp_do_one(int lnr, const char *line)
 	fprintf(stderr, "warning, got bogus tcp line.\n");
 	return;
     }
-    if ((ap = get_afntype(((struct sockaddr *) &localaddr)->sa_family)) == NULL) {
-	fprintf(stderr, "ports.cc: unsupported address family %d !\n",
-		((struct sockaddr *) &localaddr)->sa_family);
-	return;
-    }
+
     if (state == TCP_LISTEN) {
 	time_len = 0;
 	retr = 0L;
@@ -526,7 +451,7 @@ PRIVATE void FWPORTLIST::tcp_do_one(int lnr, const char *line)
 	txq = 0L;
     }
     safe_strncpy(local_addr,
-		 ap->sprint((struct sockaddr *) &localaddr, flag_not),
+		 INET_sprint((struct sockaddr *) &localaddr, flag_not),
 		 sizeof(local_addr));
     if ( strcmp( local_addr, "*" ) != 0 ) {
 	return;
@@ -561,11 +486,9 @@ PRIVATE void FWPORTLIST::udp_do_one(int lnr, const char *line)
     struct sockaddr_in6 localaddr, remaddr;
     char addr6p[8][5];
     char addr6[128];
-    extern struct aftype inet6_aftype;
 #else
     struct sockaddr_in localaddr, remaddr;
 #endif
-    struct aftype *ap;
     unsigned long rxq, txq, time_len, retr, inode;
 
     if (lnr == 0)
@@ -613,11 +536,7 @@ PRIVATE void FWPORTLIST::udp_do_one(int lnr, const char *line)
 	fprintf(stderr, "warning, got bogus udp line.\n");
 	return;
     }
-    if ((ap = get_afntype(((struct sockaddr *) &localaddr)->sa_family)) == NULL) {
-	fprintf(stderr, "ports.cc: unsupported address family %d !\n",
-		((struct sockaddr *) &localaddr)->sa_family);
-	return;
-    }
+
     switch (state) {
     case TCP_ESTABLISHED:
 	udp_state = "ESTABLISHED";
@@ -646,7 +565,7 @@ PRIVATE void FWPORTLIST::udp_do_one(int lnr, const char *line)
 
 //fprintf(stderr, "ports.cc:udp\n");
         safe_strncpy(local_addr,
-		ap->sprint((struct sockaddr *) &localaddr, flag_not),
+		INET_sprint((struct sockaddr *) &localaddr, flag_not),
 		sizeof(local_addr));
 
 	if ( strcmp( local_addr, "*" ) != 0 ) {
@@ -715,7 +634,7 @@ PUBLIC void FWPORTLIST::update_from_db()
 		FWPORT *fwport = getitem( i );
 		snprintf( service, sizeof(service),
 			"%s/%d", fwport->protocol.get(), fwport->port);
-		fwport->active = (char)linuxconf_getvalnum( K_FIREWALL, service, 0 );
+		fwport->accept = (char)linuxconf_getvalnum( K_FIREWALL, service, 0 );
 	}
 }
 
