@@ -8,7 +8,7 @@ public:
 	SSTRING user;
 	int uid;
 	int pid;
-	char active;
+	char accept;
 	
 	/*~PROTOBEG~ FWPORT */
 public:
@@ -31,3 +31,5 @@ public:
 	void update_from_db( void );
 	/*~PROTOEND~ FWPORTLIST */
 };
+
+static const char K_FIREWALL[] = "firewall";
