@@ -17,19 +17,27 @@ public:
 	int dohtml (const char *key);
 	int domenu (MENU_CONTEXT context,
 		 const char *key);
-	int execmain (int argc, char *argv[]);
+	int execmain (int argc, char *argv[], bool);
+	int message (const char *msg,
+		 int argc,
+		 const char *argv[]);
+	int probe (int level,
+		 int target,
+		 bool simulation);
 	void setmenu (DIALOG&dia,
 		 MENU_CONTEXT context);
 	void usage (SSTRINGS&tb);
     /*~PROTOEND~ MODULE_inetdconf */
 };
 
-static const char K_FIREWALL[] = "firewall";
-static const char K_ACTIVE[] = "active";
-static const char K_DYNAMIC_IP[] = "dynamic_ip";
-static const char K_LOG_DENIED[] = "log_denied";
 static const char subsys_inetdconf[] = "inetdconf";
 
 #include "inetdconf.p"
 
+PUBLIC void firewall_enable ( int argc, char **argv);
+PUBLIC void firewall_disable (void);
+PUBLIC void firewall_edit (void);
+PUBLIC int firewall_probe (int level, int target, int simulation);
+extern PUBLIC bool xinetd_conf_flag;
+
 #endif
