@@ -15,7 +15,7 @@ public:
 	MODULE_inittab (void);
 	int domenu (MENU_CONTEXT context,
 		 const char *key);
-	int execmain (int argc, char *argv[]);
+	int execmain (int argc, char *argv[], bool);
 	void setmenu (DIALOG&dia, MENU_CONTEXT context);
 	void usage (SSTRINGS&tb);
     /*~PROTOEND~ MODULE_inittab */
