@@ -2,7 +2,7 @@ extern const char **_dictionary_inittab;
 #ifndef DICTIONARY_REQUEST
 	#define DICTIONARY_REQUEST \
 	const char **_dictionary_inittab;\
-	TRANSLATE_SYSTEM_REQ _dictionary_req_inittab("inittab",_dictionary_inittab,5,0);\
+	TRANSLATE_SYSTEM_REQ _dictionary_req_inittab("inittab",_dictionary_inittab,8,0);\
 	void dummy_dict_inittab(){}
 #endif
 #ifndef MSG_U
@@ -18,3 +18,6 @@ extern const char **_dictionary_inittab;
 #define F_DEFAULTLEVEL	2
 #define T_DEFLEVEL	3
 #define F_RUNLEVEL	4
+#define I_TEXTMODE	5
+#define I_GRAPHICMODE	6
+#define I_SINGLEMODE	7
