@@ -6,6 +6,7 @@
 	#include <misc.h>
 #endif
 
+class ALIASES;
 
 class ALIAS: public ARRAY_OBJ{
 public:
@@ -16,10 +17,10 @@ public:
 	SSTRING file;
 	/*~PROTOBEG~ ALIAS */
 public:
-	ALIAS (char *line);
+	ALIAS (const char *line);
 	ALIAS (void);
 	void addoneval (const char *val);
-	int edit (PRIVILEGE *privi);
+	int edit (PRIVILEGE *privi, ALIASES&aliases);
 	int file_ok (void);
 	int filter_ok (void);
 	const char *getname (void);
@@ -28,7 +29,7 @@ public:
 private:
 	void splitline (char *ptpt);
 public:
-	void write (FILE *fout);
+	void write (FILE_CFG *fout);
 	/*~PROTOEND~ ALIAS */
 };
 
@@ -43,11 +44,12 @@ public:
 		 PRIVILEGE *_privi,
 		 bool _dodb);
 private:
-	void addline (char *buf);
+	void addline (SSTRING&buf);
 public:
 	void addnew (void);
 	int edit (const char *title);
 	ALIAS *getitem (int no);
+	bool isdup (ALIAS *ali);
 	int locate (const char *name);
 	int setalias (const char *name,
 		 int nb,
