@@ -5,6 +5,7 @@
 #include <sys/stat.h>
 #include <limits.h>
 #include <misc.h>
+#include <dialog.h>
 #include "internal.h"
 #include "../paths.h"
 #include <daemoni.h>
@@ -28,6 +29,8 @@ CONFIG_FILE f_pophash (ETC_MAIL_POPHASH,help_mailconf
 
 extern CONFIG_FILE f_mailtable;
 extern CONFIG_FILE f_sendmail_cw;
+static CONFIG_FILE f_sendmail_cV ("/etc/mail/sendmail.cV",help_nil
+	,CONFIGF_GENERATED|CONFIGF_NOARCH);
 
 /*
 	Avoid repeating the same path over and over
@@ -56,6 +59,7 @@ static MAILCONF_FILE f_rulesets ("rulesets");
 static MAILCONF_FILE f_rulesets_s0_intro ("rulesets.s0.intro");
 static MAILCONF_FILE f_rulesets_s0_90 ("rulesets.s0.90");
 static MAILCONF_FILE f_rulesets_s0_local ("rulesets.s0.local");
+static MAILCONF_FILE f_rulesets_s0_local_amavis ("rulesets.s0.local.amavis");
 static MAILCONF_FILE f_rulesets_parse1_init ("rulesets.parse1.init");
 static MAILCONF_FILE f_rulesets_parse1_virtuser ("rulesets.parse1.virtuser");
 static MAILCONF_FILE f_rulesets_parse1_middle  ("rulesets.parse1.middle");
@@ -68,11 +72,13 @@ static MAILCONF_FILE f_rulesets_s96_nodns ("rulesets.s96.nodns");
 static MAILCONF_FILE f_rulesets_s96_fewdns ("rulesets.s96.fewdns");
 static MAILCONF_FILE f_rulesets_relay ("rulesets.relay");
 static MAILCONF_FILE f_rulesets_junk ("rulesets.junk");
+static MAILCONF_FILE f_rulesets_rbl ("rulesets.rbl");
 
 static MAILCONF_FILE f_localmailer ("localmailer.std");
 static MAILCONF_FILE f_faxmailer ("faxmailer");
 static MAILCONF_FILE f_delivermailer ("localmailer.deliver");
 static MAILCONF_FILE f_procmailmailer ("localmailer.procmail");
+static MAILCONF_FILE f_amavismailer ("localmailer.amavis");
 static MAILCONF_FILE f_progmailer ("progmailer");
 static MAILCONF_FILE f_uucpmailer ("uucpmailer.std");
 static MAILCONF_FILE f_virtmailer ("virtualmailer.std");
@@ -90,7 +96,7 @@ const int TOKEN_VALLEN = 100;
 */
 static int copyfile (
 	MAILCONF_FILE &filesrc,
-	FILE *fout,
+	SSTREAM &out,
 	const char *token[],
 	const char repl[][TOKEN_VALLEN],
 	int nbtoken)
@@ -106,7 +112,7 @@ static int copyfile (
 	if (!file_exist (path)){
 		path = filesrc.refpath.get();
 	}
-	FILE *fin = filesrc.fopen(path,"r");
+	FILE_CFG *fin = filesrc.fopen(path,"r");
 	if (fin != NULL){
 		ret = 0;
 		char buf[1000];
@@ -123,7 +129,7 @@ static int copyfile (
 					strcpy (buf,buf2);
 				}
 			}
-			fputs (buf,fout);
+			out.puts (buf);
 		}
 		fclose (fin);
 	}
@@ -134,9 +140,9 @@ static int copyfile (
 	Copy a text file into fout.
 	Return 0 if ok, -1 if any error.
 */
-static int copyfile (MAILCONF_FILE &filesrc, FILE *fout)
+static int copyfile (MAILCONF_FILE &filesrc, SSTREAM &out)
 {
-	return copyfile (filesrc,fout,NULL,NULL,0);
+	return copyfile (filesrc,out,NULL,NULL,0);
 }
 
 /*
@@ -148,42 +154,60 @@ static int copyfile (MAILCONF_FILE &filesrc, FILE *fout)
 
 	Return -1 if any error
 */
-PRIVATE int MAILCONF::generate(FILE *fout)
+PRIVATE int MAILCONF::format(SSTREAM &out)
 {
-	int	ret = copyfile (f_intro,fout);
-	ret |= copyfile (f_copyright,fout);
+	int	ret = copyfile (f_intro,out);
+	ret |= copyfile (f_copyright,out);
 	THISHOST thishost;
 	const char *hname = thishost.getname1();
-	fputs ("# Alias for this host\n",fout);
+	out.puts ("# Alias for this host\n");
 	if (mainserv){
 		const char *dom = thishost.getdomain();
 		if (dom != NULL){
-			fprintf (fout,"Cw %s\n",dom);
+			out.printf ("Cw %s\n",dom);
 		}else{
 			xconf_error (MSG_U(E_NODOMAIN
 				,"Can't compute the domain name of this server\n"
 				 "It won't accept email correctly.\n"
-				 "Please enter a fully qualified domain name in the\n"
-				 "\"Basic host information\" screen."));
+				 "Please enter a fully qualified domain name (host.domain)\n"
+				 "in the \"Host name\" field (first field) of the"
+				 "\"Basic host information\" dialog.\n"
+				 ));
 		}
 	}
-	fprintf (fout,"Cw localhost %s\n",hname);
+	if (hostserv){
+		out.printf ("Cw localhost %s\n",hname);
+	}
 	if (sendmail_cw){
-		fprintf (fout,"Fw%s\n",f_sendmail_cw.getpath());
+		out.printf ("Fw%s\n",f_sendmail_cw.getpath());
 	}else{
 		for (int i=0; i<alias.getnb(); i++){
-			fprintf (fout,"Cw %s\n",alias.getitem(i)->get());
+			out.printf ("Cw %s\n",alias.getitem(i)->get());
 		}
 	}
-	fputs ("# Virtual email domain\n",fout);
+	out.puts ("# Virtual email domain\n");
 	VDOMAINS vdomains;		// Virtual domains
-	for (int i=0; i<vdomains.getnb(); i++){
-		VDOMAIN *v = vdomains.getitem(i);
-		fprintf (fout,"CV %s\n",v->domain.get());
-		for (int o=0; o<v->others.getnb(); o++){
-			SSTRING *s = v->others.getitem(o);
-			fprintf (fout,"CV %s\n",s->get());
+	out.printf ("FV%s\n",f_sendmail_cV.getpath());
+	FILE_CFG *fV = f_sendmail_cV.fopen ("w");
+	if (fV == NULL){
+		ret = -1;
+	}else{
+		for (int i=0; i<vdomains.getnb(); i++){
+			VDOMAIN *v = vdomains.getitem(i);
+			const char *domname = v->domain.get();
+			fprintf (fV,"%s\n",domname);
+			out.printf ("#CV %s\n",domname);
+			for (int o=0; o<v->others.getnb(); o++){
+				SSTRING *s = v->others.getitem(o);
+				const char *ali = s->get();
+				fprintf (fV,"%s\n",ali);
+				// We are still generated #CV lines, because the
+				// sendmail.cf content is compared with the previous one
+				// so we know if it has to be changed or not.
+				out.printf ("#CV %s\n",ali);
+			}
 		}
+		fclose (fV);
 	}
 	/* #specification: generating sendmail.cf / other module contribution
 		When generating the sendmail.cf, the mailconf module allows
@@ -196,12 +220,12 @@ PRIVATE int MAILCONF::generate(FILE *fout)
 		co-manager might be better.
 	*/
 	const char *message_argv[2];
-	message_argv[0] = (const char *)fout;
+	message_argv[0] = (const char *)&out;
 	message_argv[1] = NULL;
 	module_sendmessage ("mailconf:defclass",1,message_argv);
 	
-	fputs ("# who I masquerade as (null for no masquerading)\n",fout);
-	if (!mailas.is_empty())	fprintf (fout,"DM%s\n",mailas.get());
+	out.puts ("# who I masquerade as (null for no masquerading)\n");
+	if (!mailas.is_empty())	out.printf ("DM%s\n",mailas.get());
 
 	/* #Specification: mailconf / sendmail.cf / smarthost
 		If we specify that we are our own smarthost, this
@@ -218,45 +242,51 @@ PRIVATE int MAILCONF::generate(FILE *fout)
 		We do the same for the mailhost (Macro H of
 		sendmail.cf)
 	*/
-	fputs ("# Smart host\n",fout);
+	out.puts ("# Smart host\n");
 	if (smarthost.icmp(hname)==0){
-		fputs  ("DS\n",fout);
+		out.puts  ("DS\n");
+	}else if (ipnum_validip(smarthost.get(),false)){
+		out.printf  ("DS[%s]\n",smarthost.get());
 	}else{
-		fprintf  (fout,"DS%s\n",smarthost.get());
+		out.printf  ("DS%s\n",smarthost.get());
+	}
+	out.puts ("# Use this mailer to reach the Smart host\n");
+	if (ipnum_validip(smartmailer.get(),false)){
+		out.printf ("DN[%s]\n",smartmailer.get());
+	}else{
+		out.printf ("DN%s\n",smartmailer.get());
 	}
-	fputs ("# Use this mailer to reach the Smart host\n",fout);
-	fprintf  (fout,"DN%s\n",smartmailer.get());
 
-	fputs ("# Central host for local mail\n",fout);
+	out.puts ("# Central host for local mail\n");
 	if (mailhost.icmp(hname)==0){
-		fputs  ("DH\n",fout);
+		out.puts  ("DH\n");
 	}else{
-		fprintf (fout,"DH%s\n",mailhost.get());
+		out.printf ("DH%s\n",mailhost.get());
 	}
 
-	fputs ("# class L: names that should be delivered locally, even if we have a relay\n",fout);
-	fprintf (fout,"CL%s\n",users.deliverlocal.get());
+	out.puts ("# class L: names that should be delivered locally, even if we have a relay\n");
+	out.printf ("CL%s\n",users.deliverlocal.get());
 
-	fputs ("# class E: names that should be exposed as from this host, even if we masquerade\n",fout);
-	fprintf (fout,"CE%s\n",users.dontmasque.get());
+	out.puts ("# class E: names that should be exposed as from this host, even if we masquerade\n");
+	out.printf ("CE%s\n",users.dontmasque.get());
 
-	fputs ("# Trust users\n",fout);
+	out.puts ("# Trust users\n");
 	{
 		const char *pt = users.trust.get();
 		while (1){
 			char word[100];
 			pt=str_copyword(word,pt,sizeof(word)-1);
 			if (word[0] == '\0') break;
-			fprintf (fout,"T%s\n",word);
+			out.printf ("T%s\n",word);
 		}
 	}
 
-	fputs ("# Database for special routing\n",fout);
+	out.puts ("# Database for special routing\n");
 	if (features.mailertable){
-		fprintf (fout,"Kmailertable %s %s\n"
+		out.printf ("Kmailertable %s %s\n"
 			,features.dbformat.get(),f_mailtable.getpath());
 	}else{
-		fputs("# Not activated\n",fout);
+		out.puts("# Not activated\n");
 		if (spcs.getnb()>0){
 			xconf_notice(MSG_U(E_MTABLEON
 				,"The /etc/sendmail.cf will be generated without\n"
@@ -269,87 +299,104 @@ PRIVATE int MAILCONF::generate(FILE *fout)
 		}
 	}
 
-	fputs ("# Restrict DNS to those domain only\n",fout);
-	fputs ("CD ",fout);
+	out.puts ("# Restrict DNS to those domain only\n");
+	out.puts ("CD ");
 	int restricted_dns = 0;
 	for (int i=0; i<dnsfor.getnb(); i++){
 		SSTRING *item = dnsfor.getitem(i);
 		if (!item->is_empty()) restricted_dns = 1;
-		fprintf (fout," %s",item->get());
+		out.printf (" %s",item->get());
 	}
-	fputc ('\n',fout);
+	out.puts ("\n");
 
-	ret |= copyfile (f_stdmacros,fout);
+	ret |= copyfile (f_stdmacros,out);
 
 	if (features.relayctrl){
 		if (f_spam_ip_allow.exist()){
-			fputs ("# file containing IP numbers of machines which can use our relay\n",fout);
-			fprintf (fout,"F{LocalIP} %s\n",f_spam_ip_allow.getpath());
+			out.puts ("# file containing IP numbers of machines which can use our relay\n");
+			out.printf ("F{LocalIP} %s\n",f_spam_ip_allow.getpath());
 		}
 		if (f_spam_name_allow.exist()){
-			fputs ("# file containing names of machines which can use our relay\n",fout);
-			fprintf (fout,"F{LocalNames} %s\n",f_spam_name_allow.getpath());
+			out.puts ("# file containing names of machines which can use our relay\n");
+			out.printf ("F{LocalNames} %s\n",f_spam_name_allow.getpath());
 		}
 		if (f_spam_relay_allow.exist()){
-			fputs ("# file containing names we relay to\n",fout);
-			fprintf (fout,"F{RelayTo} %s\n",f_spam_relay_allow.getpath());
+			out.puts ("# file containing names we relay to\n");
+			out.printf ("F{RelayTo} %s\n",f_spam_relay_allow.getpath());
 		}
 	}
 	if (f_spam_deny.exist()){
-		fputs ("# file containing known spammers by email,domain,ip\n",fout);
-		fprintf (fout,"Kjunk %s -a@JUNK %s\n",features.dbformat.get()
+		out.puts ("# file containing known spammers by email,domain,ip\n");
+		out.printf ("Kjunk %s -a@JUNK %s\n",features.dbformat.get()
 			,f_spam_deny.getpath());
 	}
 	if (f_virtuser.exist()){
-		fputs ("# Virtual user table (maps incoming users\n",fout);
-		fprintf (fout,"Kvirtuser %s %s\n",features.dbformat.get()
+		out.puts ("# Virtual user table (maps incoming users\n");
+		out.printf ("Kvirtuser %s %s\n",features.dbformat.get()
 			,f_virtuser.getpath());
 	}
 	if (f_pophash.exist()){
-		fputs ("# Roaming pop users accces\n",fout);
-		fprintf (fout,"Kpopauth %s %s\n",features.dbformat.get()
+		out.puts ("# Roaming pop users accces\n");
+		out.printf ("Kpopauth %s %s\n",features.dbformat.get()
 			,f_pophash.getpath());
 	}
 
 
-	fputs ("# Deliver mail only in DNS is available\n",fout);
-	fputs (features.dnsneeded ? "OI\n" : "#OI\n", fout);
-	fputs ("# Match full user name when receiving\n",fout);
-	fputs (features.usegecos ? "OGTrue\n" : "OGFalse\n", fout);
-	fputs ("# maximum message size\n",fout);
+	out.puts ("# Deliver mail only in DNS is available\n");
+	out.puts (features.dnsneeded ? "OI\n" : "#OI\n");
+	out.puts ("# Match full user name when receiving\n");
+	out.puts (features.usegecos ? "OGTrue\n" : "OGFalse\n");
+	out.puts ("# maximum message size\n");
 	if (features.maxmsgsize > 0){
-		fprintf (fout,"O MaxMessageSize=%d\n",features.maxmsgsize);
+		out.printf ("O MaxMessageSize=%d\n",features.maxmsgsize);
 	}else{
-		fputs ("#O MaxMessageSize=1000000\n",fout);
+		out.puts ("#O MaxMessageSize=1000000\n");
 	}
 	if (features.maxrecipients > 0){
-		fprintf (fout,"O MaxRecipientsPerMessage=%d\n"
+		out.printf ("O MaxRecipientsPerMessage=%d\n"
 			,features.maxrecipients);
 	}else{
-		fputs ("#O MaxRecipientsPerMessage=xxxxx\n",fout);
+		out.puts ("#O MaxRecipientsPerMessage=xxxxx\n");
 	}
 
-	fputs ("# delivery mode\n",fout);
-	fprintf (fout,"O DeliveryMode=%s\n"
+	out.puts ("# delivery mode\n");
+	out.printf ("O DeliveryMode=%s\n"
 		, features.deferdeliv ? "deferred" : "background");
 	if (features.bogushelo){
-		fputs ("# Give a chance to broken mail client\n",fout);
-		fputs ("O AllowBogusHELO\n",fout);
+		out.puts ("# Give a chance to broken mail client\n");
+		out.puts ("O AllowBogusHELO\n");
 	}
 	{
 		static const char *usertoken[]={
-			"$(MAILMAIL)","$(MQUEUE)"
+			"$(MAILMAIL)","$(MQUEUE)","$(EXPN)","$(VERIFY)",
+			"$(TIMEOUTIDENT)",
+			"$(ETRN)","$(VERB)","$(MAILQ)","$(RUN)","$(RECEIPTS)","$(GOAWAY)",
+			"$(PROBEINTERFACE)","$(AUTHWARNINGS)",
+			"$(QUEUEWARN)","$(QUEUERETURN)"
 		};
-		char userrepl[2][TOKEN_VALLEN];
+		char userrepl[15][TOKEN_VALLEN];
 		strcpy (userrepl[0],"mail:mail");
 		if (strcmp(linuxconf_getdistdir(),"suse")==0){
 			strcpy (userrepl[0],"bin:bin");
 		}
 		strcpy (userrepl[1],configf_lookuppath("/var/spool/mqueue"));
-		ret |= copyfile (f_stdoptions,fout,usertoken,userrepl,2);
+		strcpy (userrepl[2],features.expn ? "# " : "");
+		strcpy (userrepl[3],features.verify ? "# " : "");
+		sprintf (userrepl[4],"%ds",features.tmoutident);
+		strcpy (userrepl[5],features.etrn ? "# " : "");
+		strcpy (userrepl[6],features.verb ? "# " : "");
+		strcpy (userrepl[7],features.mailq ? "# " : "");
+		strcpy (userrepl[8],features.run ? "# " : "");
+		strcpy (userrepl[9],features.receipts ? "# " : "");
+		strcpy (userrepl[10],features.goaway ? "# " : "");
+		strcpy (userrepl[11],features.probeinterface ? "# " : "");
+		strcpy (userrepl[12],features.authwarnings ? "" : "# ");
+		strcpy (userrepl[13],features.queuewarn.get());
+		strcpy (userrepl[14],features.queuereturn.get());
+		ret |= copyfile (f_stdoptions,out,usertoken,userrepl,15);
 	}
-	ret |= copyfile (f_rulesets,fout);
-	if (features.mailertable) ret |= copyfile (f_rulesets_s90,fout);
+	ret |= copyfile (f_rulesets,out);
+	if (features.mailertable) ret |= copyfile (f_rulesets_s90,out);
 	if (features.relayctrl){
 		static const char *usertoken[]={
 			"$(POPHASH)"
@@ -360,29 +407,35 @@ PRIVATE int MAILCONF::generate(FILE *fout)
 		}else{
 			strcpy (userrepl[0],"#");
 		}
-		ret |= copyfile (f_rulesets_relay,fout,usertoken,userrepl,1);
+		ret |= copyfile (f_rulesets_relay,out,usertoken,userrepl,1);
 	}
 	if (f_spam_deny.exist()){
-		ret |= copyfile (f_rulesets_junk,fout);
+		ret |= copyfile (f_rulesets_junk,out);
 	}else{
-		fputs ("#Empty junk rule\nSjunk\n\nSjunkIP\n\n",fout);
+		out.puts ("#Empty junk rule\nSjunk\n\nSjunkIP\n\n");
+	}
+	if (features.userbl){
+		ret |= copyfile (f_rulesets_rbl,out);
+	}else{
+		out.puts ("# RBL not enabled, empty ruleset\n");
+		out.puts ("Scheck_rbl\n");
 	}
 
-	ret |= copyfile (f_rulesets_s0_intro,fout);
+	ret |= copyfile (f_rulesets_s0_intro,out);
 
 	// Let a module define complex/exceptional routing
 	module_sendmessage ("mailconf:complex_s0",1,message_argv);
 	COMPLEX_ROUTES cplx;
-	cplx.rule0 (fout,alias,vdomains);
+	cplx.rule0 (out,alias,vdomains);
 	#if 0
 		// Alias for virtual email domain
 		for (i=0; i<vdomains.getnb(); i++){
 			VDOMAIN *v = vdomains.getitem(i);
 			for (int o=0; o<v->others.getnb(); o++){
 				SSTRING *s = v->others.getitem(o);
-				fprintf (fout,"R$*<@%s>\t$#virtual $@ %s $: $1\n"
+				out.printf ("R$*<@%s>\t$#virtual $@ %s $: $1\n"
 					,s->get(),v->domain.get());
-				fprintf (fout,"R$*<@%s.>\t$#virtual $@ %s $: $1\n"
+				out.printf ("R$*<@%s.>\t$#virtual $@ %s $: $1\n"
 					,s->get(),v->domain.get());
 			}
 		}
@@ -391,45 +444,53 @@ PRIVATE int MAILCONF::generate(FILE *fout)
 	if (mfax.enable){
 		const char *dom = thishost.getdomain();
 		if (dom == NULL) dom = "";
-		fprintf (fout,"R$*<@fax.%s>\t$#fax $@ $1 $: _\n",dom);
-		fprintf (fout,"R$*<@fax.%s.>\t$#fax $@ $1 $: _\n",dom);
-		fprintf (fout,"R$*<@$*.fax.%s>\t$#fax $@ $2 $: $1\n",dom);
-		fprintf (fout,"R$*<@$*.fax.%s.>\t$#fax $@ $2 $: $1\n",dom);
+		out.printf ("R$*<@fax.%s>\t$#fax $@ $1 $: _\n",dom);
+		out.printf ("R$*<@fax.%s.>\t$#fax $@ $1 $: _\n",dom);
+		out.printf ("R$*<@$*.fax.%s>\t$#fax $@ $2 $: $1\n",dom);
+		out.printf ("R$*<@$*.fax.%s.>\t$#fax $@ $2 $: $1\n",dom);
 	}
 	
-	ret |= copyfile (f_rulesets_s0_local,fout);
-	ret |= copyfile (f_rulesets_parse1_init,fout);
+        if (deliver.cmp("amavis")==0){
+            ret |= copyfile (f_rulesets_s0_local_amavis,out);
+        }else{
+            ret |= copyfile (f_rulesets_s0_local,out);
+        }
+	ret |= copyfile (f_rulesets_parse1_init,out);
 	if (f_virtuser.exist()){
-		ret |= copyfile (f_rulesets_parse1_virtuser,fout);
+		ret |= copyfile (f_rulesets_parse1_virtuser,out);
 	}
-	ret |= copyfile (f_rulesets_parse1_middle,fout);
-	if (features.mailertable) ret |= copyfile (f_rulesets_parse1_mailtable,fout);
-	ret |= copyfile (f_rulesets_parse1_remote,fout);
+	ret |= copyfile (f_rulesets_parse1_middle,out);
+	if (features.mailertable) ret |= copyfile (f_rulesets_parse1_mailtable,out);
+	ret |= copyfile (f_rulesets_parse1_remote,out);
 
-	if (features.mailertable) ret |= copyfile (f_rulesets_s0_90,fout);
+	if (features.mailertable) ret |= copyfile (f_rulesets_s0_90,out);
 
-	ret |= copyfile (f_rulesets_s3,fout);
+	ret |= copyfile (f_rulesets_s3,out);
 	if (features.nodns){
-		ret |= copyfile (f_rulesets_s96_nodns,fout);
+		ret |= copyfile (f_rulesets_s96_nodns,out);
 	}else if (restricted_dns){
-		ret |= copyfile (f_rulesets_s96_fewdns,fout);
+		ret |= copyfile (f_rulesets_s96_fewdns,out);
 	}else{
-		ret |= copyfile (f_rulesets_s96_dns,fout);
+		ret |= copyfile (f_rulesets_s96_dns,out);
 	}
 	if (deliver.is_empty()){
 		if (file_exist ("/bin/mail.local")){
-			ret |= copyfile (f_localmailer,fout);
+			ret |= copyfile (f_localmailer,out);
 		}else if (file_exist ("/usr/bin/deliver")){
-			ret |= copyfile (f_delivermailer,fout);
+			ret |= copyfile (f_delivermailer,out);
 		}else if (file_exist ("/usr/bin/procmail")){
-			ret |= copyfile (f_procmailmailer,fout);
+			ret |= copyfile (f_procmailmailer,out);
+		}else if (file_exist ("/usr/sbin/scanmails")){
+			ret |= copyfile (f_amavismailer,out);
 		}
 	}else if (deliver.cmp("deliver")==0){
-		ret |= copyfile (f_delivermailer,fout);
+		ret |= copyfile (f_delivermailer,out);
 	}else if (deliver.cmp("procmail")==0){
-		ret |= copyfile (f_procmailmailer,fout);
+		ret |= copyfile (f_procmailmailer,out);
+	}else if (deliver.cmp("amavis")==0){
+		ret |= copyfile (f_amavismailer,out);
 	}else if (deliver.cmp("mail.local")==0){
-		ret |= copyfile (f_localmailer,fout);
+		ret |= copyfile (f_localmailer,out);
 	}
 	{
 		static const char *progtoken[]={
@@ -451,10 +512,10 @@ PRIVATE int MAILCONF::generate(FILE *fout)
 				}
 			}
 		}
-		ret |= copyfile (f_progmailer,fout,progtoken,progrepl,1);
+		ret |= copyfile (f_progmailer,out,progtoken,progrepl,1);
 	}
-	ret |= copyfile (f_virtmailer,fout);
-	ret |= copyfile (f_faxmailer,fout);
+	ret |= copyfile (f_virtmailer,out);
+	ret |= copyfile (f_faxmailer,out);
 	{
 		static const char *uucptoken[]={
 			"$(MAXSIZ)","$(UUXOPT)"
@@ -465,14 +526,14 @@ PRIVATE int MAILCONF::generate(FILE *fout)
 			sprintf (uucprepl[0],"M=%d,",features.uucpmax);
 		}
 		strcpy (uucprepl[1],features.uucpnobatch ? "" : "-r");
-		ret |= copyfile (f_uucpmailer,fout
+		ret |= copyfile (f_uucpmailer,out
 			,uucptoken,uucprepl,2);
 	}
-	ret |= copyfile (f_smtpmailer,fout);
+	ret |= copyfile (f_smtpmailer,out);
 	// Let modules record mailer
 	module_sendmessage ("mailconf:defmailer",1,message_argv);
 
-	ret |= masqs.rule1(fout);
+	ret |= masqs.rule1(out);
 	return ret;
 }
 	
@@ -498,9 +559,10 @@ PRIVATE bool MAILCONF::generated_byme(char sendmail_sum[])
 PUBLIC int MAILCONF::generate_go(bool confirm)
 {
 	int ret = -1;
-	FILE *fout = f_sendmail.fopen ("w");
+	FILE_CFG *fout = f_sendmail.fopen ("w");
 	if (fout != NULL){
-		ret = generate (fout);
+		SSTREAM_FILE_CFG out (fout);
+		ret = format (out);
 		if (features.mailertable) spcs.build();
 		fclose (fout);
 		char sum[100];
@@ -528,12 +590,12 @@ PUBLIC int MAILCONF::generate_go(bool confirm)
 PUBLIC int MAILCONF::generate(bool confirm)
 {
 	int ret = -1;
-	const char *path = f_sendmail.getpath();
 	net_prtlog (NETLOG_CMD,MSG_U(I_UPDSENDMAILCF
-		,"Generating %s\n"),path);
+		,"Generating %s\n"),f_sendmail.getpath());
 	bool go = true;
 	char sum[100];
 	if (!generated_byme(sum)){
+		const char *path = f_sendmail.getpath();
 		char pathold[PATH_MAX];
 		sprintf (pathold,"%s.old",path);
 		char bufmsg[1000];
@@ -547,7 +609,7 @@ PUBLIC int MAILCONF::generate(bool confirm)
 			 "\n"
 			 "\tOk ?")
 			,path,pathold);
-		if (xconf_yesno (MSG_U(T_NOTBYME,"Attention"),bufmsg,help_nil)
+		if (dialog_yesno (MSG_U(T_NOTBYME,"Attention"),bufmsg,help_nil)
 			!=MENU_YES){
 			go = false;
 			net_prtlog (NETLOG_VERB,MSG_U(I_USERSAYNO
@@ -593,7 +655,8 @@ PUBLIC int MAILCONF::generate_if(bool confirm)
 			SSTREAM_POPEN ss (pop);
 			configf_sendexist (ss,true);
 			FILE *fout = pop.getfout();
-			ret = generate (fout);
+			SSTREAM_FILE out (fout);
+			ret = format (out);
 			pop.close();
 			if (ret != -1){
 				char line[100];
