@@ -42,7 +42,11 @@ static const char K_FALLBACK[]="vdomain_fallback";
 static const char K_STARTUID[]="vdomain_startuid";
 static const char K_FILTER[]="vdomain_filter";
 static const char K_QUOTA[]="vdomain_quota";
+static const char K_MAXUSERS[]="vdomain_maxusers";
 static const char K_GECOS[]="vdomain_gecos";
+static const char K_ACCEPTLOCK[]="vdomain_acceptlock";
+static const char K_RETRIEVELOCK[]="vdomain_retrievelock";
+
 
 static MAILCONF_HELP_FILE help_vdomain("vdomain");
 
@@ -51,14 +55,18 @@ static const char subsys_vdomain[]="vdomain";
 PUBLIC VDOMAIN::VDOMAIN(const char *line)
 {
 	domain.copyword (line);
+	oldname.copyword (line);
 	const char *dom = domain.get();
 	linuxconf_getall (K_VALIAS,dom,faliases,1);
 	linuxconf_getall (K_VOTHER,dom,others,1);
 	fallback.setfrom(linuxconf_getval (K_FALLBACK,dom));
 	startuid = linuxconf_getvalnum (K_STARTUID,dom,60000);
 	quota = linuxconf_getvalnum (K_QUOTA,dom,0);
+	maxusers = linuxconf_getvalnum (K_MAXUSERS,dom,0);
 	match_gecos = linuxconf_getvalnum (K_GECOS,dom,0);
 	filter.setfrom (linuxconf_getval(K_FILTER,dom,""));
+	acceptlock = linuxconf_getvalnum (K_ACCEPTLOCK,dom,0);
+	retrievelock = linuxconf_getvalnum (K_RETRIEVELOCK,dom,0);
 }
 
 PUBLIC VDOMAIN::VDOMAIN()
@@ -77,6 +85,9 @@ PUBLIC VDOMAIN::VDOMAIN()
 	startuid = 60000;
 	quota = 0;
 	match_gecos = 0;
+	acceptlock = 0;
+	retrievelock = 0;
+	maxusers = 0;
 }
 
 PRIVATE void VDOMAIN::showother (DIALOG &dia, int item)
@@ -94,20 +105,28 @@ PRIVATE void VDOMAIN::addother (DIALOG &dia)
 
 PUBLIC void VDOMAIN::setupdia (DIALOG &dia)
 {
+	dia.newf_title (MSG_U(T_DBASIC,"Base info"),1,"",MSG_R(T_DBASIC));
 	dia.newf_str (MSG_U(F_FALLBACK,"Fallback destination (opt)"),fallback);
 	dia.newf_num (MSG_U(F_STARTUID,"Allocate UID from"),startuid);
 	static const char *tb[]={MSG_R(I_NOLIMIT),NULL};
 	static const int tbv[]={0,0};
 	dia.newf_chkm_num (MSG_U(F_QUOTA,"Limit user inbox to (k)"),quota,tbv,tb);
+	static const char *tb2[]={MSG_R(I_NOLIMIT),NULL};
+	static const int tbv2[]={0,0};
+
+	dia.newf_title (MSG_R(T_FEATURES),1,"",MSG_R(T_FEATURES));
+	dia.newf_chkm_num (MSG_U(F_MAXUSERS,"Maximum number of users"),maxusers,tbv2,tb2);
 	dia.newf_chk ("",match_gecos,MSG_U(F_GECOS,"Match user full name"));
 	dia.newf_str (MSG_U(F_VFILTER,"Filter program + args (opt)"),filter);
+	dia.newf_chk (MSG_U(F_LOCKDOMAIN,"Lock domain"),acceptlock,MSG_U(I_LOCKACCEPT,"Incoming mail are rejected"));
+	dia.newf_chk ("",retrievelock,MSG_U(I_LOCKRETRIEVE,"Users can't retrieve their messages"));
+	
+	dia.newf_title (MSG_U(T_ALIASFILES,"Extra aliases files"),1,"",MSG_R(T_ALIASFILES));
+
 	while (faliases.getnb()<2) faliases.add (new SSTRING);
-	dia.newf_str (MSG_U(F_ALIASFILE,"Alternate Aliases files(opt)")
-		,*faliases.getitem(0));
+	dia.newf_str (MSG_U(T_FILEPATH,"File path (absolute)"),*faliases.getitem(0));
 	dia.newf_str ("",*faliases.getitem(1));
-
-
-	dia.newf_title ("",MSG_U(T_DALIAS,"Aliases for this domain"));
+	dia.newf_title (MSG_U(T_DALIAS,"Domain Aliases"),1,"",MSG_R(T_DALIAS));
 	for (int i=0; i<others.getnb(); i++){
 		showother (dia,i);
 	}
@@ -115,6 +134,33 @@ PUBLIC void VDOMAIN::setupdia (DIALOG &dia)
 	addother (dia);
 }
 
+/*
+	Rename a file or directory associated with a vdomain.
+	Do not care if it fails or not (The file or dir may not exist)
+*/
+PRIVATE void VDOMAIN::renameold (
+	const char *prefix)
+{
+	char new_path[PATH_MAX],old_path[PATH_MAX];
+	sprintf (new_path,"%s%s",prefix,domain.get());
+	sprintf (old_path,"%s%s",prefix,oldname.get());
+	rename (old_path,new_path);
+}
+
+static void vdomain_cleanupconf (const char *dom)
+{
+	linuxconf_removeall (K_VOTHER,dom);
+	linuxconf_removeall (K_VALIAS,dom);
+	linuxconf_removeall (K_STARTUID,dom);
+	linuxconf_removeall (K_QUOTA,dom);
+	linuxconf_removeall (K_MAXUSERS,dom);
+	linuxconf_removeall (K_GECOS,dom);
+	linuxconf_removeall (K_FALLBACK,dom);
+	linuxconf_removeall (K_FILTER,dom);
+	linuxconf_removeall (K_ACCEPTLOCK,dom);
+	linuxconf_removeall (K_RETRIEVELOCK,dom);
+}
+
 PUBLIC int VDOMAIN::edit (VDOMAINS &vs)
 {
 	int ret = -1;
@@ -122,7 +168,10 @@ PUBLIC int VDOMAIN::edit (VDOMAINS &vs)
 	dia.newf_str (MSG_U(F_VDOMAIN,"Virtual domain (fqdn)"),domain);
 	dia.last_noempty();
 	setupdia (dia);
-		
+	PRIVATE_MESSAGE grow;
+	dia.new_button (MSG_R(B_GROWFORM)
+		,MSG_U(I_GROWDALIAS,"Add more lines to enter domain aliases")
+		,grow);	
 	int nof = 0;
 	while (1){
 		MENU_STATUS code = dia.edit (
@@ -130,15 +179,22 @@ PUBLIC int VDOMAIN::edit (VDOMAINS &vs)
 			,MSG_U(I_DEFVDOMAIN,"")
 			,help_vdomain
 			,nof
-			,MENUBUT_CANCEL|MENUBUT_ACCEPT|MENUBUT_DEL|MENUBUT_ADD);
+			,MENUBUT_CANCEL|MENUBUT_ACCEPT|MENUBUT_DEL);
 		if (code == MENU_CANCEL || code == MENU_ESCAPE){
 			break;
-		}else if (code == MENU_ADD){
-			addother (dia);
+		}else if (code == MENU_MESSAGE){
+			if (dialog_testmessage(grow)){
+				// Remove the button, add one field, put the button back
+				dia.remove_del (dia.getnb()-1);
+				addother (dia);
+				dia.new_button (MSG_R(B_GROWFORM)
+					,MSG_R(I_GROWDALIAS)
+					,grow);	
+			}
 		}else if (code == MENU_DEL){
 			if (xconf_delok()){
 				ret = 1;
-				linuxconf_removeall (K_VOTHER,domain.get());
+				vdomain_cleanupconf (oldname.get());
 				break;
 			}
 		}else{
@@ -149,13 +205,29 @@ PUBLIC int VDOMAIN::edit (VDOMAINS &vs)
 				nof = 0;
 			}else{
 				MAILCONF mconf;
-				if (mconf.check (vs) != -1){
+				int lk = vs.lookup(this);
+				// We add the new vdomain in vs so the check includes
+				// all domains
+				if (lk == -1) vs.add (this);
+				int chk = mconf.check (vs);
+				if (lk == -1) vs.remove (this);
+				if (chk != -1){
 					faliases.remove_empty();
 					others.remove_empty();
 					domain.to_lower();
 					for (int i=0; i<others.getnb(); i++){
 						others.getitem(i)->to_lower();
 					}
+					// Check if we are doing a rename
+					if (!oldname.is_empty() && domain.cmp(oldname)!=0){
+						renameold (VHOME "/");
+						renameold (VAR_SPOOL_VMAIL "/");
+						renameold (ETC_VMAIL "/aliases.");
+						renameold (ETC_VMAIL "/passwd.");
+						renameold (ETC_VMAIL "/shadow.");
+						vdomain_cleanupconf (oldname.get());
+						oldname.setfrom (domain);
+					}
 					ret = 0;
 					break;
 				}
@@ -195,12 +267,12 @@ PUBLIC void VDOMAINS::sort()
 }
 
 
-PUBLIC VDOMAIN *VDOMAINS::getitem (int no)
+PUBLIC VDOMAIN *VDOMAINS::getitem (int no) const
 {
 	return (VDOMAIN*)ARRAY::getitem(no);
 }
 
-PUBLIC VDOMAIN *VDOMAINS::getitem (const char *domain)
+PUBLIC VDOMAIN *VDOMAINS::getitem (const char *domain) const
 {
 	VDOMAIN *ret = NULL;
 	int n = getnb();
@@ -233,7 +305,7 @@ PUBLIC VDOMAINS::VDOMAINS()
 
 	Return the VDOMAIN record or NULL if not found
 */
-PUBLIC VDOMAIN *VDOMAINS::lookup (const char *str)
+PUBLIC VDOMAIN *VDOMAINS::lookup (const char *str) const
 {
 	VDOMAIN *ret = NULL;
 	for (int i=0; i<getnb(); i++){
@@ -255,6 +327,11 @@ PUBLIC VDOMAIN *VDOMAINS::lookup (const char *str)
 	return ret;
 }
 
+PUBLIC int VDOMAINS::lookup (VDOMAIN *pt) const
+{
+	return ARRAY::lookup(pt);
+}
+
 PUBLIC int VDOMAINS::write ()
 {
 	linuxconf_setcursys (subsys_vdomain);
@@ -263,34 +340,35 @@ PUBLIC int VDOMAINS::write ()
 	for (int i=0; i<n; i++){
 		VDOMAIN *d = getitem(i);
 		const char *dom = d->domain.get();
+		vdomain_cleanupconf (dom);
 		linuxconf_add (K_VDOMAIN,K_INDEX,dom);
 		linuxconf_replace (K_VOTHER,dom,d->others);
 		linuxconf_replace (K_VALIAS,dom,d->faliases);
-		if (d->startuid == 60000){
-			linuxconf_removeall (K_STARTUID,dom);
-		}else{
+		// avoid saving default values in conf.linuxconf
+		if (d->startuid != 60000){
 			linuxconf_replace (K_STARTUID,dom,d->startuid);
 		}
-		if (d->quota == 0){
-			linuxconf_removeall (K_QUOTA,dom);
-		}else{
+		if (d->quota != 0){
 			linuxconf_replace (K_QUOTA,dom,d->quota);
 		}
-		if (d->match_gecos == 0){
-			linuxconf_removeall (K_GECOS,dom);
-		}else{
+		if (d->maxusers != 0){
+			linuxconf_replace (K_MAXUSERS,dom,d->maxusers);
+		}
+		if (d->match_gecos != 0){
 			linuxconf_replace (K_GECOS,dom,d->match_gecos);
 		}
-		if (d->fallback.is_empty()){
-			linuxconf_removeall (K_FALLBACK,dom);
-		}else{
+		if (!d->fallback.is_empty()){
 			linuxconf_replace (K_FALLBACK,dom,d->fallback);
 		}
-		if (d->filter.is_empty()){
-			linuxconf_removeall (K_FILTER,dom);
-		}else{
+		if (!d->filter.is_empty()){
 			linuxconf_replace (K_FILTER,dom,d->filter);
 		}
+		if (d->retrievelock){
+			linuxconf_replace (K_RETRIEVELOCK,dom,1);
+		}
+		if (d->acceptlock){
+			linuxconf_replace (K_ACCEPTLOCK,dom,1);
+		}
 	}
 	return linuxconf_save();
 }
@@ -324,9 +402,10 @@ PUBLIC int VDOMAINS::edit()
 	int ret = 0;
 	if (perm_rootaccess(MSG_R(P_SEEDOMAIN))){
 		int nof = 0;
+		DIALOG_LISTE dia;
 		while (1){
-			MENU_STATUS code = select (
-				MSG_U(T_VDOMAINS,"Virtual email domains")
+			MENU_STATUS code = select (dia
+				,MSG_R(T_VDOMAINS)
 				,MSG_U(I_VDOMAINS
 					,"You can define new virtual email domain\n"
 					 "virtual email hosting is a new concept.\n"
@@ -479,16 +558,20 @@ static PRIVILEGE *vdomain_lookuppriv (const char *dname)
 
 static void vdomain_editone (VDOMAIN *v, USER *like)
 {
-	CONFIG_FILE *file,*file_shadow;
-	USERS *users = v->getusers (file,file_shadow);
 	const char *dname = v->domain.get();
-	PRIVILEGE *priv = vdomain_lookuppriv (dname);
-	if (perm_access (priv,MSG_U(P_MANAGEDOM,"manage a virtual domain"))){
-		users->edit(like,priv,0);
+	CONTEXT_LOCK lk ("vdomainusers",dname);
+	if (lk.isok()){
+		CONFIG_FILE *file,*file_shadow;
+		USERS *users = v->getusers (file,file_shadow);
+		users->setmaxusers(v->maxusers);
+		PRIVILEGE *priv = vdomain_lookuppriv (dname);
+		if (perm_access (priv,MSG_U(P_MANAGEDOM,"manage a virtual domain"))){
+			users->edit(like,priv,0);
+		}
+		delete users;
+		delete file;
+		delete file_shadow;
 	}
-	delete users;
-	delete file;
-	delete file_shadow;
 }
 
 /*
@@ -496,23 +579,30 @@ static void vdomain_editone (VDOMAIN *v, USER *like)
 	Return != 0 if the has been one modification.
 */
 PUBLIC MENU_STATUS VDOMAINS::select(
+	DIALOG_LISTE &dia,
 	const char *title,
 	const char *intro,
 	int mayadd,
 	int &nof)
 {
 	sort();
-	DIALOG_LISTE dia;
-	dia.newf_head ("",MSG_U(H_VDOMAINS,"Virtual email domains"));
+	if (dia.getnb()==0){
+		dia.newf_head ("",MSG_U(H_VDOMAINS,"Virtual email domains\tStatus"));
+	}
 	int lookup[getnb()];
+	int found = 0;
 	for (int i=0; i<getnb(); i++){
 		VDOMAIN *d = getitem(i);
 		PRIVILEGE *priv = vdomain_lookuppriv (d->domain.get());
 		if (perm_checkpriv(priv)){
-			lookup[dia.getnb()-1] = i;
-			dia.new_menuitem(d->domain.get(),"");
+			lookup[found] = i;
+			dia.set_menuitem(found,d->domain.get()
+				,(d->acceptlock || d->retrievelock)
+					? MSG_U(I_LOCKED,"Locked") : "");
+			found++;
 		}
 	}
+	dia.remove_last (found+1);
 	if (dia.getnb()==1){
 		// No domain visible by the user
 		// check with root privileges
@@ -524,8 +614,12 @@ PUBLIC MENU_STATUS VDOMAINS::select(
 			}
 		}
 	}
-	if (mayadd) dia.addwhat (MSG_U(I_ONEVDOMAIN,"Select [Add] to add one virtual domain"));
-	MENU_STATUS code = dia.editmenu (title,intro,help_vdomain,nof,0);
+	int buttons = 0;
+	if (mayadd){
+		dia.addwhat (MSG_U(I_ONEVDOMAIN,"Select [Add] to add one virtual domain"));
+		buttons = MENUBUT_ADD;
+	}
+	MENU_STATUS code = dia.editmenu (title,intro,help_vdomain,nof,buttons);
 	if (code == MENU_OK) nof = lookup[nof];
 	return code;
 }
@@ -557,9 +651,10 @@ void vdomain_editusers (USER *like)
 		xconf_error (MSG_U(E_NOVDOM,"No virtual email domain defined"));
 	}else{
 		int nof = 0;
+		DIALOG_LISTE dia;
 		while (1){
-			MENU_STATUS code = vdom.select(
-				MSG_U(T_PICKVDOM,"Pick the domain")
+			MENU_STATUS code = vdom.select(dia
+				,MSG_U(T_PICKVDOM,"Pick the domain")
 				,""
 				,0,nof);
 			if (code == MENU_QUIT || code == MENU_ESCAPE){
@@ -643,9 +738,10 @@ void vdomain_editaliases ()
 		xconf_error (MSG_R(E_NOVDOM));
 	}else{
 		int nof = 0;
+		DIALOG_LISTE dia;
 		while (1){
-			MENU_STATUS code = vdom.select(
-				MSG_R(T_PICKVDOM)
+			MENU_STATUS code = vdom.select(dia
+				,MSG_R(T_PICKVDOM)
 				,""
 				,0,nof);
 			if (code == MENU_QUIT || code == MENU_ESCAPE){
@@ -673,9 +769,10 @@ void vdomain_editpolicies ()
 		xconf_error (MSG_R(E_NOVDOM));
 	}else{
 		int nof = 0;
+		DIALOG_LISTE dia;
 		while (1){
-			MENU_STATUS code = vdom.select(
-				MSG_R(T_PICKVDOM)
+			MENU_STATUS code = vdom.select(dia
+				,MSG_R(T_PICKVDOM)
 				,""
 				,0,nof);
 			if (code == MENU_QUIT || code == MENU_ESCAPE){
@@ -985,7 +1082,7 @@ int vdomain_useradd (
 	}else{
 		CONFIG_FILE *file,*file_shadow;
 		USERS *users = v->getusers (file,file_shadow);
-		ret = users->addbatch (id,"popusers",name,"/bin/false");
+		ret = users->addbatch (id,"popusers",name,"/bin/false",true);
 		delete users;
 		delete file;
 		delete file_shadow;
@@ -1006,7 +1103,7 @@ int vdomain_userdel (
 	}else{
 		CONFIG_FILE *file,*file_shadow;
 		USERS *users = v->getusers (file,file_shadow);
-		ret = users->delbatch (id);
+		ret = users->delbatch (id,DELOPER_ARCHIVE);
 		delete users;
 		delete file;
 		delete file_shadow;
@@ -1051,15 +1148,23 @@ int vdomain_passwd (
 }
 static int vdomain_setmainaliaspath (
 	const char *domain,
-	char path[PATH_MAX])
+	char path[PATH_MAX],
+	bool must_exist)		// Make sure the file exist
 {
 	int ret = -1;
-	snprintf (path,PATH_MAX-1,"%s/aliases.%s",ETC_VMAIL,domain);
-	if (file_type (path)!=0){
-		fprintf (stderr,MSG_U(E_NOALIASEFILE,"Aliases file %s does not exist\n")
-			,path);
+	VDOMAINS conf;
+	VDOMAIN *v = conf.locate (domain);
+	if (v == NULL){
+		fprintf (stderr
+			,MSG_R(E_NOVDOMAIN),domain);
 	}else{
-		ret = 0;
+		snprintf (path,PATH_MAX-1,"%s/aliases.%s",ETC_VMAIL,domain);
+		if (file_type (path)!=0 && must_exist){
+			fprintf (stderr,MSG_U(E_NOALIASEFILE,"Aliases file %s does not exist\n")
+				,path);
+		}else{
+			ret = 0;
+		}
 	}
 	return ret;
 }
@@ -1080,7 +1185,7 @@ int vdomain_setalias (
 {
 	int ret = -1;
 	char path[PATH_MAX];
-	if (vdomain_setmainaliaspath (domain,path)!=-1){
+	if (vdomain_setmainaliaspath (domain,path,false)!=-1){
 		CONFIG_FILE file (path,help_nil
 			,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
 			,"root","mail",0640
@@ -1105,7 +1210,7 @@ int vdomain_unsetalias (
 {
 	int ret = -1;
 	char path[PATH_MAX];
-	if (vdomain_setmainaliaspath (domain,path)!=-1){
+	if (vdomain_setmainaliaspath (domain,path,true)!=-1){
 		CONFIG_FILE file (path,help_nil
 			,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
 			,"root","mail",0640
