@@ -1,13 +1,16 @@
 #include <stdlib.h>
+#include <ctype.h>
 #include <string.h>
 #include "managerpm.h"
 #include "managerpm.m"
+#include <../../libmodules/filesystem/filesystem.h>
 #include <popen.h>
 #include <userconf.h>
 #include <netconf_def.h>
 #include <daemoni.h>
 #include <fstab.h>
 #include <regex.h>
+#include <proto.h>
 
 static const char K_MANAGERPM[]="managerpm";
 static const char K_DIRS[]="dirs";
@@ -25,6 +28,7 @@ static HELP_FILE help_install ("managerpm","install");
 
 /*
 	Execute the RPM command and report error messages
+	Return the rpm error code (0 if everything was fine).
 */
 int mngrpm_execrpm (
 	const SSTRING &args,
@@ -119,11 +123,64 @@ void mngrpm_setpath (const char *path, char *abspath, int size)
 		,path);
 }
 
+enum PARSEBUF_STATE {
+	PARSEBUF_START,		// Start of a line
+	PARSEBUF_STARTNAME,	// This is the first character of the package name
+	PARSEBUF_NAME,			// We are getting the rpm name
+	PARSEBUF_STARTSPACE,	// These are spaces before the hash
+	PARSEBUF_SPACE,		// These are spaces before the hash
+	PARSEBUF_STARTHASH,	// starting the see the hashed
+	PARSEBUF_HASH,			// This is a hash
+	PARSEBUF_ENDHASH,		// Ok, we have seen a full package installation
+	PARSEBUF_LOST,			// This is not valid
+};
+
+/*
+	This return the next parser start based on the character to process
+	and the number of hash seen.
+*/
+static PARSEBUF_STATE mngrpm_state (
+	PARSEBUF_STATE state,
+	char car,			// Character to process
+	int nbhash)
+{
+	PARSEBUF_STATE ret = PARSEBUF_LOST;
+	if (car == '\n'){
+		ret = PARSEBUF_START;
+	}else if (car == '#'){
+		if (state == PARSEBUF_SPACE){
+			ret = PARSEBUF_STARTHASH;
+		}else if (state == PARSEBUF_STARTHASH){
+			ret = PARSEBUF_HASH;
+		}else if (state == PARSEBUF_HASH){
+			if (nbhash == 49){
+				ret = PARSEBUF_ENDHASH;
+			}else{
+				ret = PARSEBUF_HASH;
+			}
+		}
+	}else if (car > ' '){
+		if (state == PARSEBUF_START || state == PARSEBUF_ENDHASH){
+			ret = PARSEBUF_STARTNAME;
+		}else if (state == PARSEBUF_STARTNAME || state == PARSEBUF_NAME){
+			ret = PARSEBUF_NAME;
+		}
+	}else if (isspace(car)){
+		if (state == PARSEBUF_NAME){
+			ret = PARSEBUF_STARTSPACE;
+		}else if (state == PARSEBUF_STARTSPACE || state == PARSEBUF_SPACE){
+			ret = PARSEBUF_SPACE;
+		}
+	}
+	return ret;
+}
+
 int mngrpm_domany (
 	const PACKAGES &pkgs,
 	const char *title,
 	const char *rpmoper,
-	const RPM_OPTIONS &options)
+	const RPM_OPTIONS &options,
+	bool showinfo)
 {
 	char rootopt[PATH_MAX];
 	mngrpm_setrootopt (rootopt);
@@ -155,6 +212,7 @@ int mngrpm_domany (
 			if (pop.isok()){
 				DIALOG diastatus;
 				diastatus.settype (DIATYPE_POPUP);
+				diastatus.set_formparms ("htrigger=700");
 				SSTRING curpkg,summary;
 				diastatus.newf_str (MSG_U(F_PACKAGE,"Package"),curpkg);
 				diastatus.newf_str ("",summary,80);
@@ -162,12 +220,11 @@ int mngrpm_domany (
 				diastatus.newf_gauge ("",gauge,50);
 				diastatus.newf_gauge ("",installed,nbpkg);
 				int nof = 0;
-				diastatus.show (MSG_U(T_INSTALLING,"Instaling packages")
-					,"",help_nil,nof,0);
+				diastatus.show (title,"",help_nil,nof,0);
 				diagui_flush();
-				bool logged = false;
 				DIALOG dia;
 				dia.settype (DIATYPE_NOTICE);
+				PARSEBUF_STATE state = PARSEBUF_START;
 				while (pop.wait(1000000)!=-1){
 					char buf[1000];
 					while (pop.readoutraw(buf,sizeof(buf)-1)> 0){
@@ -177,39 +234,59 @@ int mngrpm_domany (
 						bool reload_pkg = false;
 						while (*pt != '\0'){
 							char carac = *pt++;
-							if (carac == '#'){
+							PARSEBUF_STATE newstate = mngrpm_state(state,carac,gauge);
+							switch (newstate){
+							case PARSEBUF_START:
+								break;
+							case PARSEBUF_STARTNAME:	// This is the first character of the package name
+								curpkg.setfrom ("");
+								// Continue to the next, no break
+							case PARSEBUF_NAME:		// We are getting the rpm name
+								{
+									char tmp[2]={carac,'\0'};
+									curpkg.append (tmp);
+									reload_pkg = true;
+								}
+								break;
+							case PARSEBUF_STARTSPACE:	// These are spaces before the hash
+								{
+									PACKAGE *cur = pkgs.locate(curpkg.get());
+									if (cur != NULL){
+										summary.setfrom (cur->summary);
+									}else{
+										summary.setfrom ("");
+									}
+									diastatus.reload (1);
+								}
+							case PARSEBUF_SPACE:		// These are spaces before the hash
+								break;
+							case PARSEBUF_STARTHASH:
+								gauge=1;
+								reload_gauge = true;
+								break;
+							case PARSEBUF_HASH:
 								gauge++;
 								reload_gauge = true;
-							}else if (carac > ' '){
-								char tmp[2]={carac,'\0'};
-								curpkg.append (tmp);
-								reload_pkg = true;
-							}else if (carac == '\n'){
+								break;
+							case PARSEBUF_ENDHASH:
+								dia.newf_info ("",curpkg.get());
 								installed++;
 								diastatus.reload(3);
 								gauge = 0;
 								curpkg.setfrom ("");
 								reload_gauge = true;
 								reload_pkg = true;
-								logged = false;
-							}else if (carac <= ' ' && !curpkg.is_empty()
-								&& !logged){
-								PACKAGE *cur = pkgs.locate(curpkg.get());
-								if (cur != NULL){
-									summary.setfrom (cur->summary);
-								}else{
-									summary.setfrom ("");
-								}
-								diastatus.reload (1);
-								dia.newf_info ("",curpkg.get());
-								logged = true;
+								break;
+							case PARSEBUF_LOST:	// This is not valid
+								// Do nothing
+								break;
 							}
+							state = newstate;
 						}
 						if (reload_gauge || reload_pkg){
 							if (reload_pkg)	diastatus.reload(0);
 							if (reload_gauge) diastatus.reload(2);
-							diastatus.show (MSG_R(T_INSTALLING)
-								,"",help_nil,nof,0);
+							diastatus.show (title,"",help_nil,nof,0);
 							diagui_flush();
 						}
 					}
@@ -222,12 +299,14 @@ int mngrpm_domany (
 				diastatus.hide();
 				diagui_flush();
 				ret = pop.getstatus() ? -1 : 0;
-				if (dia.getnb() != 0){
-					int nof = 0;
-					dia.edit (MSG_U(N_RPMRES,"Rpm results")
-						,"",help_nil,nof,MENUBUT_CANCEL);
-				}else{
-					xconf_notice (MSG_U(N_DONE,"Done"));
+				if (showinfo || ret != 0){
+					if (dia.getnb() != 0){
+						int nof = 0;
+						dia.edit (MSG_U(N_RPMRES,"Rpm results")
+							,"",help_nil,nof,MENUBUT_CANCEL);
+					}else{
+						xconf_notice (MSG_U(N_DONE,"Done"));
+					}
 				}
 			}
 		}
@@ -258,6 +337,7 @@ PUBLIC int PACKAGES::uninstall()
 		bool nodeps = false;
 		if (testuninstall(nodeps)){
 			DIALOG dia;
+			dia.settype (DIATYPE_POPUP);
 			RPM_OPTIONS options;
 			options.nodep = nodeps ? 1 : 0;
 			mngrpm_setdelopt (dia,options);
@@ -269,7 +349,7 @@ PUBLIC int PACKAGES::uninstall()
 					break;
 				}else{
 					if (mngrpm_domany (*this,MSG_U(T_DELPKGS,"Erase packages")
-						,"-ev",options)==0){
+						,"-ev",options,true)==0){
 						ret = 0;
 						break;
 					}
@@ -303,6 +383,7 @@ PUBLIC int PACKAGES::install()
 	int ret = -1;
 	if (any_selected()){
 		DIALOG dia;
+		dia.settype (DIATYPE_POPUP);
 		RPM_OPTIONS options;
 		mngrpm_setinstopt(dia,options);
 		int nof = 0;
@@ -313,7 +394,7 @@ PUBLIC int PACKAGES::install()
 				break;
 			}else{
 				if (mngrpm_domany (*this,MSG_U(T_INSTPKGS,"Install packages")
-					,"-iv",options)==0){
+					,"-iv",options,true)==0){
 					ret = 0;
 					break;
 				}
@@ -385,9 +466,14 @@ PARSE_STATE::PARSE_STATE(
 	nbloaded = 0;
 
 	dia.settype (DIATYPE_POPUP);
-	dia.newf_gauge ("",nbloaded,nbpkg);
+	char tmp[1000];
+	dia.gui_passthrough (P_Label,"%s",diagui_quote(_title,tmp));
+	dia.newline();
+	dia.newf_gauge (NULL,nbloaded,nbpkg);
+	dia.gui_dispolast (GUI_H_CENTER,1,GUI_V_CENTER,1);
 	int nof = 0;
 	dia.show (title,"",help_nil,nof,0);
+	diagui_flush();
 }
 
 
@@ -401,7 +487,8 @@ PRIVATE void PACKAGES::parse (
 		if (buf[1] == '#'){
 			char tb[7][100];
 			const int sizetb = sizeof(tb)/sizeof(tb[0]);
-			if (str_splitline(buf+2,'\t',tb,sizetb)==sizetb){
+			tb[6][0] = '\0';
+			if (str_splitline(buf+2,'\t',tb,sizetb)>=sizetb-1){
 				for (int i=0; i<sizetb; i++) strip_end (tb[i]);
 				st.curpkg = new PACKAGE(tb[0],tb[1],tb[2],tb[3],tb[4],tb[5],tb[6]);
 				st.nbloaded++;
@@ -426,10 +513,25 @@ PRIVATE void PACKAGES::parse (
 	}else if (buf[0] != '\0' && strcmp(buf,"(none)")!=0){
 		if (st.curpkg != NULL){
 			if (st.state == requires){
-				st.curpkg->addrequire (buf);
+				/* #Specification: rpm output / hack / rpmlib
+					The RPMs contains requierements about rpm itself
+					in the form of "rpmlib(features) <= version".
+					We ignore that.
+				*/
+				if (strncmp(buf,"rpmlib(",7)!=0) st.curpkg->addrequire (buf);
 			}else if (st.state == provides
 				|| st.state == list){
-				st.curpkg->addprovide (buf);
+				/* #Specification: rpm output / hack
+					Some package do not contains any files. The rpm commands
+					output the string "(contains no files)". There does
+					not seem to be an option to get a raw output.
+
+					This string is probably translatable, so we can't trigger
+					on this. But a file name starting with an open parenthesis
+					is probably not possible either. So we discard anything
+					starting with an open parenthesis
+				*/
+				if (buf[0] != '(') st.curpkg->addprovide (buf);
 			}
 		}
 	}
@@ -442,7 +544,9 @@ PUBLIC int PACKAGES::load (
 	const char *rpmarg,
 	bool is_installed,		// Set the is_installed flag
 	int nbpkg,				// How many package to load or -1 if not known
-	const char *title)		// Title for the gauge dialog
+	const char *title,		// Title for the gauge dialog
+	SSTRINGS *tberr)		// Potentially invalid packages
+							// May be NULL
 {
 	int ret = -1;
 	char arg[PATH_MAX+strlen(rpmarg)];
@@ -460,19 +564,48 @@ PUBLIC int PACKAGES::load (
 	POPEN pop ("rpm",arg);
 	if (pop.isok()){
 		PARSE_STATE st (nbpkg,is_installed,title);
-		while (pop.wait(10) > 0){
+		SSTRING errors;
+		while (pop.wait(100) > 0){
 			char buf[1000];
 			while (pop.readout(buf,sizeof(buf)-1)!=-1){
 				parse (st,buf);
 			}
+			// hack:			
+			// Some packages may be invalid. rpm generally reports
+			// something like: foo.rpm does not appear to be ...
+			// We do a little hack. We take the first word of every
+			// error line and put it in tberr. The caller uses those
+			// words to discard invalid package. I have not found a clean
+			// to do this, beside using librpm maybe.
+			while (pop.readerr(buf,sizeof(buf)-1)!=-1){
+				errors.append (buf);
+				if (tberr != NULL){
+					SSTRING *s = new SSTRING;
+					s->copyword (buf);
+					tberr->add (s);
+				}
+			}
 		}
+		if (errors.is_filled()) xconf_error ("%s",errors.get());
 		ret = 0;
 	}else{
 		xconf_error (MSG_U(E_CANTEXEC,"Can't execute the command\nrpm %s")
 			,arg);
 	}
+	diagui_flush();
 	return ret;
-	
+}
+
+/*
+	Load the list of installed packages or packages from some directory
+*/
+PUBLIC int PACKAGES::load (
+	const char *rpmarg,
+	bool is_installed,		// Set the is_installed flag
+	int nbpkg,				// How many package to load or -1 if not known
+	const char *title)		// Title for the gauge dialog
+{
+	return load (rpmarg,is_installed,nbpkg,title,NULL);
 }
 
 /*
@@ -484,7 +617,8 @@ PUBLIC int PACKAGES::loadinstall()
 	mngrpm_setrootopt(rootopt);
 	SSTRING s;
 	s.setfromf ("%s -qa",rootopt);
-	return load (s.get(),true,-1,MSG_U(T_LOADINGINSTPKG,"Loading installed packages information"));
+	return load (s.get(),true,-1
+		,MSG_U(T_LOADINGINSTPKG,"Loading installed packages information"));
 }
 
 /*
@@ -598,6 +732,9 @@ PUBLIC int PACKAGES::loadfrompath (const char *path)
 		,MSG_U(T_LOADINGPKG,"Loading packages information"));
 	if (ret != -1){
 		remove_dups();
+		if (getnb () > 0){
+			((PACKAGE *) getitem (0))->fullname.setfrom (path);
+		}
 	}
 	return ret;
 }
@@ -608,17 +745,26 @@ PUBLIC int PACKAGES::loadfrompath (const char *path)
 PRIVATE int PACKAGES::loadfromdir_real(
 	const char *dir,
 	const char *wild,
-	const SSTRINGS &tb)
+	SSTRINGS &tb)		// Invalid package may be removed
 {
-	int size=1;
-	for (int i=0; i<tb.getnb(); i++) size += tb.getitem(i)->getlen() + 1;
-	char arg[size+4];
-	char *pt = stpcpy (arg,"-qp");
-	for (int i=0; i<tb.getnb(); i++){
-		*pt++ = ' ';
-		pt = stpcpy (pt,tb.getitem(i)->get());
+	int ret = 0;
+	if (tb.getnb() > 0){
+		int size=1;
+		for (int i=0; i<tb.getnb(); i++) size += tb.getitem(i)->getlen() + 1;
+		char arg[size+4];
+		char *pt = stpcpy (arg,"-qp");
+		for (int i=0; i<tb.getnb(); i++){
+			*pt++ = ' ';
+			pt = stpcpy (pt,tb.getitem(i)->get());
+		}
+		SSTRINGS tberr;
+		ret = load (arg,false,tb.getnb(),MSG_R(T_LOADINGPKG),&tberr);
+		for (int i=0; i<tberr.getnb(); i++){
+			const char *ivld = tberr.getitem(i)->get();
+			int no = tb.lookup(ivld);
+			if (no != -1) tb.remove_del (no);
+		}
 	}
-	int ret = load (arg,false,tb.getnb(),MSG_R(T_LOADINGPKG));
 	return ret;
 }
 
@@ -909,15 +1055,19 @@ int mngrpm_selectupd (
 	Update many package after checking that it may succeed (dependancy
 	testing).
 */
-int mngrpm_doupdate (PACKAGES &newpkgs, RPM_OPTIONS &options)
+int mngrpm_doupdate (
+	PACKAGES &newpkgs,
+	RPM_OPTIONS &options,
+	bool showinfo)
 {
 	int ret = -1;
-	bool nodeps,replacefiles;
+	bool nodeps = options.nodep==1,replacefiles=false;
+	nodeps = options.nodep != 0;
 	if (newpkgs.testinstall(nodeps,replacefiles)){
 		if (nodeps) options.nodep = 1;
 		if (replacefiles) options.replace = 1;
 		ret = mngrpm_domany (newpkgs,MSG_U(T_UPDPKG,"Update packages")
-		,"-Uvh",options);
+			,"-Uvh",options,showinfo);
 	}
 	return ret;
 }
@@ -957,6 +1107,7 @@ void mngrpm_setdirfield (DIALOG &dia, SSTRING &dir)
 	dia.last_noempty();
 }
 
+#if 0
 /*
 	Check if a given directory exist.
 	If it does not exist, try to find out if it is part of an
@@ -968,7 +1119,7 @@ void mngrpm_setdirfield (DIALOG &dia, SSTRING &dir)
 	mngrpm_unmountif() must be called by the client to do the unmount
 	if needed.
 */
-bool mngrpm_ismounted (const char *path, SSTRING &mpoint)
+bool mngrpm_ismounted (const char *path, SSTRING &mpoint, const char *cdtitle)
 {
 	bool ret = true;
 	FSTAB fstab;
@@ -992,23 +1143,28 @@ bool mngrpm_ismounted (const char *path, SSTRING &mpoint)
 		}
 	}
 	if (found != NULL && !found->is_mounted()){
-		char buf[2000];
-		snprintf (buf,sizeof(buf)-1
-			,MSG_U(I_MOUNT,"The directory %s\n"
-				"is part of an unmounted filesystem\n"
-				"Do you want to mount it ?"),path);
-		if (strcmp(found->getfs(),"iso9660")==0){
-			strcat (buf,MSG_U(I_PUTCD,"\n(Do not forget to put the CDrom in)"));
+		SSTRING buf;
+		if (cdtitle != NULL && cdtitle[0] != '\0'){
+			buf.setfromf (MSG_U(I_INSERTCD
+				,"Please insert CDROM\n\n\t%s"),cdtitle);
+		}else{
+			buf.setfromf (MSG_U(I_MOUNT
+					,"The directory %s\n"
+					"is part of an unmounted filesystem\n"
+					"Do you want to mount it ?"),path);
+			if (strcmp(found->getfs(),"iso9660")==0){
+				buf.append (MSG_U(I_PUTCD,"\n(Do not forget to put the CDrom in)"));
+			}
 		}
-		if (xconf_yesno (MSG_U(Q_MOUNT,"Mounting file system")
-			,buf,help_mount)==MENU_YES
+		if (dialog_yesno (MSG_U(Q_MOUNT,"Mounting file system")
+			,buf.get(),help_mount)==MENU_YES
 			&& perm_rootaccess(MSG_U(P_MOUNT,"mount file systems"))){
 			ret = found->domount() != -1;
 			if (ret) mpoint.setfrom (found->getmpoint());
 		}
 	}
 	if (ret){
-		int type = file_type (path);
+		int type = file_type (path,true); /* Follow symlinks. */
 		if (type == -1){
 			xconf_error (MSG_U(E_MISSING,"%s does not exist"),path);
 			ret = false;
@@ -1026,6 +1182,7 @@ void mngrpm_unmountif (SSTRING &mpoint)
 {
 	if (!mpoint.is_empty()) netconf_system_if ("umount",mpoint.get());
 }
+#endif
 
 static void mngrpm_updatedir ()
 {
@@ -1047,7 +1204,8 @@ static void mngrpm_updatedir ()
 		,""
 		,help_updatedir
 		,nof)==MENU_ACCEPT){
-		if (mngrpm_ismounted(dir.get(),mpoint)){
+		dia.hide();
+		if (filesystem_ismounted(dir.get(),mpoint,"")){
 			PACKAGES install,fromdir;
 			if (install.loadinstall() != -1
 				&& fromdir.loadfromdir(dir.get(),wild.get())!=-1){
@@ -1056,14 +1214,14 @@ static void mngrpm_updatedir ()
 						,wild.get(),dir.get());
 				}else if (mngrpm_selectupd (install,fromdir,showdup
 					,showold,shownew,options)!=-1){
-					if (mngrpm_doupdate (fromdir,options)!=-1){
+					if (mngrpm_doupdate (fromdir,options,true)!=-1){
 						break;
 					}
 				}
 			}
 		}
 	}
-	mngrpm_unmountif (mpoint);
+	filesystem_unmountif (mpoint);
 }
 
 static void mngrpm_updatepkg ()
@@ -1088,7 +1246,7 @@ static void mngrpm_updatepkg ()
 					,pkg.get());
 			}else if (mngrpm_selectupd (install,fromdir
 				,false,true,true,options)!=-1){
-				if (mngrpm_doupdate (fromdir,options) != -1) break;
+				if (mngrpm_doupdate (fromdir,options,true) != -1) break;
 			}
 		}
 	}
@@ -1247,7 +1405,8 @@ static void mngrpm_search ()
 		if (code == MENU_CANCEL || code == MENU_ESCAPE){
 			break;
 		}else{
-			if (mngrpm_ismounted (dir.get(),mpoint)){
+			dia.hide();
+			if (filesystem_ismounted (dir.get(),mpoint,"")){
 				PACKAGES fromdir;
 				if (fromdir.loadfromdir(dir.get(),wild.get())!=-1){
 					if (fromdir.getnb()==0){
@@ -1260,7 +1419,7 @@ static void mngrpm_search ()
 			}
 		}
 	}
-	mngrpm_unmountif (mpoint);
+	filesystem_unmountif (mpoint);
 }
 
 
@@ -1332,7 +1491,7 @@ void mngrpm_menu()
 	static const char *updatepkg = MSG_U(M_UPDATEPKG,"Install/Update one package");
 	static const char *updatedir = MSG_U(M_UPDATEDIR,"Install/Update many packages");
 	static const char *browseins = MSG_U(M_BROWEINS,"Browse installed packages");
-	static const char *browseunins = MSG_U(M_BROWEUNINS,"Browse un-installed packages");
+	static const char *browseunins = MSG_U(M_BROWEUNINS,"Browse non installed packages");
 	static const char *alien = MSG_U(M_ALIEN,"Show add-on packages");
 	static const char *search = MSG_U(M_SEARCH,"Search a package");
 	static const char *snapshot = MSG_U(M_SNAPSHOT,"Take a snapshot of the packages");
@@ -1376,6 +1535,7 @@ void mngrpm_menu()
 			}else if (sel == search){
 				mngrpm_search();
 			}else if (sel == snapshot){
+				xconf_notice ("Sorry not implemented");
 			}
 		}
 	}
