@@ -2,7 +2,7 @@ extern const char **_dictionary_modemconf;
 #ifndef DICTIONARY_REQUEST
 	#define DICTIONARY_REQUEST \
 	const char **_dictionary_modemconf;\
-	TRANSLATE_SYSTEM_REQ _dictionary_req_modemconf("modemconf",_dictionary_modemconf,7,0);\
+	TRANSLATE_SYSTEM_REQ _dictionary_req_modemconf("modemconf",_dictionary_modemconf,12,0);\
 	void dummy_dict_modemconf(){}
 #endif
 #ifndef MSG_U
@@ -20,3 +20,8 @@ extern const char **_dictionary_modemconf;
 #define T_MODEMCONF	4
 #define I_MODEMCONF	5
 #define P_SELDEV	6
+#define T_WAIT	7
+#define I_PROGRESS	8
+#define N_NOMODEM	9
+#define N_MODEM	10
+#define P_QRYDEV	11
