@@ -18,10 +18,13 @@ struct PACKAGE_API{
 	int (*path2pkg) (PACKAGE_API *o, const char *path, SSTRING &name, PACKAGE_VERSION &res);
 	int (*showinfo) (PACKAGE_API *o, const char *pkg);
 	// Install a selected list of package out of a dir containing more
-	int (*installpkgs) (PACKAGE_API *o, const SSTRINGS &names, const char *dir);
+	int (*installpkgs) (PACKAGE_API *o, const SSTRINGS &names, const char *dir, const char *cdtitle);
 	// Set the root where the package are installed
 	void (*setroot) (PACKAGE_API *o, const char *rootdir);
+	// Retrieve the list of installed packages
 	int (*listinstalled) (PACKAGE_API *o, SSTRINGS &lst);
+	// Retrieve the list of files owned by a package.
+	int (*listfiles) (PACKAGE_API *o, const char *package, SSTRINGS &lst);
 };
 
 #endif
