@@ -6,6 +6,8 @@ class SERVICES;
 struct SERVICECTL_API{
 	// Collect all service objects
 	int (*collect) (SERVICES &tb);
+	// Popup the control dialog for a service managing this program
+	int (*control) (const char *path);
 };
 
 #endif
