@@ -20,7 +20,7 @@ else
 	cd $DIR
 	mkdir etc etc/linuxconf var var/named etc/ppp
 	touch etc/passwd etc/group etc/fstab etc/mtab
-	touch etc/hosts etc/networks
+	touch etc/hosts etc/networks etc/conf.linuxconf
 	touch etc/aliases
 
 	for f in host.conf resolv.conf shells syslog.conf yp.conf \
