@@ -19,7 +19,7 @@ public:
 	int dohtml (const char *key);
 	int domenu (MENU_CONTEXT context,
 		 const char *key);
-	int execmain (int argc, char *argv[]);
+	int execmain (int argc, char *argv[], bool);
 	void setmenu (DIALOG&dia,
 		 MENU_CONTEXT context);
     /*~PROTOEND~ MODULE_pppdialin */
