@@ -1,11 +1,13 @@
-/* pppdialin.cc 25/11/1999 15.01.52 */
+/* _dict.cc 11/04/1998 02.28.50 */
+/* ipupupd.cc 19/12/2000 22.19.44 */
+/* pppdialin.cc 28/08/2000 18.14.02 */
 PUBLIC MODULE_pppdialin::MODULE_pppdialin (void);
 PUBLIC void MODULE_pppdialin::setmenu (DIALOG&dia,
 	 MENU_CONTEXT context);
 PUBLIC int MODULE_pppdialin::domenu (MENU_CONTEXT context,
 	 const char *key);
 PUBLIC int MODULE_pppdialin::dohtml (const char *key);
-PUBLIC int MODULE_pppdialin::execmain (int argc, char *argv[]);
+PUBLIC int MODULE_pppdialin::execmain (int argc, char *argv[], bool);
 PUBLIC PPPACCT_COMNG::PPPACCT_COMNG (DICTIONARY&_dict);
 PUBLIC void PPPACCT_COMNG::setupdia (DIALOG&dia);
 PRIVATE void PPPACCT_COMNG::saveif (CONFDB&conf,
@@ -17,8 +19,6 @@ PRIVATE void PPPACCT_COMNG::saveif (CONFDB&conf,
 PUBLIC int PPPACCT_COMNG::save (PRIVILEGE *priv);
 PUBLIC int PPPACCT_COMNG::validate (DIALOG&, int &nof);
 PUBLIC int PPPACCT_COMNG::deluser (PRIVILEGE *priv);
-/* _dict.cc 11/04/1998 02.28.50 */
-/* pppparms.cc 25/11/1999 15.05.38 */
+/* pppparms.cc 12/12/1999 22.29.06 */
 PUBLIC PPPINFO::PPPINFO (void);
 PUBLIC void PPPINFO::load (FILE *fin, const char *name);
-/* ipupupd.cc 19/11/1999 01.41.58 */
