@@ -15,7 +15,7 @@ public:
 	MODULE_rarp (void);
 	int dohtml (const char *key);
 	int domenu (MENU_CONTEXT context, const char *key);
-	int execmain (int argc, char *argv[]);
+	int execmain (int argc, char *argv[], bool);
 	int probe (int level, int , bool);
 	void setmenu (DIALOG&dia, MENU_CONTEXT context);
     /*~PROTOEND~ MODULE_rarp */
