@@ -1,12 +1,12 @@
-/* _dict.cc 03/05/98 04.27.50 */
-/* mod_rarp.cc 03/05/98 04.43.54 */
+/* _dict.cc 03/05/1998 00.27.50 */
+/* mod_rarp.cc 28/08/2000 18.13.32 */
 PUBLIC MODULE_rarp::MODULE_rarp (void);
 PUBLIC void MODULE_rarp::setmenu (DIALOG&dia, MENU_CONTEXT context);
 PUBLIC int MODULE_rarp::domenu (MENU_CONTEXT context, const char *key);
 PUBLIC int MODULE_rarp::dohtml (const char *key);
 PUBLIC int MODULE_rarp::probe (int level, int , bool);
-PUBLIC int MODULE_rarp::execmain (int argc, char *argv[]);
-/* rarp.cc 03/05/98 04.41.00 */
+PUBLIC int MODULE_rarp::execmain (int argc, char *argv[], bool);
+/* rarp.cc 07/08/2000 00.40.58 */
 PUBLIC RARP_ENTRY::RARP_ENTRY (const char *str, int noline);
 PUBLIC RARP_ENTRY::RARP_ENTRY (const char *_ip,
 	 const char *_hw,
