@@ -14,7 +14,10 @@ class MODULE_redhat: public LINUXCONF_MODULE{
 public:
 	MODULE_redhat (void);
 	int dohtml (const char *key);
-	int execmain (int argc, char *argv[]);
+	int execmain (int argc, char *argv[], bool);
+	int message (const char *msg,
+		 int argc,
+		 const char *argv[]);
     /*~PROTOEND~ MODULE_redhat */
 };
 
