@@ -2,7 +2,7 @@ extern const char **_dictionary_redhat;
 #ifndef DICTIONARY_REQUEST
 	#define DICTIONARY_REQUEST \
 	const char **_dictionary_redhat;\
-	TRANSLATE_SYSTEM_REQ _dictionary_req_redhat("redhat",_dictionary_redhat,27,3);\
+	TRANSLATE_SYSTEM_REQ _dictionary_req_redhat("redhat",_dictionary_redhat,52,3);\
 	void dummy_dict_redhat(){}
 #endif
 #ifndef MSG_U
@@ -40,3 +40,28 @@ extern const char **_dictionary_redhat;
 #define B_PKGINFO	24
 #define F_PACKAGE	25
 #define I_NOMANAGERPM	26
+#define I_BASE	27
+#define I_RUNLEVELS	28
+#define I_LEVEL0	29
+#define I_LEVEL1	30
+#define I_LEVEL2	31
+#define I_LEVEL3	32
+#define I_LEVEL4	33
+#define I_LEVEL5	34
+#define I_LEVEL6	35
+#define F_RUNLEVEL	36
+#define I_CUSTOMLEVELS	37
+#define I_RECOMMENDED	38
+#define I_BINDDESC	39
+#define I_IMAPDESC	40
+#define I_DHCPDESC	41
+#define I_NFSDESC	42
+#define Q_INSTALLPKG	43
+#define T_INSTALLPKG	44
+#define P_INSTALLPKG	45
+#define I_XFSDESC	46
+#define I_PORTMAPDESC	47
+#define I_YPSERVDESC	48
+#define I_MDKCD	49
+#define I_RHCD	50
+#define I_YPCLIENTDESC	51
