@@ -11,9 +11,15 @@
 #include <sstream.h>
 #include <netconf.h>
 #include <fviews.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <sys/ioctl.h>
+#include <net/if.h>
 #include "redhatppp.h"
 #include "redhatppp.m"
 #include "../../paths.h"
+#include <module_apis/fwinfo_apidef.h>
 
 const char scripts_path[]="/etc/sysconfig/network-scripts/ifcfg-";
 
@@ -93,6 +99,7 @@ static const char USERCTL[]="USERCTL";
 static const char BOOTPROTO[]="BOOTPROTO";
 static const char INITSTRING[]="INITSTRING";
 static const char DEBUG[]="DEBUG";
+static const char FIREWALL[]="PPP_FIREWALL";
 static const char MODE[]="MODE";
 static const char PEERDNS[]="PEERDNS";
 
@@ -147,6 +154,7 @@ public:
 	SSTRINGS chat;
 	SSTRINGS chataborts;
 	char debug;
+	char firewall;
 	int type;
 	char mode;			// slip or cslip
 	char stupidmode;	// for wvdial
@@ -244,7 +252,7 @@ PRIVATE void RHPPP::init()
 	linespeed.setfrom ("115200");
 	persist = 1;
 	defabort = 1;
-	original_defabort = 1;
+	original_defabort = 0;
 	defroute = 1;
 	hardflowctl = 1;
 	escapechars = 0;
@@ -252,8 +260,7 @@ PRIVATE void RHPPP::init()
 	modeminit.setfrom ("ATZ");
 	modemdial.setfrom ("ATDT");
 	mode = 0;
-	addspcaborts();
-	stupidmode = 0;
+	stupidmode = 1;
 }
 
 PUBLIC RHPPP::RHPPP ()
@@ -289,12 +296,13 @@ PRIVATE void RHPPP::delspcaborts()
 {
 	for (int i=0; tbaborts[i] != NULL; i++){
 		int lk = chataborts.lookup (tbaborts[i]);
-		if (lk != -1
+		while (lk != -1
 			&& (lk & 1) != 0
 			&& chataborts.getitem(lk-1)->cmp("ABORT")==0){
 			lk--;
 			chataborts.remove_del (lk);
 			chataborts.remove_del (lk);
+			lk = chataborts.lookup (tbaborts[i]);
 		}
 	}
 }
@@ -302,7 +310,8 @@ PRIVATE void RHPPP::delspcaborts()
 static void ppp_unquote (char *s)
 {
 	char tmp[1000];
-	strcpy (tmp,s);
+	strncpy (tmp,s,sizeof tmp);
+	tmp[sizeof tmp - 1] = 0;
 	char *src = tmp;
 	if (src[0] == '\'') src++;
 	int last = strlen(src)-1;
@@ -311,6 +320,31 @@ static void ppp_unquote (char *s)
 }
 
 
+static int ppp_delsecrets (
+	const char *remote)
+{
+	int ret = 0;
+	VIEWITEMS items;
+	items.read (f_pap_secrets);
+	int n = items.getnb();
+	bool found = false;
+	for (int i=0; i<n; i++){
+		VIEWITEM *it = items.getitem(i);
+		char word1[100],word2[100],word3[100];
+		const char *pt = it->line.get();
+		pt = str_copyword (word1,pt,100);
+		pt = str_copyword (word2,pt,100);
+		str_copyword (word3,pt,100);
+		if (strcmp(word2,remote)==0){
+			found = true;
+			items.remove_del(it);
+			break;
+		}
+	}
+	if (found) ret = items.write (f_pap_secrets,NULL);
+	return ret;
+}
+
 static int ppp_updsecrets (
 	const char *name,
 	const char *remote,
@@ -344,11 +378,12 @@ static int ppp_updsecrets (
 	}
 	if (!found){
 		char line[100];
-		SSTRING comment,dummy;
+		SSTRING comment;
 		comment.setfrom ("# ");
 		comment.append (MSG_U(I_ADDBYLNX,"Added by linuxconf"));
 		snprintf (line,sizeof(line)-1,"%s %s %s",name,remote,secret);
-		items.add (new VIEWITEM(comment,line,dummy));
+		items.add (new VIEWITEM(comment.get(),VIEWITEM_COMMENT));
+		items.add (new VIEWITEM(line));
 		modified = true;
 	}
 	if (modified) ret = items.write (f_pap_secrets,NULL);
@@ -390,7 +425,7 @@ PUBLIC int RHPPP::load ()
 	VIEWITEMS items;
 	if (items.read (f_ppp)!=-1){
 		ret = 0;
-		char tmp[100];
+		char tmp[1000];
 
 		name.setfrom (items.locateval (NAME,tmp));
 		wvdialsect.setfrom (items.locateval (WVDIALSECT,tmp));
@@ -407,6 +442,7 @@ PUBLIC int RHPPP::load ()
 		defabort = items.locatebval (DEFABORT);
 		original_defabort = defabort;
 		debug = items.locatebval (DEBUG);
+		firewall = items.locatebval (FIREWALL);
 
 		defroute = items.locatebval (DEFROUTE);
 		hardflowctl = items.locatebval (HARDFLOWCTL);
@@ -437,6 +473,7 @@ PUBLIC int RHPPP::load ()
 			mode = 1;
 		}
 		modeminit.setfrom (items.locateval (INITSTRING,tmp));
+		modemdial.setfrom("");
 
 		if (!papname.is_empty()) ppp_loadsecret (papname.get(),dev,secret);
 
@@ -635,8 +672,6 @@ PUBLIC int RHPPP::edit ()
 		setmodemfield (dia);
 	}
 	if (type == TYPE_PPP){
-		dia.newf_chk ("",peerdns,MSG_U(I_PEERDNS
-			,"Use the DNS of the provider"));
 		dia.newf_str (MSG_U(F_PPPOPTIONS,"PPP options"),pppoptions);
 	}
 	
@@ -672,11 +707,14 @@ PUBLIC int RHPPP::edit ()
 	dia.newf_title (MSG_U(T_NETWORKING,"Networking"),1,"",MSG_R(T_NETWORKING));
 	dia.newf_chk ("",onboot,MSG_U(I_ONBOOT,"Activate interface at boot time"));
 	dia.newf_chk ("",defroute,MSG_U(I_DEFROUTE,"Set default route"));
+	dia.newf_chk ("",firewall,MSG_U(I_FIREWALL,"Update the firewall rules"));
 	if (type != TYPE_PLIP){
 		dia.newf_chk ("",persist,MSG_U(I_PERSIST,"restart link when connection fails"));
 	}
 	int mru_field=0,mtu_field=0,discon_field=0;
 	if (type == TYPE_PPP){
+		dia.newf_chk ("",peerdns,MSG_U(I_PEERDNS
+			,"Use the DNS of the provider"));
 		dia.newf_title ("",MSG_U(T_TIMEOUT,"Timeout values in seconds"));
 		dia.newf_num (MSG_U(F_NOCON,"No connection"),retrytimeout);
 		discon_field = dia.getnb();
@@ -707,11 +745,13 @@ PUBLIC int RHPPP::edit ()
 		dia.newf_chkm (MSG_U(F_MODE,"Mode"),mode,tbmode);
 	}
 	int field_secret = 0;
+	SSTRING newsecret,newsecret2;
 	if (type == TYPE_PPP){
 		dia.newf_title (MSG_U(T_PAP,"PAP"),1,"",MSG_R(T_PAP));
 		dia.newf_str (MSG_U(F_PAPNAME,"Username"),papname);
 		field_secret = dia.getnb();
-		dia.newf_str (MSG_U(F_SECRET,"Secret"),secret);
+		dia.newf_pass (MSG_U(F_SECRET,"Secret"),newsecret);
+		dia.newf_pass (MSG_U(F_SECRET2,"Secret (confirm)"),newsecret2);
 	}
 
 	int nof = 0;
@@ -726,45 +766,60 @@ PUBLIC int RHPPP::edit ()
 			,nof
 			,MENUBUT_DEL|MENUBUT_CANCEL|MENUBUT_ACCEPT|MENUBUT_USR1|MENUBUT_USR2);
 		if (code == MENU_ESCAPE || code == MENU_CANCEL){
+			dia.restore();
 			break;
 		}else if (code == MENU_DEL){
 			if (xconf_delok ()){
-				ppp_del (device.get());
+				ppp_del (device.get(),wvdialsect.get());
 				ret = 1;
 				break;
 			}
-		}else if (code == MENU_USR1){
-			dia.save();
-			if (write() != -1){
-				ppp_connect (device.get());
-			}
-		}else if (code == MENU_USR2){
-			ppp_disconnect (device.get());
-		}else if (!ppp_checkip (ipaddr,true)){
-			nof = field_local_ip;
-		}else if (!ppp_checkip (remip,true)){
-			nof = field_remote_ip;
-		}else if (!ppp_checkip (netmask,false)){
-			nof = field_netmask;
-		}else if (!ppp_checkoptnum(mru,296,1500
-			,MSG_U(E_MRU,"Invalid MRU value"))){
-			nof = mru_field;
-		}else if (!ppp_checkoptnum(mtu,296,1500
-			,MSG_U(E_MTU,"Invalid MTU value"))){
-			nof = mtu_field;
-		}else if (!ppp_checkoptnum(disconnecttimeout,1,1000000
-			,MSG_U(E_DISCON,"Invalid disconnect timeout value"))){
-			nof = discon_field;
-		}else if (!papname.is_empty()
-			&& secret.is_empty()){
-			xconf_error (MSG_U(E_SECRET
-				,"You must provide a secret password\n"
-				 "if you provide a PAP user name"));
-			nof = field_secret;
 		}else{
-			ret = 0;
-			write ();
-			break;
+			dia.save();
+			if (newsecret.cmp(newsecret2)!=0){
+				xconf_error (MSG_U(E_PASSMATCH
+					,"Passwords do not match\n"
+					 "Please reenter the password twice"));
+				nof = field_secret;
+				newsecret.setfrom ("");
+				newsecret2.setfrom ("");
+				dia.reload (nof);
+				dia.reload (nof+1);
+				continue;
+			}
+			if (!newsecret.is_empty()) secret.setfrom (newsecret);
+			if (code == MENU_USR1){
+				if (write() != -1){
+					ppp_connect (device.get());
+				}
+			}else if (code == MENU_USR2){
+				ppp_disconnect (device.get());
+			}else if (!ppp_checkip (ipaddr,true)){
+				nof = field_local_ip;
+			}else if (!ppp_checkip (remip,true)){
+				nof = field_remote_ip;
+			}else if (!ppp_checkip (netmask,false)){
+				nof = field_netmask;
+			}else if (!ppp_checkoptnum(mru,296,1500
+				,MSG_U(E_MRU,"Invalid MRU value"))){
+				nof = mru_field;
+			}else if (!ppp_checkoptnum(mtu,296,1500
+				,MSG_U(E_MTU,"Invalid MTU value"))){
+				nof = mtu_field;
+			}else if (!ppp_checkoptnum(disconnecttimeout,1,1000000
+				,MSG_U(E_DISCON,"Invalid disconnect timeout value"))){
+				nof = discon_field;
+			}else if (!papname.is_empty()
+				&& secret.is_empty()){
+				xconf_error (MSG_U(E_SECRET
+					,"You must provide a secret password\n"
+					 "if you provide a PAP user name"));
+				nof = field_secret;
+			}else{
+				ret = 0;
+				write ();
+				break;
+			}
 		}
 	}
 	return ret;
@@ -786,35 +841,50 @@ PUBLIC int RHPPP::editnew ()
 		if (type == TYPE_PPP){
 			dia.newf_chk ("",usepap,MSG_U(I_USEPAP,"Use PAP authentication"));
 		}
-		SSTRING login,password;
+		SSTRING login,password,password2;
 		dia.newf_str (MSG_U(F_LOGIN,"Login name"),login);
-		dia.newf_str (MSG_U(F_PASSWORD,"Password"),password);
+		int field_password = dia.getnb();
+		dia.newf_pass (MSG_U(F_PASSWORD,"Password"),password);
+		dia.newf_pass (MSG_U(F_PASSWORD2,"Password (confirm)"),password2);
 		int nof=0;
 		dia.setbutinfo (MENU_USR1,MSG_U(B_CUSTOMIZE,"Customize")
 			,MSG_U(X_CUSTOMIZE,"Customize"));
-		MENU_STATUS code = dia.edit (title,"",help_ptp
-			,nof,MENUBUT_CANCEL|MENUBUT_ACCEPT|MENUBUT_USR1);
-		if (code != MENU_CANCEL && code != MENU_ESCAPE){
-			// Build the chat
-			dia.save();
-			if (!usepap){
-				chat.add (new SSTRING("ogin:"));
-				chat.add (new SSTRING(login));
-				chat.add (new SSTRING("ord:"));
-				chat.add (new SSTRING(password));
-			}
-			chat.add (new SSTRING("TIMEOUT"));
-			chat.add (new SSTRING("5"));
-			chat.add (new SSTRING("~--"));
-			chat.add (new SSTRING(""));
-			if (usepap){
-				papname.setfrom (login);
-				secret.setfrom (password);
-			}
-			if (code == MENU_USR1){
-				ret = edit();
+		while (1){
+			MENU_STATUS code = dia.edit (title,"",help_ptp
+				,nof,MENUBUT_CANCEL|MENUBUT_ACCEPT|MENUBUT_USR1);
+			if (code == MENU_CANCEL || code == MENU_ESCAPE){
+				break;
 			}else{
-				ret = write ();
+				dia.save();
+				if (password.cmp(password2)!=0){
+					xconf_error (MSG_R(E_PASSMATCH));
+					nof = field_password;
+					password.setfrom ("");
+					password2.setfrom ("");
+					dia.reload();
+				}else{
+					// Build the chat
+					if (!usepap){
+						chat.add (new SSTRING("ogin:"));
+						chat.add (new SSTRING(login));
+						chat.add (new SSTRING("ord:"));
+						chat.add (new SSTRING(password));
+					}
+					chat.add (new SSTRING("TIMEOUT"));
+					chat.add (new SSTRING("5"));
+					chat.add (new SSTRING("~--"));
+					chat.add (new SSTRING(""));
+					if (usepap){
+						papname.setfrom (login);
+						secret.setfrom (password);
+					}
+					if (code == MENU_USR1){
+						ret = edit();
+					}else{
+						ret = write();
+					}
+					break;
+				}
 			}
 		}
 	}
@@ -826,7 +896,7 @@ PUBLIC int RHPPP::editnew ()
 	Write some parts of the chat file
 */
 static void ppp_writechat (
-	FILE *fout,
+	FILE_CFG *fout,
 	SSTRINGS &chat)
 {
 	for (int i=0; i<chat.getnb(); i+=2){
@@ -849,7 +919,7 @@ PUBLIC int RHPPP::write()
 	items.read (f_ppp);
 
 	if (wvdialsect.is_empty() && !name.is_empty()){
-		// Copy the name in wvdialsect, transform spaces into _
+		// Copy the name to wvdialsect, transform spaces into _
 		char buf[name.getlen()+1];
 		const char *src = name.get();
 		char *dst = buf;
@@ -870,7 +940,7 @@ PUBLIC int RHPPP::write()
 		items.update ("MODE",mode ? "CSLIP" : "SLIP");
 	}
 	items.update (NAME,name);
-	items.update (WVDIALSECT,wvdialsect);
+	items.update (WVDIALSECT, modemport.is_empty() ? "" : wvdialsect.get());
 	items.updatebval (ONBOOT,onboot);
 	items.updatebval (USERCTL,userctl);
 	items.updatebval (PEERDNS,peerdns);
@@ -879,6 +949,7 @@ PUBLIC int RHPPP::write()
 	items.updatebval (PERSIST,persist);
 	items.updatebval (DEFABORT,defabort);
 	items.updatebval (DEBUG,debug);
+	items.updatebval (FIREWALL,firewall);
 	items.update (INITSTRING,modeminit);
 	items.updatebval (DEFROUTE,defroute);
 	items.updatebval (HARDFLOWCTL,hardflowctl);
@@ -899,40 +970,50 @@ PUBLIC int RHPPP::write()
 	items.update (BOOTPROTO,tbproto[bootproto]);
 	int ret = items.write (f_ppp,NULL);
 	if (ret != -1){
-		if (!papname.is_empty() && !secret.is_empty()){
+		if (!(papname.is_empty() || secret.is_empty())){
 			ret = ppp_updsecrets (papname.get(),device.get(),secret.get());
 		}
 		if (ret != -1 && (type == TYPE_PPP || type == TYPE_SLIP)){
-			// Read the chat
 			char pathchat[PATH_MAX];
 			sprintf (pathchat,"%s%s",ppp_chat_path,device.get());
-			CONFIG_FILE f_chat(pathchat,help_nil
-				,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
-				,"root","root",0700);
-			FILE *fout = f_chat.fopen ("w");
-			if (fout != NULL){
-				if (defabort != original_defabort){
-					// We cleanup first and add them later if needed.
-					delspcaborts();
-					if (defabort) addspcaborts();
+			if (modemport.is_empty()) {
+				wvdial_delete(wvdialsect.get());
+				ret = unlink (pathchat);
+			} else {
+				wvdial_update(wvdialsect.get(),papname.get(),
+					secret.get(),modeminit.get(),
+                                        phone.get(),modemport.get(),
+                                        linespeed.get(), modemdial.get(),
+                                        stupidmode);
+				// Read the chat
+				CONFIG_FILE f_chat(pathchat,help_nil
+					,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
+					,"root","root",0700);
+				FILE_CFG *fout = f_chat.fopen ("w");
+				if (fout != NULL){
+					//if (defabort != original_defabort){
+						// We cleanup first and add them later if needed.
+						delspcaborts();
+						if (defabort) addspcaborts();
+					//}
+					ppp_writechat (fout,chataborts);
+					fprintf (fout,"'' '%s'\n",modeminit.get());
+					if (modemdial.getlen() || phone.getlen()) {
+						fprintf (fout,"'OK' '%s%s'\n",modemdial.get(),phone.get());
+						fputs ("'CONNECT' ''\n",fout);
+					}
+					ppp_writechat (fout,chat);
+					ret = fclose (fout);
+				}else{
+					ret = -1;
 				}
-				ppp_writechat (fout,chataborts);
-				fprintf (fout,"'' '%s'\n",modeminit.get());
-				fprintf (fout,"'OK' '%s%s'\n",modemdial.get(),phone.get());
-				fputs ("'CONNECT' ''\n",fout);
-				ppp_writechat (fout,chat);
-				ret = fclose (fout);
-			}else{
-				ret = -1;
 			}
 		}
 	}
-	wvdial_update (wvdialsect.get(),papname.get(),secret.get(),phone.get()
-		,modemport.get(),linespeed.get(),stupidmode);
 	return ret;
 }
 
-int ppp_del (const char *dev)
+int ppp_del (const char *dev, const char* wvdialsect)
 {
 	char path[PATH_MAX];
 	ppp_getpath (dev,path);
@@ -943,6 +1024,10 @@ int ppp_del (const char *dev)
 		unlink (pathchat);
 		sprintf (pathchat,"%s%s",dip_chat_path,dev);
 		unlink (pathchat);
+		ppp_delsecrets(dev);
+		if (wvdialsect){
+			wvdial_delete(wvdialsect);
+		}
 	}
 	return ret;
 }
@@ -1143,7 +1228,8 @@ PUBLIC int CONFIG_FILE_PTPLIST::extract(SSTREAM &ss)
 		if (tb.lookup(dev)==-1){
 			net_prtlog (NETLOG_CMD,MSG_U(I_REMOVINGPTP,"Removing point to point configuration: %s\n")
 				,dev);
-			ppp_del (dev);
+			// Leaves entries in wvdial.conf untouched
+			ppp_del (dev,0);
 		}
 	}
 	return 0;
@@ -1151,3 +1237,143 @@ PUBLIC int CONFIG_FILE_PTPLIST::extract(SSTREAM &ss)
 
 static CONFIG_FILE_PTPLIST ptplist;
 
+static bool ppp_devexist (const char *devname)
+{
+	bool ret = false;
+	if (strncmp(devname,"redhatppp/",10)==0){
+		devname += 10;
+		SSTRINGS lst;
+		int n = ppp_list (lst);
+		for (int i=0; i<n; i++){
+			if (lst.getitem(i)->cmp(devname)==0){
+				ret = true;
+				break;
+			}
+		}
+	}
+	return ret;
+}
+
+static int ppp_getdevlist  (SSTRINGS &devs, SSTRINGS &descs)
+{
+	SSTRINGS lst;
+	int n = ppp_list (lst);
+	for (int i=0; i<n; i++){
+		SSTRING *s = new SSTRING;
+		SSTRING *d = new SSTRING;
+		const char *c = lst.getitem(i)->get();
+		s->setfromf ("redhatppp/%s",c);
+		d->setfromf (MSG_U(I_PPPDEVDESC,"Dialout configuration %s"),c);
+		devs.add (s);
+		descs.add (d);
+	}
+	return n;
+}
+
+// taken from usernet
+static int ppp_logical_to_physical  (const char *devname, SSTRING &kerneldev)
+{
+	int ret = -1;
+	/* way too much space for "/var/run/ppp-ppp??.pid" */
+	char map_file_name[100];
+	sprintf(map_file_name, "/var/run/ppp-%s.pid", devname);
+	int f = 0;
+	if ((f = open(map_file_name, O_RDONLY)) > 0) {
+		/* enough space for lock-file contents */
+		char buffer[32];
+		int n = 0;
+		if ((n = read(f, buffer, sizeof buffer)) > 0) {
+			buffer[n] = 0;
+			char* p = buffer;
+			while (p < buffer + n) {
+				if (*p == '\n')
+					*p = 0;
+				++p;
+			}
+			p = buffer;
+			while (p < buffer + n && (*p == 0 || isdigit(*p))) {
+				/* skip line */
+				while (p < buffer + n && *p++) {
+				}
+			}
+			if (*p) {
+				kerneldev.setfrom (p);
+				ret = 0;
+			}
+		}
+		close(f);
+	}
+	return ret;
+}
+
+// taken from usernet's get_interface_status_by_name
+static int ppp_getdevinfo  (const char *devname, SSTRINGS &kerneldevs)
+{
+	int ret = -1;
+	SSTRING kerneldev;
+	if (strncmp(devname,"redhatppp/",10)==0 &&
+		ppp_logical_to_physical(devname+10, kerneldev)==0){
+		int sock = 0;
+		int pfs[] = {AF_INET, AF_IPX, AF_AX25, AF_APPLETALK, 0};
+		int p = 0;
+		struct ifreq ifr;
+		while (!sock && pfs[p]) {
+			sock = socket(pfs[p++], SOCK_DGRAM, 0);
+		}
+		if (sock) {
+			memset(&ifr, 0, sizeof(ifr));
+			strcpy(ifr.ifr_name, kerneldev.get());
+			if (ioctl(sock, SIOCGIFFLAGS, &ifr) >= 0 &&
+				ifr.ifr_flags & IFF_UP) {
+				kerneldevs.add (new SSTRING(kerneldev));
+				ret = 0;
+			}
+			close(sock);
+		}
+	}
+	return ret;
+}
+
+static bool ppp_hostexist (const char *hostname)
+{
+	return ppp_devexist (hostname);
+}
+
+static int ppp_gethostlist  (SSTRINGS &hosts, SSTRINGS &descs)
+{
+	return ppp_getdevlist (hosts,descs);
+}
+static int ppp_gethostinfo  (const char *host, SSTRINGS &ips)
+{
+	int ret = -1;
+	SSTRINGS devs;
+	if (ppp_getdevinfo(host,devs)!=-1){
+		for (int i=0; i<devs.getnb(); i++){
+			const char *dev = devs.getitem(i)->get();
+			// We need the IP number associated with the device
+			IFCONFIG_INFO info;
+			if (ifconfig_getinfo_nocheck(dev,info)!=-1){
+				ips.add (new SSTRING(info.ip_addr));
+				ret = 0;
+			}
+		}
+	}
+	return ret;
+}
+
+void *redhatppp_fwinfo_api_get()
+{
+	FWINFO_API *api = new FWINFO_API;
+	api->devexist = ppp_devexist;
+	api->getdevlist = ppp_getdevlist;
+	api->getdevinfo = ppp_getdevinfo;
+	api->hostexist = ppp_hostexist;
+	api->gethostlist = ppp_gethostlist;
+	api->gethostinfo = ppp_gethostinfo;
+	return api;
+}
+
+void redhatppp_fwinfo_api_release(void *api)
+{
+	delete (FWINFO_API*)api;
+}
