@@ -1,5 +1,5 @@
 /* _dict.cc 14/11/1998 19.13.02 */
-/* ppp.cc 23/09/1999 15.21.06 */
+/* ppp.cc 24/11/2000 00.01.46 */
 PUBLIC RHPPP::RHPPP (const char *dev);
 PUBLIC void RHPPP::setdevice (const char *dev);
 PRIVATE void RHPPP::init (void);
@@ -15,12 +15,12 @@ PUBLIC int RHPPP::write (void);
 PUBLIC CONFIG_FILE_PTPLIST::CONFIG_FILE_PTPLIST (void);
 PUBLIC int CONFIG_FILE_PTPLIST::archive (SSTREAM&ss)const;
 PUBLIC int CONFIG_FILE_PTPLIST::extract (SSTREAM&ss);
-/* redhatppp.cc 06/01/1999 21.26.10 */
+/* redhatppp.cc 24/11/2000 00.01.46 */
 PUBLIC MODULE_redhatppp::MODULE_redhatppp (void);
 PUBLIC void MODULE_redhatppp::setmenu (DIALOG&dia,
 	 MENU_CONTEXT context);
 PUBLIC int MODULE_redhatppp::domenu (MENU_CONTEXT context,
 	 const char *key);
 PUBLIC int MODULE_redhatppp::dohtml (const char *key);
-PUBLIC int MODULE_redhatppp::execmain (int argc, char *argv[]);
-/* wvdial.cc 20/09/1999 21.08.12 */
+PUBLIC int MODULE_redhatppp::execmain (int argc, char *argv[], bool);
+/* wvdial.cc 24/11/2000 00.01.46 */
