@@ -4,6 +4,7 @@
 #include "samba.m"
 #include "keyword.h"
 #include <module_apis/status_api.h>
+#include <string.h>
 
 extern SAMBA_HELP_FILE help_samba;
 static SAMBA_HELP_FILE help_share ("share");
@@ -12,7 +13,10 @@ static SAMBA_HELP_FILE help_printers ("printers");
 static SAMBA_HELP_FILE help_global ("global");
 static SAMBA_HELP_FILE help_netlogon ("netlogon");
 
-static SMB_CONF *samba;
+// Dialog IDs for the virtual registry
+static char ID_MAIN[]="main";
+static char ID_SHARE[]="share";
+static char ID_LOGON[]="logon";
 
 struct SMB_FIELD_LOOKUP{
 	const char **keywords;
@@ -160,10 +164,13 @@ PUBLIC int SMB_SHARE::editgen(
 PUBLIC int SMB_SHARE::edit()
 {
 	DIALOG dia;
+	dia.set_registry_id (ID_SHARE);
 	dia.newf_str (MSG_U(F_SHARENAME,"Share name"),name);
 
 	static SMB_FIELD tbf[]={
 		{K_COMMENT,NULL,NULL,SMB_STR,NULL,MSG_U(F_COMMENT,"Comment/description")},
+
+		{NULL,NULL,NULL,SMB_BOOK,NULL,MSG_U(T_BASE,"Base info")},
 		{K_AVAILABLE,NULL,NULL,SMB_CHKON,NULL,MSG_U(F_ENABLE,"This share is enabled")},
 		{K_BROWSEABLE,NULL,NULL,SMB_CHK,NULL,MSG_U(F_BROWSEABLE,"Browsable")},
 		{K_COPY,NULL,NULL,SMB_STR,NULL,MSG_U(F_INHERIT,"Inherit settings from share")},
@@ -179,7 +186,7 @@ PUBLIC int SMB_SHARE::edit()
 
 		{NULL,NULL,NULL,SMB_BOOK,NULL,MSG_U(T_USERS,"Users")},
 		{K_USER,NULL,NULL,SMB_STR,NULL,MSG_U(F_USERLIST,"User list")},
-		{K_ONLY,K_USER,NULL,SMB_CHK,NULL,MSG_U(F_ONLYUSER,"Only user may connect")},
+		{K_ONLY,K_USER,NULL,SMB_CHK,NULL,MSG_U(F_ONLYUSER,"Only users may connect")},
 		{K_ADMIN,K_USERS,NULL,SMB_STR,NULL,MSG_U(F_ADMINUSERS,"Admin users")},
 		{K_WRITE,K_LIST,NULL,SMB_STR,NULL,MSG_U(F_WRITELIST,"Write list")},
 		{K_VALID,K_USERS,NULL,SMB_STR,NULL,MSG_U(F_VALIDUSERS,"Valid users")},
@@ -188,7 +195,9 @@ PUBLIC int SMB_SHARE::edit()
 
 		{NULL,NULL,NULL,SMB_BOOK,NULL,MSG_U(T_SCRIPTS,"Scripts")},
 		{K_PREEXEC,NULL,NULL,SMB_STR,NULL,MSG_U(F_PREEXEC,"Setup command")},
+		{K_PREEXEC,K_CLOSE,NULL,SMB_CHK,NULL,MSG_U(F_EXECCLOSE,"Close on failure")},
 		{K_ROOT,K_PREEXEC,NULL,SMB_STR,NULL,MSG_U(F_ROOTPREEXEC,"Setup command (root)")},
+		{K_ROOT,K_PREEXEC,K_CLOSE,SMB_CHK,NULL,MSG_U(F_ROOTEXECCLOSE,"Close on failure")},
 		{K_POSTEXEC,NULL,NULL,SMB_STR,NULL,MSG_U(F_POSTEXEC,"Cleanup command")},
 		{K_ROOT,K_POSTEXEC,NULL,SMB_STR,NULL,MSG_U(F_ROOTPOSTEXEC,"Cleanup command (root)")},
 		{K_MAGIC,K_SCRIPT,NULL,SMB_STR,NULL,MSG_U(F_MAGICSCRIPT,"Magic script")},
@@ -196,6 +205,8 @@ PUBLIC int SMB_SHARE::edit()
 		{NULL,NULL,NULL,SMB_BOOK,NULL,MSG_U(T_FEATURES,"Features")},
 		{K_FORCE,K_USER,NULL,SMB_STR,NULL,MSG_U(F_FORCEUSER,"Force user")},
 		{K_FORCE,K_GROUP,NULL,SMB_STR,NULL,MSG_U(F_FORCEGROUP,"Force group")},
+		{K_CREATE,K_MASK,NULL,SMB_STR,NULL,MSG_U(F_CREATEMASK,"File creation mode")},
+		{K_DIRECTORY,K_MASK,NULL,SMB_STR,NULL,MSG_U(F_DIRMASK,"Directory creation mode")},
 		{K_DONT,K_DESCEND,NULL,SMB_STR,NULL,MSG_U(F_DONTDESCEND,"Don't descent")},
 		{K_GUEST,K_ACCOUNT,NULL,SMB_STR,NULL,MSG_U(F_GUESTACCOUNTSH,"Guest account (this share)")},
 		{K_MAGIC,K_OUTPUT,NULL,SMB_STR,NULL,MSG_U(F_MAGICOUTPUT,"Magic output")},
@@ -204,9 +215,10 @@ PUBLIC int SMB_SHARE::edit()
 	return editgen (dia,tbf,(int)(sizeof(tbf)/sizeof(tbf[0])),true,help_share);
 }
 
-PUBLIC int SMB_SHARE::editglobal()
+PUBLIC int SMB_SHARE::editglobal(SMB_CONF *samba)
 {
 	DIALOG dia;
+	dia.set_registry_id (ID_MAIN);
 
 	dia.newf_title (MSG_U(T_BASIC,"Base config"),1,"",MSG_R(T_BASIC));
 	static const char *tbsmbpass[]={
@@ -276,7 +288,7 @@ PUBLIC int SMB_SHARE::editglobal()
 	
 	static SMB_FIELD tbf[]={
 		{K_UNIX,K_PASSWORD,K_SYNC,SMB_CHK,NULL,MSG_U(F_SYNCPASSFROMSMB,"Synchronise Linux from SMB passwords")},
-		{K_COMMENT,NULL,NULL,SMB_STR,NULL,MSG_U(F_SERVERDESC,"Server description")},
+		{K_SERVER,K_STRING,NULL,SMB_STR,NULL,MSG_U(F_SERVERDESC,"Server description")},
 		{K_WORKGROUP,NULL,NULL,SMB_STR,NULL,MSG_U(F_WORKGROUP,"Work group")},
 		{K_NETBIOS,K_NAME,NULL,SMB_STR,NULL,MSG_U(F_NETBIOSNAME,"Netbios name (opt)")},
 		{K_NETBIOS,K_ALIASES,NULL,SMB_STR,NULL,MSG_U(F_NETBIOSALIASES,"Netbios aliases (opt)")},
@@ -301,6 +313,7 @@ PUBLIC int SMB_SHARE::editglobal()
 		{K_PREFERRED,K_MASTER,NULL,SMB_CHK,NULL,MSG_U(F_PREFMASTER,"Preferred master")},
 		{K_DOMAIN,K_MASTER,NULL,SMB_CHK,NULL,MSG_U(F_DOMMASTER,"Domain master")},
 		{K_REMOTE,K_ANNOUNCE,NULL,SMB_STR,NULL,MSG_U(F_REMANNOUNCE,"Remote announce")},
+		{K_REMOTE,K_BROWSE,K_SYNC,SMB_STR,NULL,MSG_U(F_BROWSESYNC,"Sync. remote browsers")},
 		{K_WINS,K_SUPPORT,NULL,SMB_CHK,NULL,MSG_U(F_WINSSUPPORT,"Enable samba as a WINS server")},
 		{K_WINS,K_SERVER,NULL,SMB_STR,NULL,MSG_U(F_WINSSERVER,"WINS server")},
 		{K_INTERFACES,NULL,NULL,SMB_STR,NULL,MSG_U(F_INTERFACES,"Interfaces")},
@@ -322,6 +335,7 @@ PUBLIC int SMB_SHARE::editglobal()
 		{K_LOAD,K_PRINTERS,NULL,SMB_CHKON,NULL,MSG_U(F_LOADPRINTERS,"Show all available printers")},
 		{K_MESSAGE,K_COMMAND,NULL,SMB_COMBO,(void*)winpopupcommand,
 	 	 MSG_U(F_MSGCOMMAND,"WinPopup command")},
+		{K_STATUS,NULL,NULL,SMB_CHKON,NULL,MSG_U(F_STATUSFILE,"Update the status database")},
 
 	};
 	int ret = editgen (dia,tbf,(int)(sizeof(tbf)/sizeof(tbf[0])),false,help_global);
@@ -329,7 +343,7 @@ PUBLIC int SMB_SHARE::editglobal()
 	return ret;
 }
 
-PUBLIC int SMB_SHARE::editnetlogon(SMB_SHARE *glob)
+PUBLIC int SMB_SHARE::editnetlogon(SMB_SHARE *glob, SMB_CONF *samba)
 {
 	/* #Specification: netlogon / logon fields
 		smb.conf requires the various logon information to be stored
@@ -354,16 +368,27 @@ PUBLIC int SMB_SHARE::editnetlogon(SMB_SHARE *glob)
 		}
 	}
 	DIALOG dia;
+	dia.set_registry_id (ID_LOGON);
 	static SMB_FIELD tbf[]={
 		{K_COMMENT,NULL,NULL,SMB_STR
 				,NULL,MSG_U(F_NETLOGONCOMMENT,"netlogon share title")},
-		{K_AVAILABLE,NULL,NULL,SMB_CHK,NULL,MSG_R(F_ENABLE)},
+		{K_AVAILABLE,NULL,NULL,SMB_CHKON,NULL,MSG_R(F_ENABLE)},
 		{K_PATH,NULL,NULL,SMB_STR,NULL,MSG_R(F_PATH)},
+
+		{NULL,NULL,NULL,SMB_BOOK,NULL,MSG_U(T_LOGON,"Logon")},
 		{K_DOMAIN,K_LOGONS,NULL,SMB_CHK,NULL,MSG_U(F_DOMLOGONS,"Workgroup Logon server")},
 		{K_LOGON,K_SCRIPT,NULL,SMB_STR,NULL,MSG_U(F_LOGONSCRIPT,"Logon script")},
 		{K_LOGON,K_PATH,NULL,SMB_STR,NULL,MSG_U(F_LOGONPATH,"Logon path")},
 		{K_LOGON,K_DRIVE,NULL,SMB_STR,NULL,MSG_U(F_LOGONDRIVE,"Logon drive")},
 		{K_LOGON,K_HOME,NULL,SMB_STR,NULL,MSG_U(F_LOGONHOME,"Logon home")},
+
+		{NULL,NULL,NULL,SMB_BOOK,NULL,MSG_R(T_SCRIPTS)},
+		{K_PREEXEC,NULL,NULL,SMB_STR,NULL,MSG_R(F_PREEXEC)},
+		{K_PREEXEC,K_CLOSE,NULL,SMB_CHK,NULL,MSG_R(F_EXECCLOSE)},
+		{K_ROOT,K_PREEXEC,NULL,SMB_STR,NULL,MSG_R(F_ROOTPREEXEC)},
+		{K_ROOT,K_PREEXEC,K_CLOSE,SMB_CHK,NULL,MSG_R(F_ROOTEXECCLOSE)},
+		{K_POSTEXEC,NULL,NULL,SMB_STR,NULL,MSG_R(F_POSTEXEC)},
+		{K_ROOT,K_POSTEXEC,NULL,SMB_STR,NULL,MSG_R(F_ROOTPOSTEXEC)},
 	};
 	int ret = editgen (dia,tbf,(int)(sizeof(tbf)/sizeof(tbf[0])),false,help_netlogon);
 	if (ret == 0){
@@ -377,7 +402,7 @@ PUBLIC int SMB_SHARE::editnetlogon(SMB_SHARE *glob)
 	return ret;
 }
 
-PUBLIC int SMB_SHARE::edithomes()
+PUBLIC int SMB_SHARE::edithomes(SMB_CONF *samba)
 {
 	DIALOG dia;
 	static SMB_FIELD tbf[]={
@@ -403,16 +428,23 @@ PUBLIC int SMB_SHARE::edithomes()
 
 		{NULL,NULL,NULL,SMB_BOOK,NULL,MSG_R(T_SCRIPTS)},
 		{K_PREEXEC,NULL,NULL,SMB_STR,NULL,MSG_R(F_PREEXEC)},
+		{K_PREEXEC,K_CLOSE,NULL,SMB_CHK,NULL,MSG_R(F_EXECCLOSE)},
 		{K_ROOT,K_PREEXEC,NULL,SMB_STR,NULL,MSG_R(F_ROOTPREEXEC)},
+		{K_ROOT,K_PREEXEC,K_CLOSE,SMB_CHK,NULL,MSG_R(F_ROOTEXECCLOSE)},
 		{K_POSTEXEC,NULL,NULL,SMB_STR,NULL,MSG_R(F_POSTEXEC)},
 		{K_ROOT,K_POSTEXEC,NULL,SMB_STR,NULL,MSG_R(F_ROOTPOSTEXEC)},
+
+		{NULL,NULL,NULL,SMB_BOOK,NULL,MSG_R(T_FEATURES)},
+		{K_CREATE,K_MASK,NULL,SMB_STR,NULL,MSG_R(F_CREATEMASK)},
+		{K_DIRECTORY,K_MASK,NULL,SMB_STR,NULL,MSG_R(F_DIRMASK)},
+
 	};
 	int ret = editgen (dia,tbf,(int)(sizeof(tbf)/sizeof(tbf[0])),false,help_homes);
 	if (ret == 0) samba->write();
 	return ret;
 }
 
-PUBLIC int SMB_SHARE::editprinters()
+PUBLIC int SMB_SHARE::editprinters(SMB_CONF *samba)
 {
 	DIALOG dia;
 	static SMB_FIELD tbf[]={
@@ -432,9 +464,15 @@ PUBLIC int SMB_SHARES::write()
 	return samba->write();
 }
 
+static int cmp_by_name (const ARRAY_OBJ *o1, const ARRAY_OBJ *o2)
+{
+	SMB_SHARE *s1 = (SMB_SHARE*)o1;
+	SMB_SHARE *s2 = (SMB_SHARE*)o2;
+	return s1->name.cmp(s2->name);
+}
+
 PUBLIC int SMB_CONF::editshares()
 {
-	samba = this;
 	int ret = -1;
 	int nof = 0;
 	DIALOG_LISTE dia;
@@ -442,9 +480,9 @@ PUBLIC int SMB_CONF::editshares()
 	dia.addwhat (MSG_U(I_NEWSHARE,"Select [Add] to add a new disk share"));
 	while (1){
 		int n = shares.getnb();
-		int tblookup[n];
-		int alloc = 0;
-		bool dummy = false;
+		SMB_SHARES sorted (this);
+		sorted.neverdelete();	// We present the share sorted, but
+								// keep the original ordering in smb.conf
 		for (int i=0; i<n; i++){
 			SMB_SHARE *d = shares.getitem(i);
 			const char *name = d->name.get();
@@ -452,29 +490,29 @@ PUBLIC int SMB_CONF::editshares()
 				&& strcmp(name,K_PRINTERS)!=0
 				&& strcmp(name,K_NETLOGON)!=0
 				&& strcmp(name,K_HOMES)!=0){
-				dia.set_menuitem(alloc,name,d->getval(K_COMMENT));
-				tblookup[alloc++] = i;
+				sorted.add (d);
 			}
 		}
-		if (alloc == 0) {
-			dia.set_menuitem(alloc,"<empty list>","");
-			++alloc;
-			dummy = true;
+		sorted.sort (cmp_by_name);
+		for (int i=0; i<sorted.getnb(); i++){
+			SMB_SHARE *d = sorted.getitem(i);
+			const char *name = d->name.get();
+			dia.set_menuitem(i,name,d->getval(K_COMMENT));
 		}
-		dia.remove_last (alloc+1);
+		dia.remove_last (sorted.getnb()+1);
 		MENU_STATUS code = dia.editmenu (MSG_U(T_SHARES,"Disk shares")
 			,MSG_U(I_SHARES
 				,"You can define multiple independant entry\n"
 				 "points in your file server")
 			,help_share
-			,nof,0);
+			,nof,MENUBUT_ADD);
 		if (code == MENU_ESCAPE || code == MENU_QUIT){
 			break;
 		}else if (code == MENU_ADD){
 			SMB_SHARE *d = new SMB_SHARE;
 			ret = shares.editone (d);
-		}else if ((nof < n && nof >= 0) && ! dummy){
-			ret = shares.editone (tblookup[nof]);
+		}else if (nof < sorted.getnb() && nof >= 0){
+			ret = shares.editone (sorted.getitem(nof));
 		}
 	}
 	return ret;
@@ -483,17 +521,17 @@ PUBLIC int SMB_CONF::editshares()
 PUBLIC void SMB_CONF::editdefaults()
 {
 	SMB_SHARE *d = shares.getitem(K_GLOBAL);
-	if (d != NULL) d->editglobal();
+	if (d != NULL) d->editglobal(this);
 }
 PUBLIC void SMB_CONF::edithomes()
 {
 	SMB_SHARE *d = shares.getitem(K_HOMES);
-	if (d != NULL) d->edithomes();
+	if (d != NULL) d->edithomes(this);
 }
 PUBLIC void SMB_CONF::editprinters()
 {
 	SMB_SHARE *d = shares.getitem(K_PRINTERS);
-	if (d != NULL) d->editprinters();
+	if (d != NULL) d->editprinters(this);
 }
 
 PUBLIC void SMB_CONF::editnetlogon()
@@ -507,13 +545,12 @@ PUBLIC void SMB_CONF::editnetlogon()
 	}
 	if (d != NULL){
 		SMB_SHARE *glob = shares.getitem(K_GLOBAL);
-		if (d->editnetlogon(glob) != 0 && was_created) shares.remove_del (d);
+		if (d->editnetlogon(glob,this) != 0 && was_created) shares.remove_del (d);
 	}
 }
 
 PUBLIC void SMB_CONF::edit()
 {
-	samba = this;
 	static const char *m_def = MSG_U(M_DEFAULT,"Defaults");
 	static const char *m_homes = MSG_U(M_HOMES,"Default setup for users's home");
 	static const char *m_printers = MSG_U(M_PRINTERS,"Default setup for printers");
@@ -567,8 +604,144 @@ PUBLIC void SMB_CONF::edit()
 
 void samba_edit ()
 {
-	SMB_CONF smb;
-	smb.edit();
-	smb.sanitycheck();
+	CONTEXT_LOCK lk ("samba");
+	if (lk.isok()){
+		SMB_CONF smb;
+		smb.edit();
+		if (dialog_mode != DIALOG_TREE) smb.sanitycheck();
+	}
 }
 
+static void samedit_global()
+{
+	SMB_CONF smb;
+	smb.editdefaults();
+}
+static void samedit_netlogon()
+{
+	SMB_CONF smb;
+	smb.editnetlogon();
+}
+
+static void samedit_editshare(const char *share, bool setting)
+{
+	SMB_CONF smb;
+	SMB_SHARE *d = smb.shares.getitem(share);
+	if (d != NULL){
+		smb.shares.editone(d);
+	}else if (setting){
+		SMB_SHARE *d = new SMB_SHARE;
+		smb.shares.editone(d);
+	}
+}
+
+static void samedit_listshare(SSTRINGS &tb)
+{
+	SMB_CONF smb;
+	for (int i=0; i<smb.shares.getnb(); i++){
+		SMB_SHARE *d = smb.shares.getitem(i);
+		const char *name = d->name.get();
+		if (strcmp(name,K_GLOBAL)!=0
+			&& strcmp(name,K_PRINTERS)!=0
+			&& strcmp(name,K_NETLOGON)!=0
+			&& strcmp(name,K_HOMES)!=0){
+			tb.add (new SSTRING(name));
+		}
+	}
+}
+
+#include <modregister.h>
+
+static PUBLISH_VARIABLES_MSG samba_var_list1[]={
+	{"mngsmbpasswd",P_MSG_R(F_MNGSMBPASS)},
+	{"syncpasswds",P_MSG_R(F_SYNCPASSFROMSMB)},
+	{"servdesc",P_MSG_R(F_SERVERDESC)},
+	{"workgroup",P_MSG_R(F_WORKGROUP)},
+	{"netbiosname",P_MSG_R(F_NETBIOSNAME)},
+	{"netbiosaliases",P_MSG_R(F_NETBIOSALIASES)},
+	{"encryptpasswd",P_MSG_R(F_ENGRYPTPASS)},
+	{"security",P_MSG_R(F_SECURITY)},
+	{"maptoguest",P_MSG_R(F_MAPGUEST)},
+	{"passwdserver",P_MSG_R(F_PASSSERVER)},
+	{"passwdlevel",P_MSG_R(F_PASSLEVEL)},
+	{"passwdprogram",P_MSG_R(F_PASSPROG)},
+	{"nullpasswds",P_MSG_R(F_NULLPASSWORDS)},
+	{"allowhosts",P_MSG_R(F_ALLOWHOSTS)},
+	{"denyhosts",P_MSG_R(F_DENYHOSTS)},
+	{"oslevel",P_MSG_R(F_OSLEVEL)},
+	{"preferedmaster",P_MSG_R(F_PREFMASTER)},
+	{"domainmaster",P_MSG_R(F_DOMMASTER)},
+	{"remoteannounce",P_MSG_R(F_REMANNOUNCE)},
+	{"remotebrowsesync",P_MSG_R(F_BROWSESYNC)},
+	{"winssupport",P_MSG_R(F_WINSSUPPORT)},
+	{"winsserver",P_MSG_R(F_WINSSERVER)},
+	{"interfaces",P_MSG_R(F_INTERFACES)},
+	{"resolveorder",P_MSG_R(F_RESOLVEORDER)},
+	{"adduserscript",P_MSG_R(F_ADDUSERSCRIPT)},
+	{"deluserscript",P_MSG_R(F_DELUSERSCRIPT)},
+	{"guestaccount",P_MSG_R(F_GUESTACCOUNT)},
+	{"deadtime",P_MSG_R(F_DEADTIME)},
+	{"debuglevel",P_MSG_R(F_DBGLEVEL)},
+	{"defaultservice",P_MSG_R(F_DEFSERVICE)},
+	{"loadprinters",P_MSG_R(F_LOADPRINTERS)},
+	{"msgcommand",P_MSG_R(F_MSGCOMMAND)},
+	{NULL,NULL},
+};
+
+static REGISTER_VARIABLES samba_registry1("samba",samba_var_list1, ID_MAIN,samedit_global);
+
+
+static PUBLISH_VARIABLES_MSG samba_var_list2[]={
+	{"netlogon_comment",P_MSG_R(F_NETLOGONCOMMENT)},
+	{"netlogon_available",P_MSG_R(F_ENABLE)},
+	{"netlogon_path",P_MSG_R(F_PATH)},
+	{"netlogon_domain",P_MSG_R(F_DOMLOGONS)},
+	{"netlogon_logonscript",P_MSG_R(F_LOGONSCRIPT)},
+	{"netlogon_logonpath",P_MSG_R(F_LOGONPATH)},
+	{"netlogon_logondrive",P_MSG_R(F_LOGONDRIVE)},
+	{"netlogon_logonhome",P_MSG_R(F_LOGONHOME)},
+	{NULL,NULL},
+};
+
+static REGISTER_VARIABLES samba_registry2("samba",samba_var_list2, ID_LOGON,samedit_netlogon);
+
+static PUBLISH_VARIABLES_MSG samba_var_list3[]={
+	{"share_path",P_MSG_R(F_PATH)},
+	{"share_available",P_MSG_R(F_ENABLE)},
+	{"share_browseable",P_MSG_R(F_BROWSEABLE)},
+	{"share_forceuser",P_MSG_R(F_FORCEUSER)},
+	{"share_comment",P_MSG_R(F_COMMENT)},
+	{"share_copy",P_MSG_R(F_INHERIT)},
+	{"share_public",P_MSG_R(F_PUBLIC)},
+	{"share_guestonly",P_MSG_R(F_GUESTONLY)},
+	{"share_writable",P_MSG_R(F_WRITABLE)},
+	{"share_allowhosts",P_MSG_R(F_ALLOWHOSTS)},
+	{"share_denyhosts",P_MSG_R(F_DENYHOSTS)},
+	{"share_users",P_MSG_R(F_USERLIST)},
+	{"share_onlyusers",P_MSG_R(F_ONLYUSER)},
+	{"share_adminusers",P_MSG_R(F_ADMINUSERS)},
+	{"share_writelist",P_MSG_R(F_WRITELIST)},
+	{"share_validusers",P_MSG_R(F_VALIDUSERS)},
+	{"share_invalidusers",P_MSG_R(F_IVLDUSERS)},
+	{"share_readlist",P_MSG_R(F_READLIST)},
+	{"share_preexec",P_MSG_R(F_PREEXEC)},
+	{"share_preexecclose",P_MSG_R(F_EXECCLOSE)},
+	{"share_rootpreexec",P_MSG_R(F_ROOTPREEXEC)},
+	{"share_rootpreexecclose",P_MSG_R(F_ROOTEXECCLOSE)},
+	{"share_postexec",P_MSG_R(F_POSTEXEC)},
+	{"share_rootpostexec",P_MSG_R(F_ROOTPOSTEXEC)},
+	{"share_magicscript",P_MSG_R(F_MAGICSCRIPT)},
+	{"share_forcegroup",P_MSG_R(F_FORCEGROUP)},
+	{"share_dontdescend",P_MSG_R(F_DONTDESCEND)},
+	{"share_guestaccount",P_MSG_R(F_GUESTACCOUNTSH)},
+	{"share_magicoutput",P_MSG_R(F_MAGICOUTPUT)},
+	{"share_maxconnections",P_MSG_R(F_MAXCONNECT)},
+	{"share_createmask",P_MSG_R(F_CREATEMASK)},
+	{"share_dirmask",P_MSG_R(F_DIRMASK)},
+	{ NULL, NULL }
+};
+
+static REGISTER_VARIABLES samba_registry3("samba","shares",samba_var_list3,	ID_SHARE
+	,samedit_editshare,samedit_listshare);
+
+
