@@ -59,12 +59,14 @@ static void usage()
 {
 	xconf_error (MSG_U(T_USAGE
 		,"Module squid\n"
+		 "linuxconf --modulemain squid\n"
 		 "\n"
-		 "    No command line option\n")
+		 "    No command line option\n"
+		 "\n")
 		);
 }
 
-PUBLIC int MODULE_squid::execmain (int argc , char *argv[])
+PUBLIC int MODULE_squid::execmain (int argc , char *argv[], bool)
 {
 	int ret = LNCF_NOT_APPLICABLE;
 	const char *pt = strrchr(argv[0],'/');
@@ -76,9 +78,8 @@ PUBLIC int MODULE_squid::execmain (int argc , char *argv[])
 	if (strcmp(pt,"squid")==0){
 		ret = -1;
 		if (argc == 1){
-			// ### Place call to main menu of the module
+			squid_edit();
 		}else{
-			// ### Add some option parsing for the module
 			::usage();
 		}
 	}
