@@ -1,11 +1,11 @@
-/* _dict.cc 08/04/98 00.20.18 */
-/* squid.cc 08/04/98 00.25.02 */
+/* _dict.cc 08/04/1998 00.20.18 */
+/* squid.cc 28/08/2000 18.14.44 */
 PUBLIC MODULE_squid::MODULE_squid (void);
 PUBLIC void MODULE_squid::setmenu (DIALOG&dia, MENU_CONTEXT context);
 PUBLIC int MODULE_squid::domenu (MENU_CONTEXT context, const char *key);
 PUBLIC int MODULE_squid::dohtml (const char *key);
-PUBLIC int MODULE_squid::execmain (int argc, char *argv[]);
-/* squidedit.cc 08/04/98 02.22.26 */
+PUBLIC int MODULE_squid::execmain (int argc, char *argv[], bool);
+/* squidedit.cc 06/01/2000 14.58.42 */
 PUBLIC SQUID::SQUID (void);
 PUBLIC int SQUID::write (void);
 PUBLIC ITEM_COMMON::ITEM_COMMON (VIEWITEMS&items,
