@@ -16,7 +16,10 @@ public:
 	int dohtml (const char *key);
 	int domenu (MENU_CONTEXT context,
 		 const char *key);
-	int execmain (int argc, char *argv[]);
+	int execmain (int argc, char *argv[], bool);
+	int message (const char *msg,
+		 int argc,
+		 const char *argv[]);
 	void setmenu (DIALOG&dia, MENU_CONTEXT context);
     /*~PROTOEND~ MODULE_status */
 };
