@@ -14,11 +14,18 @@ class MODULE_suse: public LINUXCONF_MODULE{
 public:
 	MODULE_suse (void);
 	int dohtml (const char *key);
-	int execmain (int argc, char *argv[]);
+	int execmain (int argc, char *argv[], bool);
     /*~PROTOEND~ MODULE_suse */
 };
 
 class ROUTES;
+class HOSTS;
+class HOSTINFO;
+class VIEWITEMS;
+class PPPONE;
+class USER;
+class SHADOW;
+class SERVICES;
 
 #define HELP_SUSE "suse"
 
