@@ -1,5 +1,5 @@
-/* _dict.cc 14/12/98 19.23.12 */
-/* filter.cc 17/12/98 12.39.56 */
+/* _dict.cc 14/12/1998 19.23.12 */
+/* filter.cc 07/08/2000 00.52.42 */
 PRIVATE void CONFDB_SUSE::reset_modified (void);
 PUBLIC CONFDB_SUSE::CONFDB_SUSE (CONFDB *_orig);
 PUBLIC CONFDB_SUSE::~CONFDB_SUSE (void);
@@ -21,9 +21,11 @@ PUBLIC void CONFDB_SUSE::add (const char *prefix,
 	 const char *val);
 PUBLIC int CONFDB_SUSE::archive (SSTREAM&ss, const char *_sys);
 PUBLIC int CONFDB_SUSE::extract (SSTREAM&ss, const char *_sys);
-/* ipalias.cc 14/12/98 19.23.12 */
-/* routes.cc 17/12/98 12.39.50 */
-/* suse.cc 17/12/98 11.29.46 */
+/* ipalias.cc 14/12/1998 19.23.12 */
+/* routes.cc 07/08/2000 00.53.16 */
+/* suse.cc 28/08/2000 18.14.54 */
 PUBLIC MODULE_suse::MODULE_suse (void);
 PUBLIC int MODULE_suse::dohtml (const char *key);
-PUBLIC int MODULE_suse::execmain (int argc, char *argv[]);
+PUBLIC int MODULE_suse::execmain (int argc, char *argv[], bool);
+/* pam.cc 21/08/2000 22.36.34 */
+/* reduser.cc 15/03/2000 11.29.28 */
