@@ -14,10 +14,10 @@ class MODULE_treemenu: public LINUXCONF_MODULE{
 public:
 	MODULE_treemenu (void);
 	int dohtml (const char *key);
-	int execmain (int argc, char *argv[]);
+	int execmain (int argc, char *argv[], bool);
 	int message (const char *msg,
-		 int ,
-		 const char *[]);
+		 int argc,
+		 const char *argv[]);
     /*~PROTOEND~ MODULE_treemenu */
 };
 
