@@ -2,7 +2,7 @@ extern const char **_dictionary_treemenu;
 #ifndef DICTIONARY_REQUEST
 	#define DICTIONARY_REQUEST \
 	const char **_dictionary_treemenu;\
-	TRANSLATE_SYSTEM_REQ _dictionary_req_treemenu("treemenu",_dictionary_treemenu,5,2);\
+	TRANSLATE_SYSTEM_REQ _dictionary_req_treemenu("treemenu",_dictionary_treemenu,20,3);\
 	void dummy_dict_treemenu(){}
 #endif
 #ifndef MSG_U
@@ -18,3 +18,18 @@ extern const char **_dictionary_treemenu;
 #define B_ACTIVATE	2
 #define N_NOJOB	3
 #define B_HELP	4
+#define M_ABOUT	5
+#define M_INTRO	6
+#define M_TOPIC	7
+#define B_FASTQUIT	8
+#define M_COMMANDLINE	9
+#define T_ABOUT	10
+#define A_ABOUT1	11
+#define A_ABOUT3	12
+#define M_STATUS	13
+#define I_VARIOUS	14
+#define M_FILE	15
+#define I_PROFILE	16
+#define T_MODTREEMENU	17
+#define I_TEXTMODE	18
+#define I_GUIMODE	19
