@@ -17,7 +17,7 @@ public:
 	int dohtml (const char *key);
 	int domenu (MENU_CONTEXT context,
 		 const char *key);
-	int execmain (int argc, char *argv[]);
+	int execmain (int argc, char *argv[], bool);
 	int message (const char *msg,
 		 int ,
 		 const char *[]);
