@@ -22,7 +22,7 @@ PUBLIC CONFIG_OBJS::CONFIG_OBJS (CONFIG_FILE &_f)
 PUBLIC int CONFIG_OBJS::read()
 {
 	int ret = -1;
-	FILE *fin = f.fopen ("r");
+	FILE_CFG *fin = f.fopen ("r");
 	if (fin != NULL){
 		char buf[3000];
 		SSTRING comments;
@@ -54,7 +54,7 @@ PRIVATE CONFIG_OBJ *CONFIG_OBJS::getitem (int no)
 PUBLIC int CONFIG_OBJS::write()
 {
 	int ret = -1;
-	FILE *fout = f.fopen ("w");
+	FILE_CFG *fout = f.fopen ("w");
 	if (fout != NULL){
 		comment_write (first_comment,fout);
 		int n = getnb();
