@@ -11,7 +11,7 @@ class MODULE_UUCP: public LINUXCONF_MODULE{
 public:
 	MODULE_UUCP (void);
 	int domenu (MENU_CONTEXT context, const char *key);
-	int execmain (int argc, char *argv[]);
+	int execmain (int argc, char *argv[], bool);
 	void setmenu (DIALOG&dia, MENU_CONTEXT context);
 	/*~PROTOEND~ MODULE_UUCP */
 };
