@@ -35,7 +35,7 @@ PUBLIC int MODULE_UUCP::domenu (
 	return 0;
 }
 
-PUBLIC int MODULE_UUCP::execmain (int argc , char *argv[])
+PUBLIC int MODULE_UUCP::execmain (int argc , char *argv[], bool)
 {
 	int ret = -1000;
 	const char *pt = strrchr(argv[0],'/');
