@@ -16,7 +16,9 @@
 */
 
 #include <string.h>
+#include <stdlib.h>
 #include <limits.h>
+#include <grp.h>
 #include <misc.h>
 #include <usercomng.h>
 #include "wuftpd.h"
@@ -58,30 +60,6 @@ static CONFIG_FILE f_hosts ("/etc/ftphosts"
 	,subsys_ftpd);
 
 
-class VIRTUAL_FTP: public ARRAY_OBJ{
-public:
-	SSTRING addr;
-	SSTRING root;
-	VIEWITEM *root_it;		// Remember where it was found in the config
-							// file
-	SSTRING banner;
-	VIEWITEM *banner_it;
-	SSTRING logfile;
-	VIEWITEM *logfile_it;
-
-	/*~PROTOBEG~ VIRTUAL_FTP */
-public:
-	VIRTUAL_FTP (const char *_addr);
-	VIRTUAL_FTP (void);
-	int createroot (void);
-	int edit (void);
-	void updateone (const SSTRING&val,
-		 const char *keyw,
-		 VIEWITEM *&item,
-		 VIEWITEMS&items);
-	/*~PROTOEND~ VIRTUAL_FTP */
-};
-
 PUBLIC VIRTUAL_FTP::VIRTUAL_FTP(const char *_addr)
 {
 	root_it = logfile_it = banner_it = NULL;
@@ -92,16 +70,6 @@ PUBLIC VIRTUAL_FTP::VIRTUAL_FTP()
 	root_it = logfile_it = banner_it = NULL;
 }
 
-class VIRTUAL_FTPS: public ARRAY{
-	/*~PROTOBEG~ VIRTUAL_FTPS */
-public:
-	VIRTUAL_FTPS (void);
-	void edit (VIEWITEMS&items);
-	VIRTUAL_FTP *getitem (int no)const;
-	void update (VIEWITEMS&items);
-	/*~PROTOEND~ VIRTUAL_FTPS */
-};
-
 PUBLIC VIRTUAL_FTP *VIRTUAL_FTPS::getitem (int no) const
 {
 	return (VIRTUAL_FTP*)ARRAY::getitem(no);
@@ -150,23 +118,6 @@ PUBLIC void VIRTUAL_FTPS::update (VIEWITEMS &items)
 	}
 }
 
-class FTP_CTRL{
-public:
-	char f_compress;
-	char f_tar;
-	char f_chmod;
-	char f_delete;
-	char f_overwrite;
-	char f_rename;
-	char f_log_inbound;
-	char f_log_outbound;
-	/*~PROTOBEG~ FTP_CTRL */
-public:
-	FTP_CTRL (void);
-	void setall (char val);
-	/*~PROTOEND~ FTP_CTRL */
-};
-
 PUBLIC FTP_CTRL::FTP_CTRL()
 {
 	setall (0);
@@ -184,50 +135,6 @@ PUBLIC void FTP_CTRL::setall (char val)
 	f_log_outbound = val;
 }
 
-class WUFTPD {
-	VIEWITEMS access;
-	VIEWITEMS users;
-	VIEWITEMS groups;
-	VIEWITEMS hosts;
-	FTP_CTRL guest,rreal,anonymous;
-	VIRTUAL_FTPS virtuals;
-	/*~PROTOBEG~ WUFTPD */
-public:
-	WUFTPD (void);
-	void add (VIRTUAL_FTP *v);
-	void del (VIRTUAL_FTP *v);
-	void editaccess (void);
-	void editvirtuals (void);
-	void listperm (FIXPERM_SPECS&specs);
-private:
-	void load (SSTRING&val,
-		 VIEWITEMS&items,
-		 const char *key);
-	VIEWITEM *locate_alloc (const char *key1,
-		 const char *key2);
-public:
-	VIRTUAL_FTP *locatevhost (const char *vhost);
-private:
-	void setflags (const char *yes,
-		 const char *members,
-		 char &f_guest,
-		 char &f_real,
-		 char &f_anonymous);
-	void settransfers (const char *members,
-		 const char *type);
-	void store (SSTRING&val,
-		 VIEWITEMS&items,
-		 const char *key);
-	void updatectrl (const char *oper,
-		 char f_guest,
-		 char f_real,
-		 char f_anonymous);
-	void updatetrans (void);
-public:
-	int write (void);
-	/*~PROTOEND~ WUFTPD */
-};
-
 /*
 	Grab one value for a line of a VIEWITEMS object
 */
@@ -242,6 +149,28 @@ PRIVATE void WUFTPD::load (SSTRING &val, VIEWITEMS &items, const char *key)
 		val.setfrom (pt);
 	}
 }
+/*
+	Grab one value for a line of a VIEWITEMS object
+*/
+PRIVATE void WUFTPD::load (
+	int &val,
+	int defval,
+	VIEWITEMS &items,
+	const char *key1,
+	const char *key2)
+{
+	val = defval;
+	VIEWITEM *it = items.locate (key1,key2);
+	if (it != NULL){
+		const char *pt = it->line.get();
+		pt = str_skip(pt);
+		pt += strlen(key1);
+		pt = str_skip (pt);
+		pt += strlen(key2);
+		pt = str_skip (pt);
+		val = atoi(pt);
+	}
+}
 
 /*
 	Update the VIEWITEMS object with a new value for a setting
@@ -261,6 +190,29 @@ PRIVATE void WUFTPD::store (SSTRING &val, VIEWITEMS &items, const char *key)
 		}
 	}
 }
+/*
+	Update the VIEWITEMS object with a new value for a setting
+*/
+PRIVATE void WUFTPD::store (
+	int val,
+	int defval,
+	VIEWITEMS &items,
+	const char *key1,
+	const char *key2)
+{
+	VIEWITEM *it = items.locate (key1,key2);
+	if (val == defval){
+		if (it != NULL) items.remove_del (it);
+	}else{
+		char buf[strlen(key1)+1+strlen(key2)+10+2];
+		sprintf (buf,"%s %s %d",key1,key2,val);
+		if (it != NULL){
+			it->line.setfrom (buf);
+		}else{
+			items.add (new VIEWITEM(buf));
+		}
+	}
+}
 /*
 	Parse a line like: rename yes guest,anonymous
 */
@@ -327,6 +279,8 @@ PRIVATE void WUFTPD::settransfers (const char *members, const char *type)
 PUBLIC WUFTPD::WUFTPD()
 {
 	rreal.setall (1);
+	rreal.f_log_inbound = 0;
+	rreal.f_log_outbound = 0;
 	access.read (f_access);
 	users.read (f_users);
 	groups.read (f_groups);
@@ -511,42 +465,131 @@ PRIVATE void WUFTPD::updatetrans ()
 	}
 }
 
+static const char K_EMAIL[]="email";
+static const char K_GUESTGROUP[]="guestgroup";
+static const char K_BANNER[]="banner";
+static const char K_SHUTDOWN[]="shutdown";
+static const char K_DEFAULTSERVER[]="defaultserver";
+static const char K_PRIVATE[]="private";
+static const char K_ACCEPT[]="accept";
+static const char K_CONNECT[]="connect";
+static const char K_IDLE[]="idle";
+static const char K_MAXIDLE[]="maxidle";
+static const char K_IDENT[]="RFC931";
+static const char K_TIMEOUT[]="timeout";
+static const char K_DATA[]="data";
+
+static int wuftpd_checkgroup (const char *groups)
+{
+	SSTRINGS tb;
+	str_splitline (groups,' ',tb);
+	SSTRING err;
+	for (int i=0; i<tb.getnb(); i++){
+		const char *g = tb.getitem(i)->get();
+		if (getgrnam(g)==NULL){
+			err.appendf (MSG_U(E_NOGROUP,"Group %s does not exist\n"),g);
+		}
+	}
+	int ret = 0;
+	if (err.is_filled()){
+		xconf_error ("%s",err.get());
+		ret = -1;
+	}
+	return ret;
+}
+
 PUBLIC void WUFTPD::editaccess()
 {
 	DIALOG dia;
 	dia.newf_title ("",MSG_U(T_BASE,"Base configuration"));
 	SSTRING email,guestgroup,banner,shutdown;
-	load (email,access,"email");
-	load (guestgroup,access,"guestgroup");
-	load (banner,access,"banner");
-	load (shutdown,access,"shutdown");
+	load (email,access,K_EMAIL);
+	load (guestgroup,access,K_GUESTGROUP);
+	load (banner,access,K_BANNER);
+	load (shutdown,access,K_SHUTDOWN);
+
+	VIEWITEM *defprivate = access.locate (K_DEFAULTSERVER,K_PRIVATE);
+	char allow_anonymous = defprivate == NULL ? 1 : 0;
 
 	dia.newf_title (MSG_U(N_MISC,"Misc"),1,"",MSG_U(T_MISC,"Misc."));
 	dia.newf_str (MSG_U(T_EMAIL,"Email of admin"),email);
+	int field_guestgroup = dia.getnb();
 	dia.newf_str (MSG_U(T_GUESTGROUP,"Guest groups"),guestgroup);
 	dia.newf_str (MSG_U(T_BANNER,"Banner file"),banner);
 	dia.newf_str (MSG_U(T_SHUTDOWN,"Shutdown message"),shutdown);
+	dia.newf_chk ("",allow_anonymous,MSG_U(I_ALLOWANON,"Allow anonymous access"));
+
 	dia.newf_title (MSG_U(N_ACCESSCTRL,"Control"),1,"",MSG_U(T_ACCESSCTRL,"Access controls"));
-	dia.newf_title ("",MSG_U(T_REALUSERS,"Real users"));
+	dia.newf_title (MSG_U(T_REALUSERS,"Real users"),2,"",MSG_R(T_REALUSERS));
 	setupaccess (dia,rreal);
-	dia.newf_title ("",MSG_U(T_GUESTUSERS,"Guest users"));
+	dia.newf_title (MSG_U(T_GUESTUSERS,"Guest users"),2,"",MSG_R(T_GUESTUSERS));
 	setupaccess (dia,guest);
-	dia.newf_title ("",MSG_U(T_ANONYMOUS,"Anonymous"));
+	dia.newf_title (MSG_U(T_ANONYMOUS,"Anonymous"),2,"",MSG_R(T_ANONYMOUS));
 	setupaccess (dia,anonymous);
+	dia.newf_title (MSG_U(N_TIMEOUTS,"Timeouts"),1,"",MSG_U(T_TIMEOUTS,"Various timeouts"));
+	int accept,connect,data,idle,maxidle,ident;
+	load (accept,120,access,K_TIMEOUT,K_ACCEPT);
+	load (connect,120,access,K_TIMEOUT,K_CONNECT);
+	load (data,1200,access,K_TIMEOUT,K_DATA);
+	load (idle,900,access,K_TIMEOUT,K_IDLE);
+	load (maxidle,1200,access,K_TIMEOUT,K_MAXIDLE);
+	load (ident,10,access,K_TIMEOUT,K_IDENT);
+	static const char *tbdef[]={MSG_U(I_DEFAULT,"Default"),NULL};
+	static const int tb120[]={120};
+	static const int tb1200[]={1200};
+	static const int tb900[]={900};
+	static const int tb10[]={10};
+	dia.newf_chkm_num (MSG_U(F_ACCEPT,"Accept connection"),accept,120
+		,tb120,tbdef);
+	dia.newf_chkm_num (MSG_U(F_CONNECT,"Port connection"),connect,120
+		,tb120,tbdef);
+	dia.newf_chkm_num (MSG_U(F_DATA,"Data transfer"),data,1200
+		,tb1200,tbdef);
+	dia.newf_chkm_num (MSG_U(F_IDLE,"Default idle time"),idle,900
+		,tb900,tbdef);
+	dia.newf_chkm_num (MSG_U(F_MAXIDLE,"Maximum idle time"),maxidle,1200
+		,tb1200,tbdef);
+	dia.newf_chkm_num (MSG_U(F_IDENT,"Auth/Ident"),ident,10
+		,tb10,tbdef);
+
 	int nof = 0;
+	dia.setbutinfo (MENU_USR1,MSG_U(B_EDITBANNER,"Edit banner"),MSG_R(B_EDITBANNER));
 	while (1){
 		MENU_STATUS code = dia.edit (MSG_U(T_FTPBASECONF
 			,"Ftp server configuration")
 			,""
 			,help_ftp
-			,nof);
+			,nof,MENUBUT_CANCEL|MENUBUT_ACCEPT|MENUBUT_USR1);
 		if (code == MENU_CANCEL || code == MENU_ESCAPE){
 			break;
+		}else if (code == MENU_USR1){
+			dia.save();
+			if (banner.is_empty()){
+				xconf_error (MSG_U(E_NOBANNERFILE
+					,"You must fill the field \"Banner file\""));
+			}else if (perm_rootaccess(MSG_U(P_EDITBANNER
+				,"edit the banner file"))){
+				wuftpd_editbanner (banner.get());
+			}
+		}else if (wuftpd_checkgroup(guestgroup.get())==-1){
+			nof = field_guestgroup;
 		}else{
-			store (email,access,"email");
-			store (guestgroup,access,"guestgroup");
-			store (banner,access,"banner");
-			store (shutdown,access,"shutdown");
+			store (email,access,K_EMAIL);
+			store (guestgroup,access,K_GUESTGROUP);
+			store (banner,access,K_BANNER);
+			store (shutdown,access,K_SHUTDOWN);
+			store (accept,120,access,K_TIMEOUT,K_ACCEPT);
+			store (connect,120,access,K_TIMEOUT,K_CONNECT);
+			store (data,1200,access,K_TIMEOUT,K_DATA);
+			store (idle,900,access,K_TIMEOUT,K_IDLE);
+			store (maxidle,1200,access,K_TIMEOUT,K_MAXIDLE);
+			store (ident,10,access,K_TIMEOUT,K_IDENT);
+			if (allow_anonymous){
+				access.remove_del(defprivate);
+			}else if (defprivate == NULL){
+				access.add (new VIEWITEM("defaultserver private"));
+			}
+			//store (defaultserver,access,"defaultserver");
 			updatectrl ("tar",guest.f_tar,rreal.f_tar,anonymous.f_tar);
 			updatectrl ("compress",guest.f_compress,rreal.f_compress,anonymous.f_compress);
 			updatectrl ("chmod",guest.f_chmod,rreal.f_chmod,anonymous.f_chmod);
@@ -569,7 +612,7 @@ static int cmp_by_addr (const ARRAY_OBJ *o1, const ARRAY_OBJ *o2)
 
 PUBLIC int VIRTUAL_FTP::createroot()
 {
-	const char *home_ftp = configf_lookuppath("/home/ftp");
+	const char *home_ftp = configf_lookuppath("/var/ftp");
 	char buf[1000];
 	snprintf (buf,1000,"%s %s"
 		,home_ftp
@@ -619,7 +662,7 @@ PUBLIC int VIRTUAL_FTP::edit()
 				char buf[1000];
 				snprintf (buf,1000,MSG_U(I_NEWROOTDIR
 					,"The directory for anonymous ftp does not exist.\n"
-					 "Do you want to create directory %s\n"
+					 "Do you want to create the directory %s\n"
 					 "and initialise it so it become suitable for\n"
 					 "anonymous ftp operation ?")
 					,root.get());
@@ -639,25 +682,27 @@ PUBLIC int VIRTUAL_FTP::edit()
 
 PUBLIC void VIRTUAL_FTPS::edit (VIEWITEMS &items)
 {
-	DIALOG *dia = NULL;
+	DIALOG_LISTE dia;
+	dia.newf_head ("",MSG_U(H_VIRTUALS,"Address\tRoot dir"));
 	int nof = 0;
+	VIRTUAL_FTPS sorted;
+	sorted.neverdelete();
 	while (1){
-		if (dia == NULL){
-			sort (cmp_by_addr);
-			dia = new DIALOG;
-			dia->newf_head ("",MSG_U(H_VIRTUALS,"Address\tRoot dir"));
-			for (int i=0; i<getnb(); i++){
-				VIRTUAL_FTP *f = getitem(i);
-				dia->new_menuitem (f->addr.get(),f->root.get());
-			}
+		sorted.remove_all();
+		for (int i=0; i<getnb(); i++) sorted.add (getitem(i));
+		sorted.sort (cmp_by_addr);
+		for (int i=0; i<getnb(); i++){
+			VIRTUAL_FTP *f = sorted.getitem(i);
+			dia.set_menuitem (i,f->addr.get(),f->root.get());
 		}
-		MENU_STATUS code = dia->editmenu(
+		dia.remove_last (getnb()+1);
+		MENU_STATUS code = dia.editmenu(
 			MSG_U(T_VIRTUALS,"Virtual hosts")
 			,MSG_U(I_ADMINTREES
 				,"Here are the various virtual hosts for anonymous ftp.")
 			,help_virtual
 			,nof,MENUBUT_ADD);
-		bool must_delete = false;
+		bool must_update = false;
 		if (code == MENU_QUIT || code == MENU_ESCAPE){
 			break;
 		}else if (code == MENU_ADD){
@@ -665,26 +710,23 @@ PUBLIC void VIRTUAL_FTPS::edit (VIEWITEMS &items)
 			int code = f->edit();
 			if (code == 0){
 				add (f);
-				must_delete = true;
+				must_update = true;
 			}else{
 				delete f;
 			}
 		}else if (nof >=0 && nof < getnb()){
-			VIRTUAL_FTP *f = getitem(nof);
+			VIRTUAL_FTP *f = sorted.getitem(nof);
 			int code = f->edit();
 			if (code != -1){
-				must_delete = true;
+				must_update = true;
 				if (code == 1) f->addr.setfrom ("");
 			}
 		}
-		if (must_delete){
-			delete dia;
-			dia = NULL;
+		if (must_update){
 			update (items);
 			items.write (f_access,NULL);
 		}
 	}
-	delete dia;
 }
 
 PUBLIC void WUFTPD::editvirtuals()
@@ -697,9 +739,11 @@ void wuftpd_edit()
 {
 	const char *basic = MSG_U(M_BASIC,"Basic configuration");
 	const char *virtuals = MSG_U(M_VIRTUAL,"Virtual hosts");
+	const char *logs = MSG_U(M_LOGS,"Transfer logs");
 	const char *menuopt[]={
 		"",		basic,
 		"",		virtuals,
+		"",		logs,
 		NULL
 	};
 	DIALOG_MENU dia;
@@ -720,6 +764,10 @@ void wuftpd_edit()
 				wu.editaccess();
 			}else if (key == virtuals){
 				wu.editvirtuals();
+			}else if (key == logs){
+				if (perm_rootaccess(MSG_U(P_VIEWLOG,"view transfer logs"))){
+					wu.viewlogs();
+				}
 			}
 		}
 	}
