@@ -347,7 +347,7 @@ PUBLIC int BOOTRC::run(bool probe)
 			ret = ctrl.startif();
 		}else if (!probe){
 			/* #Specification: sysv init script / weirdness
-				Sysv init script as discuss earlier are an inexpensive
+				Sysv init script as discussed earlier are an inexpensive
 				way of starting a service at boot time. Creating
 				init script is easy: Too easy.
 
@@ -474,7 +474,7 @@ PRIVATE void BOOTRCS::readequiv()
 	if (!distrib_isenhanced()){
 		char path[PATH_MAX];
 		linuxconf_fixdistdir (USR_LIB_CONF_SCRIPTS,path);
-		FILE *fin = f_equiv.fopen(path,"r");
+		FILE_CFG *fin = f_equiv.fopen(path,"r");
 		if (fin != NULL){
 			char buf[300];
 			while (fgets(buf,sizeof(buf)-1,fin)!=NULL){
@@ -505,7 +505,8 @@ PUBLIC void BOOTRCS::readdir(const char *dirpath, const char *prevpath)
 		tb.sort();
 		for (int i=0; i<n; i++){
 			const char *s = tb.getitem(i)->get();
-			if (strstr(s,".rpmsave")==NULL){
+			if (strstr(s,".rpmsave")==NULL
+				&& strstr(s,".rpmorig")==NULL){
 				const char *name = s + 3;
 				char path[PATH_MAX];
 				sprintf (path,"%s/%s",dirpath,s);
@@ -750,7 +751,7 @@ int bootrc_do (
 		at probe time (checking for configuration changes requiring
 		actions).
 	*/
-	FILE *fout = f_rcd.fopen ("w");
+	FILE_CFG *fout = f_rcd.fopen ("w");
 	if (fout != NULL){
 		fprintf (fout,"%s\n",dirpath);
 		fclose (fout);
