@@ -16,6 +16,7 @@
 #include "netconf.m"
 #include "internal.h"
 #include <dialog.h>
+#include <string.h>
 
 extern NETCONF_HELP_FILE help_control;
 static NETCONF_HELP_FILE help_dropin ("dropin");
@@ -227,7 +228,7 @@ PUBLIC int DROPIN::activate_new ()
 			 "   (%s)\n"
 			 "Do you want to enable it ?")
 			,getname(),desc.get());
-		MENU_STATUS code = xconf_yesno (
+		MENU_STATUS code = dialog_yesno (
 			MSG_U(T_ACTNEW,"Activate new service")
 			,buf,help_actnew);
 		if (code==MENU_YES){
@@ -257,11 +258,23 @@ PUBLIC void RCSYSV::init_command()
 	cmd.cstop.reinit(cmd.stop);
 	cmd.creload.reinit(cmd.reload);
 	cmd.cboot.reinit(cmd.boot);
-	for (int j=0; j<processes.getnb(); j++){
+	int nbproc = processes.getnb();
+	if (nbproc > MAXSYSV_PROCESS){
+		nbproc = MAXSYSV_PROCESS;
+		fprintf (stderr,"Too many processes in sysv init script %s, max. %d\n"
+			,name.get(),MAXSYSV_PROCESS);
+	}
+	for (int j=0; j<nbproc; j++){
 		cmd.cprocess[j].reinit(*processes.getitem(j));
 	}
 	deletepids();
-	for (int i=0; i<pidfiles.getnb(); i++){
+	nbcfgpid = pidfiles.getnb();
+	if (nbcfgpid > MAXSYSV_PROCESS){
+		nbcfgpid = MAXSYSV_PROCESS;
+		fprintf (stderr,"Too many PID files in sysv init script %s, max. %d\n"
+			,name.get(),MAXSYSV_PROCESS);
+	}
+	for (int i=0; i<nbcfgpid; i++){
 		CONFIG_FILE *cfg = new CONFIG_FILE (pidfiles.getitem(i)->get()
 			,help_pidfile
 			,CONFIGF_PROBED|CONFIGF_OPTIONAL);
@@ -720,6 +733,22 @@ PUBLIC int RCSYSV::boot()
 	return ret;
 }
 
+/*
+	Record the fact that the service was restarted/reloaded.
+	Record the fact for every process managed by the service.
+*/
+PRIVATE void RCSYSV::setrestarttime()
+{
+	if (!simul_ison()){
+		daemon_loadrestartdb();
+		for (int i=0; i<processes.getnb(); i++){
+			SSTRING *s = processes.getitem(i);
+			daemon_setrestarttime (s->get());
+		}
+		daemon_saverestartdb();
+	}
+}
+
 PUBLIC int RCSYSV::startif()
 {
 	int ret = 0;
@@ -785,6 +814,9 @@ PUBLIC int RCSYSV::startif()
 			}
 		}
 		if (tbtasks.getnb() > 0){
+			net_prtlog (NETLOG_WHY,MSG_U(I_PROBECMD
+				,"The following command told me something had to be done\n"));
+			net_prtlog (NETLOG_WHY,"%s\n",buf);
 			for (int i=0; i<tbtasks.getnb() && ret == 0; i++){
 				const char *verb = tbtasks.getitem(i)->get();
 				if (strcmp(verb,"start")==0){
@@ -792,8 +824,10 @@ PUBLIC int RCSYSV::startif()
 				}else if (strcmp(verb,"stop")==0){
 					stop();
 				}else if (strcmp(verb,"restart")==0){
+					setrestarttime();
 					restart();
 				}else{
+					setrestarttime();
 					snprintf (buf,sizeof(buf)-1,"%s %s",cmd.probe.get()
 						,verb);
 					ret = netconf_system (15,buf);
@@ -893,7 +927,7 @@ PUBLIC int DROPIN::edit(const SERVICES &tb)
 		those file should never be modified by the end user (the admin).
 		This allows reliable replacement during a package update.
 
-		Further the overrides stores in /etc/conf.modules are partial. If
+		Further the overrides stored in /etc/modules.conf are partial. If
 		for example, you have changed the way a package is started by
 		changing the start command, only this change will be stored in
 		/etc/conf.linuxconf. If the package is upgrade later, the stop
@@ -1010,7 +1044,9 @@ PUBLIC DROPINS::DROPINS(bool _localconf)
 	int nb = dir_getlist(ETC_LINUXCONF_CONTROL,tb);
 	for (int i=0; i<nb; i++){
 		const char *name = tb.getitem(i)->get();
-		if (strcmp(name,".")!=0 && strcmp(name,"..")!=0){
+		if (strcmp(name,".")!=0
+			&& strcmp(name,"..")!=0
+			&& strstr(name,".OLD")==NULL){
 			add (new DROPIN (name,_localconf));
 		}
 	}
