@@ -6,6 +6,7 @@
 #include <sys/socket.h>
 #include <netdb.h>
 #include "netconf.h"
+#include "hostinfo.h"
 #include <userconf.h>
 #include "internal.h"
 #include <misc.h>
@@ -27,9 +28,9 @@ static CONFIG_FILE f_max_aliases (PROC_SYS_NET_MAX_ALIAS
 	,CONFIGF_OPTIONNAL|CONFIGF_MANAGED);
 
 
-PRIVATE void IP_ALIAS::init (int _num, const char *_ip, const char *_mask)
+PRIVATE void IP_ALIAS::init (const char *_id, const char *_ip, const char *_mask)
 {
-	num = _num;
+	id.setfrom (_id);
 	ip.setfrom (_ip);
 	mask.setfrom (_mask[0] == '\0' ? "255.255.255.255" : _mask);
 }
@@ -41,20 +42,20 @@ PRIVATE void IP_ALIAS::init (int _num, const char *_ip, const char *_mask)
 	ones.
 */
 
-PUBLIC IP_ALIAS::IP_ALIAS(int _num, const char *_ip, const char *_mask)
+PUBLIC IP_ALIAS::IP_ALIAS(const char *_id, const char *_ip, const char *_mask)
 {
-	init (_num,_ip,_mask);
+	init (_id,_ip,_mask);
 }
 
 PUBLIC IP_ALIAS::IP_ALIAS(const char *_ip, const char *_mask)
 {
-	init (-1,_ip,_mask);
+	init ("",_ip,_mask);
 }
 
 PUBLIC int IP_ALIAS::unset (const char *devname)
 {
 	char cmd[100],devali[20];
-	sprintf (devali,"%s:%d",devname,num);
+	sprintf (devali,"%s:%s",devname,id.get());
 	if (kernel_newer (2,2,0)){
 		sprintf (cmd,"%s down",devali);
 	}else{
@@ -63,6 +64,30 @@ PUBLIC int IP_ALIAS::unset (const char *devname)
 	net_printhint ("del %s\n",devali);
 	return netconf_system_if ("ifconfig",cmd);
 }
+/*
+	Return the numeric ID of an alias (eth0:N -> 0).
+	Return -1 if the ID is not numerical
+*/
+PUBLIC int IP_ALIAS::getnumid()
+{
+	return hastextid() ? -1 : id.getval();
+}
+
+/*
+	Return true if the IP alias has a textual (non numeric ID).
+*/
+PUBLIC bool IP_ALIAS::hastextid()
+{
+	bool ret = false;
+	const char *s = id.get();
+	if (*s == '\0'){
+		ret = true;
+	}else{
+		while (isdigit(*s)) s++;
+		if (*s != '\0') ret = true;
+	}
+	return ret;
+}
 
 PUBLIC int IP_ALIAS::set (int _num, const char *devname)
 {
@@ -128,16 +153,16 @@ PUBLIC IP_ALIAS *IP_ALIASES::getitem(const char *ip, const char *mask)
 PUBLIC void IP_ALIASES::readproc (const char *devname)
 {
 	int len = strlen(devname);
-	FILE *fin = f_aliases.fopen ("r");
+	FILE_CFG *fin = f_aliases.fopen ("r");
 	if (fin != NULL){
 		char buf[300];
 		while (fgets(buf,sizeof(buf)-1,fin)!=NULL){
 			if (strncmp(buf,devname,len)==0
 				&& buf[len] == ':'){
-				int id;
+				char id[100];
 				int family;
 				char ip[100];
-				sscanf (buf+len+1,"%d %d %s",&id,&family,ip);
+				sscanf (buf+len+1,"%s %d %s",id,&family,ip);
 				/* #Specification: netconf / aliases / 0.0.0.0
 					The file /proc/net/aliases contains entries
 					with an IP number of 0.0.0.0. Theses entries
@@ -147,7 +172,7 @@ PUBLIC void IP_ALIASES::readproc (const char *devname)
 				*/
 				if (strcmp(ip,"0.0.0.0")!=0){
 					char alidev[20];
-					sprintf (alidev,"%s:%d",devname,id);
+					sprintf (alidev,"%s:%s",devname,id);
 					IFCONFIG_INFO info;
 					if (ifconfig_getinfo_nocheck(alidev,info)!=-1){
 						add (new IP_ALIAS(id,ip,info.netmask));
@@ -165,8 +190,7 @@ PUBLIC void IP_ALIASES::readproc (const char *devname)
 				IFCONFIG_INFO info;
 				if (ifconfig_getinfo_nocheck(devali,info)!=-1){
 					if (info.ip_addr[0] != '\0'){
-						int id = atoi(devali+len+1);
-						add (new IP_ALIAS(id,info.ip_addr,info.netmask));
+						add (new IP_ALIAS(devali+len+1,info.ip_addr,info.netmask));
 					}
 				}
 			}
@@ -183,7 +207,8 @@ PUBLIC int IP_ALIASES::getmaxnum()
 	int n = getnb();
 	for (int i=0; i<n; i++){
 		IP_ALIAS *a = getitem(i);
-		if (a->num > ret) ret = a->num;
+		int num = a->getnumid();
+		if (num > ret) ret = num;
 	}
 	return ret;
 }
@@ -320,7 +345,7 @@ static bool ipalias_available()
 			number). If it is not there, it calls modprobe to load the
 			module ip_alias.
 		*/
-		FILE *fin = f_alias_types.fopen("r");
+		FILE_CFG *fin = f_alias_types.fopen("r");
 		if (fin == NULL){
 			xconf_error (MSG_U(E_NOALIAS,"No kernel support for devices aliasing"));
 		}else{
@@ -369,7 +394,7 @@ PUBLIC int IP_ALIASES::setup (const char *devname)
 	*/
 	if (!kernel_newer (2,2,0)){
 		max_kernel_aliases = 256;
-		FILE *fin = f_max_aliases.fopen ("r");
+		FILE_CFG *fin = f_max_aliases.fopen ("r");
 		if (fin != NULL){
 			fscanf (fin,"%d\n",&max_kernel_aliases);
 			fclose (fin);
@@ -392,7 +417,7 @@ PUBLIC int IP_ALIASES::setup (const char *devname)
 		IP_ALIAS *ap = proc.getitem(a->ip.get(),a->mask.get());
 		if (ap != NULL){
 			// Alias is already installed
-			lookup[ap->num] = 1;
+			lookup[ap->getnumid()] = 1;
 		}else{
 			toadd.add(a);
 		}
@@ -400,11 +425,23 @@ PUBLIC int IP_ALIASES::setup (const char *devname)
 	ret = 0;
 	n = proc.getnb();
 	for (i=0; i<n; i++){
+		/* #Specification: ip aliases / delete non configured ones
+			Linuxconf will add new IP aliases and delete unconfigured ones.
+			It is sometime handy to setup IP aliases on demand for other
+			needs, for example, when doing virtual servers. Kernel 2.2
+			and up supports IP alias devices with names (eth0:test).
+			Linuxconf simply ignore the aliases configured this way.
+
+			Ultimatly, we could have a preference: Don't remove un-configured
+			aliases.
+		*/
 		IP_ALIAS *a = proc.getitem(i);
-		IP_ALIAS *ap = getitem(a->ip.get(),a->mask.get());
-		if (ap == NULL){
-			// Alias is already installed and must be removed
-			ret |= a->unset (devname);
+		if (!a->hastextid()){
+			IP_ALIAS *ap = getitem(a->ip.get(),a->mask.get());
+			if (ap == NULL){
+				// Alias is already installed and must be removed
+				ret |= a->unset (devname);
+			}
 		}
 	}
 	n = toadd.getnb();
@@ -444,7 +481,7 @@ PUBLIC int IP_ALIASES::setup (const char *devname)
 							,"Raising kernel max aliases per device limit to %d\n")
 							,max_kernel_aliases);
 					}else{
-						FILE *fout = f_max_aliases.fopen ("w");
+						FILE_CFG *fout = f_max_aliases.fopen ("w");
 						if (fout != NULL){
 							max_kernel_aliases = total_needed+100;
 							net_prtlog (NETLOG_CMD,MSG_R(I_NEWMAXALIASES)
@@ -574,7 +611,11 @@ int ipalias_setup ()
 {
 	int ret = -1;
 	SSTRINGS list;
-	if (devlist_read (list) != -1){
+	HOSTS hosts;
+	NETWORKS networks;
+	HOSTINFO info;
+	if (netconf_loadinfos(hosts,networks,info) != -1
+		&& devlist_read (list) != -1){
 		SSTRING devreconf;
 		int n = list.getnb();
 		ret = 0;
@@ -582,10 +623,23 @@ int ipalias_setup ()
 			SSTRING *s = list.getitem(i);
 			net_enableprinthint (false);
 			const char *dev = s->get();
-			ret |= ipalias_setup (dev);
-			if (net_washints()){
-				if (!devreconf.is_empty()) devreconf.append (" ");
-				devreconf.append (dev);
+			INTER_INFO *ptdev = info.a;
+			// We must make sure the device is enabled
+			// If the device is not configured through linuxconf, then
+			// we don't know if it is enable or not, so we give it a try
+			bool ok = true;
+			for (int j=0; j<NB_ETH; j++,ptdev++){
+				if (ptdev->device.cmp(dev)==0){
+					ok = ptdev->enable != 0;
+					break;
+				}
+			}
+			if (ok){
+				ret |= ipalias_setup (dev);
+				if (net_washints()){
+					if (!devreconf.is_empty()) devreconf.append (" ");
+					devreconf.append (dev);
+				}
 			}
 			net_enableprinthint (true);
 		}
