@@ -12,7 +12,7 @@ static CONFIG_FILE f_netlevel (VAR_RUN_NETLEVEL,help_netlevel
 */
 void netconf_setnetlevel(int level)
 {
-	FILE *fout = f_netlevel.fopen ("w");
+	FILE_CFG *fout = f_netlevel.fopen ("w");
 	if (fout != NULL){
 		fprintf (fout,"%d\n",level);
 		fclose (fout);
@@ -25,7 +25,7 @@ void netconf_setnetlevel(int level)
 int netconf_getnetlevel()
 {
 	int level=2;
-	FILE *fin = f_netlevel.fopen ("r");
+	FILE_CFG *fin = f_netlevel.fopen ("r");
 	if (fin != NULL){
 		fscanf (fin,"%d\n",&level);
 		fclose (fin);
