@@ -28,6 +28,7 @@
 #include <module.h>
 #include <module_api.h>
 #include <module_apis/servicectl_api.h>
+#include <string.h>
 
 NETCONF_HELP_FILE help_control ("control");
 static const char INTERNSERV[]="INTERNSERV";
@@ -362,7 +363,6 @@ static void netconf_runlevel0_internal(
 	net_title (MSG_U(S_FIXPERM,"Checking files permissions"));
 	fixperm_check();
 	net_title (MSG_U(S_LILO,"Checking LILO"));
-	lilo_update();
 	char msg[10000];
 	if (!netconf_netok(msg) || netconf_sethostname() == -1){
 		xconf_error (MSG_U(E_IVLBASIC
@@ -496,7 +496,6 @@ void netconf_s00linuxconf(int level)
 	net_title (MSG_R(S_FIXPERM));
 	fixperm_check();
 	net_title (MSG_R(S_LILO));
-	lilo_update();
 	module_probe (0,level);
 }
 /*
@@ -614,7 +613,7 @@ void netconf_runlevel(
 {
 	BOOTRCS rcs (true);
 	extern CONFIG_FILE f_rcd;
-	FILE *fin = f_rcd.fopen ("r");
+	FILE_CFG *fin = f_rcd.fopen ("r");
 	if (fin != NULL){
 		char buf[PATH_MAX];
 		if (fgets(buf,sizeof(buf)-1,fin)!=NULL){
@@ -653,6 +652,7 @@ void service_control()
 					,s->getrunstatus());
 				dia.set_menuitem (j,name,buf);
 			}					   
+			dia.remove_last (services.getnb()+1);
 		}
 		MENU_STATUS code = dia.editmenu (MSG_U(T_SERVICECTL,"Service control")
 		,MSG_U(I_SERVICECTL,"You can selectivly enable or disable\n"
