@@ -25,6 +25,7 @@ public:
 	SSTRING archivecmd;
 	SSTRING postdeletecmd;
 	char showshadow;
+	char showexpire;
 	char mayedithome;
 	char mayeditshell;
 	char mayeditquota;
@@ -37,37 +38,7 @@ public:
 	/*~PROTOEND~ PASSWD_VALID */
 };
 
-class SHADOW: public ARRAY_OBJ{
-	friend class SHADOWS;
-	friend class USER;
-	friend class USERS;
-	SSTRING name;
-	SSTRING passwd;
-	int last;
-	int may;
-	int must;
-	int warn;
-	int expire;
-	int disable;
-	SSTRING reserved;
-	/*~PROTOBEG~ SHADOW */
-public:
-	SHADOW (const char *line);
-	SHADOW (void);
-	int getdisable (void)const;
-	int getexpire (void)const;
-	int getmaychange (void)const;
-	int getmustchange (void)const;
-	const char *getpwd (void)const;
-	int getwarn (void)const;
-private:
-	void initdef (void);
-public:
-	void setdialog (DIALOG&dia);
-	void write (FILE *fout);
-	int writedef (void);
-	/*~PROTOEND~ SHADOW */
-};
+class SHADOW;
 
 class SHADOWS: public ARRAY{
 	CONFIG_FILE *configf;
