@@ -66,6 +66,7 @@ PRIVATE void USER::init(
 	wrkdir.setfrom (_dir);
 	shell.setfrom (_shell);
 	special = !user_shells_isok(_shell);
+	oldname.setfrom (name);
 }
 
 PUBLIC USER::USER(
@@ -103,6 +104,8 @@ int user_splitline (const char *line, char words[9][100])
 */
 PUBLIC USER::USER(const char *line)
 {
+	enabled = 1;
+	was_enabled = 1;
 	char words[9][100];
 	user_splitline (line,words);
 	name.setfrom (words[0]);
@@ -113,6 +116,7 @@ PUBLIC USER::USER(const char *line)
 	wrkdir.setfrom (words[5]);
 	shell.setfrom (words[6]);
 	special = !user_shells_isok(words[6]);
+	oldname.setfrom (name);
 }
 PUBLIC USER::USER(struct passwd *p)
 {
@@ -140,7 +144,7 @@ PUBLIC const char *USER::gethome() const
 /*
 	Write one record of /etc/passwd
 */
-PUBLIC void USER::write(FILE *fout)
+PUBLIC void USER::write(FILE_CFG *fout)
 {
 	fprintf (fout,"%s:%s:%d:%d:%s:%s:%s\n"
 		,name.get(),passwd.get(),uid,gid,comment.get()
@@ -153,6 +157,13 @@ PUBLIC const char *USER::getname() const
 {
 	return name.get();
 }
+/*
+	Return the original login name of the user.
+*/
+PUBLIC const char *USER::getoldname() const
+{
+	return oldname.get();
+}
 /*
 	Return the long informative name of the user.
 */
@@ -257,7 +268,7 @@ PUBLIC bool USER::is_admin() const
 	Return != 0 if this user account is a special non user account
 	generally intended for machine to machine connection like uucp.
 */
-PUBLIC int USER::is_special() const
+PUBLIC bool USER::is_special() const
 {
 	return special;
 }
@@ -623,10 +634,11 @@ static void user_fixhomedia (
 	bool &chown_recur)
 {
 	DIALOG dia;
+	dia.settype (DIATYPE_POPUP);
 	char sel = 1;
 	dia.newf_radio ("",sel,0,MSG_U(F_DONOTHING,"Do nothing"));
 	dia.newf_radio ("",sel,1,MSG_U(F_CHOWNDIR,"Change ownership of the directory"));
-	dia.newf_radio ("",sel,2,MSG_U(F_CHOWNRECUR,"Change ownership recursivly"));
+	dia.newf_radio ("",sel,2,MSG_U(F_CHOWNRECUR,"Change ownership recursively"));
 	do_sethome = false;
 	chown_recur = false;
 	do_nothing = false;
@@ -679,6 +691,7 @@ PUBLIC int USER::edit(
 	bool editprivi = (may_edit & USRACCT_EDITPRIV) != 0;
 	bool editgroup = (may_edit & USRACCT_EDITGROUP) != 0;
 	bool editshell = (may_edit & USRACCT_EDITSHELL) != 0;
+	bool is_main_domain = strcmp(domain,"/")==0;
 	int categ = getcateg();
 	bool editsupgrp = (may_edit & USRACCT_EDITSUPGRP) != 0
 						&& categ == TUSER_STD;
@@ -797,6 +810,7 @@ PUBLIC int USER::edit(
 	bool override = users.has_add_override();
 	if (is_new && !override) uid = users.getnewuid();
 	if (uid != -1) struid.setfrom (uid);
+	int field_uid = dia.getnb();
 	if (is_root || policies_mayedithome()){
 		dia.newf_str (MSG_U(F_UID,"User ID(opt)"),struid);
 	}
@@ -836,10 +850,13 @@ PUBLIC int USER::edit(
 			disable_str.setfrom (buf);
 		}
 		disable_field = dia.getnb();
+
+	}
+	if (users.has_shadow() && policies_mayshowexpire()){
 		dia.newf_str (MSG_U(F_WASISDIS,"Expiration date (yyyy/mm/dd)"),disable_str);
 	}
 	comngs.setupdia (dia);
-	if (is_root || policies_mayeditquota()){
+	if ((is_root || policies_mayeditquota()) && is_main_domain){
 		qedit.setupdia (dia,MSG_U(T_DISKQUOTA,"Disk quota"));
 	}
 	PRIVILEGE_DATAS privs;
@@ -856,7 +873,7 @@ PUBLIC int USER::edit(
 			,MSG_U(X_PASSWD,"Passwd"));
 		butopt |= MENUBUT_USR1;
 	}
-	if ((is_root || policies_mayedittasks()) && strcmp(domain,"/")==0){
+	if ((is_root || policies_mayedittasks()) && is_main_domain){
 		dia.setbutinfo (MENU_USR2,MSG_U(B_TASKS,"Tasks")
 			,MSG_U(X_TASKS,"Tasks"));
 		butopt |= MENUBUT_USR2;
@@ -867,7 +884,7 @@ PUBLIC int USER::edit(
 				? MSG_U(T_NEWUSER,"User account creation")
 				: MSG_U(T_USERINFO,"User information")
 			,MSG_U(I_USERINTRO
-			 ,"You must specify at least the name\n"
+			 ,"You must specify at least the login name\n"
 			  "and the full name")
 			,help_user
 			,field
@@ -882,7 +899,11 @@ PUBLIC int USER::edit(
 				user_setaltgr (tbaltgr,altgr);
 			}
 		}else if (code == MENU_DEL){
-			if (deldialog(deloper)){
+			if (is_main_domain && is_logged()){
+				xconf_error (MSG_U(E_ISLOGGED
+					,"User is currently logged.\n"
+					 "The account can't be deleted."));
+			}else if (deldialog(deloper)){
 				ret = 1;
 				break;
 			}
@@ -927,6 +948,11 @@ PUBLIC int USER::edit(
 			}else if (groups.setalt(name.get(),altgr.get(),' ',true)==-1){
 				xconf_error (MSG_U(E_IVLDSUPGROUP,"Invalid supplementary group list"));
 				field = supgroup_field;
+			}else if (!struid.is_empty()
+				&& (struid.getval()<0 || struid.getval()>65535)){
+				xconf_error (MSG_U(E_UIDRANGE
+					,"User ID must be between 0 and 65535"));
+				field = field_uid;
 			}else{
 				if (group.is_empty() && !override){
 					group.setfrom (name);
@@ -941,7 +967,7 @@ PUBLIC int USER::edit(
 							,group.get());
 						if (name.cmp(group)==0 && privgroup){
 							//create_group = true;
-						}else if (xconf_yesno (MSG_U(T_CREATGROUP
+						}else if (dialog_yesno (MSG_U(T_CREATGROUP
 							,"Create group")
 							,buf,help_nil)!= MENU_YES){
 							continue;
@@ -977,7 +1003,7 @@ PUBLIC int USER::edit(
 							status.append(MSG_U(Q_CREATE
 								,"\nDo you want to create it ?"));
 							if (is_new
-								|| xconf_yesno(
+								|| dialog_yesno(
 								MSG_U(Q_USERHOME,"User home directory")
 								,status.get(),help_nil)==MENU_YES){
 								do_sethome = true;
@@ -1258,7 +1284,7 @@ PUBLIC int USER::editpass(
 {
 	int ret = -1;
 	if (may_override && perm_fct_change != NULL){
-		ret = (*perm_fct_change)(getname(),1);
+		ret = (*perm_fct_change)(getname(),true);
 		if (ret == -1){
 			xconf_error (MSG_U(E_PASSCHG,"Password was not changed"));
 		}
@@ -1266,6 +1292,7 @@ PUBLIC int USER::editpass(
 		int nof = 0;
 		while (1){
 			DIALOG dia;
+			dia.settype (DIATYPE_POPUP);
 			SSTRING buf1;
 			dia.newf_pass (MSG_U(F_PASSWORD,"Password"),buf1);
 			SSTRING buf2;
