@@ -1,3 +1,4 @@
+#include <utmp.h>
 #include "userconf.h"
 #include "internal.h"
 
@@ -41,10 +42,12 @@ PUBLIC int USER::getcateg() const
 */
 PUBLIC bool USER::is_like (const USER *other) const
 {
-	int other_categ = other == NULL ? TUSER_STD : other->getcateg();
-	bool ret = getcateg()==other_categ;
-	if (ret && other != NULL){
+	bool ret = false;
+	if (other != NULL){
 		ret = gid == other->gid;
+	}else{
+		int other_categ = other == NULL ? TUSER_STD : other->getcateg();
+		ret = getcateg()==other_categ;
 	}
 	return ret;
 }
@@ -64,3 +67,22 @@ PUBLIC void USER::setlike (const USER *other)
 	}
 }
 
+/*
+	Tell if the user is currently logged
+*/
+PUBLIC bool USER::is_logged()
+{
+	bool ret = false;
+	setutent();
+	struct utmp *u;
+	while ((u=getutent())!=NULL){
+		if (u->ut_type == USER_PROCESS
+			&& name.cmp(u->ut_user)==0){
+			ret = true;
+			break;
+		}
+	}
+	endutent();
+	return ret;
+}
+
