@@ -56,7 +56,7 @@ PRIVATE void USERS::readusers()
 	*/
 	nisentry = NULL;
 	nis_at_end = true;
-	FILE *fin = configf->fopen ("r");
+	FILE_CFG *fin = configf->fopen ("r");
 	if (fin != NULL){
 		char line[1000];
 		while (fgets(line,sizeof(line)-1,fin)!=NULL){
@@ -87,6 +87,7 @@ PUBLIC USERS::USERS()
 	configf = &f_passwd;
 	readusers();
 	if (shadow_exist()) shadows = new SHADOWS;
+	maxusers = 0;
 }
 
 PUBLIC USERS::USERS(
@@ -104,6 +105,16 @@ PUBLIC USERS::USERS(
 	configf = &_file;
 	readusers();
 	if (_shadow.exist()) shadows = new SHADOWS(_shadow);
+	maxusers = 0;
+}
+
+/*
+	Set the maximum users in this domain.
+	0 means no limit.
+*/
+PUBLIC void USERS::setmaxusers(int mx)
+{
+	maxusers = mx;
 }
 
 /*
@@ -292,7 +303,7 @@ PUBLIC int USERS::write(PRIVILEGE *priv)
 	int ret = -1;
 	//sortbygid();
 	if (configf != NULL){
-		FILE *fout = configf->fopen (priv,"w");
+		FILE_CFG *fout = configf->fopen (priv,"w");
 		if (fout != NULL){
 			int nbu = getnb();
 			if (nisentry != NULL && !nis_at_end) nisentry->write(fout);
@@ -311,18 +322,42 @@ PUBLIC int USERS::write(PRIVILEGE *priv)
 /*
 	Write the user accounts only if override maintenance function
 	are not in place, since these function do the modification themselves.
+
+	If the user account override function are in place, this means that
+	other processes have updated the passwd database, so our own
+	copy if invalid. Instead of writing, we reload
 */
-PUBLIC int USERS::writeif (PRIVILEGE *priv)
+PUBLIC int USERS::writeif (PRIVILEGE *priv, USER *&usr)
 {
 	int ret = -1;
 	if (user_fct_mod != NULL && may_override()){
+		SSTRING id;
+		char enabled =1;
+		if (usr != NULL){
+			id.setfrom (usr->getname());
+			enabled = usr->enabled;
+		}
 		reload();
+		if (usr != NULL){
+			usr = getitem(id.get());
+			if (usr == NULL){
+				xconf_notice ("Programming error:\n"
+					"userconf/users.cc:writeif usr==NULL");
+			}else{
+				usr->enabled = enabled;
+			}
+		}
 		ret = 0;
 	}else{
 		ret = write (priv);
 	}
 	return ret;
 }
+PUBLIC int USERS::writeif (PRIVILEGE *priv)
+{
+	USER *pt = NULL;
+	return writeif (priv,pt);
+}
 
 struct USRACC_FILTER{
 	SSTRING prefix;		// Prefix for filtering the list
@@ -429,9 +464,6 @@ PRIVATE USER *USERS::select_sorted(
 	/* do some primitive boundary checks */
 	myfrom = (filter.from < 1)  ? 1 : filter.from;
 	myto   = (filter.to < myfrom) ? myfrom: filter.to;
-	#if 0 /* dont need this I think... */
-	myto   = (myto > 65535) ? 65535 : myto;
-	#endif
 	/* #Specification: userconf / user account / root bin ...
 		Some special account are simply left out of the configuration
 		menu. These account are never edited. They make the list larger
@@ -576,6 +608,9 @@ PRIVATE int USERS::override (
 	}else if (perm_access(priv,MSG_U(P_USERDB
 		,"to maintain the user database"))){
 		ret = (*fct)(usr,getshadow(usr));
+		if (ret != 0){
+			net_showlastlog();
+		}
 	}
 	return ret;
 }
@@ -652,12 +687,16 @@ PRIVATE int USERS::editone_precut (
 	if (code != -1){
 		GROUP *g = groups.getfromgid(usr->getgid());
 		const char *group = g != NULL ? g->getname() : "";
+		SSTRING id (usr->getname());	// We must take a copy
+										// because the usr object may be deleted
+										// later.
 		const char *tbmessage[]={
-			usr->getname(),domain.get()
+			id.get(),domain.get()
 		};
 		if (code == 0){
 			if (is_new){
-				if (docreate(usr,priv,group) != -1){
+				if (docreate(usr,priv,group) != -1
+					&& writeif (priv,usr) != -1){
 					module_sendmessage ("newuser",2,tbmessage);
 					if (usr->enabled){
 						char info[100];
@@ -670,7 +709,9 @@ PRIVATE int USERS::editone_precut (
 				}
 			}else{
 				net_introlog (NETINTRO_MODIFYUSER);
-				override (priv,usr,user_fct_mod);
+				if (override (priv,usr,user_fct_mod)!=-1){
+					module_sendmessage ("chguser",2,tbmessage);
+				}
 			}
 		}else if (code == 1){
 			if (is_new){
@@ -680,7 +721,7 @@ PRIVATE int USERS::editone_precut (
 				override (priv,usr,user_fct_del);
 				remove_del (usr);
 			}else{
-				if (xconf_yesno (MSG_U(E_NOTDEL,"Account not deleted")
+				if (dialog_yesno (MSG_U(E_NOTDEL,"Account not deleted")
 					,MSG_U(I_NOTDEL
 						,"User account was not deleted\n"
 						 "because the post-deletion command failed\n"
@@ -690,7 +731,7 @@ PRIVATE int USERS::editone_precut (
 					==MENU_YES){
 					net_showlog();
 				}
-				if (xconf_yesno(MSG_U(E_DELANYWAY,"Delete anyway")
+				if (dialog_yesno(MSG_U(E_DELANYWAY,"Delete anyway")
 					,MSG_U(I_DELANYWAY
 						,"Would you like to delete this account\n"
 						 "even if the post-deletion command failed ?")
@@ -708,11 +749,11 @@ PRIVATE int USERS::editone_precut (
 			if (user_fct_mod == NULL) groups.write();
 			// The qedit.save must be done after the /etc/passwd is saved
 			if (code == 0){
-				qedit.save(priv);
+				if (domain.cmp("/")==0) qedit.save(priv);
 				if (is_new) runcreatecmd(&tmp,group);
 				comngs.save (priv);
 			}else if (code == 1){
-				qedit.deluser(priv);
+				if (domain.cmp("/")==0) qedit.deluser(priv);
 				comngs.deluser(priv);
 			}
 		}
@@ -752,7 +793,7 @@ PUBLIC int USERS::editone (
 			ret = usr->editpass(getshadow(usr),true,may_override()
 				,domain.get());
 			if (ret != -1){
-				writeif(priv);
+				writeif(priv,usr);
 			}
 		}
 	}else{
@@ -776,12 +817,23 @@ PUBLIC int USERS::addone (
 						// or NULL if only root can do this
 	unsigned may_edit)	// Which field may be edited
 {
-	USER *user = new USER;
-	user->setlike (special);
-	user->setname (name);
-	user->setgecos (fullname);
-	return editone (user,true,priv,may_edit);
-	
+	int ret = -1;
+	/* 
+	   Don't add the user if it would exceed the objects maxusers limit.
+	   Used for virtual domains with user limit (maxusers=0 means unlimited).
+	*/
+	if ((maxusers != 0) && (getnb() >= maxusers)) {
+		xconf_error(MSG_U(E_MAXUSERS
+		    ,"Adding user would exceed the userlimit for this domain\n(%d>%d) - aborting.")
+			,getnb()+1,maxusers);
+	}else{
+		USER *user = new USER;
+		user->setlike (special);
+		user->setname (name);
+		user->setgecos (fullname);
+		ret = editone (user,true,priv,may_edit);
+	}
+	return ret;
 }
 
 PUBLIC void USERS::remove_del (USER *usr)
@@ -934,7 +986,7 @@ int users_editone(
 			char buf[300];
 			sprintf (buf,MSG_U(I_USERCREATE,"User account %s does not exist\n"
 				 "Do you want to create it"),name);
-			if (xconf_yesno(MSG_U(Q_USERCREATE,"Account creation")
+			if (dialog_yesno(MSG_U(Q_USERCREATE,"Account creation")
 				,buf,help_users)==MENU_YES){
 				ret = users_addone	(name,group,fullname);
 			}
@@ -990,7 +1042,8 @@ PUBLIC int USERS::addbatch (
 	const char *id,
 	const char *group,
 	const char *name,
-	const char *shell)
+	const char *shell,
+	bool locked)		// This account will be created locked
 {
 	int ret = -1;
 	USER *u = getitem(id);
@@ -1017,10 +1070,17 @@ PUBLIC int USERS::addbatch (
 			shadow->name.setfrom (id);
 			addshadow (shadow);
 		}
-		u->update_passwd (NULL,shadow,true,domain.get());
 		if (u->sethome (NULL,group,false,*this) != -1){
 			if (docreate (u,NULL,group) != -1
-				&& write (NULL) != -1){
+				&& writeif (NULL,u) != -1){
+				if (locked){
+					u->update_passwd (NULL,shadow,true,domain.get());
+					writeif (NULL,u);
+				}
+				const char *tbmessage[]={
+					u->getname(),domain.get()
+				};
+				module_sendmessage ("newuser",2,tbmessage);
 				if (domain.cmp("/")==0) quota_applyone (id);
 				runcreatecmd (u,group);
 				ret = 0;
@@ -1033,7 +1093,7 @@ PUBLIC int USERS::addbatch (
 /*
 	Delete a user account, no question asked
 */
-PUBLIC int USERS::delbatch (const char *id)
+PUBLIC int USERS::delbatch (const char *id, USER_DELOPER oper)
 {
 	int ret = -1;
 	USER *u = getitem(id);
@@ -1042,13 +1102,20 @@ PUBLIC int USERS::delbatch (const char *id)
 			,id);
 	}else{
 		GROUPS groups;
-		if (dodel (u,DELOPER_ARCHIVE,groups,NULL)==0){
+		if (dodel (u,oper,groups,NULL)==0){
+			SSTRING name (u->getname());
 			override (NULL,u,user_fct_del);
 			remove_del (u);
 			ret = writeif(NULL);
-			if (ret != -1 && domain.cmp("/")==0){
-				QUOTA_EDIT qedit (id,QUOTA_USER);
-				qedit.deluser(NULL);
+			if (ret != -1){
+				const char *tbmessage[]={
+					name.get(),domain.get()
+				};
+				module_sendmessage ("deluser",2,tbmessage);
+				if (domain.cmp("/")==0){
+					QUOTA_EDIT qedit (id,QUOTA_USER);
+					qedit.deluser(NULL);
+				}
 			}
 		}else{
 			fprintf (stderr,MSG_U(E_PREDELFAIL
@@ -1069,28 +1136,43 @@ int users_add (
 	const char *shell)
 {
 	USERS users;
-	return users.addbatch (id,group,name,shell);
+	return users.addbatch (id,group,name,shell,true);
 }
 
 /*
 	Delete one user account
 */
-int users_del (const char *id)
+int users_del (const char *id, USER_DELOPER oper)
 {
 	USERS users;
-	return users.delbatch (id);
+	return users.delbatch (id,oper);
 }
 
-
-#ifdef TEST
-
-int main (int argc, char *argv[])
+static void users_edituser (const char *v, bool setting)
 {
-	users_edit(NULL);
-	users_edit("-");
-	users_edit("/usr/lib/uucp/uucico");
+	users_editone (v,NULL,NULL);
 }
 
-#endif
+static void users_list (SSTRINGS &tb)
+{
+	USERS users;
+	for (int i=0; i<users.getnb(); i++){
+		USER *u = users.getitem(i);
+		tb.add (new SSTRING (u->getname()));
+	}
+}
+
+#include <modregister.h>
+static PUBLISH_VARIABLES_MSG user_var_list1[]={
+	{"home",P_MSG_R(F_HOME)},
+	{"shell",P_MSG_R(F_SHELL)},
+	{"enable",P_MSG_R(I_ENABLED)},
+	{"gecos",P_MSG_R(F_FULLNAME)},
+	{"uid",P_MSG_R(F_UID)},
+	{ NULL, NULL }
+};
+
+static REGISTER_VARIABLES user_registry1("user","users",user_var_list1
+	,NULL,users_edituser,users_list);
 
 
