@@ -299,6 +299,7 @@ static int fld_select(char *mbox, int host_fld)
 	struct stat stat_buf;
 	FILE *mboxfp;
 	int lock;
+	int fd;
 
 	/* Reset folder variables */
 	fld_hostmbox = host_fld;
@@ -318,7 +319,7 @@ static int fld_select(char *mbox, int host_fld)
 
 	/* Create/save mailbox names */
 	strcpy(fld_fname, POP3_TMPFILE);
-	if (mktemp(fld_fname) == NULL)
+	if ((fd = mkstemp(fld_fname)) == -1)
 		return -1;
 	
 	/* Secure mailbox for POP3 session; copy to temporary file */
@@ -333,7 +334,7 @@ static int fld_select(char *mbox, int host_fld)
 		close(lock);
 		return -1;
 	}
-	if ((fld_fp = fopen(fld_fname,"w")) == NULL) {
+	if ((fld_fp = fdopen(fd,"w")) == NULL) {
 		fclose(mboxfp);
 		flock(lock, LOCK_UN);
 		close(lock);
@@ -584,8 +585,10 @@ static void retr_bsmtp(int msgnum, int linecnt)
 		if ((msgbody)&&(--linecnt == 0)) {
 			break;
 		} else {
-			if (*svr_buf == CR_CHAR)
+			if (*svr_buf == CR_CHAR){
 				msgbody = 1;
+				if (linecnt == 0) break;
+			}
 		}
 	}
 }
@@ -622,8 +625,10 @@ static void retr_fromsp(int msgnum, int linecnt)
 		if ((msgbody)&&(--linecnt == 0)) {
 			break;
 		} else {
-			if (*cp == CR_CHAR)
+			if (*cp == CR_CHAR){
 				msgbody = 1;
+				if (linecnt == 0) break;
+			}
 		}
 	}
 }
@@ -772,11 +777,12 @@ void fld_release()
 			fail(FAIL_FILE_ERROR);
 		temp_fname[0] = '\0';
 	} else {
+		int fd;
 		/* Copy from the original mailbox to a temp file */
 		strcpy(temp_fname, POP3_TMPFILE);
-		if (mktemp(temp_fname) == NULL)
+		if ((fd = mkstemp(temp_fname)) == -1)
 			fail(FAIL_FILE_ERROR);
-		if ((tempfp = fopen(temp_fname,"w")) == NULL)
+		if ((tempfp = fdopen(fd,"w")) == NULL)
 			fail(FAIL_FILE_ERROR);
 
 		while (fgetl(svr_buf,SVR_BUFSIZ,mboxfp) != NULL) {
