@@ -3,20 +3,31 @@
 
 #define MAXHLINE	200
 
-struct MAILADR{
+class MAILADR{
+public:
 	char adr[MAXHLINE+1];
 	char name[MAXHLINE+1];
+	/*~PROTOBEG~ MAILADR */
+public:
+	MAILADR (void);
+	/*~PROTOEND~ MAILADR */
 };
 
-struct MAILHEADER {
+class MAILHEADER {
+public:
 	int nbreceived;
-	struct MAILADR from;
-	struct MAILADR to;
+	MAILADR from;
+	MAILADR to;
 	char reply[MAXHLINE+1];
 	char subject[MAXHLINE+1];
 	char msgid[MAXHLINE+1];
 	char replyid[MAXHLINE+1];
 	char date[MAXHLINE+1];
+	char priority[MAXHLINE+1];
+	/*~PROTOBEG~ MAILHEADER */
+public:
+	MAILHEADER (void);
+	/*~PROTOEND~ MAILHEADER */
 };
 
 #include "mailhead.p"
